#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This is a top level script whose purpose is to configure a Data
# Collector.  The steps are as follows:
#
#   1) Create the PCL and PDL to hold the persistent data.
#      The PCL is the data dictionary.  It holds the name,
#      description, initial value and prompt message key for
#      all common data items.
#
#   2) Apply any relevant environment variables to the PDL.
#      For each data item there may be zero or more environment
#      variables which may set the value of the data item.
#
#   3) Interpret the command line flags and put the results in the PDL.
#      For each data item there may be zero or more flags (command
#      line options) which set the value of the data item.
#
#   4) Handle the silent install file by reading it in and updating the
#      PDL with the options it specifies.
#
#   5) Make various inquiries to the operating system and prompts to
#      the user to set the final state of the PDL, such as running
#      findServers.  Routines which update the value of data items
#      in the PDL are also responsible for setting a tag to indicate
#      the source of the data item value.
#
#   6) Create the dcinput.txt file for non-z/OS platforms.
#
#   7) Create and populate the runtime server instance for z/OS.
#
#   8) Tailor the input.properties files (per server).
#
#   9) Run configDataCollector, passing the input.properties files.
#
#####################################################################

'''
    Configure the data collector for ITCAM/AD
'''

fixlev = "40296" # Last APAR, defect or story to update this module

import os
import sys
import platform
import zipfile
import traceback
import codecs

# Outermost exception handler
try:

    # Eliminate need to prefix import our modules with "lib."
    sys.path.append("lib")

    # Jython 2.5.2 compatibility
    _Jython_vrm = os.sys.version_info[0:3]
    assert _Jython_vrm >= (2,5,2), "Unsupported Jython version %s" % ".".join([str(_vrm) for _vrm in _Jython_vrm])

    # Java 1.5 compatibility
    _Java_vrm = [int(_vrm) for _vrm in platform.java_ver()[0].split(".")[0:2]]
    assert _Java_vrm >= [1,5], "Unsupported Java JVM version %s" % ".".join([str(_vrm) for _vrm in _Java_vrm])

    import createPCL_config as createPCL
    import createPDL
    import get_wasroot
    import tcpipstack
    import populateFindServers
    import createservers
    import inputproperties
    import configDC
    import platformUtil
    import configtema
    import configms
    import configtt
    import configsoa
    import configsmf
    import configgc
    import configtpv
    import configde
    import configdcless
    import wasbackup
    import display_config
    import configmaxheap
    import silentInputFileProcessor
    import configplugin
    import createDCInput
    import configzos
    import checkprobes
    import configUtils
    import configconnection
    import configproxy
    import findconfigservers
    import installSRVPGMs
    import checkversion
    import utils

    from constants import constants
    from ConfigurationException import ConfigurationException

    import resourcebundle
    from resourcebundle import getmsg

    import debugUtil as log
    from debugUtil import logDebug as debug

    from ConsoleWriter import ConsoleWriter
    from PrintWrapper import PrintWrapper

    # Exception handler for expected exceptions such
    # as ConfigurationException, KeyboardException and SystemExit.
    try:

        #############
        # Initialize
        #############

        # Instantiate the global constants class
        common_constants = constants()

        # Set the print wrapper for stdout and stderr.
        PrintWrapper(common_constants.STDOUT)
        PrintWrapper(common_constants.STDERR)

        # Initialize the resource bundle
        resourcebundle.set_resourcebundle()

        # There should be no debug() statement before this point
        debug("Entry config %s" % fixlev)

        debug("  Jython version: %s" % ".".join([str(_vrm) for _vrm in os.sys.version_info[0:3]]))
        debug("  Java version: %s" % platform.java_ver()[0])
        debug("  Java home: %s" % os.environ.get("JAVA_HOME"))

        # Get the approximate build timestamp from the install bundle zip directory
        try:
            zipFilename = None

            # Try environment
            if zipFilename is None:
                if "ITCAM_CONFIGHOME" in os.environ:
                    zipFilename = os.path.normpath(os.path.join(os.environ["ITCAM_CONFIGHOME"], "bin", "lib", "installbundle.jar"))
                    if not os.path.isfile(zipFilename):
                        debug("Install bundle file %s not found from ITCAM_CONFIGHOME" % zipFilename)
                        zipFilename = None

            # Try command line
            if zipFilename is None:
                if len(sys.argv) > 0 and sys.argv[0]:
                    zipFilename = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), os.pardir, "bin", "lib", "installbundle.jar"))
                    if not os.path.isfile(zipFilename):
                        debug("Install bundle file %s not found from sys.argv[0]" % zipFilename)
                        zipFilename = None

            # Try current working directory
            if zipFilename is None:
                zipFilename = os.path.normpath(os.path.join(os.getcwd(), "lib", "installbundle.jar"))
                if not os.path.isfile(zipFilename):
                    debug("Install bundle file %s not found from os.getcwd()" % zipFilename)
                    zipFilename = None

            if zipFilename is not None:
                zipObject = zipfile.ZipFile(zipFilename,"r")
                zipInfos = zipObject.infolist()
                zipObject.close()
                if len(zipInfos) > 0: debug("  Build timestamp: %04i-%02i-%02i %02i:%02i:%02i" % zipInfos[0].date_time)
            else: debug("Unable to locate installbundle.jar, build timestamp not available.")

        except Exception, errmsg:
            debug("Error encountered getting build timestamp:\n\n%s" % traceback.format_exc())
            debug("Execution continuing")

        #########################################
        # Create the PDL and insert data from
        # environment and command line arguments
        #########################################

        pdl = createPDL.make_pdl(createPCL.make_pcl())

        log.logDumpPCL(pdl["PCL"][0], "Initial creation of PCL with %d entries" % len(pdl["PCL"][0]))
        log.logDumpPDL(pdl, "Initial creation of PDL with %d entries" % len(pdl), short=False)

        createPDL.proc_envvar(pdl)
        createPDL.proc_cliopt(pdl,sys.argv)
        
        ttOfferingFileName = pdl["CONFIGHOME"][0] + os.sep + "offering.id"
        debug("ttOfferingFileName = %s " % ttOfferingFileName)
        propsdict=utils.getProperties(ttOfferingFileName)        
        if propsdict is None or type(propsdict) is not dict:
            propsdict={'IOFFERING':'IM'}
            debug("  product offering is defaulted to: IM ")
        createPDL.proc_propsdict(pdl,propsdict)            

        reconfigFlag = pdl["RECONFIG"][0]

        ############################
        # Display a startup message
        ############################

        if reconfigFlag:
            logbase = "reconfig"
            print "\n\n%s\n" % getmsg("FLOWERBOX_RECONFIG","Reconfigure Data Collector")
        else:
            logbase = "config"
            print "\n\n%s\n" % getmsg("FLOWERBOX_CONFIG","Configure Data Collector")

        # Perhaps all we are going to do is display the help text.
        if pdl["HELP"][0]:
            log.set_logging(False)
            print "%s\n" % getmsg("HELP_CONFIG")
            sys.exit(0)

        # Default log and message file names
        if pdl["LOG"][0] is None:
            pdl["LOG"][0:2] = logbase+"-trace.log", "config"
        if pdl["MESSAGE"][0] is None:
            pdl["MESSAGE"][0:2] = logbase+"-message.log", "config"

        # Default the config home.
        if pdl["CONFIGHOME"][0] is None:
            if len(sys.argv) > 0 and sys.argv[0]:
                cwd = os.path.dirname(os.path.abspath(sys.argv[0]))
                debug("Current working directory based on sys.arg[0]: %s" % cwd)
            else:
                cwd = os.path.abspath(os.getcwd())
                debug("Current working directory based on os.getcwd(): %s" % cwd)
            if os.path.basename(cwd) == "bin":
                config_home = os.path.join(cwd, os.pardir)
                debug("Configuration home default: %s" % config_home)
                pdl["CONFIGHOME"][0:2] = config_home, "config"
            else:
                raise ConfigurationException(getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home"))

        # Normalize the configuration home
        config_home = os.path.normpath(pdl["CONFIGHOME"][0])
        if config_home != pdl["CONFIGHOME"][0]:
            debug("Configuration home normalized to: %s" % config_home)
            pdl["CONFIGHOME"][0:2] = config_home, "config"

        debug("reconfigFlag: %s" % reconfigFlag)


        # Initialize the log utility and interrogate version.properties.
        log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
        if log.isLogBad():
            print "%s" % getmsg("CFG5027W", "Unable to open log file {0}, diagnostic output will be directed to the console.", [pdl["LOG"][0]])
        else:
            # Redirect sys.stdout
            if pdl["LOGCONSOLE"][0] is True:
                log_file = log.getLogFilename()
                log_dir = os.path.dirname(log_file)
                console_wtr = ConsoleWriter(os.path.join(log_dir, logbase+"-console.log"))

                # Start with the same flower box printed to console above

                if reconfigFlag:
                    console_wtr.log("%s" % getmsg("FLOWERBOX_RECONFIG","Reconfigure Data Collector"))
                else:
                    console_wtr.log("%s" % getmsg("FLOWERBOX_CONFIG","Configure Data Collector"))

        configUtils.logVersionProperties(pdl)

        print "%s" % getmsg("CFG5043I", "Log file name: {0}", [log.getLogFilename()])

        # Get the product name
        # Get the current version of this release

        product = getmsg("ITCAMAD","ITCAM for Application Diagnostics")
        version = checkversion.get_current_version(pdl)
        if version == "-1":
            raise ConfigurationException(getmsg("CFG1070E", "Invalid version number - {0}", [version]))
        pdl["VERSION"][0:2] = version, "config"

        feature_version = checkversion.get_feature_version(pdl)
        if feature_version == "-1":
            raise ConfigurationException(getmsg("CFG1070E", "Invalid liberty feature version number - {0}", [feature_version]))
        pdl["ITCAMFEATUREVERSION"][0:2] = feature_version, "config"

        log.logDumpPDL(pdl, "PDL after environment variables, command line arguments and initialization")

        ########################
        # Get Silent Properties
        ########################

        if pdl["SILENT"][0]:
            debug("Silent configuration file is supplied: %s" % pdl["SILENT"][0])
            if reconfigFlag:
                raise ConfigurationException(getmsg("CFG1139E", "Reconfig is not supported in silent mode."))
            else:
                files = silentInputFileProcessor.processFile(pdl,pdl["SILENT"][0],2)
                log.logDumpPDL(pdl, "PDL after silent configuration file applied")

        # Determine the local host name
        tcpipstack.get_iphost(pdl)

        probes = False
        if reconfigFlag:
            haveServersFlag = findconfigservers.find_configservers(pdl, common_constants.CONFIG)
            if not haveServersFlag:
                raise ConfigurationException(getmsg("CFG1089I", "No application servers previously configured with version {0} of {1} found, nothing to do.", [version, product]))
            else:

                # Set the values from env variable again, because, the reconfig is reading the values
                # from input.properties and sets the default values for each of the prompts, so in the
                # case of batch mode which uses the env variables or command arguments, the env variables
                # and command arguments are read again and replaced in the pdl to be the current values.

                if pdl["BATCH"][0]:
                    createPDL.proc_envvar(pdl)
                    createPDL.proc_cliopt(pdl,sys.argv)

                # Check each probes, and determine if the server needs to be notified or not.

                probes = findconfigservers.find_configuredprobes(pdl, common_constants.RECONFIG)

        ##############################################
        # Install services programs if this is OS/400
        ##############################################

        if platformUtil.IS_OS400_OS():
            srvpgmRC = installSRVPGMs.installSRVPGMs()
            # Error message has already been issued by installSRVPGMs() so just exit abnormally
            if srvpgmRC > 0:
                sys.exit(1)

        ###################
        # Identify servers
        ###################

        # For not reconfiguration case, invoke findconfigservers.py
        # If we are reconfiguring we already have the servers

        if reconfigFlag:
            server_found = True
        else:
            # Prompt for WAS location and resolve it
            get_wasroot.get_wasroot(pdl)
            # Call findServers
            server_found = populateFindServers.populateFindServers(pdl)

        # Dump the contents of the PDL.
        log.logDumpPDL(pdl,"PDL environment and command line options applied", findservers=True)

        # In the case of silent, all the servers that are provided in the silent file are
        # checked to make sure that the servers are not already configured by older or newer
        # version.

        if server_found and pdl["SILENT"][0]:
            server_found = silentInputFileProcessor.check_servers(pdl)

        if server_found:

            # At this point we need a loop to allow reprompting after the summary is displayed.
            while True:

                # Prompt for servers and details
                # If reconfigFlag is true, then skip prompt for servers and details;
                # instead, we use input.properties to populate servers

                if not reconfigFlag:

                    createservers.process_servers(pdl)

                    # If server has already been configured, then check its probe settings
                    probes = findconfigservers.find_configuredprobes(pdl)

                    # Set connection data in pdl
                    configconnection.set_connection(pdl)

                else:

                    # Get the current JVM mode now so it displays in display_config.
                    if platformUtil.IS_zOS_OS():
                        configzos.set_jvmmode(pdl)

                # Configure SOA DC (not for OS400)
                if not platformUtil.IS_OS400_OS():
                    configsoa.config_soa(pdl)

                # Configure Tema
                configtema.config_tema(pdl)

                # Configure Managing Server
                configms.config_ms(pdl)

                # Config TTAPI
                continue_config = configtt.config_tt(pdl)
                if not continue_config:
                    raise ConfigurationException(getmsg("CFG1998I", "Configuration of Data Collector has been bypassed"))

                # Configure TPV
                configtpv.config_tpv(pdl)

                # Configure DE
                configde.config_de(pdl)

                # Configure SMF
                configsmf.config_smf(pdl)

                # Configure GC
                configgc.config_gc(pdl)

                # Configure max heap
                if pdl["SILENT"][0]:
                    configmaxheap.config_maxheap(pdl)

                # Display a configuration summary page and prompt for next action.
                doConfig = display_config.display(pdl)
                if doConfig:
                    break

            # Fix selected file names
            configUtils.fixFilenames(pdl)

            # Dump the contents of the now-completed PDL.
            log.logDumpPDL(pdl,"findServers has been applied to the PDL", findservers=False)

            ###############################################################################
            # At this point the data in the PDL should be complete.  We are ready to start
            # creating the runtimes.
            ###############################################################################

            if not pdl["BYPASSCONFIG"][0] and pdl["RETURNCODE"][0] == 0:

                # Perform any server modifications
                if reconfigFlag or not pdl["NOCONFIGDC"][0]:

                    # Prompt for Websphere backup and backup current configuration if requested
                    continue_config = wasbackup.do_backup(pdl)

                    if not continue_config:
                        raise ConfigurationException(getmsg("CFG1998I", "Configuration of Data Collector has been bypassed"))

                    # If z/OS, execute platform specific configuration tasks
                    if platformUtil.IS_zOS_OS():
                        configzos.config_zos(pdl)

                    # Config plugin
                    configplugin.process_plugin(pdl)

                    # The runtime directory will be created at DC startup, based on the manual inputs file.
                    createDCInput.createManualInputFile(pdl)

                    # Create input.properties files
                    inputproperties.process_inputproperties(pdl)

                    # Call configDataCollector if not bypassed and no return code has been set
                    probe_serverindex_list = None
                    assert isinstance(pdl["RETURNCODE"][0],int), "Invalid return code has been set"
                    if not pdl["BYPASSCONFIG"][0] and pdl["RETURNCODE"][0] == 0:

                        # Check the probe values for any changes.  If there are changes, the server index
                        # that needs to be notified is acquired.

                        if probes:
                            debug("Checking servers for probe changes... ")
                            #print "\n%s" % getmsg("CFG2022I", "Checking servers for probe changes... ")
                            probe_serverindex_list = checkprobes.probe_checking(pdl)
                            if len(probe_serverindex_list) > 0:
                                debug("Found servers with modified probes, processing probes.... ")

                        # Do the configuration.

                        configDC.configDataCollector(pdl, probe_serverindex_list)

                        # Configure port consolidator scripts if there is an MS connection

                        configproxy.config_proxyserverctrl_script(pdl, True)

                    # If z/OS, update access permisssions and ownership
                    if platformUtil.IS_zOS_OS():
                        configUtils.update_runtime_access_zOS(pdl)

                # Create DC-less JMX server connection
                # (This is after other configuration in hopes
                # the user and password have been validated.)

                if reconfigFlag or pdl["DCLESS"][0]:
                    configdcless.makeruntime(pdl)

            if pdl["BYPASSCONFIG"][0]:
                print "%s" % getmsg("CFG1997I", "Configuration of Data Collector has been bypassed at user's request")


        # Check for return code that has been set to non-zero.
        # Assume that an error messages has been written at that time that RETURNCODE was set.
        if pdl["RETURNCODE"][0] != 0:
            print "\n%s" % getmsg("CFG5045E", "The configuration process is being terminated due to errors listed above")
            sys.exit(1)

        # If we get here everything should be okay, and we exit with RC(0)
        sys.exit(0)

    # This exception is raised when a problem is detected which may possibly
    # be corrected by the user.
    except ConfigurationException, instance:

        print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["config.py"])
        print "%s" % instance.parameter

        log.logException("Configuration error in config.py")
        log.logDumpStack("Stack data for diagnostic purposes")

        # Default log file name because ConfigurationException might be raised
        # before the the log has been set.
        if pdl["LOG"][0] is None:
            pdl["LOG"][0:2] = "config-trace.log", "config"
        # Initialize the log utility.
        log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0])
        if log.isLogBad():
            print "%s" % getmsg("CFG5027W", "Unable to open log file {0}, diagnostic output will be directed to the console.", [pdl["LOG"][0]])

        sys.exit(1)

    # assert during execution, detected internal program error
    except AssertionError:

        print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["config.py"])

        log.logException("Assertion error in config.py")
        log.logDumpStack("Stack data for diagnostic purposes")
        sys.exit(1)

    # User entered terminate character in prompt
    except KeyboardInterrupt:

        print "%s" % getmsg("CFG1998I", "Configuration of Data Collector has been bypassed")
        print "%s" % getmsg("CFG1088I", "Exiting configuration script")
        sys.exit(0)

    # For sys.exit(0) we will quietly exit and suppress dumping the log to the console.
    # For sys.exit(1) or indeed otherwise, we presume an erroneous situation and diagnose.
    except SystemExit, exitcode:

        if exitcode.code == 0: sys.exit(0)
        print "%s" % getmsg("CFG1088I", "Exiting configuration script")
        sys.exit(1)


######################################################
# This is the last-ditch attempt to gather diagnostic
# information for unexpected failures.
######################################################

except SystemExit, exitcode:

    # Suppress trace for intentional system exit
    if exitcode.code == 0: sys.exit(0)

    # In case logging was never started, we need to dump the log buffer and set final exit code.
    log.set_logging()
    sys.exit(1)

except:

    # In unlikely case an exception happened before imports were
    # completed, we need to do some minimal imports to insure
    # we can put out diagnostic information.

    import debugUtil as log
    from resourcebundle import getmsg as getmsg

    _exctype, _excval, _exctb = sys.exc_info()

    print
    try:
        print "%s" % getmsg("CFG1002E", "Unexpected exception detected in config.py")
    except Exception:
        print "Unexpected exception detected in config.py"
    print

    try:
        log.logException("Error in main routine")
        log.logDumpStack("Stack data for diagnostic purposes")
    except Exception:
        _type, _val, _tb = sys.exc_info()
        print
        try:
            print "%s" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "Logging routine failed, traceback follows"
        print
        traceback.print_exception(_type,_val,_tb)
        print
        print "Original traceback"
    traceback.print_exception(_exctype,_excval,_exctb)
    print

    # In case logging was never started, we need to dump the log buffer.
    log.set_logging()
    sys.exit(1)
