###########################################################################   
# Licensed Materials - Property of IBM
#
# 5724-L62    
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.     
#  
############################################################################
# This module contains functions that create the ITCAM for AD 7.2
# configuration directories used for z/OS. 
#############################################################################
    
import sys
import os
import shutil
import time
import traceback
from time import strftime

import java
from java.lang import System

sys.path.append("lib")
import debugUtil 
import debugUtil as log
import configUtils
import shared_globals as globals
import createPCL_createcfg as createPCL
import createPDL as createPDL
import utils   
import resourcebundle
from resourcebundle import getmsg as getmsg
import doprompt as prompt
from ConfigurationException import ConfigurationException


fixlev  = "HAAD720 AK00000"


launching_script = "createcfg.sh"
createcfg_script = "createcfg.py"


# User related information:
login_user = "" 
user_name  = "" 
user_group = "" 

# Temp log if we crash before log set
tmplog = "/tmp/itcam72_createcfg.log"

####################################################
#   print_help
####################################################
def _print_help():           
    """
        Display the help page
    """
    help_text= \
    """   
    
*******************************************************
     Create configuration options
*******************************************************

This script creates a configuration home directory.
You need to run this script only once. Multiple Data
Collectors can share the same configuration home.


Command line syntax is:

  createcfh.sh  [-batch]
                [-debug]
                [-config  <config home>]
                [-install <install home>]
                [-owner   <Userid[:group]>]
                [-access  <permissions>]
                [-help]

       -batch      turns off prompting (alias: -noprompt)
       -debug      debugging messages will be produced
       -config     sets runtime configuration home directory
       -install    sets install home directory
       -owner      sets ownership of the created directories
       -access     sets permissions to the created directories
       -help       displays this message and quits

       ITCAM_BATCH   Sets default for -batch, value is y or n
       ITCAM_DEBUG   Sets default for -debug, value is y or n
       ITCAM_CONFIG  Sets default for -config (alias CONFIG_HOME)
       ITCAM_INSTALL Sets default for -install
       ITCAM_OWNER   Sets default for -owner, such as such as wsadmin
       ITCAM_ACCESS  Sets default for -access, such as 664            
    """                                                                        

    msg = getmsg("HELP_CREATECFG", help_text)
    print msg      
               
#####################################################################
# Configure the configuration home directories.                     
#####################################################################

def _do_config(pdl):
    """
        Creates and populates the configuration directories.
    """
    log.logDebug("do_config entry")

    #
    # Initialization
    #
    # 1. Obtain the needed variables from the pdl
    config_home     = pdl["CONFIGHOME"][0]
    install_home    = pdl["INSTALLHOME"][0]
    itcam_owner     = pdl["OWNER"][0]
    itcam_access    = pdl["ACCESS"][0]
    LOG             = pdl["LOG"][0]
     # 2. log the input values
    log.logDebug("  config_home="     + str(config_home) )
    log.logDebug("  install_home="    + str(install_home) )
    log.logDebug("  itcam_owner="     + str(itcam_owner) )
    log.logDebug("  itcam_access="    + str(itcam_access) )

    if itcam_owner  is None or itcam_owner  == 'none': itcam_owner  = ""
    if itcam_access is None or itcam_access == 'none': itcam_access = ""
 
    #
    # 1. Check if config_home exists 
    #    Clear the Jython class files
    #
    if not os.path.isdir(config_home):
        success, msg = _create_config_home_dir(config_home)
        if not success:
            print "%s" % getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home")
            raise SystemExit
    else:
        log.logDebug("  Configuration home exists: " + config_home)
        clear_jython_class_files(pdl)

    #
    # 2. Populate the configuration home directory
    #
    log.logDebug(" Populate config home directory: %s" % config_home)

    ermsg=""
    bindir        = config_home + '/bin'
    binlibdir     = config_home + '/bin/lib'
    binjythondir  = config_home + '/bin/jython'
    logsdir       = config_home + '/logs' 
    datadir       = config_home + '/data' 
    runtimedir    = config_home + '/runtime'
    customdir     = config_home + '/runtime/custom'
    itcam_properties_file = config_home + '/itcam.properties'
    itcamdcdir    = config_home + '/itcamdc'
    toolkitdir    = config_home + '/toolkit'
    pluginsdir    = config_home + '/plugins'
    gdcdir        = config_home + '/gdc'
    ttdcdir       = config_home + '/ttdc'
    soadcdir      = config_home + '/soadc'

    #  a) bin and bin/lib directories ok ?
    dirs = [bindir, binlibdir, binjythondir]  
    for dir in dirs:
        if  configUtils.isLink(dir):
            ermsg=ermsg + "\nSymbolic link to %s directory exists in config_home" % (os.path.basename(dir))
        elif os.path.exists(dir)  and  not  os.path.isdir(dir):
            ermsg=ermsg + "\nA \"bin\" exists in config_home which is not a directory"

    #   b) runtime and runtime\custom ok ?
    dirs = [runtimedir, customdir ]
    for dir in dirs:
        if os.path.exists(dir) and not os.path.isdir(dir):
            ermsg=ermsg + "\nA \"" + os.path.basename(dir) + "\" exists in config_home which is not a directory"

    #   c) itcam.properties file ok ?  
    if configUtils.isLink(itcam_properties_file):
        ermsg=ermsg + "\nSymbolic link to itcam.properties file exists in config_home"
    elif os.path.exists(itcam_properties_file) and not os.path.isfile(itcam_properties_file):
        ermsg=ermsg + "\nAn \"itcam.properties\" exists in %s and is not a file" % ( config_home )

    #   d) symlink dirs ok ?
    dirs = [itcamdcdir, toolkitdir, pluginsdir, gdcdir, ttdcdir, soadcdir ]
    for dir in dirs:
        if os.path.exists(dir) and not configUtils.isLink(dir):
            dirname=dir.replace(config_home+'/','')
            ermsg=ermsg + "\nA \"" + dirname  + "\" exists in config_home which is not a symbolic link"
 
    # 3. Check for an old-format directory structure. 
    if  os.path.isfile(install_home + '/itcam.properties'):
        ermsg=ermsg + "\nThe itcam.properties file exists in install_home instead of config_home"
    elif  os.path.exists(install_home + '/itcam.properties'):
        ermsg=ermsg + "\nAn \"itcam.properties\" object exists in install_home"

    if configUtils.isLink(install_home + '/runtime'):
        ermsg=ermsg + "\nThere is a symbolic link to the runtime directory in install_home"
    elif os.path.exists(install_home + '/runtime'):
        ermsg=ermsg + "\nAn \"runtime\" object exists in install_home"

    if ermsg != "":
        print getmsg("CFG3015E", "The following directory problems were found: {0}", [ermsg])
        print getmsg("CFG3016I","Correct the problems and rerun the script")
        raise SystemExit
 
    # 5. Create the configuration home directories
    for ent in [runtimedir, customdir, bindir, binlibdir, binjythondir, logsdir, datadir  ]:
        log.logDebug("  Creating configuration home directories")
        if not os.path.isdir(ent):
            if os.path.exists(ent):
                print getmsg("CFG3017E","Directory object {0} already exists in {1}", [ent, config_home])
                print getmsg("CFG3028I","Remove the file and rerun the script")
                raise SystemExit
            os.mkdir(ent)
            if not os.path.isdir(ent):
                msg = getmsg("CFG5021E", "Failed to create {0}", [ent])
                print "%s" % msg
                raise SystemExit 
            rc = os.system('chmod a+rwX ' + ent)
            log.logDebug("  Successfully created %s subdirectory" % ent)
        else:
            log.logDebug("  Reusing existing %s subdirectory." % ent)

    # 6. Create the configuration home symbolic links
    log.logDebug("  Creating config home symbolic links")
    for ent in [itcamdcdir, toolkitdir, pluginsdir, gdcdir, ttdcdir , soadcdir]:
        log.logDebug("    directory name=" + ent)
        failed=0
        if  configUtils.isLink(ent):
            os.rmdir(ent)
            if  configUtils.isLink(ent):
                print getmsg("CFG3026W","Unable to remove an existing symbolic link {0}",[ent])
                continue
        elif os.path.exists(ent):
            print getmsg("CFG3027E","{0} object already exists in {1}", [ent , config_home])
            print getmsg("CFG3028I","Remove the file and rerun the script")
            sys.exit(0) 
              
        ent_install_dir=ent.replace(config_home,install_home)
        cmd="ln -s %s %s"   %( ent_install_dir,   ent )
        rc=configUtils.issueOSCommand(cmd)
        if rc == 0:
            log.logDebug("  Successfully created %s symbolic link." % ent )
        else:
            log.logDebug("  Failed to create %s symbolic link." % ent )
            assert False, "  Create symlink failed for  %s." % ent
    
    # 7. Create the file symbolic links in bin, bin/lib, and bin/jython
    log.logDebug("  Configuring symlinks for directory scripts")
    scripts_in_list = ['/bin', '/bin/lib', '/bin/jython' ]
    for subdir in scripts_in_list:
        _create_symbolic_links(subdir, install_home, config_home)
    
    # 8. Edit-in the setmode.cntl file
    install_file = install_home + "/bin/setmode.cntl"
    config_file  = config_home  + "/bin/setmode.cntl"
    log.logDebug("  Configuring setmode.cntl from install dir: " + install_file)
    if os.path.exists(install_file):
        substitutionsList = ['@{ITCAMDCHOME}:' + config_home ]
        rc = configUtils.scanReplaceFile(install_file, config_file, substitutionsList)
        if rc == 0:
            log.logDebug("  Successfully edited setmode.cntl")
        else:
            log.logDebug("  Failed to successfully edit setmode.cntl; refer to earlier messages")
    else:
        log.logDebug("*Warning*  setmode.cntl template not found")
        print getmsg("CFG3024W","The setmode.cntl template not found at {0}", [install_file])
    log.logDebug("  Configuring symlinks for bin directory scripts..complete")

    # 8 Recreate runtime - removed

    # 9 Copy several itcam properties files 
    #   Do not perform substitutions. Leave @{DC_HOME} and @{FILESEP} as is.
    #   This matches the behavior of the distributed DC.
    log.logDebug("  Configuring itcam properties files") 
    files = [ 'itcam.properties', 'itcam_ms.properties', 'itcam_wr.properties', 'itcam_tt.properties', 
              'itcam_ecam.properties', 'itcam_soa.properties.template' ]
    for file in files:
        install_file=install_home+"/itcamdc/etc/"+file
        if file.endswith(".template"):
             file = file.replace(".template", "")
        config_file=config_home+"/"+file                                    
        try:
            log.logDebug("  Copying file: %s" % file)
            shutil.copyfile(install_file, config_file)
            os.system('chmod a+rwX ' + config_file)
            log.logDebug("  Copying file: %s - successful" % file)
        except Exception, msgVar:
            log.logDebug("  Exception copying properties file to: %s" % config_file)
            print getmsg("CFG3025E","Error copying file {0}", [install_file])    
            raise SystemExit
        
    
    # 11. Create the global custom properties file
    gbl_custom_props='/custom/toolkit_global_custom.properties'
    runtimedir_gbl_custom_props = runtimedir + gbl_custom_props
    if not os.path.isfile(runtimedir_gbl_custom_props):
        log.logDebug("  Creating global custom properties")
        src=install_home+'/itcamdc/etc/toolkit_global_custom.properties'
        target_full_name=runtimedir_gbl_custom_props
        target = runtimedir + '/custom'
        try:
            shutil.copyfile(src, target_full_name)
            os.system('chmod a+rwX ' + target_full_name)
            log.logDebug("  Successfully created global custom properties")
        except Exception, msgVar:
            log.logDebug("  Exception creating global custom properties file: %s" % target_full_name)
            print getmsg("CFG3025E","Error copying file {0}", [src])    
            raise 
    else:
        log.logDebug("  Global custom properties already exists")

    # 12. Leave some scat and set ownership if requested
    scat_text = strftime("%a  %b %d %H:%M:%S %Z %Y") + " %s %s"   % (createcfg_script, fixlev)
    scat_file_name=config_home + '/runtime/.' + createcfg_script + ".txt"
    cmd="echo " + scat_text + " > " +  scat_file_name
    rc = configUtils.issueOSCommand(cmd)
    if rc != 0:
       log.logDebug("*WARNING* Problems creating the hidden scat file %s" % scat_file_name)

    # 13. Update permissions, and change owner and access if requested
    try:  
       log.logDebug("updating permissions to config_home: %s" % config_home)
       # 1. set default permissions
       os.system('chmod -hRf a+rwX ' + config_home) 
       # 2. call method to set owner and access using pdl values     
       if itcam_owner != "" or itcam_access != "":
           configUtils.changeowner(config_home, itcam_owner, itcam_access)
       # 3. Set search permisison
       os.system('chmod -hRf a+X ' + config_home)                  
    except:
        log.logException("Exception setting ownership/permissions - %s " % config_home)

    
    print getmsg("CFG3090I", "Configuration home directory created and configured: {0}", [config_home]) + "\n"
     
    log.logDebug("do_config exit")


#############################################################
# Create the symbolic links for scripts and jython jar  
#############################################################
def _create_symbolic_links(subdir, installhome, confighome):
    """
        create symbolic links
        @param subdir       - sub directory, such as /bin and /bin/lib 
        @param installhome  - path to INSTALL home
        @param confighome   - path to CONFIG home
    """ 
    log.logDebug("    create_symbolic_links entry")
    log.logDebug("     Configuring symlinks for scripts and jython jar in %s " % (subdir) )
    log.logDebug("      installhome=" + installhome)
    log.logDebug("      confighome=" + confighome)
    
    dirlist=os.listdir(installhome+subdir)  
    files_needing_symlink = []

    # Look at each file in this install bin directory
    for file in dirlist:
        if file.endswith('.sh') or file.endswith('.jacl') or file.endswith('.py') or file == 'jython.jar':
            files_needing_symlink.append(file)

    for file in files_needing_symlink: 
        path_install=installhome+subdir+'/'+file
        path_config=confighome+subdir+'/'+file
        failed=0
        log.logDebug("      path_install=" + path_install) 
        log.logDebug("      path_config=" + path_config)     
        log.logDebug("      name=" + file)
       
        # .1 If the file is not an ordinary file, continue
        if  ( not os.path.isfile(path_install) ):
             log.logDebug("      fails isfile test, bypassed")
             continue
        # .2 If script is this script continue. It's not run from the CONFIG dir
        if file == launching_script:
            continue
        # .3 If the bin/config file with the this name is a symlink, remove it
        if configUtils.isLink(path_config):
            log.logDebug("      file is a symlink: " + path_config )
            os.remove(path_config)
            log.logDebug("       remove issued")
            if  configUtils.isLink(path_config):
                print getmsg("CFG3026W","Unable to remove existing symbolic link {0}",[path_config])
                continue
        # else if it is regular file, it's  unexpected
        elif  os.path.isfile(path_config):
            print getmsg("CFG3027E","{0}/{1} object already exists in  {2}",[subdir, file, confighome]) 
            print getmsg("CFG3028I","Remove the file and rerun the script")
            raise SystemExit
 
        # .4 Create the symlink
        #      Model: ln -s path_to_real_script_in_install symlink_name
        rc=configUtils.issueOSCommand('ln -s ' + path_install + ' ' + path_config)
        if rc == 0:
            log.logDebug(" Successfully created symbolic link for %s/%s " % ( subdir, file))
        else:
            log.logDebug("   *ERROR* Failed to create symbolic link for %s/%s " % (subdir, file))    
            assert("*ERROR* Failed to create symbolic link for %s/%s " % (subdir, script))
        
    log.logDebug("    create_symbolic_links exit") 

#############################################################
# Create the configuration home directory and the data dir
#############################################################
def _create_config_home_dir(config_home):
    """
        Create the configuration home directory
        @param config_home  - name of the configuration home directory
        return True|False   - based on success
    """
    log.logDebug("create_config_home_dir - enter")
    log.logDebug("  Creating config home: %s" % config_home)
    try:
        if not os.path.isdir(config_home):
            os.makedirs(config_home)  
            os.system('chmod a+rwX ' + config_home)
            log.logDebug("  Successfully created: %s" % config_home)
        else:
           log.logDebug("  Directory exists")
        if not os.path.isdir(config_home+"/data"):
               os.mkdir(config_home+"/data")
        log.logDebug("create_config_home_dir - exit success")       
        return True , "success"
    except Exception, msgVar:
        log.logException("Creating config home %s failed - %s" % (config_home, msgVar)) 
        log.logDebug("create_config_home_dir - exit failed")  
        return False, msgVar      
 

#############################################################
# Clear Jython class files
# Input: "CONFIGHOME"
############################################################# 
def clear_jython_class_files(pdl):
     """
         clear_jython_class_files(pdl)
     """
     log.logDebug(" clear_jython_class_files - entry")
     config_home     = pdl["CONFIGHOME"][0]
     if config_home == None or config_home == "" or not os.path.isdir(config_home):
        print "%s" % getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home")
        raise SystemExit
     #assert config_home is not None and config_home != "", "ITCAM_CONFIG must be provided"
     #assert os.path.isdir(config_home),"ITCAM_CONFIG = %s is not a valid directory" % (config_home)
     bindir    = config_home+'/bin'
     binlibdir = config_home+'/bin/lib'
     if not os.path.isdir(bindir): 
         return 
     log.logDebug(" Clearing configuration Jython $py.class files in: %s"  % config_home) 
     log.logDebug(" ... in directories %s and %s " % (bindir ,binlibdir))  
     binList     = os.listdir(bindir) 
     binlibList  = os.listdir(binlibdir) 
     for file in binList:
         if file.endswith('$py.class'):
             os.remove(bindir+'/'+file)
     if not os.path.isdir(binlibdir): 
         return             
     for file in binlibList:      
         if file.endswith('$py.class'):
             os.remove(binlibdir+'/'+file)
     log.logDebug("   Clearing configuration Jython class files complete")       
     log.logDebug(" clear_jython_class_files - exit")


#############################################
# Display configuration choices
#############################################
def _display_config_summary(pdl):
    """
        display_config_summary(pdl): 
    """
    log.logDebug("display_config_summary - enter")
    config_home  = utils.getData(pdl, "CONFIGHOME")
    install_home = utils.getData(pdl, "INSTALLHOME")
    itcam_owner  = utils.getData(pdl, "OWNER")
    itcam_access = utils.getData(pdl, "ACCESS")    
 
    msgInstallHome = getmsg("SMPEINSTALLPATH","SMP/E install path")  
    msgConfigHome  = getmsg("CONFIGHOMEPATH", "Config home path")
   
    if itcam_owner is None or itcam_owner == "none":
        msgOwner =  getmsg("OWNERREMAINS", "Owner will remain")
        owner    =  "[%s:%s]" % (user_name, user_group)     
    else:
        msgOwner = getmsg("OWNERWILLBE",  "Owner will be")
        owner    = itcam_owner
    if itcam_access is None or itcam_access  == "none":
        msgAccess = ""  
    else:
        msgAccess = getmsg("ACCESSWILLBE", "Access will be")
 
    print 
    print getmsg("CFG3009I","Summary of chosen configuration parameters")  
    print
    print getmsg("CFG3010I","ITCAM will create an ITCAM for Application Diagnostics configuration home \
                 with the following parameters: \n")
    print
    print "   1) %20s : %-s "  % ( msgInstallHome, str(install_home))
    print  
    print "   2) %20s : %-s "  % ( msgConfigHome,  str(config_home))
    print 
    print "   3) %20s : %-s "  % ( msgOwner,       str(owner))
    print
    if msgAccess != "":
        print "   4) %20s : %-s "  % ( msgAccess,  str(itcam_access) )
    print 

    log.logDebug("display_config_summary - exit")
    
#############################################
# Display startup banner and messages
#############################################
def _display_startup_messages():
    """
        display startup messages on the console 
    """
    log.logDebug("display_startup_messages - enter")
    UID_string = configUtils.CommandAndResponse('id')[0]
    print
    print "%s" % getmsg("FLOWERBOX_CREATECFG","Create Configuration Home for Data Collector")
    print
    print getmsg("CFG3021I","You only need to run this script once.\n" + 
          "All Data Collectors can share the same configuration home.")
    print 
    print "        %s  "   % ( strftime("%a  %b %d %H:%M:%S %Z %Y") )
    print 
    print "        %s  "   %  (str(UID_string))
    log.logDebug("display_startup_messages - exit")               
   
#############################################
# Get the physical path of the directory. 
# Jython os.path commands are not giving 
# physical paths 
#############################################
def _get_physical_path(dir): 
    """
        get physical directory path
        @param  dir  
        returns the physical path 
    """ 
    log.logDebug("get_physical_path entry for: %s" % dir) 
    cmdi = "set +o logical; cd " + dir + " && pwd"
    real_path  = configUtils.CommandAndResponse(cmdi)[0].rstrip("\n")
    log.logDebug("get_physical_path exit with real_path: %s" % real_path )
    return real_path
   
#############################################
# Test the bundle messages
#############################################
def test_msgs():
    """
        test messages
    """
    config_home  = "/u/itcam72"
    install_home = "/usr/lpp/itcam/WebSphere/DC"
    OWNER        = "IBMUSER"
    ACCESS       = "775"
    ermsg        = "This is a test error message"
    ent          = "/usr/lpp/itcam/WebSphere/DC/testdir"
    install_file = "/usr/lpp/itcam/WebSphere/DC/testdir/test.txt"
    src          = "/usr/lpp/itcam/WebSphere/DC/testdir/src"
    path_config  = "/usr/lpp/itcam/WebSphere/DC/testdir/test"
    subdir       = "/u/subdir"
    file         = "/u/file" 
    confighome   = "/u/itcam72"
    log          = "/tmp/templog.log"
    var1         = [str(config_home), str(install_home)]
    msgVar       = "ERROR TEXT"
    
    print getmsg("CFG1088I", "Exiting...")
    print getmsg("CFG3005I",\
         """For the SMP/E installation directory path you should enter   
            the logical path (symbolic link) instead of the physical path 
            (canonical path name).  Doing this will provide flexibility   
            in cloning and for service migration. 
         """)
    print getmsg("CFG3006I", \
          """You may set a new owner for the configuration
             home files and directories.  
                                        
              The owner may be entered as 'user' or 'user:group'. 
              An owner of 'none' will bypass setting the owner.   
              
              **IMPORTANT**: Root access is required to change owner.   
          """)
    print getmsg("CFG3007I",\
         """You may set permissions for the configuration  
            home files and directories. 
                                       
             Permissions are entered in chmod format, which may be either     
             numeric, like  664, or symbolic, like: 'a+rX,u+w,g+w,o-w'        
             Permissions specified as 'none' will bypass setting permissions. 
        """)
    print getmsg("CFG3008I", "Type 'a' to accept, and  continue to build the configuration home,  or -\n \
                  type 'r' to respecify, or -\n   \
                  type 'q' to quit"  )    
    print getmsg("CFG3009I","Summary of chosen configuration parameters")  
    print getmsg("CFG3010I","ITCAM will create an ITCAM for Application Diagnostics configuration home \
                  with the following parameters: \n")
    print getmsg("CFG3012I","Both config home and install home are required \n \
                  config home:{0},  install home {1}",var1)
    print getmsg("CFG3013E","Unable to create configuration home directory at \n \
                  {0}", [str(config_home)])
    print getmsg("CFG1014I","Correct the problem and rerun the script")
    print getmsg("CFG3015E","The following directory problems were found: {0}", [ermsg])
    print getmsg("CFG3016I","Correct the problems and rerun the script")
    print getmsg("CFG3017E","Directory object {0} already exists in {1}", [ent, config_home])
    print getmsg("CFG3018E","Error creating directory {0} - {1}",[config_home, msgVar])  
    print getmsg("CFG3019E","Unable to create configuration home directory at {0}", [config_home] )
    print getmsg("CFG3022E","Invalid Installation home specified: {0}",[install_home]) 
    print getmsg("CFG3023I","Re-enter the configuration home when prompted")
    print getmsg("CFG3024W","The setmode.cntl template not found at {0}", [install_file])
    print getmsg("CFG3025E","Error copying file {0}", [src])  
    print getmsg("CFG3026W","Unable to remove an existing symbolic link {0}",[ent])
    print getmsg("CFG3027E","{0} object already exists in {1}", [ent , config_home])  
    print getmsg("CFG3028I","Remove the file and rerun the script")
    print getmsg("CFG3029I","Temporarily logging to: {0}", [log])
    print getmsg("CFG3030I","Configuring ITCAM configuration home: {0} \n\
          using installation directory: {1}", var1)
    print getmsg("CFG3031E","Invalid Configuration home specified: {0}", [config_home]) 
    print getmsg("CFG3032E","Configuration home must be different from installation home") 
    print getmsg("CFG3090I","Configuration home directory created and configured: {0}", [config_home])
    print getmsg("CFG3998I","Create configuration has been cancelled")   
    print getmsg("CFG3999E","Create configuration script has failed and has terminated")
    print "   1) %20s : %-s "  % ( getmsg("SMPEINSTALLPATH","SMP/E install path"), str(install_home))
    print "   2) %20s : %-s "  % ( getmsg("CONFIGHOMEPATH", "Config home path"),  str(config_home))
    print "   3) %20s : %-s "  % ( getmsg("CHGOWNERTO", "Change owner to"),  str(OWNER))
    print "   4) %20s : %-s "  % ( getmsg("CHGACCESSTO","Change access to"), str(ACCESS))  
    print  getmsg("OWNERREMAINS", "Owner will remain")
    print  getmsg("OWNERWILLBE",  "Owner will be")
    print  getmsg("ACCESSWILLBE", "Access will be")
 
 
#############################################
#   Prompt for the owner 
#############################################
def _get_owner(pdl):
   """
      prompt for the owner
      @param pdl 
   """
   log.logDebug("get_owner entry") 
   print getmsg("CFG3006I", \
       """You may set a new owner for the configuration 
           home files and directories.  
                                         
           The owner may be entered as 'user' or 'user:group'. 
           An owner of 'none' will bypass setting the owner.   
           Root access is required to change owner.   
        """) 
   prompt.do_prompt("OWNER", pdl)
   itcam_owner = utils.getData(pdl, "OWNER")  
   log.logDebug("get_owner exit ") 
   return itcam_owner

 
#############################################
#   Prompt for the access permissions 
#############################################
def _get_access(pdl):
    """
       prompt for the access permissions
       @param pdl 
    """ 
    log.logDebug("get_access entry") 
    print getmsg("CFG3007I",\
        """You may set permissions for the configuration  
           home files and directories. 
                                     
           Permissions are entered in chmod format, which may be either     
           numeric, like  664, or symbolic, like: 'a+rX,u+w,g+w,o-w'        
           Permissions specified as 'none' will bypass setting permissions. 
        """)
    prompt.do_prompt("ACCESS", pdl)
    itcam_access = utils.getData(pdl, "ACCESS") 
    log.logDebug("get_access exit")         
    return itcam_access 

#############################################
#   Prompt for the installation home
#############################################
def _get_install_home(pdl):
    """
       prompt for the installation home
       @param pdl 
    """ 
    log.logDebug("get_install_home - entry")
    batch_mode = pdl["BATCH"][0]
    install_home = ""
    
    # The initial default should be based on current execution directory
    # if the user has not supplied one. That task is done in _validate_input(pdl)
    
    while True:
        print 
        print getmsg("CFG3005I",\
            """For the SMP/E installation directory path you should enter   
               the logical path (symbolic link) instead of the physical path 
               (canonical path name). Doing this will provide flexibility    
               in cloning and service migration.
            """)
        prompt.do_prompt("INSTALLHOME", pdl)
        install_home = utils.getData(pdl, "INSTALLHOME") 
        log.logDebug("   install_home from prompt: %s" % str(install_home))  
        # Validate - The do_prompt method validates the directory exists
        jython_jar = install_home+"/bin/jython/jython.jar"
        if  not os.path.isdir(install_home) or not os.path.isfile(jython_jar):
            print
            print getmsg("CFG3022E","Invalid Installation home specified: {0}",[install_home])
            if not batch_mode: continue
            raise SystemExit    
        # Continue below to collect config home name          
        break       
    log.logDebug("get_install_home - exit: %s" % install_home)        
    return install_home  

#############################################
#   Prompt for the configuration home
#############################################
def _get_config_home(pdl): 
    """
       prompt for the config home
       @param pdl 
    """  
    log.logDebug("get_config_home - entry") 
    batch_mode = pdl["BATCH"][0]  
    while True:
        prompt.do_prompt("CONFIGHOME", pdl)
        config_home = utils.getData(pdl, "CONFIGHOME") 
        log.logDebug(" config_home from prompt:%s " %  str(config_home))
        # Validate
        errors = False
        if (config_home is None) or (config_home == "") or \
           (config_home.startswith("/") is False)       or \
           (config_home == "/"):
                log.logDebug(" Invalid config_home specified")
                print getmsg("CFG3031E","Invalid Configuration home specified: {0}", [config_home])
                errors = True
        else:
            if os.path.isdir(config_home):
                real_path_install = _get_physical_path(install_home)
                real_path_config  = _get_physical_path(config_home)
                if real_path_install == real_path_config:
                    log.logDebug(" Error config_home value must be different from install_home") 
                    print getmsg("CFG3032E","Configuration home must be different from installation home") 
                    errors = True
        if errors:
            if not batch_mode: continue
            raise SystemExit
            #assert False,"The configuration home is invalid: %s" % config_home               
        #  Continue below to collect owner: 
        break        
    log.logDebug("get_config_home - exit: %s" % config_home)      
    return config_home           

#############################################
#  Set the logging, assumes CONFIGHOME is set
############################################# 
def _set_logging(pdl, logfile):
    """
        set logging 
    """
    config_home = pdl["CONFIGHOME"][0]
    if (config_home is None) or (not os.path.isdir(config_home)):
        print "%s" % getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home")
        raise SystemExit
        #assert False, "ITCAM_CONFIG variable must be provided and be a valid config home %s" % config_home
    if logfile is None:
        logfile = config_home+"/data/createcfg.log"
        log.set_logging(logfile) 
    log.logDebug("set_logging exit using logfile: %s" % str(logfile)) 


#############################################
#  Validate input
############################################# 
def _validate_input(pdl):
    """
        validate input
        If INSTALLHOME not supplied, set it's default based on current executing directory  
    """  
    log.logDebug("validate_input entry")  
    if utils.is_initial_value(pdl, "INSTALLHOME"):
        cwd =  os.getcwd() 
        if cwd.endswith("/bin"):
            install_home = cwd.replace("/bin","") 
            # If not the config home directory
            if not os.path.isdir(install_home+"/data"):
                utils.setData(pdl,"INSTALLHOME", install_home , "createcfg")
    log.logDebug("validate_input exit")  
        
#####################################################################
#                                                                   #
# Main                                                              #
#                                                                   #
#####################################################################

try:
   
    # Initialize the resource bundle
    resourcebundle.set_resourcebundle()
    
    # Obtain information about the user 
    login_user = os.getlogin()
    user_name  = configUtils.CommandAndResponse('id -nu')[0]
    user_group = configUtils.CommandAndResponse('id -ng')[0]
    if user_name is not None:   user_name= user_name.rstrip("\n")
    if user_group is not None:  user_group=user_group.rstrip("\n")
    
    # Create the data structures
    pdl = createPDL.make_pdl(createPCL.make_pcl())
    # Merge in the environemntal variables and command line arguments 
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    log.logDumpPDL(pdl, "PDL after environment variables and command line arguments")
   
    # Logging - determine if log location has been set externally
    require_log = True 
    logfile =  pdl["LOG"][0]
    if logfile is not None:
        log.set_logging(logfile)

    # Globalized characters
    ACCEPT = getmsg("ACCEPT_CHAR","a")
    RESPECIFY = getmsg("RESPECIFY_CHAR","r")

    # Special handling for mocking
    if len(ACCEPT) > 1 and ACCEPT.endswith("]"): ACCEPT = ACCEPT[-2:-1]
    if len(RESPECIFY) > 1 and RESPECIFY.endswith("]"): RESPECIFY = RESPECIFY[-2:-1]
    
    # Batch mode flag 
    batch_mode = pdl["BATCH"][0] 
    
    # validate_input
    _validate_input(pdl)
     
    # Display startup banner and messages                         
    _display_startup_messages()
    
    #
    # Option 1: Help display  
    #
    if pdl["HELP"][0]:
        require_log = False
        _print_help() 
        sys.exit(0)
    
    #
    # Option 2: Clear Jython $py.class files  
    #   External interface has been removed. 
    #   But a rerun of this script on an existing config_home will clear these $py.class files
    #   by a call to "clear_jython_class_files(pdl)"  
    #
 
    #
    # Option 3: Test the resource bundle
    #
    if pdl["TESTMSGS"][0]:
        require_log = False 
        test_msgs()
        sys.exit(0) 
     
    #
    # Option 4: Create a configuration home 
    #    -  collect input
    #    -  display configuration choices
    #    -  prompt to accept or re-enter 
    #    -  create the config home directory 
    #    -  redrive prompts if not successfull 
    #    -  configure the configuration directories
  
    # 1. Collect the input
    while True:
        # .1 Installation home 
        #      Enter SMP/E installation directory path.   
        #      Example:   /smp/usr/lpp/itcam/WebSphere/DC
        #
        install_home = _get_install_home(pdl)
      
        # .2  Configuration home 
        #       Enter directory name in which to create the ITCAM configuration root. 
        #     The Jython os.path commands do not return the real physical path when a symbolic link is used. 
        config_home = _get_config_home(pdl)
       
        # .3 Owner for the configuration home
        itcam_owner = _get_owner(pdl)
         
        # .4 Access permissions for the configuration home 
        itcam_access = _get_access(pdl)
       
        # .5  Display Configuration choices
        _display_config_summary(pdl)
      
        # .6 Prompt:  'a' to accept, and proceed to configure 
        #             'r' to reenter choices
        #              No default answer 
        validAnswers = [ACCEPT,RESPECIFY]   
        while True:
            utils.setData(pdl,"PRCRTCFGDISPLAY","","createConfig")  
            prompt.do_prompt("PRCRTCFGDISPLAY", pdl)
            answer =  utils.getData(pdl, "PRCRTCFGDISPLAY")
            if answer in validAnswers:
                break
            else:
                if batch_mode == True:
                    break
                print getmsg("CFG5013E", "Invalid response") + "\n"
                continue
        # Re-drive prompts?        
        if answer == RESPECIFY: continue
     
        # .7 Create the config home directory 
        #    Set logging
        #    Redrive prompts if not successfull 
        success, exc_msg = _create_config_home_dir(config_home)
        if success:
            if logfile is None:
                logfile = config_home+"/data/createcfg.log"
            log.set_logging(logfile)
            break
        else:
            print getmsg("CFG3018E","Error creating directory {0} - {1}",[config_home, exc_msg]) + "\n"
            if not batch_mode: 
                print getmsg("CFG3023I","Re-enter the configuration home when prompted") + "\n"
                continue           
            else:
                raise SystemExit 
                       
    # 2. Configure the configuration directories
    var1 =[config_home, install_home]
    print getmsg("CFG3030I","Configuring ITCAM configuration home: {0} \n\
       using installation directory: {1}", var1) + "\n" 
 
    _do_config(pdl)

#########################################
# Main exception catch
#########################################
#former assert, unrecoverable error caused by environment settings or input parameters
#fixable by user, not a program error, error text goes to the console
except ConfigurationException, (instance):
    print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["createcfg.py"])
    print "%s" % instance.parameter #print exception message 
    sys.exit(1)    
    
#assert during execution 
except AssertionError:
    import sys, traceback
    
    print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["createcfg.py"])
    try:
        log.logException("Assertion error in createcfg.py")
        log.logDumpStack("Stack data for diagnostic purposes")
    except:
        _type, _val, _tb = sys.exc_info()
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
        traceback.print_exception(_type,_val,_tb)
        print "\nOriginal traceback"
    sys.exit(1)

except KeyboardInterrupt:   #User entered terminate character in prompt
    print "%s" % getmsg("CFG1088I", "Exiting...") 
    print getmsg("CFG3998I","Create configuration has been cancelled")
    # Do not flog the console with log buffer messages 
    if logfile is None:
        logfile = tmplog
        log.set_logging(tmplog)
        print getmsg("CFG3033I","Log file written to {0}",[tmplog])    
    sys.exit(0) 
 
except SystemExit:   # Generic exit (after an exception was already caught and handled)
    print "%s" % getmsg("CFG1088I", "Exiting...") 
    if logfile is None:
        logfile = tmplog
        log.set_logging(tmplog) 
        print getmsg("CFG3033I","Log file written to {0}",[tmplog]) 
    sys.exit(1) 
        
except:
    _exctype, _excval, _exctb = sys.exc_info()
    ermsg = str(_excval)
    ertype= str(_exctype)
    log.logException("Create configuration script has failed")
    print
    print getmsg("CFG3999E","Create configuration script has failed and has terminated")
    print
    print   "*.......   %s, %s " % ( ertype , ermsg )
    traceback.print_exception(_exctype,_excval,_exctb)
    log.set_logging()
    sys.exit(1)
