#!/bin/sh

#####################################################################         
# Licensed Materials - Property of IBM
#  
# 5724-L62     
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.      
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
##################################################################### 

usage() {
    msgid="GDC_EXTRACTOR_USAGE"
    usage_str="Usage: $0 [-h] [[-d config_home] | [-r response_file]] archive_file\
      archive_file: the archive file to extract\
      -d config_home: the directory to extract the archive file to\
      -r response_file: the response file for silent installation\
      -h: display this help message"
    if [ -f "$WORKDIR/installbundle.jar" ]; then 
        "$JAVA_HOME"/bin/java -cp "$WORKDIR"/installbundle.jar com.ibm.tivoli.itcam.install.common.resources.MessagePrinter $msgid "$usage_str"  
    else
        echo $usage_str
    fi
}

exit_error() {
    rc=$1
    msgid=$2
    message_text=$3
    if [ -f "$WORKDIR/installbundle.jar" ]; then 
        "$JAVA_HOME"/bin/java -cp "$WORKDIR"/installbundle.jar com.ibm.tivoli.itcam.install.common.resources.MessagePrinter $msgid "$message_text"  
    else 
        echo $message_text
    fi
    
    if [ -d "$TMPDIR" ]; then
        rm -rf "$TMPDIR"
    fi
    exit $rc
}

RESPONSE_FILE=
SILENT=
RESPONSE_FILE_REQUIRED=
WORKDIR=`dirname "$0"`

# Disable JIT compiler to avoid segmenation fault due to Java bug
export JAVA_COMPILER=NONE

if [ -z "$JAVA_HOME" ]; then
    echo ERROR: Unable to locate Java Home
    exit 1
fi

# getopts
set -- `getopt hd:r: $*`
if [ $? != 0 ]; then
    usage
    exit 1
fi
while [ "$1" != "--" ]; do
    case $1 in
    -h) usage; exit 0;;
    -d) shift; ITCAM_CONFIGHOME=$1; SILENT=y;;
    -r) shift; RESPONSE_FILE=$1; SILENT=y; RESPONSE_FILE_REQUIRED=y;;
    esac
    shift
done
shift

#Check if we have a valid response file for the silent install with -r flag
if [[ -n $RESPONSE_FILE_REQUIRED ]]; then
    #strip single quotes from file name
    RESPONSE_FILE="${RESPONSE_FILE%\'}"
    RESPONSE_FILE="${RESPONSE_FILE#\'}"
    [[ -n $RESPONSE_FILE ]] || exit_error 0 "CFG5055I" "CFG5055I Skipping DC extraction, EP_RSP_FILE property is not set" 
    [[ -f $RESPONSE_FILE ]] || exit_error 1 "CFG5056E" "CFG5056E Specified response file $RESPONSE_FILE not found on the system"
fi  

# Get the name of the file to extract
GDC_ARCHIVE="$1"
if [ -z "$GDC_ARCHIVE" ]; then
    usage
    exit_error 1 "CFG5058E" "CFG5058E Specify the archive file to extract"
fi
if [ ! -f "$GDC_ARCHIVE" ]; then
    exit_error 1 "CFG5063E" "CFG5063E Specified archive file doesn't exist."
fi


TMPDIR=/tmp/tempdir_$$
(umask 077 && mkdir $TMPDIR) || exit 1

DC_VERSION="7.3.0.10.0"

# The list of files to extract
FILE_LIST="\
$DC_VERSION/bin/jython/jython.jar \
$DC_VERSION/bin/lib/installbundle.jar \
$DC_VERSION/bin/lib/gdc_extract.py \
$DC_VERSION/bin/lib/debugUtil.py \
$DC_VERSION/bin/lib/platformUtil.py \
$DC_VERSION/bin/lib/resourcebundle.py \
$DC_VERSION/bin/lib/ConfigurationException.py \
$DC_VERSION/bin/lib/PrintWrapper.py \
$DC_VERSION/bin/lib/constants.py \
$DC_VERSION/bin/lib/utils.py"


# Extract the file list from the archive to the tmp dir
"$JAVA_HOME"/bin/java -jar "$WORKDIR"/gdc_extract.jar $TMPDIR "$GDC_ARCHIVE" $FILE_LIST
RC=$?
[ $RC -eq 0 ] || exit_error $RC "CFG5060E" "CFG5060E Failure in Java bootstrap"

CLASSPATH=$TMPDIR/$DC_VERSION/bin/jython/jython.jar:$TMPDIR/$DC_VERSION/bin/lib/installbundle.jar

# The file to which the value of ITCAM_CONFIGHOME will be written by the jython script
TMPFILE=$TMPDIR/$DC_VERSION/configdir.dat

# execute the extracted jython bootstrap
JAVA_OPTS=
if [ -n "$RESPONSE_FILE"  ]; then
    JAVA_OPTS=-Dresponse.file="$RESPONSE_FILE"
elif [ -n "$ITCAM_CONFIGHOME"  ]; then
    JAVA_OPTS=-Dconfig.home="$ITCAM_CONFIGHOME"
fi
JAVA_OPTS=$JAVA_OPTS" -Ddc.version=$DC_VERSION"
"$JAVA_HOME"/bin/java $JAVA_OPTS -classpath "$CLASSPATH" org.python.util.jython "$TMPDIR"/$DC_VERSION/bin/lib/gdc_extract.py "$GDC_ARCHIVE" "$TMPFILE"
RC=$?
[ $RC -eq 0 ]  || exit_error $RC "CFG5057E" "CFG5057E Failed to extract files from archive"

# read DC_HOME from the output file
[ -f "$TMPFILE" ] || exit_error 1 "CFG5059E" "CFG5059E No temporary directory"

ITCAM_CONFIGHOME=`cat $TMPFILE`/$DC_VERSION
[ -n "$ITCAM_CONFIGHOME" ] || exit_error 1 "CFG5062E" "CFG5062E Failed to get the value of ITCAM_CONFIGHOME"

export ITCAM_CONFIGHOME

# Clean up temporary dir
rm -rf "$TMPDIR"

# Execute the DC configurator (unless silent mode)
if [ -z "$SILENT" ]; then
    "$ITCAM_CONFIGHOME/bin/config.sh"
    RC=$?
    [ $RC -eq 0 ] || exit_error $RC "CFG5045E" "CFG5045E The configuration process is being terminated due to errors listed above."
fi

if [ -t 0 ]; then
    # jython turns echo off
    stty echo
fi

exit 0
