######################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This class inherits from the WsadminTask, which executes a findservers call to
   get the list of servers for a specific profile.
   This class overwrites the method calls that are specifically for the findservers call.
   The actual call to the findservers jython call is done in the base class.
'''

fixlev = "38001" # Last APAR, defect or story to update this module

import debugUtil as log
import utils
import os

from WsadminTask import WsadminTask
from constants import constants
from resourcebundle import getmsg
from debugUtil import logDebug as debug


class FindserversTask(WsadminTask):

    wtype = -1
    common_constants = None
    wasProfileHome = None
    wroot = None
    profile = None
    conntype = None
    host = None
    port = None
    log_file_name = None
    dat_filename = None
    dat_fileloc = None
    findservers = None


    def __init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call, findserver_dat_filename):
        '''
           This is the constructor routine.

           @param self - Reference to this object
           @param pdl - PDL reference
           @param wtype - Caller's type like config, unconfig,.....
           @param inputlist_fname - input list file name
           @param logf - log file name
           @param jython_call - Jython name to be called
           @param findserver_dat_filename - findserver dat file name
        '''

        debug("Entry FindserversTask.__init__ %s" % fixlev)

        WsadminTask.__init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call)
        self.wtype = wtype
        self.findservers = getmsg("FINDSERVERS", "findservers ")
        debug("Config task type is : %s" % (wtype))
        self.log_file_name = logf
        debug("Log file name is %s" % (self.log_file_name))
        self.dat_filename = findserver_dat_filename
        debug("dat file location is %s" % (self.dat_filename))
        #
        # Rename any existing dat files.
        #
        datadir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data"))
        dat_fileloc = os.path.normpath(os.path.join(datadir, self.dat_filename))
        if self.dat_filename is not None:
            utils.rename_archival_file(dat_fileloc)
        #
        # Instantiate common constants class
        #
        self.common_constants = constants()
        debug("Constants reference is: %s" % (self.common_constants))

        debug("Exit FindserversTask.__init__")


    def process(self, serverindex = None):
        '''
           As the servers have not been discovered yet, to get the wsadmin data is
           different for findservers in the way that searching the pdl to get the
           data is using the non index pdl data.
           Once the wsadmin execution is finished, the return code from wsadmin is checked
           for successful or unsuccessful execution.

           @param self - Reference to this object
           @param serverindex - Not used in this task, and it has a value of None
        '''

        debug("Entry FindserversTask.process %s" % fixlev)

        try:
            #
            # If this is as a result of findservers task, the processing is a little
            # different, as the servers have not been discovered yet, so setting the
            # wsadmin call is different for findservers.
            #
            debug("Processing findservers...")
            self.create_inputlist()
            command = self.constructCommand()
            wasprofile = utils.getData(self.pdl, "PROFILE")
            wasprofilehome = utils.getData(self.pdl, "PROFILEHOME")
            msgl =[wasprofile]
            message = getmsg("CFG2036I", "Searching for servers under profile: {0} ", msgl)
            print "%s" % message
            debug("Processing findservers call ")
            debug("starting the command thread...")
            #
            # Start wsadmin command.  The wsdadmin runs in a separate thread
            #
            self.start_wsadmin(command, wasprofilehome, wasprofile)

        except Exception, errmsg:
            utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
            msg1 = "Error occurred processing findservers call - %s" % (errmsg)
            log.logException(msg1)
            msgl =[]
            msgl.append(self.findservers)
            msgl.append(errmsg)
            print getmsg("CFG2038E", "Error occurred while processing {0} call - {1}", msgl)
            raise errmsg

        debug("Exit FindserversTask.process")


    def get_progress_file_location(self, server_indexlist = None):
        '''
           This routine gets the progress file location for this task.

           @param inputlist_file - input list file fd
           @param server_indexlist - Not used in this task, and it has a value of None
           @return - Progress file location
        '''

        debug("Entry FindserversTask.get_progress_file_location %s" % fixlev)

        wasprofile = utils.getData(self.pdl, "PROFILE")
        progressfile_prefix = wasprofile + "." + "findservers" + "." + "progress"
        progressfile_loc = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data", progressfile_prefix))

        debug("Exit FindserversTask.get_progress_file_location")

        return progressfile_loc


    def set_inputlist_entries(self, inputlist_file, server_indexlist = None):
        '''
           This routine sets the input list entry record for the findservers.  It contains
           the wasprofile and the dat file location separated by "^"

           @param inputlist_file - input list file fd
           @param server_indexlist - Not used in this task, and it has a value of None
        '''

        debug("Entry FindserversTask.set_inputlist_entries %s" % fixlev)

        datadir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data"))
        #
        # Set the findservers record which is the profile name and the dat file location
        # separated by a "^"
        #
        wasprofile = utils.getData(self.pdl, "PROFILE")
        dat_fileloc = os.path.normpath(os.path.join(datadir, self.dat_filename))
        debug("Progress file loc: " + dat_fileloc)
        self.dat_fileloc = dat_fileloc
        record = wasprofile + "^" + dat_fileloc
        inputlist_file.write(record)
        inputlist_file.write("\n")

        debug("Exit FindserversTask.set_inputlist_entries")


    def set_wsadmin_data(self, serveri = None):
        '''
           This routine sets the wsadmin data from PDL.  This is the default implementation which
           gets the data from the PDL based on the server index

           @param serveri - Not used in this task, and it has a value of None
        '''

        debug("Entry FindserversTask.set_wsadmin_data %s" % fixlev)

        self.wasProfileHome = utils.getData(self.pdl, "PROFILEHOME")
        debug("wasProfileHome: " + self.wasProfileHome)
        self.wroot = utils.getData(self.pdl, "WASROOT")
        debug("WASROOT: " + self.wroot)
        self.profile = utils.getData(self.pdl, "PROFILE")
        debug("profile: " + self.profile)
        self.conntype = utils.getData(self.pdl, "CONNTYPE")
        self.host = utils.getData(self.pdl, "HOST")
        self.port = utils.getDataString(self.pdl, "PORT")

        debug("Exit FindserversTask.set_wsadmin_data")


    def get_server_indexlist(self):
        '''
           This routine gets the server index list of current group.  This list
           is saved when each group is processed.

           @return - Server index list
        '''

        debug("get_server_indexlist of FindserversTask is called")
        return self.server_indexlist


    def get_wasprofilehome(self):
        '''
           This routine gets the WAS profile home

           @return - WAS profile home
        '''

        debug("get_wasprofilehome of FindserversTask is called")
        return self.wasProfileHome


    def get_wasroot(self):
        '''
           This routine gets WAS root.

           @return - WAS root
        '''

        debug("get_wasroot of FindserversTask is called")
        return self.wroot


    def get_profile_name(self):
        '''
           This routine gets WAS profile name

           @return - WAS profile name
        '''

        debug("get_profile_name of FindserversTask is called")
        return self.profile


    def get_washost(self):
        '''
           This routine gets WAS host name.

           @return - WAS host name
        '''

        debug("get_washost of FindserversTask is called")
        return self.host


    def get_wasport(self):
        '''
           This routine gets WAS port.

           @return - WAS port number
        '''

        debug("get_wasport of FindserversTask is called")
        return self.port


    def get_wasconntype(self):
        '''
           This routine gets WAS connection type.

           @return - WAS password
        '''

        debug("get_wasconntype of WsadminTask is called")
        return self.conntype


    def get_logfile_loc(self, serveri = None):
        '''
           This routine gets log file location

           @param serveri - Not used in this task, and it has a value of None
           @return - Log file location
        '''

        debug("get_logfile_loc of FindserversTask is called")
        datadir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data"))
        wasprofile = utils.getData(self.pdl, "PROFILE")
        logfile_loc  = os.path.normpath(os.path.join(datadir, wasprofile + "." + self.log_file_name))

        return logfile_loc


    def get_dat_logfile_loc(self):
        '''
           This routine gets dat log file location

           @return - dat log file location
        '''

        return self.dat_fileloc
