######################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This class inherits from the WsadminTask, which executes a unconfigdatacolltorc call to
   unconfigure selected servers in a profile.  This class
   overwrites the method calls that are specifically for the unconfig call.
   The actual call to the unconfig jython call is done in the base class.
'''

fixlev = "38001" # Last APAR, defect or story to update this module

import os
import sys

import msunconfig
import debugUtil as log
import utils
import codecs

from WsadminTask import WsadminTask
from constants import constants
from Functhread import Functhread
from Progress import Progress
from resourcebundle import getmsg
from debugUtil import logDebug as debug


class UnconfigTask(WsadminTask):

    common_constants = None


    def __init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call):
        '''
           This is the constructor routine.

           @param self - Reference to this object
           @param pdl - PDL reference
           @param wtype - Caller's type like config, unconfig,.....
           @param inputlist_fname - input list file name
           @param logf - log file name
           @param probe_serverindex_list - A list of server indexes to be notified because of probe changes
        '''

        debug("Entry __init__")

        WsadminTask.__init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call)

        #
        # Instantiate common constants class
        #
        self.common_constants = constants()

        debug("Exit __init__")



    def set_inputlist_entries(self, inputlist_file, server_indexlist):
        '''
           This routine saves the following info in the unconfig inputlist:
           1st position = 1 if server needs to be notified
                          0 no need to notifiy server
           2nd position = Cell name
           3rd position = Node name
           4th position = Server name
           5th position = input list location
           The entries are separated by a "^".

           @param inputlist_file - input list file fd
           @param server_indexlist - Index list of the selected servers for a given group
        '''

        debug("Entry set_inputlist_entries - %s - %s" % (inputlist_file, server_indexlist))

        debug("unconfigtask set_inputlist_entries is called")
        runtimedir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "runtime"))
        for serveri in server_indexlist:
            inputPropertyFilePrefix = utils.getServerBasedFilePrefix(self.pdl, serveri)
            wasserver = utils.getData(self.pdl, "SERVERS", serveri)
            #
            # Save the input list entries.  The entries are separated by a "^".
            #
            inputPropertyFile = wasserver + "^" + os.path.normpath(os.path.join(runtimedir, inputPropertyFilePrefix + "input.properties"))
            debug("inputPropertyFile: " + inputPropertyFile)
            inputlist_file.write(inputPropertyFile)
            inputlist_file.write("\n")

        debug("Exit set_inputlist_entries")

    def start_msunconfig(self, server_indexlist, progressfile_loc):
        '''
           This routine starts ms unconfiguration in a separate command, and monitors the
           progress file and displays the info from progress file into the display.

           @param server_indexlist - A list of server indexes to be unconfigured from MS
           @param progressfile_loc - progres file location.
        '''

        debug("Entry start_msunconfig - %s - %s" % (server_indexlist, progressfile_loc))

        # Look for eligible-for-MS-unconfure servers
        server_indexlist_ms = []
        for ix in server_indexlist:
            if msunconfig.server_MS_configured(self.pdl, ix):
                server_indexlist_ms.append(ix)
        if len(server_indexlist_ms) == 0:
            debug("No servers in server_indexlist are eligible for MS unconfig")
            debug("Exit start_msunconfig - early exit")
            return
        debug("start_msunconfig of UnconfigTask is called")
        #
        # Make sure the file gets empty if it is being used previously.
        #
        progress_file = codecs.open(progressfile_loc, mode="w", encoding="utf-8")
        progress_file.close()
        #
        # start wsadmin in a separate thread.
        #
        # t1 = Functhread(self._ms_unconfig, server_indexlist, progressfile_loc)
        t1 = Functhread(self._ms_unconfig, server_indexlist_ms, progressfile_loc)
        debug("starting the MS unconfig thread...")
        t1.start()
        sys.stdout.write("  %s"  % getmsg("MSCHECKING", "Checking for MS Configuration...."))
        sys.stdout.flush();
        #
        # Start reading the progress file and display all the data in the file
        # to the console.
        #
        self._check_progress(progressfile_loc)
        #
        # Now wait till the ms unconfig thread is finished.
        #
        t1.join()

        debug("Exit start_msunconfig")


    def _ms_unconfig(self, server_indexlist, progress_file_loc):
        '''
           This call runs in a separate thread which runs thru a list of servers
           to do MS unconfiguration.

           @param server_indexlist - A list of server indexes to be unconfigured from MS
           @param progress_file_loc - progress file location
        '''

        ms_configured = False

        debug("Entry _ms_unconfig - %s - %s" % (server_indexlist, progress_file_loc))

        try:
            self.thread_rc = self.START
            debug("MS unconfig thread started")
            progress_obj = Progress(self.common_constants.MSUNCONFIG, None, progress_file_loc)
            #
            # Open the progress file
            #
            wascell = utils.getData(self.pdl, "WASCELL", server_indexlist[0])
            wasnode = utils.getData(self.pdl, "WASNODE", server_indexlist[0])
            wasserver = utils.getData(self.pdl, "WASSERVER", server_indexlist[0])
            wasprofile = utils.getData(self.pdl, "PROFILE", server_indexlist[0])
            msmsg = getmsg("MSUNCONFIG", "MS unconfig ")
            debug("Processing %s call for  Cell: %s  Node: %s  Profile: %s" % (msmsg, wascell, wasnode, wasprofile))
            #
            # Process each server to do MS unconfig.
            # If MS is now or was once connected to MS, call msunconfig to delete the controller id
            #
            for serveri in server_indexlist:
                #
                # Remove DC from MS database if DC/MS communication was configured.
                #
                msConnect = utils.getData(self.pdl, "MSCONNECTLIST", serveri)
                kernelIP = utils.getData(self.pdl, "KERNELIPLIST", serveri)
                wascell = utils.getData(self.pdl, "WASCELL", serveri)
                wasnode = utils.getData(self.pdl, "WASNODE", serveri)
                wasserver = utils.getData(self.pdl, "WASSERVER", serveri)
                debug("Connected to MS now? 1 - true; 0 - false: " + str(msConnect))
                if msConnect == "1" or kernelIP is not None:
                    ms_configured = True
                    #
                    # Write the start message for the console in the progress file.
                    #
                    progress_obj.write_progress_start(wasserver)
                    removeFromMS = msunconfig.msunconfig(self.pdl, serveri, progress_obj)
                    debug("removeFromMS: " + str(removeFromMS))
                    msgl =[]
                    msgl.append(wascell)
                    msgl.append(wasnode)
                    msgl.append(wasserver)
                    msgl.append(kernelIP)
                    if not removeFromMS:
                        infomsg = getmsg("MSUNCONFIG_NOT_REMOVED", "Unable to remove Data Collector {0}.{1}.{2} from Managing Server {3}", msgl)
                        progress_obj.write_progress(infomsg)
                        progress_obj.write_progress_finish(wasserver, self.common_constants.UNSUCCESSFUL)
                    else:
                        infomsg = getmsg("MSUNCONFIG_REMOVED", "Removed Data Collector {0}.{1}.{2} from Managing Server {3}", msgl)
                        progress_obj.write_progress(infomsg)
                        #
                        # Write the finish message for the console in the progress file.
                        #
                        progress_obj.write_progress_finish(wasserver, self.common_constants.SUCCESSFUL)
            #
            # If ms not configured for any of the servers, then issue a message.
            #
            if ms_configured == False:
                msmsg = getmsg("NOMSCONFIGURED", "No MS configured for the servers")
                progress_obj.write_progress(msmsg)

            self.thread_rc = self.FINISHED

        except Exception, errmsg:
            self.ex_thread_rc = self.FINISHED
            log.logException("An error occurred while executing the ms unconfig in a separate thread - %s" % (errmsg))
            #
            # Write the finisht message for the console in the progress file.
            #
            if progress_obj != None:
                progress_obj.write_progress_finish(wasserver, self.common_constants.UNSUCCESSFUL)


        #
        # Close the progress file
        #
        if progress_obj != None:
            progress_obj.close_file();

        debug("Exit _ms_unconfig")
