#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This class executes a wsadmin call, and makes the call in a separate thread
   and checks the status of wsadmin execution.
   This class behaves as an abstract class and should not be executed by itself.
   It must be inherited by other classes, and the drived class must implement the
   abstract method calls for its own environment.
'''

fixlev = "46315" # Last APAR, defect or story to update this module

import os
import sys
import time
import codecs

import debugUtil as log
import utils
import platformUtil
import wasdiscover
import doprompt
import re

from constants import constants
from Functhread import Functhread
from debugUtil import logDebug as debug
from resourcebundle import getmsg
from subprocess import call

import wlpUnconfigDC
import wlpConfigDC


class WsadminTask:

    START = 0
    FINISHED = 1
    NOSERVERS = -1

    common_constants = None
    pdl = None
    inputlist_filename = None
    log_file_name = None
    log_file_loc = None
    progressfile_loc = None
    jython_call = None
    server_indexlist = None
    caller_type = "None"
    thread_rc = -1
    wtype = -1
    wasProfileHome = None
    wroot = None
    profile = None
    conntype = None
    userid = None
    password = None
    host = None
    port = None
    retry = None


    def __init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_modname):
        '''
           This is the constructor routine.

           @param self - Reference to this object
           @param pdl - PDL reference
           @param wtype - Caller's type like config, unconfig,.....
           @param inputlist_fname - input list file name
           @param logf - log file name
           @param jython_modname - Jython module that is called by wsadmin
        '''

        debug("Entry WsadminTask.__init__ %s" % fixlev)

        #
        # Instantiate common constants class
        #
        self.common_constants = constants()

        #
        # Based on the type, set its corresponding string
        #
        self.wtype = wtype
        if wtype == self.common_constants.CONFIG :
            self.caller_type = getmsg("CONFIG", "config ")
        elif wtype == self.common_constants.RECONFIG:
            self.caller_type = getmsg("RECONFIG", "reconfig ")
        elif wtype == self.common_constants.UNCONFIG:
            self.caller_type = getmsg("UNCONFIG", "unconfig ")
        elif wtype == self.common_constants.MIGRATE:
            self.caller_type = getmsg("MIGRATE", "migrate ")
        elif wtype == self.common_constants.FINDSERVERS:
            self.caller_type = getmsg("FINDSERVERS", "findservers ")
        elif wtype == self.common_constants.CONFIGTEMPLATE:
            self.caller_type = getmsg("CONFIGTEMPLATE", "configuring XD template ")
        elif wtype == self.common_constants.DELETETEMPLATE:
            self.caller_type = getmsg("DELETETEMPLATE", "deleting XD template ")
        elif wtype == self.common_constants.DISCOVERTEMPLATE:
            self.caller_type = getmsg("DISCOVERTEMPLATE", "discovering XD template ")
        else:
            self.caller_type = "NOTYPE"
        debug("Caller type is - %s" % (self.caller_type))

        #
        # Initialize rest of the fields.
        #
        self.pdl = pdl_obj
        self.inputlist_filename = inputlist_fname
        debug("input list file name is %s" % (self.inputlist_filename))
        self.log_file_name = logf
        debug("Log file name is %s" % (self.log_file_name))
        self.jython_call = jython_modname
        debug("Jython call is  %s" % (self.jython_call))

        debug("Exit WsadminTask.__init__")


    def process(self, serverindex = None):
        '''
           Gets all the selected servers index and group them all together, and based
           on each group, it creates the input file that contains the progress file
           location, and calls the abstract call if user needs to add its own entries
           into the file, and calls wsadmin in a separate thread.
           Once the wsadmin execution is finished, the return code from wsadmin is checked
           for successful or unsuccessful execution.

           @param self - Reference to this object
           @param serverindex - It has the server index for the selected servers.
        '''

        debug("Entry WsadminTask.process %s" % fixlev)

        wascell = "None"
        wasnode = "None"
        wasprofile = "None"

        try:
            #
            # For each server index entry and group all the servers
            # that belongs to the same cell, node.
            #
            group_servers = utils.group_servers(self.pdl, serverindex)
            group_liberty_servers = {}
            keys=group_servers.keys()
            for key in keys:
                if key == self.common_constants.LIBERTY_CELL: #check if liberty server, and remove it from the original list
                    group_liberty_servers[key] = group_servers[key]
                    del group_servers[key]
                    break
            #
            # For each group which contains the index to SERVERS pdl, create the input list
            # and execute command wsadmin to configure it.  If there are no profiles to
            # process (there is a possibility that previous call to the pofiles could have
            # a successful connection, so no need to attempt again), then exit.
            #
            if len(group_servers) > 0:
                for group_name in group_servers.keys():
                    try:
                        index_list = None
                        index_list = group_servers[group_name]
                        self.server_indexlist = index_list
                        self.create_inputlist(index_list)
                        command = self.constructCommand(index_list[0])
                    except KeyboardInterrupt:
                        debug("Exiting the ITCAM Data Collector for WebSphere Configuration Utility.")
                        print "%s" % getmsg("CFG1088I", "Exiting...")
                        sys.exit(0)                                                
                    except:
                        etype, value, tbck = sys.exc_info()
                        utils.setData(self.pdl, "RETURNCODE", 1, "configDC")
                        if index_list != None:
                            wascell = utils.getData(self.pdl, "WASCELL", index_list[0])
                            wasnode = utils.getData(self.pdl, "WASNODE", index_list[0])
                            wasprofile = utils.getData(self.pdl, "PROFILE", index_list[0])
                        msgl =[wascell, wasnode, wasprofile]
                        #message = getmsg("CFG2019E", "Error found during {0} call process for Cell: {1}  Node: {2}  Profile: {3}. Next line has the error description.", msgl)
                        if self.wtype == self.common_constants.CONFIG :
                            message = getmsg("CFG2087E", "Error found during wsadmin execution for Configuration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.RECONFIG:
                            message = getmsg("CFG2088E", "Error found during wsadmin execution for Reconfiguration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.UNCONFIG:
                            message = getmsg("CFG2089E", "Error found during wsadmin execution for Unconfiguration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.MIGRATE:
                            message = getmsg("CFG2090E", "Error found during wsadmin execution for Migration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.FINDSERVERS:
                            message = getmsg("CFG2091E", "Error found during wsadmin execution for Find Servers for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.CONFIGTEMPLATE:
                            message = getmsg("CFG2092E", "Error found during wsadmin execution for XD template configuration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.DELETETEMPLATE:
                            message = getmsg("CFG2093E", "Error found during wsadmin execution for XD template deletion for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.DISCOVERTEMPLATE:
                            message = getmsg("CFG2094E", "Error found during wsadmin execution for XD template discovery for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        print "%s" % message
                        print "%s" % value
                        msgl =[self.caller_type, wascell, wasnode, wasprofile]
                        message = "Error found during {0} call process for Cell: {1}  Node: {2}  Profile: {3} "
                        message = utils.format_message(message, msgl)
                        debug(message)
                        log.logException("Error occurred in calling %s type  %s   tback  %s" % (self.caller_type, etype, tbck))
                        continue
                    #
                    # Execute the command.  The command is execute in a separate
                    # thread.  The reason for that is to be able to show the progress
                    # status for the user.
                    #
                    wascell = utils.getData(self.pdl, "WASCELL", index_list[0])
                    wasnode = utils.getData(self.pdl, "WASNODE", index_list[0])
                    wasprofile = utils.getData(self.pdl, "PROFILE", index_list[0])
                    wasprofilehome = utils.getData(self.pdl, "PROFILEHOME", index_list[0])
                    if wasprofilehome != None:
                        wasprofilehome = wasprofilehome.strip()
                        debug("wasprofilehome: " + wasprofilehome)
                    msgl =[wascell, wasnode, wasprofile]
                    #message = getmsg("CFG2025I", "Processing {0} call for  Cell: {1}  Node: {2}  Profile: {3}  ........", msgl)
                    if self.wtype == self.common_constants.CONFIG :
                        message = getmsg("CFG2095I", "Processing Configuration call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.RECONFIG:
                        message = getmsg("CFG2096I", "Processing Reconfiguration call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.UNCONFIG:
                        message = getmsg("CFG2097I", "Processing Unconfiguration call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.MIGRATE:
                        message = getmsg("CFG2098I", "Processing Migration call Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.FINDSERVERS:
                        message = getmsg("CFG2099I", "Processing Find Servers call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.CONFIGTEMPLATE:
                        message = getmsg("CFG2100I", "Processing Configuring XD template call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.DELETETEMPLATE:
                        message = getmsg("CFG2101I", "Processing Deleting XD template call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    elif self.wtype == self.common_constants.DISCOVERTEMPLATE:
                        message = getmsg("CFG2102I", "Processing Discovering XD template call for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                    print "%s" % message
                    msgl = [self.caller_type, wascell, wasnode, wasprofile]
                    message = "Processing {0} call for  Cell: {1}  Node: {2}  Profile: {3}"
                    message = utils.format_message(message, msgl)
                    debug(message)
                    debug("starting the command thread...")
                    #
                    # Start wsadmin command.  The wsdadmin runs in a separate thread
                    #
                    self.start_wsadmin(command, wasprofilehome, wasprofile, index_list, wascell, wasnode)
            else:
                if len(group_liberty_servers) == 0:
                    message = getmsg("CFG2039I", "No profiles to process, possible previous calls did not connect to the profiles successfully")
                    print "%s" % message
                    print "\n"

        except Exception, errmsg:
            utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
            msg1 = "Error occurred processing %s call - %s" % (self.caller_type, errmsg)
            log.logException(msg1)
            msgl =[]
            msgl.append(self.caller_type)
            msgl.append(errmsg)
            print getmsg("CFG2038E", "Error occurred while processing {0} call - {1}", msgl)
            raise errmsg

        if len(group_liberty_servers) > 0:
            for group_name in group_liberty_servers.keys():
                try:
                    index_list = None
                    index_list = group_liberty_servers[group_name]
                    self.server_indexlist = index_list
                    self.create_inputlist(index_list)
                except:
                    debug(message)
                    log.logException("Error occurred in calling %s type  %s   tback  %s" % (self.caller_type, etype, tbck))
                    continue
                wascell = utils.getData(self.pdl, "WASCELL", index_list[0])
                wasnode = utils.getData(self.pdl, "WASNODE", index_list[0])
                wasprofile = utils.getData(self.pdl, "PROFILE", index_list[0])
                msgl =[]
                msgl.append(wascell)
                msgl.append(wasnode)
                msgl.append(wasprofile)
                if self.wtype == self.common_constants.CONFIG or self.wtype == self.common_constants.MIGRATE:
                    message = getmsg("CFG2103I", "Processing Configuration call for Liberty profile: {0}.{1}", msgl)
                    print "%s" % message
                    wlpConfigDC.startConfig(self.pdl, self.server_indexlist, self.wtype)
                elif self.wtype == self.common_constants.RECONFIG:
                    message = getmsg("CFG2104I", "Processing Reconfiguration call for Liberty profile: {0}.{1}", msgl)
                    print "%s" % message
                    wlpConfigDC.startConfig(self.pdl, self.server_indexlist, self.wtype)
                elif self.wtype == self.common_constants.UNCONFIG:
                    message = getmsg("CFG2105I", "Processing Unconfiguration call for Liberty profile: {0}.{1}", msgl)
                    print "%s" % message
                    wlpUnconfigDC.startUnconfig(self.pdl, self.server_indexlist)
                msgl = [self.caller_type, wascell, wasnode, wasprofile]
                message = "Processing {0} call for  Liberty Cell: {1}  Node: {2}  Profile: {3}"
                message = utils.format_message(message, msgl)
                debug(message)

        debug("Exit WsadminTask.process")


    def create_inputlist(self, server_indexlist = None):
        '''
           This routine creates the inputlist for a given profile.
           The progress file has the information that is read by the main thread, and will be displayed to the console
           as part of the progress processing.  The progress file has a prefix
           which is a cell.node.profile.
           Once the progress entry is saved in the input list, method "set_inputlist_entries"
           is called, which each caller if required to add entries into the input list
           must overwrite the method call and add its own entry, otherwise, the default
           implementation does nothing.

           @param server_indexlist - Index list of the selected servers for a given group
        '''

        inputlist_file = None

        debug("Entry WsadminTask.create_inputlist %s - %s" % (fixlev,server_indexlist))

        datadir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data"))
        if not utils.createDataDir(self.pdl["CONFIGHOME"][0]):
            # To do: a failure here should really terminate.  Other stuff is written to data.
            datadir = os.path.normpath(self.pdl["CONFIGHOME"][0])
            debug("Alternative data home in use: %s" % datadir)

        inputlist_file_loc = os.path.normpath(os.path.join(datadir, self.inputlist_filename))
        debug("Input list file location - %s" % (inputlist_file_loc))

        try:
            #
            # Write the common record in the input list file, which contains the progress
            # file location and the log file location.
            #
            inputlist_file = codecs.open(inputlist_file_loc, mode="w", encoding="utf-8")
            self._set_common_task_record(server_indexlist)
            inputlist_file.write(self.progressfile_loc)
            inputlist_file.write("\n")
            #
            # Call the abstract method call set_inputlist_entries, which has the default
            # implementation of nothing, but if caller has overwritten it, then the caller's
            # method will be called.
            #
            self.set_inputlist_entries(inputlist_file, server_indexlist)

        except Exception, errmsg:
            msg1 = "Error occurred processing input list file - %s - %s" % (inputlist_file_loc, errmsg)
            log.logException(msg1)
            raise errmsg

        finally:
            if inputlist_file != None:
                inputlist_file.close()

        debug("Exit WsadminTask.create_inputlist")


    def constructCommand(self, serveri = None):
        '''
           Constructs the wsadmin command.

           @param self - Reference to this object
           @param serveri - Server index or None
        '''

        user = None
        password = None

        debug("Entry WsadminTask.constructCommand %s" % fixlev)

        #
        # Create the wsadmin data. The wsadmin data are the information needed
        # to setup the wsadmin.
        #
        self.set_wsadmin_data(serveri)
        #
        # Set required fields for wsadmin
        #
        debug("osfamily: %s" % platformUtil.getOSFamily())
        dchome = self.pdl["CONFIGHOME"][0]
        debug("dchome: %s" % dchome)
        if platformUtil.isWindows():
            dchome=utils.replaceBackSlash(dchome)
        pythonpath = os.path.normpath(os.path.join(dchome, "bin")) + os.pathsep + os.path.normpath(os.path.join(dchome, "bin", "lib"))
        datadir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data"))
        wasProfileHome = self.get_wasprofilehome()
        if wasProfileHome is not None:
            wasProfileHome = wasProfileHome.strip()
            debug("wasProfileHome: %s" % wasProfileHome)
        wroot = self.get_wasroot()
        profile = self.get_profile_name()
        wsadmin_classpath = os.path.normpath(os.path.join(dchome, "toolkit", "lib", "jiti.jar"))
        wsadmin_classpath = wsadmin_classpath + os.pathsep + os.path.normpath(os.path.join(dchome, "itcamdc", "lib", "ppe.installer.jar"))
        if platformUtil.IS_zOS_OS():
            wsadmin_classpath = wsadmin_classpath + os.pathsep + os.path.normpath(os.path.join(dchome, "itcamdc", "lib", "installbundle.jar"))
        else:
            wsadmin_classpath = wsadmin_classpath + os.pathsep + os.path.normpath(os.path.join(dchome, "bin", "lib", "installbundle.jar"))
        wsadmin_classpath = utils.replaceBackSlash(wsadmin_classpath)
        conntype = self.get_wasconntype()
        port = self.get_wasport()
        #
        # Check for WAS security
        #
        wasdiscover.checkWASSecurity(self.pdl, wasProfileHome, wroot, profile, self.retry)
        userlist = self.pdl["USERLIST"][0]
        if userlist is not None:
            if userlist.has_key(wasProfileHome):
                user = userlist[wasProfileHome]
                debug("user: %s" %(user))
            else:
                user = self.pdl["USER"][0]
        else:
            user = self.pdl["USER"][0]
        user = self._character_escape(user)
        #
        # If userid was entered as blank, then set it as blank within double quotes
        # as this will cause the wsadmin to get an error which is detected to do
        # a retry.
        #
        if user is not None and len(user) == 0:
            user = '"  "'
            debug("USER is blank")
        passwordlist = self.pdl["PASSWORDLIST"][0]
        if passwordlist is not None:
            if passwordlist.has_key(wasProfileHome):
                password = passwordlist[wasProfileHome]
            else:
                password = self.pdl["PASSWORD"][0]
        else:
            password = self.pdl["PASSWORD"][0]
        password = self._character_escape(password)
        #
        # If password was entered as blank, then set it as blank within double quotes
        # as this will cause the wsadmin to get an error which is detected to do
        # a retry.
        #
        if password is not None and len(password) == 0:
            password = '"  "'
            debug("PASSWORD is blank")
        #
        # Create the command
        #
        if platformUtil.IS_OS400_OS():
            command = os.path.normpath(os.path.join(wasProfileHome, "bin", platformUtil.getWSAdminScript()))
        else:
            command = os.path.normpath(os.path.join(wroot, "bin", platformUtil.getWSAdminScript()))
        command = utils.add_quotes(command)

        if platformUtil.isZ():
            command = command + " -javaoption -Ditcam.debug=-debug"
        if conntype is not None and conntype.strip() != "None" and conntype.strip() != "" and len(conntype) > 0:
            command = command + " -conntype " + conntype
        if port is not None and port.strip() != "None" and port.strip() != "" and len(port) > 0:
            command = command + " -port " + port

        if profile != None:
            command = command + " -profileName " + utils.add_quotes(profile)

        if user is not None:
            command = command + " -user " + user
        if password is not None:
            command = command + " -password " + password
        #
        # The back slash is changed to forward /, as the config home is in ....\bin for
        # windows, and \b is translated as back character and that cause the file location
        # has an invalid character in it.
        #
        inputlist_filename = os.path.normpath(os.path.join(datadir, self.inputlist_filename))
        inputlist_filename = utils.replaceBackSlash(inputlist_filename)
        inputlist_filename = utils.add_quotes(inputlist_filename)
        debug("Input list location -  '%s'" % (inputlist_filename))

        wsadmin_classpath = utils.add_quotes(wsadmin_classpath)
        jython_call = utils.add_quotes(os.path.normpath(os.path.join(dchome, "bin", "lib", self.jython_call)))

        command = command + " -wsadmin_classpath " + wsadmin_classpath + " -lang jython -f " + jython_call + " -javaoption \"-Dpython.path=" + pythonpath + "\" " + inputlist_filename

        #
        # Get the log file location and set it in the command.
        #
        self.log_file_loc = self.get_logfile_loc(serveri)

        debug("Redirecting output into '%s'" % self.log_file_loc)
        if not platformUtil.isWindows():
            #in case of non-windows making redirect part of the command string
            redir_output = " > %s 2>&1" % (utils.add_quotes(self.log_file_loc))
            command = command + redir_output
        if password is not None and password != "\" \"":
            commandForLog = re.sub("-password\s+[^\s]+", "-password ********", command)
        else:
            commandForLog = command
        debug("command: " + commandForLog)
        #
        # Rename any existing log files.
        #
        if self.log_file_loc is not None:
            utils.rename_archival_file(self.log_file_loc)

        debug("Exit WsadminTask.constructCommand")

        return command


    def start_wsadmin(self,
                      command,
                      wasprofilehome_loc,
                      wasprofile_name,
                      server_indexlist = None,
                      wascell = None,
                      wasnode = None):
        '''
           This routine starts the wsadmin and check its return code, and based
           on the return code, process accordingly.

           @param command - Command to be executed
           @param wasprofilehome_loc - WAS profile home location
           @param wasprofile_name - WAS profile name
           @param server_indexlist - A list of servers within a group (profile)
           @param wascell - WAS cell or None
           @param wasnode - WAS node or None
        '''

        retry_count = 0
        message = ""

        debug("Entry WsadminTask.start_wsadmin %s" % fixlev)

        while True:
            #
            # Open and close the progress file so it is reset.
            #
            progress_file = codecs.open(self.progressfile_loc, mode="w", encoding="utf-8")
            progress_file.close()
            #
            # Start wsadmin command.  The wsdadmin runs in a separate thread
            #
            self._start_wsadmin_thread(command, self.progressfile_loc)
            #
            # Check the wsadmin status.
            #
            rc = utils.getData(self.pdl, "RETURNCODE")
            rc = int(rc)

            # Always process the log for diagnostic purposes
            error_msg, is_user_passwd_problem, is_was_connection_problem = utils.get_was_error_msg(self.log_file_loc)

            # Handle known special cases where there really isn't an error preventing wsadmin from running
            # 1. WASX7011E: Cannot find file "/tmp/dummy.py" with wsadmin rc = 99
            if rc == 99:
                if error_msg == "" or len(error_msg) == 0:
                    debug("  WARNING: wsadmin rc = 99 but no error messages indicating failure")
                    debug("    rc reset to 0; processing continues")
                    rc = 0
                    utils.setData(self.pdl, "RETURNCODE", 0, "WsadminTask_upd")

            if rc == 0:
                #
                # If we have password problem, prompt the user for retrying or abort.
                #
                if is_user_passwd_problem:
                    process, retry_count, command = self._retry_uid_password(error_msg,
                                                                             retry_count,
                                                                             wasprofilehome_loc,
                                                                             wasprofile_name,
                                                                             server_indexlist,
                                                                             wascell,
                                                                             wasnode)
                    if not process:
                        for serveri in range(len(self.pdl["SERVERS"][0])):
                            profilehome = self.pdl["PROFILEHOME"][0][serveri]
                            wasprofile = self.pdl["PROFILE"][0][serveri]
                            wascell_pdl = self.pdl["WASCELL"][0][serveri]
                            wasnode_pdl = self.pdl["WASNODE"][0][serveri]
                            wasserver = self.pdl["SERVERS"][0][serveri]
                            debug("%s=%s %s=%s server_indexlist=%s" % (profilehome, wasprofilehome_loc, wasprofile, wasprofile_name, server_indexlist))
                            if (profilehome == wasprofilehome_loc and wasprofile == wasprofile_name and wascell_pdl == wascell and wasnode_pdl == wasnode):
                                debug("setting return code to 99")
                                self.pdl["CONFIGRETURNCODE"][0][serveri] = 99
                        break
                elif is_was_connection_problem:
                    message = getmsg("CFG5037E", "Cannot connect to WebSphere. Make sure the WebSphere server or the node agent is up and running.")
                    print "%s" % message
                    print "\n"
                    break
                elif error_msg != None and error_msg != "" and len(error_msg) > 0:
                    debug("Error occurred in %s call process: %s" % (self.caller_type, error_msg))
                    if wascell is None:
                        msgl =[self.caller_type, wasprofile_name]
                        message = getmsg("CFG2031E", "Error found during wsadmin execution for {0} call process for Profile: {1}", msgl)
                    else:
                        msgl =[self.caller_type, wascell, wasnode, wasprofile_name]
                        message = getmsg("CFG2020E", "Error found during wsadmin execution for {0} call process for Cell: {1}  Node: {2}  Profile: {3}. ", msgl)
                    print "%s" % message
                    print "\n"
                    msgl = [self.caller_type, wascell, wasnode, wasprofile_name, error_msg]
                    message = "Error found during {0} call process for Cell: {1}  Node: {2}  Profile: {3}  - {4}"
                    message = utils.format_message(message, msgl)
                    log.logError(message)
                    #
                    # If there are errors, do not raise exception. set pdl[RETURNCODE][0] to 1, and indicate in
                    # GROUPSERVERS for the profile that wsadmin did not execute successfully.
                    #.
                    utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
                    break
                else:
                    #
                    # Call msunconfig if MS needs to be unconfigured.
                    #
                    self.start_msunconfig(server_indexlist, self.progressfile_loc)
                    if wascell is None:
                        msgl =[wasprofile_name]
                        #message = getmsg("CFG2032I", "Successfully executed {0} for Profile: {1}", msgl)
                        if self.wtype == self.common_constants.CONFIG :
                            message = getmsg("CFG2071I", "Successfully executed Configuration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.RECONFIG:
                            message = getmsg("CFG2072I", "Successfully executed Reconfiguration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.UNCONFIG:
                            message = getmsg("CFG2073I", "Successfully executed Unconfiguration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.MIGRATE:
                            message = getmsg("CFG2074I", "Successfully migrated Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.FINDSERVERS:
                            message = getmsg("CFG2075I", "Successfully found servers for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.CONFIGTEMPLATE:
                            message = getmsg("CFG2076I", "Successfully configured XD template for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.DELETETEMPLATE:
                            message = getmsg("CFG2077I", "Successfully deleted XD template for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.DISCOVERTEMPLATE:
                            message = getmsg("CFG2078I", "Successfully discovered XD template for Profile: {0}", msgl)
                    else:
                        msgl =[wascell, wasnode, wasprofile_name]
                        #message = getmsg("CFG2021I", "Successfully executed {0}  Cell: {1}  Node: {2}  Profile: {3}", msgl)
                        if self.wtype == self.common_constants.CONFIG :
                            message = getmsg("CFG2063I", "Successfully executed Configuration for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.RECONFIG:
                            message = getmsg("CFG2064I", "Successfully executed Reconfiguration for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.UNCONFIG:
                            message = getmsg("CFG2065I", "Successfully executed Unconfiguration for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.MIGRATE:
                            message = getmsg("CFG2066I", "Successfully migrated Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.FINDSERVERS:
                            message = getmsg("CFG2067I", "Successfully found servers for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.CONFIGTEMPLATE:
                            message = getmsg("CFG2068I", "Successfully configured XD template for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.DELETETEMPLATE:
                            message = getmsg("CFG2069I", "Successfully deleted XD template for Cell: {0}  Node: {1}  Profile: {2}", msgl)
                        elif self.wtype == self.common_constants.DISCOVERTEMPLATE:
                            message = getmsg("CFG2070I", "Successfully discovered XD template for Cell: {0}  Node: {1}  Profile: {2}", msgl)

                    print "%s" % message
                    print "\n"
                    msgl = [self.caller_type, wascell, wasnode, wasprofile_name]
                    message = "Successfully executed {0}  Cell: {1}  Node: {2}  Profile: {3} "
                    message = utils.format_message(message, msgl)
                    debug(message)
                    break

            else:
                debug("  WARNING: command rc not zero. rc= " + str(rc) )
                utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
                if is_user_passwd_problem:
                    process, retry_count, command = self._retry_uid_password(error_msg,
                                                                             retry_count,
                                                                             wasprofilehome_loc,
                                                                             wasprofile_name,
                                                                             server_indexlist,
                                                                             wascell,
                                                                             wasnode)
                    if not process:
                        for serveri in range(len(self.pdl["SERVERS"][0])):
                            profilehome = self.pdl["PROFILEHOME"][0][serveri]
                            wasprofile = self.pdl["PROFILE"][0][serveri]
                            wascell_pdl = self.pdl["WASCELL"][0][serveri]
                            wasnode_pdl = self.pdl["WASNODE"][0][serveri]
                            wasserver = self.pdl["SERVERS"][0][serveri]
                            debug("%s=%s %s=%s server_indexlist=%s" % (profilehome, wasprofilehome_loc, wasprofile, wasprofile_name, server_indexlist))
                            if (profilehome == wasprofilehome_loc and wasprofile == wasprofile_name and wascell_pdl == wascell and wasnode_pdl == wasnode):
                                debug("setting return code to 99")
                                self.pdl["CONFIGRETURNCODE"][0][serveri] = 99
                        break
                elif is_was_connection_problem:
                    message = getmsg("CFG5037E", "Cannot connect to WebSphere. Make sure the WebSphere server or the node agent is up and running.")
                    print "%s" % message
                    print "\n"
                    break
                elif wascell is None:
                    msgl =[self.caller_type, wasprofile_name]
                    message = getmsg("CFG2035W", "Possible failure for {0} call process for Profile: {1}.  Check the log for more information.", msgl)
                    if is_was_connection_problem:
                        message = getmsg("CFG5037E", "Cannot connect to WebSphere. Make sure the WebSphere server or the node agent is up and running.")
                    print "%s" % message
                    print "\n"
                    msgl =[self.caller_type, wasprofile_name, error_msg]
                    tmessage = "Possible failure found during {0} call process for Profile: {1}  - {2}"
                    tmessage = utils.format_message(tmessage, msgl)
                    log.logError(tmessage)
                    break
                else:
                    msgl =[self.caller_type, wascell, wasnode, wasprofile_name]
                    message = getmsg("CFG2040W", "Possible failure for {0} call process for Cell: {1}  Node: {2}  Profile: {3}.  Check the log for more information.", msgl)
                    print "%s" % message
                    print "\n"
                    msgll =[self.caller_type, wascell, wasnode, wasprofile_name, error_msg]
                    tmessage = "Possible failure for {0} call process for Cell: {1}  Node: {2}  Profile: {3} - {4}"
                    tmessage = utils.format_message(tmessage, msgll)
                    log.logError(tmessage)
                    break

        debug("Exit WsadminTask.start_wsadmin")


    def _retry_uid_password(self,
                            error_msg,
                            retry_count,
                            wasprofilehome_loc,
                            wasprofile_name,
                            server_indexlist,
                            wascell,
                            wasnode):
        '''
           Prompt the user for retrying the userid and password.

           @param error_msg - Error message for the uid/pwd
           @param retry_count - Count to retry for uid/pwd
           @param wasprofilehome_loc - WAS profile home location
           @param wasprofile_name - WAS profile name
           @param server_indexlist - A list of servers within a group (profile)
           @param wascell - WAS cell or None
           @param wasnode - WAS node or None
           @return - true/false to continue and retry count and command
        '''

        process = True
        command = None

        debug("Entry WsadminTask._retry_uid_password %s" % fixlev)

        print "   %s" % getmsg("WRONG_UID_PWD", "- Incorrect user or password if global security is activated")
        if retry_count == self.common_constants.MAX_TRY:
            utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
            max_retry_msg = getmsg("CFG5003E", "Error: Maximum number of retries have been reached.")
            print max_retry_msg
            process = False
        else:
            self.retry = doprompt.prompt_retry(self.pdl)
            if not self.retry:
                utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
                process = False
            else:
                wasdiscover.clear_WASsecurity(self.pdl, wasprofilehome_loc)
                self.pdl["USECLIENTPROPS"][0:2] = False, "WsadminTask._retry_uid_password"
                try:
                    debug("retry: " + str(self.retry))
                    if server_indexlist is None:
                        command = self.constructCommand()
                    else:
                        command = self.constructCommand(server_indexlist[0])
                    retry_count = retry_count + 1
                except Exception, errmsg:
                    etype, value, tbck = sys.exc_info()
                    utils.setData(self.pdl, "RETURNCODE", 1, "WsadminTask")
                    log.logException("Error occurred while constructing the command - %s" % (errmsg))
                    if wascell is None:
                        msgl =[wasprofile_name]
                        #message = getmsg("CFG2031E", "Error found during wsadmin execution for {0} call process for Profile: {1}", msgl)
                        if self.wtype == self.common_constants.CONFIG :
                            message = getmsg("CFG2079E", "Error found during wsadmin execution for Configuration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.RECONFIG:
                            message = getmsg("CFG2080E", "Error found during wsadmin execution for Reconfiguration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.UNCONFIG:
                            message = getmsg("CFG2081E", "Error found during wsadmin execution for Unconfiguration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.MIGRATE:
                            message = getmsg("CFG2082E", "Error found during wsadmin execution for Migration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.FINDSERVERS:
                            message = getmsg("CFG2083E", "Error found during wsadmin execution for Find Servers for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.CONFIGTEMPLATE:
                            message = getmsg("CFG2084E", "Error found during wsadmin execution for XD template configuration for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.DELETETEMPLATE:
                            message = getmsg("CFG2085E", "Error found during wsadmin execution for XD template deletion for Profile: {0}", msgl)
                        elif self.wtype == self.common_constants.DISCOVERTEMPLATE:
                            message = getmsg("CFG2086E", "Error found during wsadmin execution for XD template discovery for Profile: {0}", msgl)
                        print message
                        msgl =[self.caller_type, wasprofile_name, errmsg]
                        message = "Error found during {0} call process for Profile: {1} - {2} "
                        message = utils.format_message(message, msgl)
                        log.logError(message)
                    else:
                        msgl =[wascell, wasnode, wasprofile_name]
                        #message = getmsg("CFG2020E", "Error found during {0} call process for Cell: {1}  Node: {2}  Profile: {3}. ", msgl)
                        if self.wtype == self.common_constants.CONFIG :
                            message = getmsg("CFG2087E", "Error found during wsadmin execution for Configuration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.RECONFIG:
                            message = getmsg("CFG2088E", "Error found during wsadmin execution for Reconfiguration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.UNCONFIG:
                            message = getmsg("CFG2089E", "Error found during wsadmin execution for Unconfiguration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.MIGRATE:
                            message = getmsg("CFG2090E", "Error found during wsadmin execution for Migration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.FINDSERVERS:
                            message = getmsg("CFG2091E", "Error found during wsadmin execution for Find Servers for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.CONFIGTEMPLATE:
                            message = getmsg("CFG2092E", "Error found during wsadmin execution for XD template configuration for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.DELETETEMPLATE:
                            message = getmsg("CFG2093E", "Error found during wsadmin execution for XD template deletion for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        elif self.wtype == self.common_constants.DISCOVERTEMPLATE:
                            message = getmsg("CFG2094E", "Error found during wsadmin execution for XD template discovery for Cell: {0}  Node: {1}  Profile: {2}.", msgl)
                        print "%s" % message
                        print "%s" % value
                        msgl =[self.caller_type, wascell, wasnode, wasprofile_name, errmsg]
                        message = "Error found during {0} call process for Cell: {1}  Node: {2}  Profile: {3} - {4} "
                        message = utils.format_message(message, msgl)
                        log.logError(message)
                    log.logError("Error occurred in calling %s type  %s   tback  %s" % (self.caller_type, etype, tbck))
                    process = False

        if command is not None:
            commandForLog = re.sub("-password\s+[^\s]+", "-password ********", command)
        else:
            commandForLog = command
        debug("Exit WsadminTask._retry_uid_password - process = %s, retry_count = %s, command = %s" % (process, retry_count, commandForLog))

        return process, retry_count, command


    def _start_wsadmin_thread(self, command, progressfile_loc):
        '''
           This routine starts the wsadmin in a separate command, and monitors the
           progress file and displays the info from progress file into the display.

           @param command - Command to be executed
           @param progressfile_loc - progres file location.
        '''
        command_debug = re.sub("-password\s+[^\s]*", "-password ********", command)
        debug("Entry WsadminTask._start_wsadmin_thread %s - %s - %s" % (fixlev, command_debug, progressfile_loc))

        #
        # start wsadmin in a separate thread.
        #
        t1 = Functhread(self._execute_command, command)
        debug("starting wsadmn thread...")
        t1.start()
        sys.stdout.write("  %s"  % getmsg("CONNECTING", "Connecting...."))
        sys.stdout.flush();
        #
        # Start reading the progress file and display all the data in the file
        # to the console.
        #
        self._check_progress(progressfile_loc)
        #
        # Now wait till the wsadmin thread is finished.
        #
        t1.join()

        debug("Exit WsadminTask._start_wsadmin_thread")


    def _execute_command(self, command):
        '''
           This call runs in a separate thread which executes the provided command.

           @param command - Command to be executed
        '''
        command_debug = re.sub("-password\s+[^\s]*", "-password ********", command)
        debug("Entry WsadminTask._execute_command %s - %s" % (fixlev, command_debug))

        try:
            self.thread_rc = self.START
            debug("wsadmin thread started")
            if platformUtil.isWindows():
                #for windows we redirect subprocess output, due to recent chnages in java command string can not have redirect symbol in
                if self.log_file_loc is not None:
                    fhandler = open(self.log_file_loc, "w")
                else:
                    fhandler = open(os.devnull, "w")
                rc = call(command, stdout = fhandler, stderr = fhandler)
                fhandler.close()
            else:
                #in case of non-windows command has redirect portion as part of command string
                rc = call(command, shell=True)
            self.thread_rc = self.FINISHED
            debug("  command rc: " + str(rc) )
            utils.setData(self.pdl, "RETURNCODE", rc, "WsadminTask")

        except Exception, errmsg:
            self.thread_rc = self.FINISHED
            log.logException("An error occurred while executing the wsadmin command in a separate thread - %s" % (errmsg))

        debug("Exit WsadminTask._execute_command")


    def _check_progress(self, progressfile_loc):
        '''
           Checks the progress file peridically and prints the messages to the
           console .

           @param progressfile_loc - Progress file path location
        '''

        done = False
        last_time = False
        repeat = False
        first_time = True
        progress_file = None

        debug("Entry WsadminTask._check_progress %s" % fixlev)

        #
        # Start reading the progress file and display all the data in the file
        # to the console.
        #
        try:
            progress_file = codecs.open(progressfile_loc, mode="r", encoding="utf-8")
            while not done:
                where = progress_file.tell()
                if where == 0:
                    sys.stdout.write(self.common_constants.PROGRESS_DOT)
                    sys.stdout.flush();
                line = progress_file.readline()

                if not line:
                    #
                    # If wsadmin already finished, go thru the loop one more time to
                    # make sure that all the data is read from the progress file.
                    #
                    if last_time == True:
                        #
                        # If no records in the progress file yet, then make sure that any other
                        # message is set to the next line.
                        #
                        if first_time == True or repeat == True:
                            sys.stdout.write(self.common_constants.PROGRESS_DOT_NL)
                            sys.stdout.flush();
                            repeat = False
                        done = True
                    if self.thread_rc == self.FINISHED:
                        last_time = True
                    if repeat == True:
                        sys.stdout.write(self.common_constants.PROGRESS_DOT)
                        sys.stdout.flush();
                    time.sleep(self.common_constants.PROGRESS_WAIT_TIME)
                    progress_file.seek(where)
                else:
                    if first_time == True:
                        first_time = False
                        sys.stdout.write(self.common_constants.PROGRESS_DOT_NL)
                        sys.stdout.flush();
                    debug("Progress record - %s" % (line))
                    line = line.rstrip("\r\n")
                    #line = str(line)
                    if repeat == True:
                        sys.stdout.write(self.common_constants.PROGRESS_DOT_NL)
                        sys.stdout.flush();
                    #
                    # Check for '^', and based on the type set the message to the console.
                    # Decode the first slot, as the split changes to byte string, so we are
                    # converting it back to unicode.
                    #
                    msg = line.split(self.common_constants.SEPARATOR_FOR_REPEAT_MESSAGE)
                    debug("Progress record before - %s" % (msg))
                    msg_unicode = msg[0]
                    debug("Progress record before - %s" % (msg_unicode))
                    if len(msg) > 1:
                        if msg[1] == self.common_constants.NOINDENT:
                            sys.stdout.write("     " + msg_unicode + "\n")
                            sys.stdout.flush();
                            repeat = False
                        else:
                            sys.stdout.write("         " + msg_unicode)
                            sys.stdout.flush();
                            repeat = True
                    else:
                        sys.stdout.write("         " + msg_unicode + "\n")
                        repeat = False

        finally:
            #
            # Close the file.
            #
            if progress_file != None:
                progress_file.close()

        debug("Exit WsadminTask._check_progress")


    def _set_common_task_record(self, server_indexlist):
        '''
           This is the default implementation to create the common task record in the input list.
           It contains the progress file location.

           @param server_indexlist - Index list of the selected servers for a given group
        '''

        debug("Entry WsadminTask._set_common_task_record %s - %s" % (fixlev, server_indexlist))

        #
        # Get progress file location.
        #
        self.progressfile_loc = self.get_progress_file_location(server_indexlist)
        debug("Progress log location is - %s" % (self.progressfile_loc))

        debug("Exit WsadminTask._set_common_task_record")


    def get_progress_file_location(self, server_indexlist):
        '''
           This is the default implementation to get the progress file location.

           @param server_indexlist - Index list of the selected servers for a given group
        '''

        debug("Entry WsadminTask.get_progress_file_location %s" % fixlev)

        #
        # The progress file has the information that will be displayed to the console
        # as part of the progress processing.  The progress file has a prefix
        # which is a cell.node.profile.
        #
        wascell = utils.getData(self.pdl, "WASCELL", server_indexlist[0])
        wasnode = utils.getData(self.pdl, "WASNODE", server_indexlist[0])
        wasprofile = utils.getData(self.pdl, "PROFILE", server_indexlist[0])
        progressfile_prefix = wasprofile + "." + wascell + "." + wasnode + "." + "progress"
        progressfile_loc = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data", progressfile_prefix))

        debug("Exit WsadminTask.get_progress_file_location - %s" % (progressfile_loc))

        return progressfile_loc


    def set_wsadmin_data(self, serveri):
        '''
           This routine sets the wsadmin data from PDL.  This is the default implementation which
           gets the data from the PDL based on the server index

           @param server_indexlist - Index of a server that belongs to a group
        '''

        debug("Entry WsadminTask.set_wsadmin_data %s" % fixlev)

        self.wasProfileHome = utils.getData(self.pdl, "PROFILEHOME", serveri)
        debug("wasProfileHome: " + self.wasProfileHome)
        self.wroot = utils.getData(self.pdl, "WASINSTALLROOT", serveri)
        debug("WASINSTALLROOT: " + self.wroot)
        self.profile = utils.getData(self.pdl, "PROFILE", serveri)
        debug("profile: " + self.profile)
        self.conntype = utils.getData(self.pdl, "CONNTYPELIST")
        if self.conntype != None:
            self.conntype = self.conntype[serveri]
            debug("conntype: " + self.conntype)
        self.host = utils.getData(self.pdl, "HOSTLIST")
        if self.host != None:
            self.host = self.host[serveri]
            debug("host: " + self.host)
        self.port = utils.getData(self.pdl, "PORTLIST")
        if self.port != None:
            self.port = self.port[serveri]
            debug("port: " + self.port)

        debug("Exit WsadminTask.set_wsadmin_data")


    def get_server_indexlist(self):
        '''
           This routine gets the server index list of current group.  This list
           is saved when each group is processed.

           @return - Server index list
        '''

        return self.server_indexlist


    def get_wasprofilehome(self):
        '''
           This routine gets the WAS profile home

           @return - WAS profile home
        '''

        debug("get_wasprofilehome of WsadminTask is called")
        return self.wasProfileHome


    def get_wasroot(self):
        '''
           This routine gets WAS root.

           @return - WAS root
        '''

        debug("get_wasroot of WsadminTask is called")
        return self.wroot


    def get_profile_name(self):
        '''
           This routine gets WAS profile name

           @return - WAS profile name
        '''

        debug("get_profile_name of WsadminTask is called")
        return self.profile


    def get_washost(self):
        '''
           This routine gets WAS host name.

           @return - WAS host name
        '''

        debug("get_washost of WsadminTask is called")
        return self.host


    def get_wasport(self):
        '''
           This routine gets WAS port.

           @return - WAS port number
        '''

        debug("get_wasport of WsadminTask is called")
        return self.port


    def get_wasconntype(self):
        '''
           This routine gets WAS password.

           @return - WAS password
        '''

        debug("get_wasconntype of WsadminTask is called")
        return self.conntype


    def get_logfile_loc(self, serveri):
        '''
           This routine gets log file location

           @param serveri - Index of a server in pdl.
           @return - Log file location
        '''

        debug("Entry get_logfile_loc - %s" % (serveri))

        debug("get_logfile_loc of WsadminTask is called")
        datadir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "data"))
        wascell = utils.getData(self.pdl, "WASCELL", serveri)
        wasnode = utils.getData(self.pdl, "WASNODE", serveri)
        wasprofile = utils.getData(self.pdl, "PROFILE", serveri)
        logfile_loc  = os.path.normpath(os.path.join(datadir, wasprofile + "."  + wascell + "." + wasnode + "." + self.log_file_name))

        debug("Entry get_logfile_loc - %s" % (logfile_loc))

        return logfile_loc


    def set_inputlist_entries(self, inputlist_file, server_indexlist):
        '''
           This routine is the default implementation that does nothing.  Caller could
           overwrite this routine to add its own entries to the input list file.

           @param inputlist_file - input list file fd
           @param server_indexlist - Index list of the selected servers for a given group
        '''

        debug("Default set_inputlist_entries is called")


    def start_msunconfig(self, server_indexlist, progressfile_loc):
        '''
           This routine is the default implementation that does nothing.

           @param server_indexlist - A list of server indexes to be unconfigured from MS
           @param progressfile_loc - progres file location.
        '''

        debug("start_msunconfig of WsadminTask is called")


    def _character_escape(self, rawstr):
        '''
           This routine will escape platform problematic characters.
        '''

        # Don't log input because it may be a password
        debug("Entry _character_escape %s" % fixlev)

        # Note that in the following URL WebSphere tells you not to use any special characters in the user and password:
        #
        #   http://pic.dhe.ibm.com/infocenter/wasinfo/v7r0/index.jsp?topic=%2Fcom.ibm.websphere.nd.doc%2Finfo%2Fae%2Fae%2Fcsec_chars.html
        #
        # Network Deployment
        #   Administering Applications and Their Envionment
        #     Security
        #       Authentication Users
        #         Characters that are Valid for User IDs and Passwords

        ans = rawstr

        # For Windows no problem characters have yet been identified, so far
        # it is sufficient to enclose the user and password in double quotes.
        # (And and we'll need a different escaping algorithmn for Windows.)
        # The caret (^) is the escape character on Windows.
        if platformUtil.isWindows():
            escape = "^"
            badchars = ""

        # For Unix-like platforms (which in our universe is anything but Windows), we must
        # escape problematic characters using the backslash (\).
        else:
            escape = "\\"
            badchars = "\\()'|$;<>&\" \t"

        # Note that if the escape character is in the bad characters list it must come first
        assert badchars.rfind(escape) <= 0, "Logic error, fix position of escape in badchars"

        # Effect the escapes as required, but remember that the user and password may be None.
        if ans is not None:
            for badchar in badchars:
                ans = ans.replace(badchar,escape+badchar)

        return ans
