#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012,2013 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module checks the probes values for each server, and compares it to the
# current probe value.  The already configured servers, their probes 
# (MS, TT, TEMA, SOA) are saved, and based on the new probe values as a
# result of the interview, the new values are compared to the old values
# and if there are different, then an mbean call is issued for the server
# to pickup the new modified probes.  
#
# Author: Mehrdad Ashrafian
#
#####################################################################@
# Update for every APAR or defect affecting this module
fixlev  = "PM80120"

import utils
import debugUtil as log
import resourcebundle as msg
import platformUtil

from debugUtil import logDebug as debug


#
# The dictionary contains the actual probe points like MS, Tema, TT, and SOA.  The
# names are the pdl names which contains the probe point values.  The value for
# each key indicates if the dynamic allocation needs to be issued or not.  1 means
# yes, and 0 means no.
# Make sure to add to this list, put the entries with option of 1 at the end.
#
probe_keys = [
               "TEMACONNECTLIST,0",
               "MSCONNECTLIST,0",
               "TTAPILIST,0",
               "TPVLIST,0",
               "SOADCLIST,1",
               "DELIST,1"  
             ]


#
# The list contains the items or attributes for the probe points.  For example, for MS
# probe point, we have the MS host and port.  The items defines the pdl names which 
# contains the values.
#
probe_items = {
               "TEMACONNECTLIST": ["TEMAHOSTLIST", "TEMAPORTLIST", "ALIASLIST"],
               "MSCONNECTLIST": ["KERNELIPLIST", "KERNELPORT01LIST"],
               "TTAPILIST": ["TTTCHOSTLIST", "TTTCPORTLIST", "TTAPIDIRLIST", "TTAPISSNLIST"],
               "SOADCLIST": [],
               "TPVLIST": [],
               "DELIST": []
              }   

  
        
        
def examine_probes(pdl, prev_configured_probe_dict, serveri):
    '''
       Get each item from the probe dictionary and check it against the current
       values.
       
       @param pdl - PDL reference
       @param prev_configured_probe_dict - Dictionary which contains the probe values for a server
       @param serveri - server index
       @return - A dictionary which the key is the server index and its value of 1 or 0, which
                 0 means to start the server and 1 means not to restart as it is dynamically refreshed
    '''
    
    server_dict = None
        
    debug("Entry examine_probes")
    
    #
    # If any yes/no probe value (previous and current) are different, then that means we need to call
    # mbean, otherwise, if the probe values (yes/no) are the same and have the value of "1" which 
    # means the answer was "yes", then the specific items for the probe is checked (like the host and 
    # port) and if they are different, then that is also a situation to call the mbean.
    # 
    for probe_key_entry in probe_keys:
        probe_key = probe_key_entry.split(",");
        prev_probe_value = prev_configured_probe_dict[probe_key[0]]
        current_probe_value = utils.getData(pdl, probe_key[0], serveri)
        log.logDebug("Probe key: %s   Previous probe value: %s   current probe value: %s" % (probe_key, prev_probe_value, current_probe_value))
        wasserver = utils.getData(pdl, "WASSERVER", serveri) 
        wasnode = utils.getData(pdl, "WASNODE", serveri)  
        wascell = utils.getData(pdl, "WASCELL", serveri)
        if (prev_probe_value != current_probe_value):
            log.logDebug("Server mbean will be called for - %s  %s  %s " % (wascell, wasnode, wasserver))
            server_dict = {}
            server_dict[serveri] = probe_key[1]
            break
        else:
            callmbean = examine_probe_items(pdl, probe_key[0], prev_configured_probe_dict, serveri)
            if (callmbean == True):
                log.logDebug("Server mbean will be called for -  %s  %s  %s " % (wascell, wasnode, wasserver))
                server_dict = {}
                server_dict[serveri] = probe_key[1] 
                break
     

    debug("Exit examine_probes - %s" % (server_dict))  
             
    return server_dict           
            
        
def examine_probe_items(pdl, probe_key, prev_configured_probe_dict, serveri):
    '''
       For each probe key (which is the answer yes/no from the interview), check
       its items (like host, port) and if there is a difference, then that is 
       an indication to call mbean.
       
       @param pdl - PDL reference
       @param probe_key - Probe key 
       @param prev_configured_probe_dict - Dictionary which contains the probe values for a server
       @param serveri - server index
       @return - True if needs to call mbean otherwise False
    '''
    
    mbeancall = False
    
    debug("Entry examine_probe_items")
    
    #
    # The items for the probe is checked.  The items for a probe is like its host and port
    # in the case of MS, TT, TEMA.  So, if there is a descrepency, then the mbean will be called. 
    # 
    probe_itemlist = probe_items.get(probe_key)
    for probe_item_entry in probe_itemlist:
        prev_probe_itemvalue = prev_configured_probe_dict[probe_item_entry]
        current_probe_itemvalue = utils.getData(pdl, probe_item_entry)
        if current_probe_itemvalue != None and len(current_probe_itemvalue) > 0:
            current_probe_itemvalue = current_probe_itemvalue[serveri]
            log.logDebug("Probe key (ITEMS): %s   Previous probe item value (%s): %s   current probe item value (%s): %s" % (probe_key, probe_item_entry, prev_probe_itemvalue, probe_item_entry, current_probe_itemvalue))
            if current_probe_itemvalue != None and current_probe_itemvalue != "None":
                if (prev_probe_itemvalue != current_probe_itemvalue):
                    mbeancall = True
                    break
     
    debug("Exit examine_probe_items - %s" % (mbeancall))
            
    return mbeancall    


def probe_checking(pdl):
    '''
       Checks the current configured probe values for a server to the interviewed
       values, and if there are differences, the mbean call is issued for the server.
       
       @param pdl - PDL reference
    '''
    
    server_index = []
    
    debug("Entry probe_checking")
    
    try:
        configured_probes = utils.getData(pdl, "CONFIGUREDPROBES")
        
        for serveri in configured_probes.keys():
            prev_configured_probe_dict = configured_probes.get(serveri)
            log.logDebug("Configured probe dictionary: %s" % (prev_configured_probe_dict))
            sindexes = examine_probes(pdl, prev_configured_probe_dict, serveri)
            if (sindexes != None) or platformUtil.IS_OS400_OS():
                server_index.append(sindexes)
                
    except Exception, errmsg: 
        msg1 = "Error occurred processing probe checking - %s" % (errmsg)  
        log.logException(msg1)
        msgl =[]
        msgl.append(errmsg)
        print msg.getmsg("CFG2018E", "Error occurred while processing check probes - {0}", msgl) 
        utils.setData(pdl, "RETURNCODE", 1, "checkprobes")           
     
    debug("Entry probe_checking - %s " % (server_index))
        
    return server_index  

