#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module prompts the user for the upgrade location, and checks its
# version for a valid version.
#
#####################################################################

fixlev = "38001" # Last APAR, defect or story to update this module

import codecs
import string
import os

import doprompt
import utils
import platformUtil
import debugUtil as log

from debugUtil import logDebug as debug
from resourcebundle import getmsg


def convert_version(version):
    '''
       Converts the version from a string format such as "7.2.0" to
       a list of integers.

       Lists of integers will compare as expected for product
       versions, and will also compare correctly with the integer -1.
    '''

    debug("Entry convert_version %s - %s" % (fixlev,version))

    # If we can't convert to a numeric list, return [-1] so that it can be
    # compared or can be indexed to return the major version.

    try:
        if version is None: raise ValueError
        intlist = [int(version.split(".")[i]) for i in range(version.count(".")+1)]
        if intlist == []: raise ValueError
        debug("Converted version string '%s' to list of integers: %s" % (version, intlist))
    except ValueError:
        debug("Unable to convert version string '%s' to a list of integers." % (version))
        intlist = [-1]

    debug("Exit convert_version - %s" % (intlist))

    return intlist

def get_version(pdl, proplist):
    '''
       Checks the property list for "probe.release.version", and once it is
       found, the value is returned to the caller.

       @param pdl - PDL reference
       @param proplist - Property list which contains the properties from version.properties file
    '''

    debug("Entry get_version() %s, property list =\n\n%s" % (fixlev,"\n".join(proplist)))

    version = None
    release_ver = None
    ptf_ver = None
    patch_ver = None

    #
    # Check for the property "probe.release.version" in the property list.
    #
    for prop_entry in proplist:
        prop_nowhite = str(prop_entry).translate(string.maketrans("",""), string.whitespace)
        if prop_nowhite.startswith("probe.release.version="):
            release_ver = prop_nowhite.split("=",1)[1]
            debug("Release version is - %s" % (release_ver))
        elif prop_nowhite.startswith("probe.ptf.version="):
            ptf_ver = prop_nowhite.split("=",1)[1]
            debug("PTF version is - %s" % (ptf_ver))
        elif prop_nowhite.startswith("probe.patch.level="):
            patch_ver = prop_nowhite.split("=",1)[1]
            debug("Patch level version is - %s" % (patch_ver))
    #
    # If ptf and patch have blank value, then replace it with value zero '0'.
    #
    ptf_ver = ptf_ver.strip(" ")
    patch_ver = patch_ver.strip(" ")
    if len(ptf_ver) == 0:
        ptf_ver = "0"
    if len(patch_ver) == 0:
        patch_ver = "0"
    debug("PTF version is - %s" % (ptf_ver))
    debug("Patch level version is - %s" % (patch_ver))

    #
    # Check the release version and append the ptf version to it.  The overall release
    # version should be in the form of x.x.x.x, so we make sure that release version has
    # 3 digits, and if it does not have 3 digits, we force the positions to zero, and then
    # append the ptf version as the last digit.
    #
    if release_ver is not None and ptf_ver is not None and patch_ver is not None:
        dcnt = release_ver.count(".") + 1
        if dcnt > 4:
            print getmsg("CFG1068E", "Invalid version property - {0}", [release_ver])
            debug("CFG1068E Invalid version property - dcnt gt 4 %s" % (release_ver))
        else:
            itr = 4 - dcnt
            while (itr > 1):
                release_ver = release_ver + "." + "0"
                itr -= 1
            # Do not add the patch version if it is not numeric
            release_ver = release_ver + "." + ptf_ver
            if patch_ver.isdigit():
                release_ver = release_ver + "." + patch_ver
            else:
                debug("Patch version is non-numeric, not included in release: %s" % patch_ver)
                release_ver = release_ver + ".0"
            debug("Release version is - %s" % (release_ver))
            verintlist = convert_version(release_ver)
            if verintlist != -1:
                version = release_ver
    else:
        print getmsg("CFG1068E", "Invalid version property - {0}", [release_ver])
        print getmsg("CFG1068E", "Invalid version property - {0}", [ptf_ver])
        print getmsg("CFG1068E", "Invalid version property - {0}", [patch_ver])
        debug("Invalid version property - %s %s " % (release_ver, ptf_ver))

    debug("Exit get_version() = '%s'" % version)

    return version


def check_version(pdl, upgrade_proplist, current_proplist, print_error):
    '''
       Compare the versions from upgrade to the current and
       making sure the versions are valid.
       print_error - boolean flag, if False invalid version error message will not be printed
    '''

    stat = False
    upgrade_version_intlist = -1

    debug("Entry check_version %s - %s - %s" % (fixlev, upgrade_proplist, current_proplist))

    #
    # Get the versions for upgrade and current.
    #
    upgrade_version = get_version(pdl, upgrade_proplist)
    current_version = get_version(pdl, current_proplist)
    #
    # Check the upgrade version against the current version.  The
    # upgrade version must be a lesser version than the current.
    # The version is in the form of x.x.x.x, and it might not have
    # all the items, so it is converted to an integer.
    #
    if upgrade_version != None and current_version != None:
        upgrade_version_intlist = convert_version(upgrade_version)
        if upgrade_version_intlist != -1:
            current_version_intlist = convert_version(current_version)
            debug("Upgrade integer version %s" % (upgrade_version_intlist))
            debug("Current integer version %s" % (current_version_intlist))
            if current_version_intlist > upgrade_version_intlist:
                stat = True
            else:
                log.logError("Can not upgrade from a higher version to lower or equal version - %s - %s" % (upgrade_version_intlist, current_version_intlist))
                msgl = []
                msgl.append("%s" % upgrade_version_intlist)
                msgl.append("%s" % current_version_intlist)
                errmsg = getmsg("CFG1072E", "Can not upgrade from a higher version to lower or equal version - {0} {1}", msgl)
                if print_error:
                    print errmsg

    debug("Exit check_version - %s - %s" % (stat, upgrade_version_intlist))

    return stat, upgrade_version_intlist


def read_version(pdl, versionproperty_path):
    '''
       Read the version.properties and save each line in a
       list for later use.

       @param pdl - Reference to PDL
       @param versionproperty_path - Path to version.properties

    '''
    prop_entries = []

    debug("Entry read_version %s - %s" % (fixlev, versionproperty_path))

    if not os.path.exists(versionproperty_path):
        debug("Invalid path to version.properties file - %s" % (versionproperty_path))
        errmsg = getmsg("CFG1066E", "Invalid path to version.properties - {0}", versionproperty_path)
        print errmsg
    else:
        if platformUtil.isZ():
            debug("z/OS system to read version.properties in EBCDIC")
            input_properties_file = codecs.open(versionproperty_path, mode="r", encoding="cp1047")
        else:
            input_properties_file = open(versionproperty_path, "r")

        # In case this is a DOS file we remove the CR as well as the LF.
        prop_entries = [record.rstrip("\r\n") for record in input_properties_file.readlines()]
        input_properties_file.close()

    if len(prop_entries) <= 0:
        errmsg = getmsg("CFG1067I", "No properties found in  version.properties file")
        debug("No properties found in  version.properties file")
        print errmsg

    debug("Exit read_version -\n  %s" % "\n  ".join(prop_entries))

    return prop_entries

def get_current_version(pdl):
    '''
       Get the current version by reading the version.properties.

       @param pdl - PDL reference
    '''

    version = "-1"

    debug("Entry get_current_version %s" % fixlev)

    current_filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "version.properties"))
    current_prop_entries = read_version(pdl, current_filename)
    if len(current_prop_entries) > 0:
        debug("Current Version Properties - %s" % (current_prop_entries))
        version = get_version(pdl, current_prop_entries)

    debug("Exit get_current_version - %s" % (version))

    return version

def check_migrate_version(pdl, dc_home_list=None, print_error=True):
    '''
       Prompts the user for migrate location.  Once the location is
       acquired, look for the version.properties file which resides
       in  MIGRATEHOME/itcamdc/etc, and read the file to get the upgrade
       version.
       Also, the current config version.properties is read to get the
       current version number.

       @param pdl - PDL reference
    '''

    version_checked = False
    upgrade_version = -1

    debug("Entry check_migrate_version %s" % fixlev)
    migratehome_list = []
    try:
        
        if dc_home_list != None and len(dc_home_list) > 0:
            #if we know where DC installs are, do not prompt user for the DC home, check all available
            migratehome_list = dc_home_list
        else:
            doprompt.do_prompt("MIGRATEHOME", pdl)
            migratehome = utils.getData(pdl, "MIGRATEHOME")
            migratehome_list.append(migratehome)
        current_filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "version.properties"))
        current_prop_entries = read_version(pdl, current_filename)
        final_migratehome_dic = {}
        for migratehome in migratehome_list:
            if migratehome != None:
                #
                # If the last character in the migrate in home is '/' or '\', then just
                # simply remove it.
                #
                debug("Migrate home is(b) %s " % (migratehome))
                if migratehome.endswith("/") or migratehome.endswith("\\"):
                    migratehome = migratehome[0: len(migratehome)-1]
                    debug("Migrate home is %s " % (migratehome))
                    #utils.setData(pdl, "MIGRATEHOME", migratehome, "checkversion")
                upgrade_filename = os.path.normpath(os.path.join(migratehome, "itcamdc", "etc", "version.properties"))
                #current_filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "version.properties"))
                #
                # Read upgrade and current version.properties, and check to make sure
                # that the upgrade version is less than the current version.
                #
                upgrade_prop_entries = read_version(pdl, upgrade_filename)
                #current_prop_entries = read_version(pdl, current_filename)
                if len(upgrade_prop_entries) > 0 and len(current_prop_entries) > 0:
                    debug("Upgrade Version Properties - %s" % (upgrade_prop_entries))
                    debug("Current Version Properties - %s" % (current_prop_entries))
                    version_checked, upgrade_version = check_version(pdl, upgrade_prop_entries, current_prop_entries, print_error)
                    if version_checked:
                        final_migratehome_dic[migratehome] = upgrade_version
                    else:
                        debug("Invalid migrate_home %s, removing from list" % migratehome )
            else:
                debug("UPGRADEHOME is not set")
                errmsg = getmsg("CFG1064E", "UGRADEHOME pdl value is not set")
                print errmsg

    except Exception, errmsg:
        log.logException("Error occurred while checking for upgrade version - %s" % (errmsg))
        print getmsg("CFG1065E", "Error occurred while checking for upgrade version - {0}", [errmsg])
        raise

    debug("Exit check_migrate_version - %s - %s" % (version_checked, upgrade_version))
    debug("Exit heck_migrate_version - %s" % final_migratehome_dic)
    #return a dictionary with dchome as a key and version as a value
    #if dictionary length is 0 there were no valid migrate homes found 
    return final_migratehome_dic
    #return version_checked, upgrade_version

def get_feature_version(pdl):
    '''
    return version number for was liberty feature
    used only when configuring DC for WAS Liberty
    '''
    #liberty.feature.version=72.004
    feature_version = "-1"

    debug("Entry get_feature_version %s" % fixlev)
    current_version = get_current_version(pdl)
    current_version = current_version.replace(".", "")
    feature_version = current_version[0:3] + "." + current_version[3:]
    #current_filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "version.properties"))
    #current_prop_entries = read_version(pdl, current_filename)
    #if len(current_prop_entries) > 0:
    #    debug("Current Version Properties - %s" % (current_prop_entries))
    #    for prop in current_prop_entries:
    #        prop_nowhite = str(prop).translate(string.maketrans("",""), string.whitespace)
    #        if prop_nowhite.startswith("liberty.feature.version="):
    #            feature_version=prop_nowhite.split('=',1)[1]
    #            break
    debug("Exit get_current_version - %s" % (feature_version))

    return feature_version

# Unit testing

if __name__ == "__main__":

    import sys
    from time import strftime

    import createPDL
    import createPCL_config as createPCL

    print
    print "%s *Debug*  Unit testing of checkversion.py" % strftime("%H:%M:%S")
    print

    log.set_logging("./checkversion.log")

    pcl = createPCL.make_pcl()
    pdl = createPDL.make_pdl(pcl)
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    log.logDumpPDL(pdl)

    for ver in ["7.2.0", u"7.2.0", "7.2.0", "7", "", " ", "7.2.1b"]:
        print "  version of '%s' = %s" % (ver, convert_version(ver))
    print

    print "  Version gotten = '%s'" % get_version(pdl,[" probe.release.version  = 7.2.0  "])
