#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2013, 2014 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
  Configuration of DC-less JMX server
'''

fixlev = "57346" # Last APAR, defect or story to update this module

import os
import re
import codecs
import array
import errno

from xml.dom import minidom, Node
from time import strftime

import utils
import debugUtil as log
import platformUtil
import wasdiscover

from ConfigurationException import ConfigurationException
from resourcebundle import getmsg as getmsg
from debugUtil import logDebug as debug


def makeruntime(pdl):
    '''
      Create the connections properties file and the JMX DC input file.
    '''

    debug("Entry makeruntime %s" % fixlev)

    # Make sure the user and passwords have been prompted

    wasprofilehome = utils.getData(pdl, "PROFILEHOME", 0)
    wasroot = utils.getData(pdl, "WASINSTALLROOT", 0)
    wasprofilename = utils.getData(pdl, "PROFILE", 0)
    wasdiscover.checkWASSecurity(pdl, wasprofilehome, wasroot, wasprofilename)

    # Create the patterns

    p_user = re.compile(r"\s*com\.ibm\.(SOAP|CORBA)\.loginUserid\s*=\s*([^\s]+)\s*$")
    p_password = re.compile(r"\s*com\.ibm\.(SOAP|CORBA)\.loginPassword\s*=\s*([^\s]+)\s*$")

    enc = platformUtil.getEncoding()

    useclientpropsdict = pdl["USECLIENTPROPSLIST"][0]
    userdict = pdl["USERLIST"][0]
    passworddict = pdl["PASSWORDLIST"][0]

    conntype = pdl["DCLESSCONNTYPE"][0]
    if conntype is None: conntype = "SOAP"
    assert conntype in ("SOAP","RMI"), "Value of DCLESSCONNTYPE is %s" % conntype

    # Read in the old connections properties file

    conndict = readConnFile(pdl)
    newfile_flag = len(conndict.keys()) == 0
    debug("New file status of connection properties is %s" % newfile_flag)

    # Set new homes

    # For reconfig, JAVAHOME will be a list, or might not be resolved
    javahome = pdl["JAVAHOME"][0]
    if type(javahome) is type([]):
        if len(javahome) == 0:
            javahome = None
        else:
            javahome = javahome[0]
    if javahome is not None:
        javahome = utils.replaceBackSlash(javahome)
        conndict["java.home"] = javahome

    conndict["jmx.port"] = pdl["JMXADMINPORT"][0]
    conndict["jmx.host"] = "localhost"

    localhostip = pdl["LOCALHOSTIP"][0]
    debug(" localhostip is set to %s" % localhostip)
    conndict["jmx.localhostip"] = localhostip

    # Remove itcam.home property
    if "itcam.home" in conndict.keys(): del conndict["itcam.home"]

    # Create new connection dictionary entries for each selected server.
    # Existing entries will be replaced for matching servers.

    for serveri in range(len(pdl["SERVERS"][0])):

        # We don't create entries for WLP (only happends during reconfiguration)
        if pdl["WLPHOME"][0][serveri] is not None: continue
        #skip servers where config failed due to invalid user credentials, updating connection.properties with invalid credentials causes problems with WAS
        if pdl["CONFIGRETURNCODE"][0][serveri] == 99: 
            debug("Invalid credentials provided for server %s-%s, skipping" % (pdl["PROFILE"][0][serveri], pdl["SERVERS"][0][serveri]))
            continue
    
        profilehome = pdl["PROFILEHOME"][0][serveri]
        wasprofile = pdl["PROFILE"][0][serveri]
        wascell = pdl["WASCELL"][0][serveri]
        wasnode = pdl["WASNODE"][0][serveri]
        wasserver = pdl["SERVERS"][0][serveri]

        # As in configtema, do our best to get the true path
        wasinstallroot = pdl["WASINSTALLROOT"][0][serveri]
        wasinstallroot = os.path.normpath(wasinstallroot)
        wasinstallroot = os.path.abspath(wasinstallroot) or wasinstallroot
        wasinstallroot = os.path.normcase(wasinstallroot)
        wasinstallroot = utils.replaceBackSlash(wasinstallroot)

        serveralias = pdl["ALIASLIST"][0][serveri]

        # Get the global security flag to force user and password to be filled in PDL

        securityfilename = os.path.normpath(os.path.join(profilehome, "config", "cells", wascell, "security.xml"))
        debug("Security file: %s" % securityfilename)
        securityenabled_flag = False
        secdoc = None
        try:
            secdoc = minidom.parse(securityfilename)
            secroot = secdoc.documentElement
            debug("XML: Successfully parsed %s document" % secroot.nodeName)
            securityenabled = secroot.getAttribute("enabled")
            debug("Global security enabled attribute = \"%s\"" % securityenabled)
            if securityenabled == "true": securityenabled_flag = True
        except Exception, errmsg:
            debug("Exception processing security XML: %s" % errmsg)
            log.logException("Error in configdcless.makeruntime routine for security.xml")
            log.logDumpStack("Stack data for diagnostic purposes")
        finally:
            if secdoc is not None: secdoc.unlink()
        debug("Global security flag is %s" % securityenabled_flag)

        serverindexfilename = os.path.normpath(os.path.join(profilehome,"config","cells",wascell,"nodes",wasnode,"serverindex.xml"))
        debug("Server index file: %s" % serverindexfilename)

        soapfilename = os.path.normpath(os.path.join(profilehome,"properties","soap.client.props"))
        rmifilename = os.path.normpath(os.path.join(profilehome,"properties","sas.client.props"))
        if conntype.upper() == "SOAP":
            propsfilename = soapfilename
        else:
            propsfilename = rmifilename
        debug("Client props file: %s" % propsfilename)

        # Go find the SOAP and RMI connection hosts and ports from the serverindex.xml

        # defaults in case it doesn't work out
        connhost = "localhost"
        connport = "8880"

        srvdoc = None
        endpointname = None
        if conntype == "RMI":
            endpointname = "BOOTSTRAP_ADDRESS"
        else:
            endpointname = "SOAP_CONNECTOR_ADDRESS"
        try:
            srvdoc = minidom.parse(serverindexfilename)
            srvroot = srvdoc.documentElement
            debug("XML: Successfully parsed %s document" % srvroot.nodeName)
            # Locate the server entry for this server
            break_flag = False
            for serverEntry in srvroot.childNodes:
                if serverEntry.nodeType != Node.ELEMENT_NODE or serverEntry.nodeName != "serverEntries": continue
                debug("XML: %s %s %s" % (serverEntry.nodeName,serverEntry.getAttribute("serverName"),serverEntry.getAttribute("serverType")))
                if serverEntry.getAttribute("serverType") != "APPLICATION_SERVER": continue
                if serverEntry.getAttribute("serverName") != wasserver: continue
                # Locate the special endpoint
                for specialEndpoint in serverEntry.childNodes:
                    if specialEndpoint.nodeType != Node.ELEMENT_NODE or specialEndpoint.nodeName != "specialEndpoints": continue
                    if specialEndpoint.getAttribute("endPointName") != endpointname: continue
                    debug("XML:   %s %s" % (specialEndpoint.nodeName,specialEndpoint.getAttribute("endPointName")))
                    # Locate the endpoint (there will only be one):
                    for endPoint in specialEndpoint.childNodes:
                        if endPoint.nodeType != Node.ELEMENT_NODE or endPoint.nodeName != "endPoint": continue
                        debug("XML:     %s port=%s host=%s" % (endPoint.nodeName,endPoint.getAttribute("port"),endPoint.getAttribute("host")))
                        connhost = endPoint.getAttribute("host")
                        connport = endPoint.getAttribute("port")
                        break_flag = True
                    if break_flag: break
                if break_flag: break
        except Exception, errmsg:
            debug("Exception processing server index XML file: %s" % errmsg)
            log.logException("Error in configdcless.makeruntime routine for serverindex.xml")
            log.logDumpStack("Stack data for diagnostic purposes")
        finally:
            if srvdoc != None: srvdoc.unlink()
        debug("For server %s we have %s %s:%s" % (wasserver,conntype,connhost,connport))

        if not securityenabled_flag:
            user = None
            password = None

        else:

            # Get the USECLIENTPROPS flag, the USER and the PASSWORD.  This is actually
            # specified on a per-profile basis, so for the real values we have to use
            # the profile home dictionaries.

            if useclientpropsdict is not None and profilehome in useclientpropsdict:
                useclientprops = useclientpropsdict[profilehome]
            else:
                useclientprops = pdl["USECLIENTPROPS"][0]
                debug("No USECLIENTPROPSLIST flag for profile %s" % profilehome)

            if userdict is not None and profilehome in userdict:
                user = userdict[profilehome]
            else:
                debug("No USERLIST for profile %s" % profilehome)
                user = pdl["USER"][0]

            if passworddict is not None and profilehome in passworddict:
                password = passworddict[profilehome]
            else:
                debug("No PASSWORDLIST for profile %s" % profilehome)
                password = pdl["PASSWORD"][0]

            display_password = "********"
            if password is None: display_password = None
            debug("From PDL, user(%s) password(%s)" % (user,display_password))

            # User and password extraction will not need to be done if the connection
            # supported empty values for the user and password.  But as this is written,
            # that is not the case.  This could therefore be removed when that improvement is implemented.

            if useclientprops:

                inlns = []
                inputfile = None
                try:
                    inputfile = codecs.open(propsfilename,"r",encoding=enc)
                    inlns = inputfile.readlines()
                except IOError, errmsg:
                    debug("IOError accessing client props file: %s" % errmsg)
                finally:
                    if inputfile is not None: inputfile.close()
                # Remove comments and blank lines.
                # In case this is a DOS file we remove the CR as well as the LF.
                # Put comment back on for pattern matching.
                inlns = [inln.rstrip("\r\n")+"\n" for inln in inlns if not inln.startswith("#") and inln.strip() != ""]

                debug("client properties file %s contents:\n\n    %s" % (propsfilename,"\n    ".join([ln.strip("\n") for ln in inlns])))

                for inln in inlns:
                    # Note that the patterns are written so as not to hit on blank values for user or password
                    m_user = p_user.match(inln)
                    if m_user:
                        debug("user line: %s" % inln.rstrip("\n"))
                        user = m_user.group(2)
                    m_password = p_password.match(inln)
                    if m_password:
                        debug("password line: %s" % inln.split("=",1)[0]+"=********")
                        password = m_password.group(2)

            else: debug("USECLIENTPROPS is not set, did not read client props file")

        debug("Have user(%s) password(********)" % user)

        if password is not None: password = _encodePassword(pdl,password,pdl["PROFILEHOME"][0][serveri])

        sep = "|"

        wasqual, stackqual = get_wasqual_stackqual(pdl, serveri)
        amplatform = wasqual + stackqual

        connstr = amplatform

        connstr += sep + normalizeString(connhost)
        connstr += sep + normalizeString(connport)

        connstr += sep + normalizeString(wascell)
        connstr += sep + normalizeString(wasnode)
        connstr += sep + normalizeString(wasprofile)
        connstr += sep + normalizeString(wasserver)

        connstr += sep + normalizeString(user)
        connstr += sep + normalizeString(password)

        connstr += sep + conntype.lower()
        connstr += sep + normalizeString(wasinstallroot)

        connstr += sep + normalizeString(serveralias)

        connkey = (normalizeString(wasinstallroot), normalizeString(wasprofile), normalizeString(wascell), normalizeString(wasnode), normalizeString(wasserver))
        conndict[connkey] = connstr

    _writeConnFile(pdl, conndict)

    print
    print "%s" % getmsg("CFG1181I","Registration for resource (PMI) monitoring by ITCAM Agent for WebSphere Applications successfully completed.")
    print

    if newfile_flag:
        print "%s" % getmsg("CFG1187I","You must restart your Agent for WebSphere Applications to activate monitoring.")
        print

    debug("Exit makeruntime")


def readConnFile(pdl):
    '''
      Read the properties file, return a dictionary of the properties.
    '''

    debug("Entry readConnFile %s" % fixlev)

    connfilename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "runtime", "custom", "connections.properties"))
    debug("Connections properties file name: %s" % connfilename)

    lines = []
    if os.path.isfile(connfilename):
        debug("Connection properties file exists and will be read")

        if platformUtil.IS_zOS_OS():
            enc = "cp1047"
        else:
            enc = "utf-8"

        infile = None
        try:
            infile = codecs.open(connfilename, "r", enc)
            # In case this is a DOS file we remove the CR as well as the LF.
            lines = [line.rstrip("\r\n") for line in infile.xreadlines()]
        except IOError, errmsg:
            if errmsg.errno == 2:
                debug("No existing connections.properties file found")
            else:
                debug("Unexpected I/O error: %s" % errmsg)
        finally:
            if infile is not None: infile.close()

        debug("Existing connections properties:\n\n    %s" % "\n    ".join(lines))

    else: debug("No connections properties file exists, a skeleton dictionary will be returned")

    conndict = {}
    for line in lines:
        if len(line.strip()) == 0: continue
        if line[0] == "#": continue
        props = line.split("=",1)
        if len(props) != 2:
            debug("Invalid line ignored: %s" % line)
            continue
        if props[0] == "connections":
            debug("The %s=%s property will be ignored" % (props[0], props[1]))
            continue

        #                              0     1         2    3=cell      4=node       5=profile    6=server 7    8                                                      9    10                               11
        # connection1.connectionstring=was85|localhost|8880|MyWAS85Cell|MyAppSrvNode|MyAppSrvProf|server1|ejack|205-117-93-251-199-206-116-6-25-51-114-9-80-128-80-140|soap|C:\Utl\IBM\WebSphere\AppServer85|MyAlias

        # connection1.connectionstring=
        #  was85|           [0] WebSphere type
        #  hermesjacksone|  [1] hostname
        #  8895|            [2] connection port
        #  MyCell|          [3] cell
        #  HermesNode|      [4] node
        #  MyAppSrv|        [5] profile
        #  server3|         [6] server
        #  ejack|           [7] user
        #  password|        [8] password
        #  soap|            [9] connection type
        #  C:\Utl\IBM\WebSphere\AppServer855nd| [10] WAS install home
        #  MyAlias          [11] Server Alias

        if props[0].startswith("connection") and props[0].endswith(".connectionstring"):
            debug("Connection string: %s" % props[1])
            connprops = props[1].split("|")
            if len(connprops) < 11:
                debug("Invalid connection string, property will be ignored")
                continue
            connkey = (utils.replaceBackSlash(connprops[10]), connprops[5], connprops[3], connprops[4], connprops[6])
            conndict[connkey] = props[1]
        else:
            debug("Property %s = \"%s\"" % (props[0], props[1]))
            conndict[props[0]] = props[1]

    # Display the dictionary for debugging
    debug("conndict dictionary created from existing file:\n\n    %s" % "\n    ".join(["%s = %s" % (connkey,conndict[connkey]) for connkey in sorted(conndict.keys())]))

    debug("Exit readConnFile")
    return conndict


def _writeConnFile(pdl, conndict):
    '''
      Create a new connections properties file from the connections dictionary.
    '''

    debug("Entry _writeConnFile %s" % fixlev)
    debug("conndict to create new file:\n\n    %s" % "\n    ".join(["%s = %s" % (connkey,conndict[connkey]) for connkey in sorted(conndict.keys())]))

    # Insure that the [Config Home]/runtime/custom directory path exists.  If not, create it.

    runtimecustom = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0],"runtime","custom"))
    if os.path.exists(runtimecustom):
        debug("Existing runtime custom directory: %s" % runtimecustom)
    else:
        debug("Creating runtimecustom directory: %s" % runtimecustom)
        if not os.path.exists(runtimecustom):
            try:
                os.makedirs(runtimecustom)
                debug("Created runtimecustom directory: %s" % runtimecustom)
            except OSError, errmsg:
                raise ConfigurationException(getmsg("CFG5021E", "Failed to create {0}", [runtimecustom] ))

    connfilename = os.path.normpath(os.path.join(runtimecustom,"connections.properties"))
    debug("Connections properties file name: %s" % connfilename)

    if platformUtil.IS_zOS_OS():
        enc = "cp1047"
    else:
        enc = "utf-8"

    # Collect output lines.  The simple properties have a string as the key, connection strings have a tuple.
    lns = ["# Created by configdcless._writeConnFile %s at %s" % (fixlev, strftime("%Y-%m-%d %X"))]
    connpropi = 0
    for connkey in sorted(conndict.keys()):
        if type(connkey) is not type(()):
            lns.append("%s=%s" % (connkey, conndict[connkey]))
        else:
            connpropi += 1
            lns.append("connection%i.connectionstring=%s" % (connpropi, conndict[connkey]))

    debug("New connections properties lines:\n\n    %s" % "\n    ".join(lns))

    # Create the connections file
    connfile = None
    try:
        connfile = codecs.open(connfilename, mode="w", encoding=enc)
        connfile.writelines(["%s\n" % ln for ln in lns])
    finally:
        if connfile is not None: connfile.close()

    debug("Exit _writeConnFile")
    return


def normalizeString(s):
    '''
      Handle embedded separators and missing data
      Any time we see the separator character "|", we replace it by "%7C".
      And if we see "%", we replace that by "%25".
      The consumer of the connection string can therefore easily recreate the original value specified.
    '''

    snew = s
    if snew is None: snew = ""
    snew = snew.replace("%","%25")
    snew = snew.replace("|","%7C")
    snew = snew.replace("\\","%5C")
    if s != snew and s is not None: debug("Replacement string: %s -> %s" % (s,snew))

    return snew


def _toBytes(numstr):
    '''
      Take a string of dash-delimited numbers and convert it to a Java byte array.
    '''

    try:
        # Convert the string list of numbers in the range 0:255 to a byte array of numbers in the range -128:127
        ans = array.array("b",[(0xFF&int(i)) - ((int(i)&0x80)<<1) for i in numstr.split("-")])
    except ValueError, errmsg:
        debug("Invalid number string passed to _toBytes: %s" % numstr)
        log.logException("Error in configdcless.makeruntime routine for serverindex.xml")
        #log.logDumpStack("Stack data for diagnostic purposes")
        raise ConfigurationException(getmsg("CFG5034E", "Invalid value specified by {0} for {1}: {2}", ["encryption","AES key",numstr]))

    return ans


def _toEncode(bytes):
    '''
      Take a Java byte array and convert it to a string of positive numbers delimited by dashes.
    '''

    a = []
    for b in bytes:
        if b >= 0:
            a.append("%i" % b)
        else:
            a.append("%i" % (256+b))
    ans = "-".join(a)

    return ans


def _encodePassword(pdl, password, wasProfileHome):
    '''
       Encode password using PropFilePasswordEncrypt
    '''

    debug("Entry _encodePassword(********, %s) %s" % (wasProfileHome, fixlev))

    # Create a temporary file to contain the password to encode

    pswdfilename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "data", "temp.dat"))
    debug("Temporary file name: %s" % pswdfilename)

    pswdfile = None
    try:
        pswdfile = codecs.open(pswdfilename, "w", platformUtil.getEncoding())
        pswdfile.write("encode=%s\n" % password)
        pswdfile.close()
        pswdfile = None
    except IOError, errmsg:
        debug("Temporary file write I/O error: %s" % errmsg)
        if pswdfile is not None: pswdfile.close()
        raise

    # Call PropFilePasswordEncrypt

    suf = platformUtil.isWindows() and ".bat" or ".sh"
    encode_script = os.path.normpath(os.path.join(wasProfileHome, "bin", "PropFilePasswordEncoder"+suf))
    debug("Encode script: %s" % encode_script)
    assert os.path.isfile(encode_script), getmsg("CFG5022E", "internal error {0}", ["_encodePassword: %s is not a file." % encode_script])

    encpswd_cmd = "%s %s encode" % (utils.add_quotes(encode_script), utils.add_quotes(pswdfilename))
    debug("Encode command: %s" % encpswd_cmd)
    rc, cmdoutput = utils.runCommand(encpswd_cmd)
    if cmdoutput is None or len(cmdoutput) == 0:
        debug("RC(%i) from encode command, no output" % rc)
    else:
        debug("RC(%i) from command, output:\n\n    %s" % (rc, "\n    ".join([line.rstrip("\r\n") for line in cmdoutput if line.strip() != ""])))

    # Read the (presumably) encoded password

    pswdfile = None
    lines = []
    try:
        pswdfile = codecs.open(pswdfilename, "r", platformUtil.getEncoding())
        lines = [line.rstrip("\r\n") for line in pswdfile.readlines()]
        pswdfile.close()
        pswdfile = None
    except IOError, errmsg:
        debug("Temporary file read I/O error: %s" % errmsg)
        if pswdfile is not None: pswdfile.close()
        raise

    # Extract the new value

    assert len(lines) == 1, getmsg("CFG5022E", "internal error {0}", ["_encodePassword() lines"])
    props = lines[0].split("=",1)
    assert len(props) == 2 and props[0] == "encode", getmsg("CFG5022E", "internal error {0}", ["_encodePassword() props"])
    ans = props[1]

    # Only display the password if it was transformed
    display_ans = ans == password and "*"*8 or ans

    # Over-write of the temporary file and delete it

    pswdfile = None
    try:
        pswdfile = codecs.open(pswdfilename, "w", platformUtil.getEncoding())
        pswdfile.write("="*80)
        pswdfile.close()
        pswdfile = None
    except IOError, errmsg:
        debug("Temporary file rewrite I/O error: %s" % errmsg)
        if pswdfile is not None: pswdfile.close()

    if os.path.isfile(pswdfilename): os.remove(pswdfilename)

    debug("Exit _encodePassword(): %s" % display_ans)
    return ans


def get_wasqual_stackqual(pdl,ix):
    '''
        Scan the version files for WBI, WPS, etc
        Returns a two digit version-release number  and id if located
        Ex.  "6.1.0.0"  -->   "61"
    '''

    debug("Entry get_wasqual_stackqual %s" % fixlev)

    wasQualifier = "was%s" % utils.getData(pdl, "WASVERSION", ix)
    debug("WAS version: %s" % wasQualifier)

    # stack product qualifier, inserted into runtime and am_platform names
    stackQualifier = ""

    debug("ix=%i  server=%s  subProduct=%s" % (ix, utils.getData(pdl, "WASSERVER", ix),utils.getData(pdl, "SUBPRODUCT", ix)))
    subProduct = utils.getData(pdl, "SUBPRODUCT", ix)
    WBIHOME = utils.getData(pdl, "WBIHOME", ix)
    WPSHOME = utils.getData(pdl, "WPSHOME", ix)
    debug("Subproduct: %s, WBIHOME: %s, WPSHOME: %s" % (subProduct, WBIHOME, WPSHOME))

    # Check for WBI in subProduct once findservers recognizes WBI servers properly
    if WBIHOME is not None and WBIHOME != "":
        # Locate WBI (process server) version file
        path = os.path.normpath(os.path.join(WBIHOME, "properties", "version", "WBI.product"))
        path1 = os.path.normpath(os.path.join(WBIHOME, "properties", "version", "BPM.product"))

        if os.path.exists(path1):
            path = path1

        debug("WBI product file: %s" % path)

        assert os.path.exists(path), getmsg("CFG1122E", "{0} home exists but {1} file is missing", ["WBI", "WBI.product"])
        wbi_version, wbi_id  = scan_version_file(path)
        debug("WBI version/id : %s/%s" %  (wbi_version, wbi_id))
        stackQualifier = ".prs%s" % wbi_version

    if platformUtil.IS_AIX_OS():
        wasinstallroot = pdl["WASINSTALLROOT"][0][ix]
        debug("wasinstallroot is set to: %s" % (wasinstallroot))
        # Locate BPM (BPM server) version file to determine whether it is a BPM server or not
        #
        # It was found in Linux and Windows environment, checking for WBI_INSTALL_ROOT on the findServers 
        # script was sufficient to determine whether the server is BPM or not. However in certain installations
        # in aix, it was observed WBI_INSTALL_ROOT is not present.
        #
        # From that angle we are introducing this additional conditional checking which is now only acting
        # on AIX systems. However in reality we can simply rely on the existence of the BPM.product file and this
        # checking should work for all environments.
        #
        # So in future we can consider to merge this code block with the above conditional logic.
        # (after significant testing on Linux and Windows)
        path = os.path.normpath(os.path.join(wasinstallroot, "properties", "version", "BPM.product"))
        debug("BPM product file: %s" % path)
        if os.path.exists(path):
            wbi_version, wbi_id  = scan_version_file(path)
            debug("BPM version/id : %s/%s" %  (wbi_version, wbi_id))
            stackQualifier = ".prs%s" % wbi_version

    # Check for WPS in subProduct which is populated by the findservers
    if subProduct == "WPS" and WPSHOME is not None and WPSHOME != "":
        # Locate WPS (portal server) version file
        path = os.path.normpath(os.path.join(WPSHOME, "wps.properties"))
        debug("WPS product file: %s" % path)
        assert os.path.exists(path), getmsg("CFG1122E", "{0} home exists but {1} file is missing", ["WPS", "wps.properties"])
        wps_version, wps_id  = scan_version_file(path)
        debug("WPS version/id: %s/%s" % (wps_version, wps_id))
        stackQualifier = ".wps%s" % wps_version

    # To do: If ESB is to be implemented, this is the place to do it.

    debug("Exit get_wasqual_stackqual: %s, %s" % (wasQualifier, stackQualifier))
    return wasQualifier, stackQualifier


def scan_version_file(path):
    '''
        Scan the version files for WBI, WPS, etc
        Returns a two digit version-release number  and id if located
        Ex.  "6.1.0.0"  -->   "61"
    '''

    debug("Entry scan_version_file(%s) %s" % (path, fixlev))

    vers = ""
    id = ""

    try:
        verFile = open(path, "r")
        inLines = verFile.read().splitlines()
        verFile.close()

        debug("Version file lines:%s" % (len(inLines) == 0 and " None" or "\n    %s" % "\n    ".join(inLines)))

        for rec in inLines:
            rec = rec.strip()
            if rec.startswith("version="):
                debug("Version property line: %s" % rec)
                rec = rec.replace("version=","")
                versList=rec.split(".")
                vers = versList[0] + versList[1]
            elif rec.startswith("<version>"):
                debug("Version tag line: %s" % rec)
                rec = rec.replace("<version>","").replace("</version>","")
                versList=rec.split(".")
                vers = versList[0] + versList[1]
            elif rec.startswith("<id>"):
                debug("ID tag line: %s" % rec)
                rec = rec.replace("<id>","").replace("</id>","")
                id  = rec.lower()

    except Exception, msgVar:
        message = "Exception %s processing version file: %s " %(msgVar, path)
        log.logException(message)
        raise message

    debug("Exit scan_version_file, result = %s %s" % (vers, id))
    return vers, id


#############
# Unit Test #
#############

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL

    from debugUtil import set_logging
    from resourcebundle import set_resourcebundle

    from createPDL import getFormats

    set_logging("./configdcless.log")
    set_resourcebundle()

    pcl = createPCL.make_pcl()
    pdl = createPDL.make_pdl(pcl)

    pdl["CONFIGHOME"][0:2] = os.curdir, "configdcless.__main__"
    print "CONFIGHOME: %s" % pdl["CONFIGHOME"][0]
    pdl["PROFILEHOME"][0:2] = [r"C:\Utl\IBM\WebSphere\AppServer855test"], "configdcless.__main__"
    print "PROFILEHOME: %s" % pdl["PROFILEHOME"][0]
    datahome = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "data"))
    if not os.path.exists(datahome): os.mkdir(datahome)

    print "Encoded test #1: '%s'" % _encodePassword(pdl,"a")
    print "Encoded test #2: '%s'" % _encodePassword(pdl,"password")

    pass
