#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module setsup the plugin for WAS based on WAS version.  For WAS61
# the bundlemanager jar file is copied into the WAS plugins directory.  
# For WAS7 and higher, the bundle metadata xml file is created in the
# data/wsBundleMetaData directory (If not already created), which points to the
# bundlemanager jar file location.
#
# Author: Mehrdad Ashrafian
#
#####################################################################

import os
import shutil
import debugUtil as log
import utils
import platformUtil

from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg

BUNDLEMETADATA_DIR = "wsBundleMetaData"
BUNDLEMETADATA_FILENAME = "itcam_wsBundleMetaData.xml"
BUNDLEMETADATA_TEMPLATE = "itcam_wsBundleMetaData_template.xml"
BUNDLEMANAGER_JAR = "com.ibm.tivoli.itcam.bundlemanager_7.2.0.jar"

def process_was6_plugin(pdl, wasinstallroot):
    '''
       Process WAS6.1 for plugin.  It copies the bundle manager jar
       into the plugins directory in the application server home.
       This is done for all the servers tht are needed to be configured.
       
       @param pdl - PDL reference
       @param wasinstallroot 
    '''
    
    debug("Entry process_was6_plugin - %s" % (wasinstallroot))
    
    was_plugin_dir = wasinstallroot + os.sep + "plugins"
    bundlemanager_jar = pdl["CONFIGHOME"][0] + os.sep + "plugins" + os.sep + BUNDLEMANAGER_JAR
    if (os.path.exists(was_plugin_dir)):
        if (os.path.isfile(bundlemanager_jar)):
            try:
                if platformUtil.IS_zOS_OS():
                    target_full_name = was_plugin_dir + os.sep  + BUNDLEMANAGER_JAR
                    shutil.copyfile(bundlemanager_jar, target_full_name)
                    # os.system('chmod a+rwX ' + target_full_name)  
                else:     
                    shutil.copy(bundlemanager_jar, was_plugin_dir)
                log.logDebug("Bundle Manager jar  %s copied successfully to %s" % (bundlemanager_jar, was_plugin_dir))
            except OSError, errarg:
                print "%s" % getmsg("CFG1128E", "Could not copy file {0} to {1} - {2}", [bundlemanager_jar, was_plugin_dir, errarg])
        else:
            log.logException('Bundle Manager jar file does not exist in the plugins directory')    
            raise
    else:
        log.logException('plugins directory does not exist in %s' % (wasinstallroot))    
        raise
            
    debug("Exit process_was6_plugin")    
    
       

def process_was7_higher_plugin(pdl):
    '''
       Process was7 and higher application server plugin.
    '''
    
    debug("Entry process_was7_higher_plugin")
    
    #
    # If the bundle metadata is already present, skip processing it, as
    # the file is already been copied over from the previous config.
    #
    file_already_present = check_bundle_metadata(pdl)
    if (file_already_present != 1):
        #
        # Read the itcam_wsBundleMetaData_template.xml, and save the entries in a 
        # list for later use.
        #
        prop_entries = read_template(pdl)
        #
        # Replace the variables in the xml file.
        #
        replaced_variables = replace_variables(prop_entries, pdl)
        log.logDebug(replaced_variables)
        #
        # Write the modified bundle metadata into the data/wsMetaData dir.
        #
        write_bundle_metadata(pdl,replaced_variables)   
        
    debug("Exit process_was7_higher_plugin")    

def check_bundle_metadata(pdl):
    '''
       Check the bundle metadata xml file in the runtime/wsBundleMetaData directory,
       and if already exists, then skip plugin processing as it is
       already set
       @param  pdl: pdl reference 
    '''
    
    stat = 0
    
    debug("Entry check_bundle_metadata")
    
    
    #
    # Create wsBundleMetaData directory in the runtime dir if not already
    # exist.
    #
    wsmetadata_dir = pdl["CONFIGHOME"][0] + os.sep + "runtime" + os.sep + BUNDLEMETADATA_DIR
    utils.create_dir_tree(wsmetadata_dir)
    filename = wsmetadata_dir +  os.sep + BUNDLEMETADATA_FILENAME
    if (os.path.isfile(filename)):
        stat = 1
        log.logDebug("File %s already exist, skipping plugin processing" % (filename))
    
    debug("Exit check_bundle_metadata")
    
    return stat
    

def read_template(pdl):
    '''
       Read the itcam_wsBundleMetaData_template.xml and save each line in a
       list for later use.
    '''
    prop_entries = []
    
    debug("Entry read_template")
    
    filename = pdl["CONFIGHOME"][0] + os.sep + "itcamdc" + os.sep + "etc" + os.sep + "was" + os.sep + BUNDLEMETADATA_TEMPLATE
    bundle_metadata_file = open(filename, 'r')
    for prop_data in bundle_metadata_file.readlines():
        prop_entries.append(prop_data)
    bundle_metadata_file.close()         

    debug("Exit read_template")
    
    return prop_entries 


def replace_variables(prop_entries, pdl):  
    '''
        Replaces the variables in the template with the provided
        values.
        
        @param prop_entries - template bundle metadata xml as a list 
        @param pdl - pdl reference
    '''

    misses = {}
    replacevar = ["CONFIGHOME"]
    replaced_prop_entries = []
    
    debug("Entry replace_variables")
    
    #
    # Examine the metadata xml entries.
    # Replace the variables in the template file.
    # 
    lineno = -1
    for prop_entry in prop_entries:
        lineno = lineno+1
        for varentry in replacevar:
            var_value = utils.getData(pdl, varentry)
            assert type(var_value) == type(""), getmsg("CFG5019E", "Unexpected value type {0} for PDL key {1}", [type(var_value), varentry])              
            if var_value != None:
                prop_entry = prop_entry.replace("@{"+varentry+"}",var_value)
                
        replaced_prop_entries.append(prop_entry)
        #
        # Check each entry for variable that is not substituted, and put
        # a message in the log.
        #
        i = 0
        while True:
            i = prop_entry.find("@{",i)
            if i == -1: 
                break
            i = i + 2
            j = prop_entry.find("}",i)
            if j == -1: 
                break
            miss = prop_entry[i-2:j+1]
            misses[miss] = (lineno,prop_entry)
                
    if len(misses.keys()) > 0:
        print "\n%s" % getmsg("CFG1050E","Some substitution values were not replaced in the itcam_wsBundleMetaData file")
        for miss in misses.keys():
            print "    %s" % miss
    
    debug("Exit replace_variables")
        
    return replaced_prop_entries     

def write_bundle_metadata(pdl, replaced_prop_entries):
    '''
       Writes the bundle metadata xml template after replacing
       variables to the runtime directory
    '''
  
    debug("Entry write_bundle_metadata")
  
    #
    # Write the input.properties in ASCII
    #  
    bundle_metadata_filename = pdl["CONFIGHOME"][0] + os.sep + "runtime" + os.sep + BUNDLEMETADATA_DIR + os.sep + BUNDLEMETADATA_FILENAME
    log.logDebug("Bundle MetaData XML file name: " + bundle_metadata_filename)
    bundle_metadata_file = open(bundle_metadata_filename, 'w')
    for prop_data in replaced_prop_entries:
        bundle_metadata_file.write(prop_data)
    bundle_metadata_file.close() 
    log.logDebug("Bundle MetData XML file saved successfully in - " + bundle_metadata_filename)  
        
    debug("Exit write_bundle_metadata")
    
  
def process_plugin(pdl):
    '''
       Process plugin based on WAS version.
       @param pdl - PDL reference
    '''
    
    debug("Entry process_plugin")
    
    try:
        #
        # For each server, check its version, and process plugin accordingly.
        # For WAS 7 and up, there is a property that is set which points to 
        # a directory which contains the XML file that has the information about
        # the plugin bundle manager location, so the directory is created in the
        # runtime and the xml file is created.  If xml file is already created,
        # processing is skipped.
        # For WAS6.1, the actual plugin bundle manager is copied into the WAS
        # install directory under the plugins directory.
        # Group the servers, based on the Cell and node.  Based on a group, the
        # plugin is set.
        #
        group_servers = utils.group_servers(pdl)
        #
        # For each group which contains the index to SERVERS pdl, create the input list
        # and execute command wsadmin to configure it.  
        #
        for group_index in group_servers.keys():
            index_list = group_servers[group_index]
            #
            # Pick the first entry as all the servers in a group belong to the
            # the same WAS home.
            #
            serveri = index_list[0]
            wasversion = utils.getData(pdl, "WASVERSION", serveri)
            wasinstallroot = utils.getData(pdl, "WASINSTALLROOT")[serveri] 
            wasversion = str(wasversion)[0]
            wasversion = int(wasversion)
            if (wasversion > 6):
                process_was7_higher_plugin(pdl)
            else:
                process_was6_plugin(pdl, wasinstallroot)    

        
        
    except:
        log.logException('Error while processing plugins')    
        raise
    
    debug("Exit process_plugin")
