####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    Update configuration for zOS specific features

    1. Verify existance of critical files
    2. Verify directory exists: <CONFIGHOME>/[node].[server]/lib
    3. Create symlink for native shared library:
'''

fixlev = "41683" # Last APAR, defect or story to update this module

import sys
import os

import debugUtil as log
import utils
import configUtils
import shared_globals
from debugUtil import logDebug as debug
from resourcebundle import getmsg
from ConfigurationException import ConfigurationException


###
# Update the configuration or zOS specific items for the specified server
###

def update_zos(pdl, server, serveri):
    '''
        Update configuration for zOS specific items for the specified server
        @param pdl
        @param SERVER  server name
        @param serveri server index
    '''

    debug("Entry update_zos %s - entry for server %s" % (fixlev, server))

    # 1. Get variables we need
    CONFIGHOME = utils.getData(pdl, "CONFIGHOME")
    WASVERSION = utils.getData(pdl, "WASVERSION", serveri)
    WASNODE    = utils.getData(pdl, "WASNODE",    serveri)
    JVMVERSION = utils.getData(pdl, "JVMVERSION", serveri)
    JVMMODE    = utils.getData(pdl, "JVMMODE",  serveri)
    debug(".. server: %s,  %s,  wasver: %s, node: %s, jvmversion: %s, jvmmode: %s" %(server, CONFIGHOME, WASVERSION, WASNODE, JVMVERSION, JVMMODE) )

    # Directory we need in config home
    libdir  =  "%s/%s/%s.%s/lib" % (CONFIGHOME, shared_globals.CONFIGHOME_NODE_SERVER_ROOT, WASNODE, server)

    # 2. Verify <CONFIGHOME>/[node].[server]/lib directory structure exists. If not, create it.
    #    Note: libdir = <CONFIGHOME>/[WASNODE].[SERVER]/lib
    if not verify_dir_tree(libdir):
        msg = getmsg("CFG5021E", "Failed to create {0}", [libdir] )
        raise ConfigurationException(msg)
    ## this is old assert rc, "Unable to create required lib configuration directory for %s " % libdir

    # 4. Create symlink  CONFIGHOME/[node].[server]/lib/libam.so
    create_libam_symlink(CONFIGHOME, libdir, JVMMODE, JVMVERSION)

    # 5. Update variable to pass to inputproperties.py so LIBPATH gets set
    update_pdl(pdl, WASNODE, server, serveri)

    debug("Exit update_zos - for server %s" % (server))


###
#  update_pdl for input.properties
#
#  Note the input.properties.template for server.environment LIBPATH entry
#  Our zOS LIBPATH entry for the native shared library currently needs to be in this format:
#       ${ITCAMDCHOME}/<CONFIGHOME_NODE_SERVER_ROOT>/[node].[server]
###

def  update_pdl(pdl, node, server, serveri):
    '''
         update_pdl for_input_properties
         @param pdl - reference to the PDL
         @param node     node name
         @param server   server name
         @param serveri  server index
    '''

    debug("Entry configzos.update_pdl %s - serverix: %s " % (fixlev, serveri))

    node_server_root = shared_globals.CONFIGHOME_NODE_SERVER_ROOT
    zospath =  "${ITCAMDCHOME}/" + node_server_root + "/" + node + "." + server
    utils.setData(pdl, "ZOSLIBPATH", zospath, "configzos", serveri)
    debug("  zospath: %s" % str(zospath))

    debug("Exit configzos.update_pdl")


###
# Verify existance of critical files
#   Crash if they do not exist
###

def verify_critical_files(CONFIGHOME, JVMMODE, JVMVERSION, WASVERSION, WASMAJORVER):
    '''
        verify_critical_files(CONFIG_HOME, JVMVERSION, WASVERSION)
    '''
    # Determine itcamdc/etc directory location of was$WASVERSION files
    wassetc_ver = str(WASVERSION)
    if not os.path.exists(CONFIGHOME+"/itcamdc/etc/was/was"+WASVERSION):
        wassetc_ver = str(WASMAJORVER)
        debug("   wassetc_ver= %s" % wassetc_ver)
    ents=[]
    ents.append(CONFIGHOME+"/toolkit/etc/was/was"+WASVERSION +"/jmxconfig_was" + WASVERSION + ".xml")
    ents.append(CONFIGHOME+"/itcamdc/etc/was/was"+wassetc_ver+"/dc_config_was" + WASVERSION + ".properties")
    if JVMMODE == "64bit":
        ents.append(CONFIGHOME+"/toolkit/lib/libam_ibm_"+JVMVERSION+"_64.so")
    else:
        ents.append(CONFIGHOME+"/toolkit/lib/libam_ibm_"+JVMVERSION+".so")
    bad_hit = False
    errMsg = ""
    for ent in ents:
        if not os.path.exists(ent):
            bad_hit = True
            errMsg = errMsg + "*Error**  Missing ITCAM support files for WAS %s: %s \n" % (WASVERSION, os.path.basename(ent))
    if bad_hit == True:
        assert False, getmsg("CFG1115E", "Unable to continue without these files: {0}", [errMsg])


###
# Verify existance of a directory tree.  If not create it
###

def verify_dir_tree(libdir, mode=None):
    '''
        verify_dir_tree
        @param libdir  - path to directory tree to verify
    '''

    debug("Entry verify_dir_tree %s - %s" % (fixlev, libdir))

    if not os.path.exists(libdir):
        debug("Directory " +  libdir + " does not exist, will attempt to create.")
        try:
            if mode is None:
                os.makedirs(libdir)
            else:
                os.makedirs(libdir, mode)
        except:
            log.logException("  Cannot create: %s" % libdir )
            debug("....Unable to create it")
        debug("Exit verify_dir_tree")
        return os.path.exists(libdir)
    else:
        #directory already exists
        debug("Exit verify_dir_tree - directory already exists")
        return True


###
# Create symlink "libam.so" for the libam.so shared library
# in format:  <CONFIGHOME?/[node].[server]/lib/libam.so
#   - CONFIGHOME/data/
#   - CONFIGHOME/data/cell_node_server
#   - CONFIGHOME/data/cell_node_server/lib
#   - libdir: in this format <CONFIGHOME?/[node].[server]/lib
###

def create_libam_symlink (CONFIGHOME, libdir, JVMMODE, JVMVERSION):
    '''
         create_libam_symlink(CONFIGHOME, libdir, JVMMODE, JVMVERSION)
         @param libdir  -  config lib directory  where we create the symlink
         @param  JVMMODE
         @param JVMVERSION
    '''

    debug("Entry create_libam_symlink %s for %s, jvmmode: %s, version: %s" % (fixlev, libdir, JVMMODE, JVMVERSION)  )

    symlink=libdir + "/libam.so"

    #  1. Remove any existing link
    if configUtils.isLink(symlink):
        debug("    Removing: " + symlink)
        try:
            os.remove(symlink)
        except:
            log.logException("create_libam_symlink remove symlink failed")
        if configUtils.isLink(symlink):
            debug("     **ERROR**  unable to remove symlink")
            msg = getmsg("CFG1116E", "Unable to remove {0} symlink {1}", ["libam.so", symlink])
            raise ConfigurationException(msg)

    #  2.  Create a symlink named libam.so that refers to the real shared library in install toolkit/lib dir
    if JVMMODE == "64bit":
        libam_name = "libam_ibm_%s_64.so" % JVMVERSION
    else:
        libam_name ="libam_ibm_%s.so" % JVMVERSION

    # symlink config_so_path = libdir + '/libam.so'
    debug("    Creating symlink: " + symlink )
    #    ln -s  <real link>     <symlink>
    cmd="ln -s "+CONFIGHOME+"/toolkit/lib/"+libam_name+"  "+symlink
    rc=configUtils.issueOSCommand(cmd)
    if rc != 0:
        debug(" command rc !=0 for: %s" % cmd)
        assert 0,"*ERROR* Create symlink failed for  %s" % libam_name
    # Confirm success
    assert configUtils.isLink(symlink), "*ERROR* Create symlink failed for  %s" % libam_name

    debug("Exit create_libam_symlink - complete for  for: %s" % libam_name )


###
# Initialize variable(s) in the PDL.
###

def init_pdl(pdl, varlist):
    '''
        Initialize the pdl for variables set by configzos
        @param pdl     - reference to the PDL
        @param varlist - a list of variables to initialize

    '''

    debug("Entry init_pdl() %s for: %s" % (fixlev, varlist))

    for name in varlist:
        # Is a test for None sufficient ?
        if  type(pdl[name][0]) != type([]) or type(pdl[name][0]) != type(()):
            servern = len(pdl["SERVERS"][0])
            pdl[name][0:2] = [""]*servern, "configzos"

    debug("Exit init_pdl()")

###
# Configure for zOS all servers in the PDL
###

def config_zos(pdl):
    '''
        config_zos  -  Configure zOS
        @param pdl
    '''

    debug("Entry config_zos %s" % fixlev)

    try:

        # 1. Verify there is at least one server being configured
        if pdl["SERVERS"][0] is None:
            assert 0, "ERROR* - no servers located in the PDL data container "

        # 2. Init pdl
        varlist= ["ZOSLIBPATH"]
        init_pdl(pdl, varlist)

        # 3. For each server, perform the updates
        for serveri in range(len(pdl["SERVERS"][0])):
            server = pdl["SERVERS"][0][serveri]
            debug("  Updating zos for server: " + server )
            update_zos(pdl, server, serveri )
            debug("  Updating zos complete for server: %s " % server )
        # Log the pdl entries we updated
        debug("pdl JVMMODE: %s "    % str(pdl["JVMMODE"][0]))
        debug("pdl ZOSLIBPATH: %s " % str(pdl["ZOSLIBPATH"][0]))

    except:
        log.logException("**ERROR** Error in configzos - unable to update the configuration")
        raise

    debug("Exit config_zos - exit")


###
#   Sets the JVMmode( 31bit, 64bit) variable for each server in SERVERS list.
#   by updating the JVMMODE list in the PDL, based on properties in the server's was.env.
#
#   Used for operation modes where wsadmin is not being called.
###

def set_jvmmode(pdl):
    '''
        Sets the JVMmode( 31bit, 64bit) variable for each server in SERVERS list.
        by updating the JVMMODE list in the PDL based on properties in the server's was.env
        Used for operation modes where wsadmin is not being called.
        @param pdl
    '''

    debug("Entry set_jvmmode() %s" % fixlev)

    init_pdl(pdl, ["JVMMODE"])
    # For each server, locate and set the JVM mode(into the pdl)
    for serveri in range(len(pdl["SERVERS"][0])):
        server_name = pdl["SERVERS"][0][serveri]
        PROFILEHOME = utils.getData(pdl, "PROFILEHOME", serveri)
        WASCELL     = utils.getData(pdl, "WASCELL",     serveri)
        WASNODE     = utils.getData(pdl, "WASNODE",     serveri)
        # Ex.  /u/WAS700/w7cell/w7nodea/AppServer/profiles/default/config/cells/w7cell/nodes/w7nodea/servers/W7SR005
        was_env_path = PROFILEHOME+"/config/cells/"+WASCELL+"/nodes/"+WASNODE+"/servers/"+server_name+"/was.env"
        if not os.path.isfile(was_env_path):
            print "%s" % getmsg("CFG1169W", "Unable to determine current JVM mode, was.env was not found at {0}", [was_env_path])
            jvmmode = utils.getData(pdl, "JVMMODE", serveri)
            debug("  JVMMODE = %s as previously configured" % jvmmode)
        else:
            jvmmode = get_jvmmode_from_wasenv(was_env_path)
            debug("  JVMMODE = %s for was.env at %s" %(jvmmode,was_env_path))
            utils.setData(pdl, "JVMMODE", jvmmode, "configzos", serveri)

    debug("Exit set_jvmmode()")


###
# Get the JVM mode( 31-bit vs 64-bit) from the was.env file
#
# Look first for was.com.ibm.websphere.zos.jvmmode (31bit | 64bit),
# which is not found on WAS 8.  In WAS 7, we see it present on TVT4017 for 31bit but not for 64bit servers.
# If the zos.jvmmode is not found, look at the JAVA_HOME property.
###

def get_jvmmode_from_wasenv(was_env_path):
    '''
       Get the JVM mode (31bit vs 64bit) from the was.env file
       @param was_env_path - path to a was.env file
       returns "31bit" or "64bit"
    '''

    debug("Entry get_jvmmode_from_wasenv %s - for %s" % (fixlev, was_env_path))

    mode = ""
    try:
        wasenv_props    = configUtils.loadProperties(was_env_path)
    except Exception, msgVar:
        debug("Unable to load was.env at %s -  exception %s" % (was_env_path, msgVar))
        assert False,"Unable to read was.env file to locate JVM mode"
    zos_jvmmode = configUtils.getProperty(wasenv_props, "was.com.ibm.websphere.zos.jvmmode")
    if zos_jvmmode is not None:
        mode = zos_jvmmode
    else:
        java_home = configUtils.getProperty(wasenv_props, "JAVA_HOME")
        assert (java_home is not None),"JAVA_HOME not located in %s" % was_env_path
        java_home = java_home.rstrip()
        if java_home.endswith("java64"):
            mode = "64bit"
        else:
            mode = "31bit"

    debug("Exit get_jvmmode_from_wasenv - mode: %s" % mode)
    return mode


###
# Unit testing
###

if __name__ == "__main__":

    import createPDL
    import createPCL_config as createPCL

    print "**Test configzos** running from main .................."
    pcl = createPCL.make_pcl()
    pdl = createPDL.make_pdl(pcl)
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    log.logDumpPDL(pdl,short=True)

    logfile = utils.getData(pdl,"LOG")
    print " Set logging to: " + str(logfile)
    log.set_logging()

    #CONFIGHOME set in environmental variables of calling script
    utils.setData(pdl,"SERVERS",    ["b8sr00a","b8sr00b"], "*Debug*")
    utils.setData(pdl,"JVMMODE",    ["31bit","64bit"],     "*Debug*")
    utils.setData(pdl,"PROFILES",   ["default","default"], "*Debug*")
    utils.setData(pdl,"STANDALONE", [0,1],                 "*Debug*")
    utils.setData(pdl,"WASCELL",    ["beta","beta"],       "*Debug*")
    utils.setData(pdl,"WASNODE",    ["tvt7006","tvt7006"], "*Debug*")
    utils.setData(pdl,"WASSERVER",  ["b8sr00a","b8sr00b"], "*Debug*")
    utils.setData(pdl,"JVMVERSION", ["15","16"],           "*Debug*")
    utils.setData(pdl,"WASVERSION", ["80","80"],           "*Debug*")
    utils.setData(pdl,"WASMAJORVER","8",                   "*Debug*")
    utils.setData(pdl,"WASVER",     ["8.0.0.1","8.0.0.1"], "*Debug*")

    # Update for zos
    config_zos(pdl)

    # Display the pdl
    print "**Test configzos** dump PDL............."
    log.logDumpPDL(pdl,short=True,findservers=True)

    # Test get_jvmmode_from_wasenv(was_env_path)
    p1 = "/u/itcam72/252/test/was7_java64_was.env"
    p2 = "/u/itcam72/252/test/was7_jvmmode31_was.env"
    p3 = "/u/itcam72/252/test/was7_jvmmode64_was.env"
    p4 = "/u/itcam72/252/test/was7_java31_was.env"
    pathList = [p1, p2, p3, p4]
    test_wasenv = True
    if test_wasenv:
        for path in pathList:
            print "jvmmode test using: " + path
            mode = get_jvmmode_from_wasenv(path)
            print "..." + mode

    utils.setData(pdl,"JVMMODE",     None    ,     "*Debug*")
    profile_home = "/u/WAS800B/beta/tvt7006/AppServer/profiles/default"
    utils.setData(pdl,"PROFILEHOME", [profile_home, profile_home], "*Debug*")
    set_jvmmode(pdl)
    print "**Test configzos after set_jvmmode** dump PDL............."
    log.logDumpPDL(pdl,short=True,findservers=True)
