#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
# This module creates the Parametric Configuration List (PCL), which
# is the data dictionary for the configuration scripts.  This data
# described may be set from environment variables, command line
# argument flags, or console prompting.  Alternatively it may be
# discovered from inspection of the current environment.
#
# Each entry in the PCL consists of a tuple with this content:
#   [0] = Unique data element name
#   [1] = Format: A string that indicates the acceptable data
#   [2] = Initial Value: Default value
#   [3] = Environment Variable: tuple of environment variables
#   [4] = Command Line Interface Options: dictionary of command
#         line options, possibly with values associated.
#   [5] = A tuple of strings for the prompt and help message IDs.
#
# Some data is discovered by inspection, such as the host name.
# For such data, environment and option flags are not used and
# empty lists are specified in the PCL.
#
# There will be a different make_pcl routine for every script.
# this one is for the config script.
#
#####################################################################

'''
    Create the Parametric Control List (PCL) for the config script.
    A PCL is the data dictionary for the configuration script.
'''

fixlev = "40296" # Last APAR, defect or story to update this module

from createPDL import getFormats
from debugUtil import debug


#####################################################################

def make_pcl():
    '''
        Make the tuple for the Parse Control List, which specifies all parametric data
    '''

    debug("make_pcl():config entry %s" % fixlev)

    # Note that this is too early to use platformUtil.
    # (We used to use System.getProperty("os.name") for platform specific
    # defaults, but that is no longer necessary.)

    #######################################################################
    #
    # The valid formats are as follows:
    #
    # Discover    - The value will be discovered by our code.  For
    #               example, LOCALHOSTNAME will be discovered by using
    #               java.net.InetAddr.
    #
    # YesNo       - A flag which may be True or False, such as DEBUG.
    #
    # YesNoLeave  - A flag which may be True, False or None.  For example,
    #               WAS SMF recording may be turned on, turned off or
    #               left alone.
    #
    # Data        - Some complex data structure.
    #
    # File        - A file name.
    #
    # FileExists  - A file name which must exist in the file system.
    #               (Existence is only checked during prompting.)
    #
    # IPAddress   - The data value must be an IP address in V4 format
    #               dotted notation.
    #
    # Hostname    - A string that could be a valid host name.
    #
    # Note that some data is per server, such as the names of the servers.
    #
    # ServerList  - A list, each entry is the name of a server.
    #
    # StringList  - A list, each entry is a string.
    #
    # Port        - A port number.
    #
    # Drectory   - A POSIX or DOS directory path.
    #
    # Data        - Some complex data type, such as the results of findServers.
    #
    # FileNoBlank - a file name that doesn't contain space characters in
    #######################################################################

    # Dictionary of valid formats; this is kept in the PCL for validation purposes only.
    # This is a dictionary on the theory that would improve lookup times.  Since the list
    # is short, it should probably be a tuple.

    valid_formats = getFormats()

    # Create the PCL, a tuple to describe the variables
    # To reference a message key, use this: pdl["PCL"][pdl["DEBUG"][2]][5] - maybe worth a function.
    # Note: (CFG0000I,CFG0000I) are place-holder keys, it should be replaced with keys for prompt and help for prompt.)
    # todo: Come up with a scheme for prompting messages.
    # Message keys may be replace by None if prompting is not allowed for this key.

    pcl = (

        # This is used to validate formats
        ("FORMATS",          "Data",        valid_formats,   (),                          {},                                      None),

        # RETURNCODE will be set non-zero for any warning condition or error (processing may continue after a warning)
        ("RETURNCODE",       "Discover",    0,               (),                          {},                                      None),

        # Version is our current product version, discovered by reading version.properties.
        # It is a string.  To compare version strings, first convert them to lists using
        # utils.convert_version.
        ("VERSION",          "Discover",    None,            (),                          {},                                      None),

        # If the help flag is used, a help message is issued and the script immediately exits.
        ("HELP",             "YesNo",       False,           ("ITCAM_HELP",),             {"-help":True, "-nohelp":False},         ("CFG0000I","CFG0000I")  ),

        # LOG and MESSAGE have to be defaulted in the caller, since the PCL is used by multiple modules.
        ("LOG",              "File",        None,            ("ITCAM_LOG",),              {"-log":None, "-nolog":False},           ("CFG0000I","CFG0000I")),
        ("MESSAGE",          "File",        None,            ("ITCAM_MESSAGE",),          {"-message":True, "-nomessage":False},   ("CFG0000I","CFG0000I")),

        # The debug key is not in use.
        ("DEBUG",            "YesNo",       False,           ("ITCAM_DEBUG",),            {"-debug":True, "-nodebug":False},       ("CFG0000I","CFG0000I")),
        ("BATCH",            "YesNo",       False,           ("ITCAM_BATCH",),            {"-batch":True, "-nobatch":False, "-noprompt":True, "-prompt":False}, ("CFG0000I","CFG0000I")),
        ("LOGCONSOLE",       "YesNo",       True,            ("ITCAM_LOGCONSOLE",),       {"-logconsole":True, "-nologconsole":False,}, ("CFG0000I","CFG0000I")),
        ("SILENT",           "FileExists",  None,            ("ITCAM_SILENT",),           {"-silent":None},                        ("CFG1021I","CFG1021I")),
        ("SERVERS",          "ServerList",  None,            ("ITCAM_SERVER","ITCAM_SERVERS"), {"-server":None},                   ("CFG1021I","HELPPR_SERVERS")),
        ("SERVERFILE",       "FileExists",  None,            ("ITCAM_SERVERFILE",),       {"-serverfile":None},                    ("CFG0000I","CFG0000I")),
        ("WASROOT",          "PathExists",  None,            ("ITCAM_WASROOT",),          {"-wasroot":None},                       ("CFG1015I","HELPPR_WASROOT")),
        ("WSADMIN",          "FileExists",  None,            ("ITCAM_WSADMIN",),          {"-wsadmin":None},                       ("CFG0000I","CFG0000I")),
        ("RECONFIG",         "YesNo",       False,           ("ITCAM_RECONFIG",),         {"-reconfig":True, "-noreconfig":False}, ("CFG0000I","CFG0000I")),
        ("MIGRATE",          "YesNo",       False,           ("ITCAM_MIGRATE",),          {"-migrate":True, "-nomigrate":False},   ("CFG0000I","CFG0000I")),
        ("MIGRATESOA71WASROOT", "PathExists", None,          (),                          {},                                      ("CFG1131I","HELPPR_MIGRATESOA71WASROOT")),
        ("LISTRESP",         "Discover",    None,            ("ITCAM_LISTRESP",),         {"-listresp":None},                      ("CFG0000I","CFG0000I")),
        ("CREATETEMPLATE",   "YesNo",       False,           ("ITCAM_CREATETEMPLATE",),   {"-createtemplate":True, "-nocreatetemplate":False},   ("CFG0000I","CFG0000I")),
        ("DELETETEMPLATE",   "YesNo",       False,           ("ITCAM_DELETETEMPLATE",),   {"-deletetemplate":True, "-nodeletetemplate":False},   ("CFG0000I","CFG0000I")),
        ("TEMPLATEOS",       "Discover",    None,            ("ITCAM_TEMPLATEOS",),       {"-templateos":None},                    ("CFG0000I","CFG0000I")),
        ("TEMPLATEJDKVER",   "Discover",    None,            ("ITCAM_TEMPLATEJDKVER",),   {"-templatejdkver":None},                ("CFG0000I","CFG0000I")),
        ("TEMPLATEINTERP",   "Discover",    None,            ("ITCAM_TEMPLATEINTERP",),   {"-templateinterp":None},                ("CFG0000I","CFG0000I")),
        ("TEMPLATEDCHOME",   "Directory",   None,            ("ITCAM_TEMPLATEDCHOME",),   {"-templatedchome":None},                ("CFG1162I","HELPPR_TEMPLATEDCHOME")),
        ("TEMPLATELIST",     "StringList",  None,            (),                          {},                                      ("CFG2053I","HELPPR_TEMPLATELIST")),



        ("RECONFIGDIR",      "PathExists",  None,            ("ITCAM_RECONFIGDIR",),      {"-reconfigdir":None},                   ("CFG0000I","CFG0000I")),

        ("TCPIPINTERFACES",  "StringList",  None,            (),                          {},                                      ("CFG1098I","HELPPR_TCP")),
        ("TCPIPINTERFACE",   "IPAddress",   None,            (),                          {},                                      None),
        ("LOCALHOSTNAME",    "Hostname",    None,            ("ITCAM_LOCALHOSTNAME",),    {"-localhostname":None},                 None),
        ("LOCALHOSTIP",      "IPAddress",   None,            ("ITCAM_LOCALHOSTIP",),      {"-localhostip":None},                   None),
        ("DEFAULTHOSTNAME",  "Hostname",    None,            ("ITCAM_DEFAULTHOSTNAME",),  {"-defaulthostname":None},               None),
        ("DEFAULTHOSTIP",    "IPAddress",   None,            ("ITCAM_DEFAULTHOSTIP",),    {"-defaulthostip":None},                 None),

        # KERNELIP and KERNELPORT01 are used to connect to the MS to locate the MSHOME, as well as
        # being configured into the runtime.
        ("KERNELIP",         "Hostname",    "127.0.0.1",     ("ITCAM_KERNELIP","KERNEL_IP",), {"-kernelip":None},                  ("CFG1010I","HELPPR_MSKERNELIP")),
        ("KERNELPORT01",     "Port",        "9122",          ("ITCAM_KERNELPORT01","KERNEL_PORT01",), {"-kernelport01":None},      ("CFG1011I","HELPPR_MSPORT")),
        ("KERNELPORT02",     "Port",        "9123",          ("ITCAM_KERNELPORT02","KERNEL_PORT02",), {"-kernelport02":None},      ("CFG0000I","CFG0000I")),

        # The best value for MSHOME is gotten by contacting the MS and asking it.
        # Whether configuration should be setting this value is another question beyond our scope.
        ("MSHOME",           "Directory",   "/opt/IBM/itcam/WebSphere/MS", ("ITCAM_MSHOME","MSHOME"), {"-mshome":None},            ("CFG1012I","HELPPR_MSHOME")),

        ("BINDIP",           "Hostname",    None,            ("ITCAM_BINDIP","BINDIP",),  {"-bindip":None},                        ("CFG0000I","CFG0000I")),
        ("EXPORTIP",         "Hostname",    None,            ("ITCAM_EXPORTIP","EXPORTIP",), {"-exportip":None},                   ("CFG0000I","CFG0000I")),
        ("RFSPORT01",        "Port",        "9120",          ("ITCAM_RFSPORT01","RFS_PORT01",), {"-rfsport01":None},               ("CFG1146I","HELPPR_MSRFSPORT")),
        ("RFSPORT02",        "Port",        "9121",          ("ITCAM_RFSPORT02","RFS_PORT02",), {"-rfsport02":None},               ("CFG0000I","CFG0000I")),
        ("RMIPORTSET",       "YesNo",       False,           ("ITCAM_RMIPORTSET",),       {"-rmiportset":True,"-normiportset":False},  ("CFG1060I","HELPPR_MSADV")),
        ("PROBERMIPORT",     "PortRange",   "8200-8299",     ("ITCAM_PROBERMIPORT",),     {"-probermiport":None},                  ("CFG1061I","HELPPR_MSRMIPORT")),
        ("PROBECONTROLLERRMIPORT", "PortRange", "8300-8399", ("ITCAM_PROBECONTROLLERRMIPORT",), {"-probermicontrollerport":None},  ("CFG1062I","HELPPR_MSRMICPORT")),

        # Flag to indicate whether a JMX connection is required.
        ("DCLESS",           "Discover",    False,           (),                          {},                                      None),
        # The following is not yet active because only SOAP is supported for the DC-less connnection
        ("DCLESSCONNTYPE",   "String",      "SOAP",          ("ITCAM_DCLESSCONNTYPE",),   {"-dclessconntype":None},                ("CFG0000I","CFG0000I")),

        ("TEMADCAPPSERVER",  "YesNo",       True,            ("ITCAM_TEMADCAPPSERVER",),  {"-temadcappserver":True,"-notemadcappserver":False}, ("CFG1176I","HELPPR_TEMADCAPPSERVER")),
        ("NOCONFIGDC", "Discover",          None,            (),                          {},                                      None),

        # Flag to indicate whether a connection to a TEMA is required
        ("TEMACONNECT",      "YesNo",       False,           ("ITCAM_TEMACONNECT",),      {"-temaconnect":True,"-notemaconnect":False}, ("CFG1013I","HELPPR_ADTEMA")),
        ("TEMAHOST",         "Hostname",    "127.0.0.1",     ("ITCAM_TEMAHOST",),         {"-temahost":None},                      ("CFG1006I","HELPPR_ADTEMAHOST")),
        ("TEMAPORT",         "Port",        "63335",         ("ITCAM_TEMAPORT",),         {"-temaport":None},                      ("CFG1007I","HELPPR_ADTEMAPORT")),
        ("SERVERALIAS",      "Alphanumeric", None,           ("ITCAM_SERVERALIAS",),      {"-serveralias":None},                   ("CFG1167I","HELPPR_ADSERVERALIAS")),
        ("JMXADMINPORT",     "Port",        "63355",         ("ITCAM_JMXADMINPORT",),     {"-jmxadminport":None},                  ("CFG1182I","HELPPR_ADJMXADMINPORT")),

        ("SOADC",            "YesNo",       False,           ("ITCAM_SOADC",),            {"-soadc":True,"-nosoadc":False},        ("CFG1034I","HELPPR_SOADC")),
        ("TPV",              "YesNo",       False,           ("ITCAM_TPV",),              {"-tpv":True,"-notpv":False},            ("CFG1135I","HELPPR_TPV")),
        ("DE",               "YesNo",       False,           ("ITCAM_DE",),               {"-de":True,"-node":False},              ("CFG1138I","HELPPR_DE")),
        # DEPRO is a copy of DE for prompting and display
        ("DEPRO",            "YesNo",       False,           (),                          {},                                      ("CFG1185I","HELPPR_DEPRO")),
        #Flag to indicate that we are dealing with SCM AI install (SaaS or on-Prem)
        ("SMAI",             "Discover",    None,           (),                          {},                                      None),
    

        ("WSHOST",           "Hostname",    None,            ("ITCAM_WSHOST",),           {"-wshost":None},                        ("CFG0000I","CFG0000I")),
        ("DMGRHOST",         "Hostname",    None,            ("ITCAM_DMGRHOST",),         {"-dmgrhost":None},                      ("CFG0000I","CFG0000I")),
        ("OWNER",            "String",      None,            ("ITCAM_OWNER",),            {"-owner":None},                         ("CFG0000I","CFG0000I")),
        ("ACCESS",           "String",      None,            ("ITCAM_ACCESS",),           {"-access":None},                        ("CFG0000I","CFG0000I")),
        ("PROFILE",          "String",      None,            ("ITCAM_PROFILE",),          {"-profile":None},                       ("CFG0000I","CFG0000I")),
        ("CONNTYPE",         "String",      None,            ("ITCAM_CONNTYPE",),         {"-conntype":None},                      ("CFG0000I","CFG0000I")),
        ("HOST",             "Hostname",    None,            ("ITCAM_HOST",),             {"-host":None,},                         ("CFG0000I","CFG0000I")),
        ("PORT",             "Port",        None,            ("ITCAM_PORT",),             {"-port":None,},                         ("CFG1145I","HELPPR_PORT")),
        ("USER",             "String",      None,            ("ITCAM_USER",),             {"-user":None,},                         ("CFG1018I","HELPPR_WASUID")),
        ("PASSWORD",         "Password",    None,            ("ITCAM_PASSWORD",),         {"-password":None,},                     ("CFG1019I","HELPPR_WASPWD")),
        ("MASTERINPUT",      "YesNo",       False,           ("ITCAM_MASTERINPUT",),      {"-masterinput":True, "-nomasterinput":False}, ("CFG0000I","CFG0000I")),

        # Flag to indicate whether a connection to an MS is required
        ("MSCONNECT",        "YesNo",       False,           ("ITCAM_MSCONNECT",),  {"-msconnect":True,"-nomsconnect":False}, ("CFG1009I","HELPPR_MS")),
        ("SMF",              "YesNoLeave",  None,            ("ITCAM_SMF",),              {"-smf":True, "-nosmf":False, "-leavesmf":None}, ("CFG1014I","HELPPR_SMF")),
        ("GCOUTPUT",         "YesNo",       False,           ("ITCAM_GCOUTPUT",),         {"-gcoutput":True,"-nogcoutput":False},  ("CFG1020I","HELPPR_GCOUTPUT")),
        ("GCLOGPATH",        "FileNoBlank", None,            ("ITCAM_GCLOGPATH",),        {"-gclogpath":None},                     ("CFG1024I","HELPPR_GCPATH")),
        ("GC_PARMS",         "String",      None,            ("ITCAM_GCPARMS",),          {"-gcparms":None},                       ("CFG0000I","CFG0000I")),
        ("GCOUTPUTDIST",     "YesNo",       False,           ("ITCAM_GCOUTPUTDIST",),     {"-gcoutputdist":True,"-nogcoutputdist":False},  ("CFG1023I","HELPPR_GCD")),

        # Flag to indicate whether to connect to the MS to acquire the MSHOME value
        ("MSHOMECONNECT",    "YesNo",       True,            ("ITCAM_MSHOMECONNECT",),    {"-mshomeconnect":True,"-nomshomeconnect":False}, ("CFG0000I","CFG0000I")),
        ("TRANSFORM",        "YesNo",       False,           ("ITCAM_TRANSFORM",),        {"-transform":True,"-notransform":False},("CFG0000I","CFG0000I")),
        ("CLEANUP",          "YesNo",       False,           ("ITCAM_CLEANUP",),          {"-cleanup":True,"-nocleanup":False},    ("CFG1043I","CFG1043I")),
        ("CLEANUPPLUGINS",   "YesNo",       False,           ("ITCAM_CLEANUPPLUGINS",),   {"-cleanupplugins":True,"-nocleanupplugins":False}, ("CFG0000I","CFG0000I")),
        ("PLUGINS",          "YesNo",       False,           ("ITCAM_PLUGINS",),          {"-plugins":True,"-noplugins":False},    ("CFG0000I","CFG0000I")),
        ("HEAPSIZE",         "YesNo",       False,           ("ITCAM_HEAPSIZE",),         {"-heapsize":True,"-noheapsize":False},  ("CFG1027I","HELPPR_HEAP")),
        ("MAXHEAPSIZE",      "String",      "0",             ("ITCAM_MAXHEAPSIZE",),      {"-maxheapsize":None},                   ("CFG0000I","CFG0000I")),

        # TT API
        ("TTAPI",            "YesNo",       False,           ("ITCAM_TTAPI",),            {"-ttapi":True,"-nottapi":False},        ("CFG1031I","HELPPR_TT")),
        # TT API library directory - zOS only
        ("TTAPIDIR",         "PathExists",  None,            ("ITCAM_TTAPIDIR",),         {"-ttapidir":None,"-nottapidir":None},   ("CFG1033I","HELPPR_TTAPIDIR")),
        # TT API SSN name - zOS only
        ("TTAPISSN",         "String",      "CYTZ",          ("ITCAM_TTAPISSN",),         {"-ttapissn":None,"-nottapissn":None},   ("CFG1032I","HELPPR_TTAPISSN")),
        # TT API jar file
        ("TTAPIJAR",         "File",        None,            ("ITCAM_TTAPIJAR",),         {"-ttapijar":None,"-nottapijar":None},   ("CFG0000I","CFG0000I")),

        # TT API Transactions Collector host and port - distributed only
        ("TTTCHOST",         "Hostname",    "127.0.0.1",     ("ITCAM_TTTCHOST",),         {"-tthost":None,},                       ("CFG1035I","HELPPR_TTHOST")),
        ("TTTCPORT",         "Port",        "5457",          ("ITCAM_TTTCPORT",),         {"-ttport":None,},                       ("CFG1036I","HELPPR_TTPORT")),

        # Emergency patch
        ("ITCAMDCINPUT",     "File",        None,            ("ITCAMDC_INPUT",),         {"-dci":None,"-nodci":None},              ("CFG0000I","CFG0000I")),

        # Flag for using client properties
        ("USECLIENTPROPS",   "YesNo",       False,           ("ITCAM_USECLIENTPROPS",),  {"-useclientprops":True,"-nouseclientprops":False}, ("USECLIENTPROPS","HELPPR_USECLIENTPROPS")),


        # CONFIGHOME is the configuration home.  On distributed, it is the same as the install
        # location, but on z/OS it is a distinct directory which contains symbolic links to
        # the install routines and real configuration files.  Identified with ITCAMDCHOME.
        ("CONFIGHOME",       "PathExists",  None,            ("ITCAM_CONFIG","ITCAM_CONFIGHOME","ITCAM_DCHOME"), {"-config":None,"-confighome":None,"-dchome":None,},("CFG0000I","CFG0000I")),

        ("MIGRATEHOME",      "PathExists",  None,            ("ITCAM_MIGRATEHOME",),     {"-migratehome":None},                   ("CFG1085I","HELPPR_MIGRATEHOME")),


        # RUNHOME is CONFIGHOME+sep+"runtime".
        ("RUNHOME",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # Bypass flags for debugging.
        ("BYPASSFINDSERVERS", "File",       None,            ("ITCAM_BYPASSFINDSERVERS",),{"-bypassfindservers":True, "-nobypassfindservers":False},    None),
        ("BYPASSCONFIG",     "YesNo",       False,           ("ITCAM_BYPASSCONFIG",),     {"-bypassconfig":True, "-nobypassconfig":False},              None),
        ("BYPASSCHECK",      "YesNo",       False,           ("ITCAM_BYPASSCHECK",),      {"-bypasscheck":True, "-nobypasscheck":False},                None),
        ("BYPASSPROMPT",     "YesNo",       False,           ("ITCAM_BYPASSPROMPT",),     {"-bypassprompt":True, "-nobypassprompt":False},              None),

        # GCPARMS is set based on the value of GCOUTPUT.
        ("GCPARMS",          "Discover",    "",              (),                          {},                                      ("CFG0000I","CFG0000I")),


        # TTAPILIBPATH zOS only, by server,  is for the LIBPATH entry in input.properties: RUNTIMEP/lib/ttapi. Consumer: inputproperties
        ("TTAPILIBPATH",     "Discover",    "",              (),                          {},                                      ("CFG0000I","CFG0000I")),
        # TTAPICLASSPATH zOS only, by server. is for the CLASSPATH entry in input.properties: RUNTIMEP/lib/ttapi/cytapi4j.jar. Consumer: inputproperties
        ("TTAPICLASSPATH",   "Discover",    "",              (),                          {},                                      ("CFG0000I","CFG0000I")),
        # ZOSLIBPATH zOS only, by server,  is for the LIBPATH entry in input.properties.template:  Consumer: inputproperties.py
        ("ZOSLIBPATH",       "Discover",    "",              (),                          {},                                      ("CFG0000I","CFG0000I")),

        # Data returned by findServers
        ("FINDSERVERS",      "Data",        None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # Atomic data returned by findServers
        ("WASCELL",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASNODE",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASCELLMODIFIED",  "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASNODEMODIFIED",  "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASINSTALLROOT",   "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASPLUGINSROOT",   "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("USERINSTALLROOT",  "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASCONFIGROOT",    "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WASPROPSDIR",      "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("RMIHOST",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("RMIPORT",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("JAVAHOME",         "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("LOGROOT",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WPSHOME",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WBIHOME",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("WLPHOME",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),


        # Per server data returned by findServers
        ("WASSERVER",            "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("SERVERSMODIFIED",      "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("PROFILEMODIFIED",      "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("STANDALONE",           "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("JVMMODE",              "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("SHARECLASSESMODIFIED", "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("SERVERID",             "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("CURRENTMAXHEAPSIZE",   "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("CURRENTGCLOGLOCATION", "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("MIGRATEDDCHOME",       "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("MIGRATEDVERSION",      "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("MIGRATEDCONFIGHOME",   "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        ("SUBPRODUCT",           "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        #Return code from server config per server, GT 0 indicates an error during configuration, code 99 flags invalid credentials
        ("CONFIGRETURNCODE", "Discover",           0,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # Discovery data, also per server

        # RUNLOC is [platform].[node].server
        # Note: This is per server, so it must be a list matching SERVERS
        ("RUNLOC",           "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # RUNTIME is CONFIGHOME+sep+runtime+sep+RUNLOC (RUNLOC=[platform].[node].server
        # Note: This is per server, so it must be a list matching SERVERS
        ("RUNTIME",          "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # RUNTIMEP is ${ITCAMDCHOME}+sep+"runtime"+sep+RUNLOC
        # ${ITCAMDCHOME} will be resolved by WebSphere at runtime.
        ("RUNTIMEP",         "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # AMPLATFORM is the platform, including the version                          
        ("AMPLATFORM",       "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # WASVER is the printable WebSphere version
        ("WASVER",           "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # WASVERSION is the numeric two digit version (51, 60, 61, 70, 80, 85, etc.)
        ("WASVERSION",       "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # WASMAJOR is the WebSphere version (5, 6, 7, 8, etc.)
        ("WASMAJORVER",      "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # JVMVERSION is the Java Virtual Machine version supported by this WebSphere
        ("JVMVERSION",       "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # VERSIONQUOTES is a flag that indicates whether quotes are required in a certain situation.  It is almost obsolete.
        ("VERSIONQUOTES",    "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # CONFIGURED is a flag that indicates if an application server has ITCAM configuration or not.
        ("CONFIGURED",    "Discover",       None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # DCVERSION stores current ITCAM version.
        ("DCVERSION",    "Discover",       None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        # BACKUP
        ("BACKUP",            "YesNo",      False,          ("ITCAM_BACKUP",),           {"-backup":True,"-nobackup":False},       ("CFG1025I","HELPPR_BACKUP")),

        # WAS BACKUP file
        ("BACKUPDIR",         "File",       None,           ("ITCAM_BACKUPDIR",),        {"-backupdir":None,"-nobackupdir":None},  ("CFG1026I","HELPPR_BACKUPDIR")),
        ("BACKUPSUFFIX",      "String",     None,           ("ITCAM_BACKUPSUFFIX",),     {"-backupsuffix":None,"-nobackupsuffix":None},  ("CFG1086I","CFG1086I")),
        ("CONTINUECONFIG",    "YesNo",      False,          ("ITCAM_CONTINUECONFIG",),   {"-configcontinue":True,"-nocontinueconfig":False},       ("CFG2013I","CFG2013I")),


        # Miscellaneous prompts:
        #   Accept change configuration choices
        ("PRCONFIGDISPLAY",   "String",      "a",            (),                          {},                                     ("CFG1041I","HELPPR_SUMMARY")),

        #####################################################
        # The following discovered lists are not per server.
        #####################################################

        # WASHOMES is a list of WebSphere homes discovered on distributed.  It is not set on z/OS.
        ("WASHOMES",         "StringList",  None,            ("ITCAM_WASHOMES","ITCAM_WASHOME"), {"-washomes":None, "-washome":None}, ("CFG1016I","HELPPR_WASHOME")),

        # PROFILES is a list of profile names discoverd on distributed.  It is not set on z/OS.
        ("PROFILES",         "StringList",  None,            ("ITCAM_PROFILENAMES","ITCAM_PROFILENAME"), {"-profileName":None},    ("CFG1017I","HELPPR_PROFILES")),

        # WSADMINS is a list of wsadmin locations discovered on z/OS..
        ("WSADMINS",         "StringList",  None,            (),                          {},                                       ("CFG1106I","HELPPR_WSADMINS")),


        # PROFILEHOME is a home directory for the selected profile
        ("PROFILEHOME",         "Discover",  None,            (),                         {},                                         ("CFG0000I","CFG0000I")),

        # USER and password list which each entry in the list is a dictionary which its key is WASHOME.WASPROFILE
        ("USERLIST",           "Discover",    None,            ("ITCAM_USERLIST",),           {"-userlist":None,},                      ("CFG80000I","CFG0000I")),
        ("PASSWORDLIST",       "Discover",    None,            ("ITCAM_PASSWORDLIST",),       {"-passwordlist":None,},                  ("CFG0000I","CFG0000I")),
        ("USECLIENTPROPSLIST", "Discover",    None,            ("ITCAM_USECLIENTPROPSLIST",), {"-useclientpropslist":None,},            ("CFG0000I","CFG0000I")),


        # FULLSERVERS is a list for each selected servers each entry in the list represents
        # selected servers, and it has a format of cell.node.server
        ("FULLSERVERS",      "Discover",    None,            ("ITCAM_FULLSERVERS",),     {"-fullservers":None,},                     ("CFG0000I","CFG0000I")),


        ("PREAD72MIGRATE",   "Discover",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        #For SCMA AI installs list of DC homes that can be used to migrate
        ("CCMDCHOMES",       "StringList",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),
        
        ("CCMMIGRATEHOMES",  "StringList",    None,            (),                          {},                                      ("CFG0000I","CFG0000I")),

        #
        # Discovery data at the time of unconfig/reconfig/upgrade.  Each pdl is a list which its index
        # corresponds to the index in SERVERS pdl.
        #
        ("CONNTYPELIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("HOSTLIST",                   "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("PORTLIST",                   "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TEMACONNECTLIST",            "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TEMAHOSTLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TEMAPORTLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("MSCONNECTLIST",              "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("MSHOMECONNECTLIST",          "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("KERNELIPLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("KERNELPORT01LIST",           "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("MSHOMELIST",                 "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("RMIPORTSETLIST",             "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("PROBECONTROLLERRMIPORTLIST", "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("PROBERMIPORTLIST",           "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TTAPILIST",                  "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TTAPISSNLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TTAPIDIRLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TTTCHOSTLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TTTCPORTLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("SOADCLIST",                  "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("SMFLIST",                    "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("GCOUTPUTLIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("GCLOGPATHLIST",              "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("GCOUTPUTDISTLIST",           "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("HEAPSIZELIST",               "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("CONFIGUREDPROBES",           "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TPVLIST",                    "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("DELIST",                     "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TEMPLATEOSLIST",             "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TEMPLATEJDKVERLIST",         "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("TEMPLATEINTERPLIST",         "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("ALIASLIST",                  "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),


        ("RFSPORT01LIST",              "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("UNCONFIG",                   "YesNo",       False,           (),                       {},                                         ("CFG0000I","CFG0000I")),

        #parameter related to liberty
        ("WLPINSTALLDIR",             "Path",        None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        ("WLPUSRDIR",                 "Path",        None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        # ITCAMFEATUREVERSION stores current itcam feature version used in WAS Liberty configation
        ("ITCAMFEATUREVERSION",       "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),
        # IOFFERING stores whether it is apm, apd or am
        ("IOFFERING",                 "Discover",    None,            (),                       {},                                         ("CFG0000I","CFG0000I")),

    )


    debug("make_pcl() exit")
    return pcl
