#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
####################################################################

#####################################################################
# This module creates the Parametric Configuration List (PCL), which
# is the data dictionary for the configuration scripts.  This data
# described may be set from environment variables, command line
# argument flags, or console prompting.  Alternatively it may be
# discovered from inspection of the current environment.
#
# Each entry in the PCL consists of a tuple with this content:
#   [0] = Unique data element name
#   [1] = Format: A string that indicates the acceptable data
#   [2] = Initial Value: Default value
#   [3] = Environment Variable: tuple of environment variables
#   [4] = Command Line Interface Options: dictionary of command
#         line options, possibly with values associated.
#
# Some data is discovered by inspection, such as the host name.
# For such data, environment and option flags are not used and
# empty lists are specified in the PCL.
#
# There will be a different make_pcl routine for every script.
# This one is used by createcfg.
#
#####################################################################

'''
    Create the Parametric Control List (PCL) for the createcfg script.
    A PCL is the data dictionary for the configuration script.
'''

from createPDL import getFormats

debug=0    #  0 - debug off;  1 - debug on

def set_debug(flag):
    """
        Method to set the internal debug flag
    """
    if flag == 1: debug = 1
    else:         debug = 0
#enddef

def make_pcl(): # test
    '''
        Make the tuple for the Parse Control List, which specifies all parametric data
    '''

    from java.lang import System
    osname = System.getProperty("os.name")


    if  osname.startswith("Windows"):
        default_wsadmin = "C:\\WebSphere\\bin\wsadmin.bat" # todo: fix
        default_wasroot = "C:\\WebSphere" # todo: fix

    elif osname == "z/OS":
        default_wsadmin = "/u/WAS80/bin/wsadmin.sh" # todo: fix
        default_wasroot = "/u/WAS80"

    # todo: extend the supported platforms AIX, HP-UX, ....
    else:
        assert False, "%s is an unsupported platform" % osname

     # Dictionary of valid formats; this is kept in the PCL for validation purposes only.

    valid_formats = getFormats()


    # Create the PCL, a tuple to describe the variables

    pcl = (
            ("LOG",              "File",        None,            ("ITCAM_LOG",),              {"-log":True, "-nolog":False},           ("CFG0000I","CFG0000I")  ),
            ("MESSAGE",          "File",        None,            ("ITCAM_MESSAGE",),          {"-message":True, "-nomessage":False},   ("CFG0000I","CFG0000I")),
            ("HELP",             "YesNo",       False,           ("ITCAM_HELP",),             {"-help":True, "-nohelp":False},         ("CFG0000I","CFG0000I")  ),
            ("DEBUG",            "YesNo",       False,           ("ITCAM_DEBUG",),            {"-debug":True, "-nodebug":False},       ("CFG0000I","CFG0000I")  ),
            ("BATCH",            "YesNo",       False,           ("ITCAM_BATCH",),            {"-batch":True, "-nobatch":False,
                                                                                               "-noprompt":True, "-prompt":False},     ("CFG0000I","CFG0000I")  ),

            ("OWNER",            "String",      "none",          ("ITCAM_OWNER",),            {"-owner":None, },                       ("CFG3003I","CFG3003I")  ),
            ("ACCESS",           "String",      "none",          ("ITCAM_ACCESS",),           {"-access":None,},                       ("CFG3004I","CFG3004I")  ),

            ("CONFIGHOME",       "String",      "/u/itcam72",    ("ITCAM_CONFIG",
                                                                  "ITCAM_CONFIGHOME", ),      {"-config":None,},                       ("CFG3002I","CFG3002I")  ),
            # TODO remove ITCAM_INSTALLHOME and "ITCAM_CONFIGHOME
            ("INSTALLHOME",      "Path",        "/smp/usr/lpp/itcam/WebSphere/DC",
                                                                  ("ITCAM_INSTALL",
                                                                  "ITCAM_INSTALLHOME",),      {"-install":None,},                      ("CFG3001I","CFG3001I")  ),

            ("CLEARJY",          "YesNo",       False,            ("ITCAM_CLEARJY",),         {"-clearjy":True, "-noclearjy":False},  ("CFG0000I","CFG0000I")  ),

            # Prompt for to accept or to re-enter create config choices
            ("PRCRTCFGDISPLAY",  "String",      "a",             (),                          {},                                     ("CFG3008I","CFG3008I")  ),

            ("TESTMSGS",         "YesNo",       False,           ("ITCAM_TESTMSGS",),         {"-testmsgs":True, "-notestmsgs":False},("CFG0000I","CFG0000I")  ),



            ("FORMATS",          "Data",        valid_formats,   (),                          {},                                     ("CFG0000I","CFG0000I")  ),
          )

    # Do some validation of the PCL

    for pc in pcl:
        if pc[0] == "FORMATS":
            valid_formats = pc[2]
            if debug == 1:  print " valid formats: " + str(valid_formats)
            break

    nms = {}
    envs = {}
    flgs = {}


    if debug == 1: print "len .." + str(len(pcl))
    r= range(len(pcl))
    if debug == 1: print "..range = " + str(r)


    for pcli in range(len(pcl)):
        if debug == 1: print "  pcli " + str(pcli)
        pc = pcl[pcli]           # pc is one row
        if debug == 1: print "pc: "  + str(pc)
        nm = pc[0]
        if debug == 1: print "nm:  " + nm
        assert nm.upper() == nm, "Name %s, used in PCL[%d', is not upper case" % (nm, pcli)
        # mec  assert nm not in nms, "Name %s is used in multiple PCL entries %d and %d" % (nm, nms[nm], pcli)
        # name not in nms yet of course nothing is..  assert nm not in nms.keys(), "Name %s is used in multiple PCL entries %d and %d" % (nm, nms[nm], pcli)


        nms[nm] = pcli
        if debug == 1:  print "nms[nm]= " + str(nms[nm])
        fmt = pc[1]
        val = pc[2]
        envpcl = pc[3]
        flgpcl = pc[4]

        if debug == 1: print "val: " + str(val)
        if debug == 1: print "envpcl: " + str(envpcl)
        if debug == 1:  print "flgpcl: " + str(flgpcl)

        # Check the format is valid and that the initialization
        # value is valid for the format
        if False: pass

        elif fmt == "YesNo":
            assert val in {True:None, False:None}.keys(), "Initializer for %s is '%s', which is invalid for %s format" % (nm, val, fmt)

        elif fmt == "YesNoLeave":
            assert val in {True:"", False:"", "leave":""}.keys(), "Initializer for %s is '%s', which is invalid for %s format" % (nm, val, fmt)

        #mec elif fmt in valid_formats:
        elif fmt in valid_formats.keys():
            # todo: Improve checking for some items
            pass

        elif fmt == None or fmt == "":
            assert False, "Missing format for %s" % nm
        else:
            assert False, "Unknown format '%s' for PCL entry for %s" % (fmt, nm)

        # Verify that the environment variables are unique
        firstEnvLoop=0
        for env in envpcl:
            if debug == 1: print "env: " + env
            if not firstEnvLoop==0:
                assert env not in envs.keys(), "Duplicate environment variable %s used by PCL entries %s and %s" % (env, envs[env], nm)
                firstEnvLoop=1
            envs[env] = nm

        # Verify that the command line flags are unique
        # for flg in flgpcl:

        first=0
        for flg in flgpcl.keys():
            if debug == 1:  print "flg:  " + flg
            if not first == 0:
                assert flg not in flgs.keys(), "Command line flag %s is used by PCL entries %s and %s" % (flg, flgs[flg], nm)
                first=1
            flgs[flg] = nm

    return pcl
