#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
# This module contains functions that create and initialize the
# Parametric Data List (PDL).  The PDL is a dictionary whose key is
# the name of the data item.  The value of each entry is a tuple
# consisting of:
#   0. The data of the items.
#   1. A tag indicating what set the data value
#   2. An index into the PCL indicating the associated PCL entry
#
# make_pdl
#   This function is passed the PCL, which is used to create the
#   inital PDL with all values set to the initial defaults.
#
# proc_envvar
#   This function is passed the PCL and the PDL.  It looks for
#   environment variables as specified in the PCL; when found,
#   the value in the PDL is updated based on the associated format
#   found in the PCL.
#
# proc_cliopt
#   This function is passed the PCL, PDL and command line arguments.
#   Option flags will be interpreted per the format in the PCL and
#   the PDL will be updated accordingly.
#
#####################################################################
#
# To update a PDL entry, use a list containing
#
#   0) The data
#   1) A tag that indicates what set the value, usually the name of
#      function that set the value.
#   2) The index into the PCL; initially set by make_pdl
#
# For example, suppose the function get_iphost has discovered the
# local host name.  To place the value into the PDL, the following
# locution would be used:
#
#   pdl["HOSTNAME"][0] = hostname
#   pdl["HOSTNAME"][1] = "get_iphost"
#
# Then in subsequent code, to access the host name index into the
# PDL in the following manner:
#
#   hostname = pdl["HOSTNAME"][0]
#
#####################################################################

'''
    Create the Parametric Data List (PDL)
'''

import os
import re

from ConfigurationException import ConfigurationException
from resourcebundle import getmsg
from debugUtil import debug


#####################################################################

def make_pdl(pcl):
    '''
        Given a PCL, make the dictionary for the Parametric Data List.
    '''

    debug("make_pdl() entry")

    pdl = {}

    # Save the PCL in the PDL so there will be only one object to
    # carry around.

    pdl["PCL"] = [pcl, "Initial", None]

    for i in range(len(pcl)):
        v = pcl[i]
        nm = v[0]
        val = v[2]
        assert nm not in pdl.keys(), "%s is duplicated in PDL" % nm
        pdl[nm] = [val,"Initial",i]

    debug("make_pdl() exit, %d entries in pdl" % len(pdl))
    return pdl


#####################################################################

def getFormats():
    '''
        Return a dictionary of the valid formats.  Key is the format name,
        value is a list of properties:
        [0] - None (Reserved for a planned validity check routine reference)
    '''

    #######################################################################
    #
    # The valid formats are as follows:
    #
    # Discover    - The value will be discovered by our code.  For
    #               example, LOCALHOSTNAME will be discovered by using
    #               java.net.InetAddr.
    #
    # YesNo       - A flag which may be True or False, such as DEBUG.
    #
    # YesNoLeave  - A flag which may be True, False or None.  For example,
    #               WAS SMF recording may be turned on, turned off or
    #               left alone.
    #
    # Data        - Some complex data structure.
    #
    # File        - A file name.
    #
    # FileExists  - A file name which must exist in the file system.
    #               (Existence is checked during prompting only.)
    #
    # PathExists  - A directory path which must exist in the file system.
    #               (Existence is checked during prompting only.)
    #
    # IPAddress   - The data value must be an IP address in V4 format
    #               dotted notation.
    #
    # Hostname    - A string that could be a valid host name.
    #
    #
    # Note that some data is per server, such as the names of the servers.
    #
    # ServerList  - A list, each entry is the name of a server.
    #
    # StringList  - A list, each entry is a string.
    #
    # Port        - A port number.
    #
    # Directory   - A POSIX or DOS directory path, quite likely not a valid path on the current system.
    #
    # Alphanumeric - a string that contains alpha-numeric characters, dot, dash and underscore
    #
    #######################################################################

    # Notes:
    #  - The validation routines should probably be moved to their own module.
    #  - Some parameters, such as FileExists, will be sensitive to the caller.
    #  - todo: Complete the validation routines for all formats.

    ans = {
           "YesNo":[validateYesNo],
           "YesNoLeave":[validateYesNoLeave],
           "File":[validateNothing],
           "Path":[validateNothing],
           "FileExists":[validateFileExists],
           "PathExists":[validatePathExists],
           "Hostname":[validateNothing],
           "Password":[validateNothing],
           "IPAddress":[validateNothing],
           "String":[validateNothing],
           "StringList":[validateNothing],
           "ServerList":[validateNothing],
           "Port":[validateNothing],
           "PortRange":[validateNothing],
           "Directory":[validateNothing],
           "Discover":[validateNothing],
           "Data":[validateNothing],
           "FileNoBlank": [validateNothing],
           "Alphanumeric" : [validateStringChars]
          }

    return ans


#####################################################################

def validateNothing(nm,env,val):
    '''
        Validate a value for a format - Dummy routine that does nothing

        nm - The PDL key which will be set
        callid - Caller's identify
              Environment - Setting value from environment variables
              CommandLine - Setting value from command line flag
              Prompt - Setting the value from a prompt
        val - Proposed value to be set
    '''

    assert env in ["Environment", "CommandLine", "Prompt"], "Unexpected caller: %s" % env
    
    return True


#####################################################################

def validateYesNo(nm,env,val):
    '''
        Validate a value for a format - YesNo

        nm - The PDL key which will be set
        callid - Caller's identify
              Environment - Setting value from environment variables
              CommandLine - Setting value from command line flag
              Prompt - Setting the value from a prompt
        val - Proposed value to be set
    '''

    assert env in ["Environment", "CommandLine", "Prompt"], "Unexpected caller: %s" % env
    
    if env != "Environment":
        ans = True
    elif str(val).upper() in ["Y","N","YES","NO","1","2"]:
        ans = True
    else:
        ans = False

    return ans


#####################################################################

def validateYesNoLeave(nm,env,val):
    '''
        Validate a value for a format - YesNoLeave

        nm - The PDL key which will be set
        callid - Caller's identify
              Environment - Setting value from environment variables
              CommandLine - Setting value from command line flag
              Prompt - Setting the value from a prompt
        val - Proposed value to be set
    '''

    assert env in ["Environment", "CommandLine", "Prompt"], "Unexpected caller: %s" % env

    if env != "Environment":
        ans = True
    elif str(val).upper() in ["Y","N","L","YES","NO","LEAVE","1","2","3"]:
        ans = True
    else:
        ans = False

    return ans


#####################################################################

def validateFileExists(nm,env,val):
    '''
        Validate a value for a format
        nm - The PDL key which will be set
        callid - Caller's identify
              Environment - Setting value from environment variables
              CommandLine - Setting value from command line flag
              Prompt - Setting the value from a prompt
        val - Proposed value to be set
    '''

    assert env in ["Environment", "CommandLine", "Prompt"], "Unexpected caller: %s" % env

    # To do: Investigate whether the path exists
    # To do: Issue message if within Prompt

    if not env == "Prompt":
        ans = True
    else:
        if os.path.isfile(val):
            ans = True
        else:
            ans = False

    return ans


#####################################################################

def validatePathExists(nm,env,val):
    '''
        Validate a value for a format
        nm - The PDL key which will be set
        callid - Caller's identify
              Environment - Setting value from environment variables
              CommandLine - Setting value from command line flag
              Prompt - Setting the value from a prompt
        val - Proposed value to be set
    '''

    assert env in ["Environment", "CommandLine", "Prompt"], "Unexpected caller: %s" % env
    
    if not env == "Prompt":
        ans = True
    else:
        if os.path.isdir(val):
            ans = True
        else:
            ans = False

    return ans

#####################################################################
def validateStringChars(nm,env,val):
    '''
        Validate a value for a format - Alphanumeric

        nm - The PDL key which will be set
        callid - Caller's identify
              Environment - Setting value from environment variables
              CommandLine - Setting value from command line flag
              Prompt - Setting the value from a prompt
        val - Proposed value to be set
    '''

    assert env in ["Environment", "CommandLine", "Prompt"], "Unexpected caller: %s" % env

    if env != "Environment":
        ans = True
    elif re.match('^[\w\.-]+$', val):
        ans = True
    else:
        ans = False

    return ans
    

#####################################################################

def proc_envvar(pdl):
    '''
        Apply the environment variables to the PDL
    '''

    debug("proc_envvar(pdl) entry")

    # Verify no unsupported formats

    valid_formats = pdl["FORMATS"][0]
    pcl = pdl["PCL"][0]
    for pc in pcl:
        assert pc[1] in valid_formats.keys(), "Format '%s' for %s is not supported" % (pc[1],pc[0])

    # Walk through the PCL, looking for associated environment variables with values

    # Validate the PCL index
    for pcli in range(len(pcl)):
        nm = pcl[pcli][0]
        assert pcli == pdl[nm][2], "PCL and PDL are not consistent, PCL entry %d has name %s, but that PDL entry is %d" % (pcli, nm, pdl[nm][2])

    header_flag = False
    for pc in pcl:
        nm = pc[0]
        fmt = pc[1]
        ver = valid_formats[fmt][0]
        envs = pc[3]
        pcli = pdl[nm][2]

        # Walk through the environment variables
        # Use the first one that has a value
        for env in envs:
            if env not in os.environ.keys(): continue
            val = os.environ[env]
            if not header_flag:
                header_flag = True
                print "%s" % getmsg("ENVIRONMENT_VARIABLES", "Environment Variables:")
                print
            if env == "ITCAM_PASSWORD":
                print "  %s=%s" % (env, "********")
            else:
                print "  %s=%s" % (env, val)


            # Case statement for format type
            if ver is not None:
                if not ver(nm, "Environment", val):
                    raise ConfigurationException(getmsg("CFG5034E", "Invalid value specified by {0} for {1}: {2}", [env,nm,val]))

            # Set the tag in anticipation, since validation has successfully completed.
            pdl[nm][1] = "Environment"

            # Answer is Yes, No or Leave
            if fmt == "YesNoLeave" or fmt == "YesNo":
                if val == None:
                    pdl[nm][0] = None
                elif val == True:
                    pdl[nm][0] = True
                elif val == False:
                    pdl[nm][0] = False
                elif type(val) == type("str") and val.upper().startswith("Y"):
                    pdl[nm][0] = True
                elif type(val) == type("str") and val.upper().startswith("N"):
                    pdl[nm][0] = False
                elif fmt == "YesNoLeave" and val.upper().startswith("L"):
                    pdl[nm][0] = None
                else:
                    msg = getmsg("CFG5025E", "Unexpected value {0} for environment variable {1} format {2} for {3}", [val, env, fmt, nm])
                    raise ConfigurationException(msg)

            # A ServerList is a blank-separated list of servers
            elif fmt.endswith("List"):
                if val == None:
                    pdl[nm][0] = None
                else:
                    pdl[nm][0] = val.split()

            # Anything else is a simple parameter
            else:
                pdl[nm][0] = val

            break

    if header_flag: print
    debug("proc_envar() exit")


#####################################################################

def proc_cliopt(pdl,opts):
    '''
        Set our parametric data from command line options
    '''

    debug("proc_cliopt(pdl,opts) entry")
    command_debug = " ".join([str(opt) for opt in opts])
    command_debug = re.sub("-password\s+[^\s]+", '-password ********', command_debug)
    debug("command line options = %s" % command_debug)

    # Validate that all formats are supported

    valid_formats = pdl["FORMATS"][0]

    pcl = pdl["PCL"][0]
    for pc in pcl:
        assert pc[1] in valid_formats.keys(), "Format '%s' for %s is not supported" % (pc[1],pc[0])

    # Make a temporary dictionary of indexes that relate
    # the command line flags to the respective PCL entry.
    flgs = {}
    for pcli in range(len(pcl)):
        for flg in pcl[pcli][4].keys():
            assert flg not in flgs.keys(), "Flag %s is duplicated for %s" % (flg, pcl[pcli][1])
            flgs[flg] = pcli

    # Walk the options and set PDL values per the associated format
    # We use the skip flag to skip either the command entry or the
    # value of an option.

    # First skip over the command entry.  (There are situations where
    # it will not be present.)
    if len(opts) > 0 and not opts[0].startswith("-"):
        skip = True
    else:
        skip = False

    # Loop over the command line arguments
    header_flag = False
    for opti in range(len(opts)):
        # Skip if we've already handled this entry
        if skip: skip = False; continue

        # Get the current entry as the option and the
        # next entry as its potential value.
        opt = opts[opti]
        if opti+1 < len(opts):
            val = opts[opti+1]
        else:
            val = None

        # Validate the option
        if opt not in flgs.keys():
            msg = getmsg("CFG5023E", "Command line option {0} is not a known flag", [opt])
            raise ConfigurationException(msg)

        # Get the PDL entry name, the format, the option flags, and the PCL index
        # "ents" is a dictionary of option flags and their meanings.
        nm = pcl[flgs[opt]][0]
        fmt = pcl[flgs[opt]][1]
        ver = valid_formats[fmt][0]
        ents = pcl[flgs[opt]][4]
        ent = ents[opt]
        pcli = pdl[nm][2]

        # "True" or "False" indicate that the meaning is implicit in the flag and
        # no argument follows.  The "YesNoLeave" format has "None" implicit as a value.
        if ent is True or ent is False or (fmt == "YesNoLeave" and ent == None):
            debug("Flag %s implicitly sets %s to '%s'" % (opt,nm,ent))
            pdl[nm][0:2] = ent, "CommandLine"

        # If the flag does not implicitly specify a value such as True or False, we must
        # use the next entry in the command line as the value.
        else:
            skip = True

            # If a verification routine was provided for the format,
            # invoke it now.
            if ver is not None:
                if not ver(nm,"CommandLine",val):
                    raise ConfigurationException(getmsg("CFG5035E", "Invalid value specified by {0} for {1}: {2}", [opt,nm,val]))

            # Check that a value was specified for the option flag, indicate an error if not.
            if val == None:
                msg = getmsg("CFG5024E", "Missing parameter for {0} command line option", [opt])
                raise ConfigurationException(msg)

            # ServerList and StringList entries are additive starting with the first command line entry
            elif fmt.endswith("List"):
                # If this is the first time we have seen this parameter on the
                # command line, start fresh.  In any case, if not already present, append the value and skip.
                
                if pdl[nm][1] != "CommandLine":
                    pdl[nm][0:2] = [], "CommandLine"
                      
                if val not in pdl[nm][0]:       
                    pdl[nm][0].append(val)
                    debug("Flag %s appends to %s the value %s giving %s" % (opt,nm,val,pdl[nm][0]))
                else:
                    debug("Flag %s not appending to %s the value %s, remaining %s" % (opt,nm,val,pdl[nm][0])) 

            # If it's not a list, every time the flag is encountered the value is replaced.
            else:
                if opt == "-password":
                    debug("Flag %s sets %s to %s" % (opt,nm,"********"))
                else:
                    debug("Flag %s sets %s to %s" % (opt,nm,val))
                pdl[nm][0:2] = val, "CommandLine"

        if not header_flag:
            header_flag = True
            print "%s" % getmsg("COMMAND_LINE_FLAGS", "Command Line Flags:")
            print
        if skip:
            if opt == "-password":
                print "  %s %s" % (opt,"********")
            else:
                print "  %s %s" % (opt,val)
        else:
            print "  %s" % opt
            
    if header_flag: print
    debug("proc_cliopt() exit")


def proc_propsdict(pdl,propsdict):
    '''
        Set our parametric data from a properties dictionary
    '''

    debug("proc_propsdict(pdl,propsdict) entry")
    
    # loop over the properties dictionary and set PDL values    
    for k,v in propsdict.items():
        debug("  setting product offering property %s:%s in PDL" % (k,v))        
        if k in pdl.keys():
            pdl[k][0] = v
            pdl[k][1] = 'Initial'
            
    debug("proc_propsdict(pdl,propsdict) exit")

#####################################################################
# Unit Testing
#####################################################################

if __name__ == "__main__":

    import debugUtil as log
    from debugUtil import set_logging
    from resourcebundle import set_resourcebundle

    set_logging("./createPDL.log")
    set_resourcebundle()

    from createPCL_config import make_pcl
    #from createPCL_createcfg import make_pcl

    # Do some validation of the PCL

    pcl = make_pcl()
    print "%d entries in real PCL" % len(pcl)
    pdl = make_pdl(pcl)
    print "%d entries in real PDL" % len(pdl)

    try:
        #proc_envvar(pdl)
        #proc_cliopt(pdl,["command","-silent","option"])
        proc_cliopt(pdl,["command","-server","server1","-server","server2"])
    except ConfigurationException, errmsg:
        print "ConfigurationException: %s" % errmsg

    log.logDumpPCL(pcl)
    log.logDumpPDL(pdl)
    raise SystemExit

    #####

    hit_flag = False
    for pc in pcl:
        if pc[0] == "FORMATS":
            hit_flag = True
            valid_formats = pc[2]
            break
    assert hit_flag, "FORMATS entry not found in PCL"

    nms = {}
    envs = {}
    flgs = {}
    for pcli in range(len(pcl)):
        pc = pcl[pcli]
        nm = pc[0]
        assert nm.upper() == nm, "Name %s, used in PCL[%d', is not upper case" % (nm, pcli)
        if nm in nms.keys(): assert nm not in nms.keys(), "Name %s is used in multiple PCL entries %d and %d" % (nm, nms[nm], pcli)
        nms[nm] = pcli
        fmt = pc[1]
        val = pc[2]
        envpcl = pc[3]
        flgpcl = pc[4]

        # Check the format is valid and that the initialization
        # value is valid for the format
        if fmt == "YesNo":
            assert val in {True:None, False:None}.keys(), "Initializer for %s is '%s', which is invalid for %s format" % (nm, val, fmt)

        elif fmt == "YesNoLeave":
            assert val in {True:"", False:"", None:""}.keys(), "Initializer for %s is '%s', which is invalid for %s format" % (nm, val, fmt)

        elif fmt in valid_formats.keys():
            # todo: Improve checking for some items
            pass

        elif fmt == None or fmt == "":
            assert False, "Missing format for %s" % nm
        else:
            assert False, "Unknown format '%s' for PCL entry for %s" % (fmt, nm)

        # Verify that the environment variables are unique
        for env in envpcl:
            if env in envs.keys(): assert env not in envs.keys(), "Duplicate environment variable %s used by PCL entries %s and %s" % (env, envs[env], nm)
            envs[env] = nm

        # Verify that the command line flags are unique

        for flg in flgpcl.keys():
            assert flg not in flgs.keys() #, "Command line flag %s is used by PCL entries %s and %s" % (flg, flgs[flg], nm)
            flgs[flg] = nm

    for pcli in range(len(pcl)):
        assert len(pcl[pcli]) == 6, "%s has %d items in it's entry" % (pcl[pcli][0], len(pcl[pcli]))

    testpcl = (
                ("0001", "ServerList", ["A","B","C"],  ("ENVVAR0001",), {"-flag0001":None}, "MSG0000I"),
                ("0002", "YesNo",      True,           ("ENVVAR0002",), {"-flag0002":True}, "MSG0000I"),
                ("0003", "YesNo",      False,          ("ENVVAR0003",), {"-flag0003":False}, "MSG0000I"),
                ("0004", "YesNoLeave", None,           ("ENVVAR0004",), {"-flag0004":None}, "MSG0000I"),
                ("0005", "Path",       "C:\\bin",      ("ENVVAR0005",), {"-flag0005":None}, "MSG0000I"),
                ("0006", "File",       "C:\\test.dat", ("ENVVAR0006",), {"-flag0006":None}, "MSG0000I"),
                ("0007", "FileExists", "C:\\test.dat", ("ENVVAR0007",), {"-flag0007":None}, "MSG0000I"),
                ("0008", "StringList", ["A","B","C"],  ("ENVVAR0008",), {"-flag0008":None}, "MSG0000I"),
                ("0009", "Hostname",   "ibm.com",      ("ENVVAR0009",), {"-flag0009":None}, "MSG0000I"),
                ("0010", "IPAddress",  "127.0.0.1",    ("ENVVAR0010",), {"-flag0010":None}, "MSG0000I"),
                ("0011", "Port",       "8890",         ("ENVVAR0011",), {"-flag0011":None}, "MSG0000I"),
                ("0012", "PortRange",  "8890-8900",    ("ENVVAR0012",), {"-flag0012":None}, "MSG0000I"),
                ("0013", "Directory",  "C:\\bin",      ("ENVVAR0013",), {"-flag0013":None}, "MSG0000I"),
                ("0014", "String",     "A string",     ("ENVVAR0014",), {"-flag0014":None}, "MSG0000I"),
                ("0015", "Password",   "nodisplay",    ("ENVVAR0015",), {"-flag0015":None}, "MSG0000I"),
                ("0016", "Discover",   None,           ("ENVVAR0016",), {"-flag0016":None}, "MSG0000I"),

                # This is used to validate formats
                ("FORMATS",          "Data",        valid_formats,               (),                          {}),
              )

    testpdl = make_pdl(testpcl)
    log.logDumpPCL(testpcl)
    log.logDumpPDL(testpdl)

    testdict = {}
    for testpc in testpcl:
        testfmt = testpc[1]
        if testfmt not in testdict.keys():
            testdict[testfmt] = None
    msgdict = {}
    testmsg = ""
    for realpc in pcl:
        realfmt = realpc[1]
        if realfmt not in testdict.keys():
            if realfmt not in msgdict:
                msgdict[realfmt] = None
                if testmsg != "":
                    testmsg = testmsg+", "
                testmsg = testmsg+realfmt
    assert testmsg == "", "These formats are not represented in the test exercise PCL: %s" % testmsg

    for nm in sorted(testpdl):
        testpd = testpdl[nm]
        print "\n%s" % nm
        pcli = testpd[2]
        if nm == "PCL" and pcli == None: continue
        if nm == "FORMATS": continue
        print "  pcli = '%s' for %s" % (pcli,nm)
        assert nm == testpcl[pcli][0], "Name %s for PCL entry %d does not match %s" % (testpcl[pcli][0], pcli, nm)
        #nm = testpcl[pcli][0]
        fmt = testpcl[pcli][1]
        init = testpcl[pcli][2]
        envs = testpcl[pcli][3]
        flgs = testpcl[pcli][4]
        print "  fmt=%s init=%s envs=%s flgs=%s" % (fmt,init,envs,flgs)

        for env in envs:
            testpd[0] = "[unset]"
            testpd[1] = "Unset"
            if fmt in {"YesNo":None, "YesNoLeave":None}.keys():
                envval = {True:"Yes", False:"No", None:"Leave"}[init]
            elif fmt == "ServerList" or fmt == "StringList":
                envval = " ".join(init)
            else:
                envval = init
            print "  Setting envvar %s to %s (%s)" % (env, envval, init)
            os.environ[env] = envval
            print "  envvar %s set to '%s'" % (env, os.environ[env])
            proc_envvar(testpdl)
            assert testpdl[nm][1] == "Environment", "proc_envvar() %s tag is %s instead of %s" % (nm, testpdl[nm][1], "Environment")
            assert testpdl[nm][0] == init, "proc_envvar() set %s to '%s' instead of '%s'" % (nm, testpdl[nm][0], init)
            del os.environ[env]

        for flg, flgval in flgs.items():
            testpd[0] = "[unset]"
            testpd[1] = "Unset"
            initval = init
            if fmt in {"YesNo":None, "YesNoLeave":None}.keys():
                flgval = {True:"Yes", False:"No", None:"Leave"}[init]
                print "  Setting cliopt %s as %s format %s" % (flg,flgval,fmt)
                proc_cliopt(testpdl,("command.py",flg))
            elif fmt == "ServerList" or fmt == "StringList":
                testopts = ["command.py"]
                for flgvali in init:
                    testopts.append(flg)
                    testopts.append(flgvali)
                proc_cliopt(testpdl,testopts)
            else:
                if initval is None: initval = "Dummy Value"
                print "  Setting cliopt '%s %s' format=%s" % (flg, flgval, fmt)
                proc_cliopt(testpdl,("command.py",flg,initval))
            assert testpdl[nm][1] == "CommandLine", "proc_cliopt() tag for %s is %s instead of %s" % (nm, testpdl[nm][1], "CommandLine")
            assert testpdl[nm][0] == initval, "proc_cliopt() set %s to '%s' instead of %s/%s" % (nm, testpdl[nm][0], flgval, initval)
