#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module creates the SERVERS pdl.  First, the cli option is checked and
   if it does exist, then no need to prompt the user to choose the servers, otherwise
   a list of all the servers that was found is displayed for selection.
'''

fixlev = "40544" # Last APAR, defect or story to update this module

import os
import utils
import debugUtil as log
import doprompt
import platformUtil

from types import ListType, DictType

from debugUtil import logDebug as debug
from ConfigurationException import ConfigurationException
from resourcebundle import getmsg

SERVERS = "SERVERS"
FULLSERVERS = "FULLSERVERS"
FIND_SERVERS = "FINDSERVERS"
WAS_VERSION = "WAS_VERSION"
WAS_SERVER = "WASSERVER"
WAS_NODE = "WASNODE"
WAS_CELL = "WASCELL"
WAS_PROFILE = "PROFILE"
SERVER_ID = "SERVER_ID"
GCPARMS = "GCPARMS"
GCLOGPATH = "GCLOGPATH"
CONFIGHOME = "CONFIGHOME"
AM_HOME = "AM_HOME"
RUNTIMEP = "RUNTIMEP"
RUNTIME = "RUNTIME"
AGENTLIB = "AGENTLIB"
GC_PARMS = "GC_PARMS"
DEFAULTDCVERSION = "VERSION"


def read_findservers_and_prompt(pdl):
    '''
       Prompt the user for the list of available servers, and once a server
       or servers are selected, it creates the SERVERS pdl.
       Prompt routine puts the selected servers in FINDSERVERS,
       so, the original FINDSERVERS is deleted.
    '''

    debug("Entry read_findservers_and_prompt %s" % fixlev)

    fservers = pdl.get(FIND_SERVERS)
    if fservers is None:
        log.logError("No FINDSERVERS pdl defined")
        raise Exception("No FINDSERVERS pdl defined")

    if fservers[0] is None:
        debug("FINDSERVERS pdl is defined but no value")
        raise Exception("FINDSERVERS pdl is defined but no value")

    # Before we prompt for the servers we must convert the ALIASLIST to a dictionary so
    # that later we can associate the aliases with the correct servers in transferdata_to_pdl.
    # But in silent mode the ALIASLIST might aready be a list, in which case we leave it alone.

    if type(pdl["ALIASLIST"][0]) is ListType and type(pdl["WASSERVER"][0]) is ListType:
        debug("Converting ALIASLIST to dictionary")
        # We have a list of aliases that corresponds to the list of servers.
        # A dictionary is needed so we can put it back together.
        aliasDict = {}
        for serveri in xrange(len(pdl["SERVERS"][0])):
            wasCell = pdl["WASCELL"][0][serveri]
            wasNode = pdl["WASNODE"][0][serveri]
            wasServer = pdl["WASSERVER"][0][serveri]
            aliasKey = (wasCell, wasNode, wasServer)
            debug("  Alias dictionary entry %s : \"%s\"" % (pdl["ALIASLIST"][0][serveri], aliasKey))
            aliasDict[aliasKey] = pdl["ALIASLIST"][0][serveri]
        pdl["ALIASLIST"][0:2] = aliasDict, "createservers"

    doprompt.do_prompt("SERVERS", pdl)

    if not pdl["BATCH"][0]:
        check_findservers(pdl)
    else:
        # An application server can be one of the following:
        # 1. pristine;
        # 2. configured by 7.2;
        # 3. configured by an older ITCAM;
        # 4. installed by MSVE
        # For interactive mode, the checking is done before prompting in doprompt.py.
        # For batch/silent mode, we need to perform the check here.
        check_findservers_forbatch(pdl)

    debug("Exit read_findservers_and_prompt")


def check_findservers(pdl):
    '''
       Each entry in the server list is checked against the FINDSERVERS
       pdl entries.  If any are not found, we have an error, because
       this is only called in batch mode.

       @param pdl - PDL reference
       @param serverlist - A list of all the selected servers
    '''

    debug("Entry check_findservers %s" % fixlev)

    serverlist = pdl.get(FULLSERVERS)[0]
    findservers = pdl.get(FIND_SERVERS)[0]
    assert findservers != None, "%s entry not found in the PDL" % FIND_SERVERS

    # Each entry in the FINDSERVERS is a dictionary.

    index = 0
    findservers_indexlist = []

    # check each entry in the serverlist against the entries in
    # FINDSERVERS, and if there is match, remember the index in
    # FINDSERVERS for later use.

    for lentry in serverlist:
        serverfound = False
        for index in range(len(findservers)):
            sentry = findservers[index]

            # For each entry in the server list, check it against
            # this FINDSERVER entry and if the server profile, cell
            # node, and name matches then make a note of it.

            if (lentry[WAS_PROFILE].strip() == sentry[WAS_PROFILE].strip() and
                lentry[WAS_CELL].strip() == sentry[WAS_CELL].strip() and
                lentry[WAS_NODE].strip() == sentry[WAS_NODE].strip() and
                lentry[WAS_SERVER].strip() == sentry[WAS_SERVER].strip()):
                debug("FINDSERVERS matched")
                findservers_indexlist.append(index)
                serverfound = True
                debug("Server entry found in FINDSERVERS- " + lentry[WAS_PROFILE].strip() +":"+ lentry[WAS_CELL].strip()+":"+lentry[WAS_NODE].strip()+":"+lentry[WAS_SERVER].strip())
                break
        if not serverfound:
            messg = getmsg("CFG5047E", "Invalid Profile:{0} , Cellname:{1} , nodename:{2} , servername:{3}  combination in the silent input file",[lentry[WAS_PROFILE].strip(),lentry[WAS_CELL].strip(),lentry[WAS_NODE].strip(),lentry[WAS_SERVER].strip()])
            raise ConfigurationException(messg)

    # Transfer the server data to PDL

    transferdata_to_pdl(pdl, findservers_indexlist)

    debug("Exit check_findservers")


def check_findservers_forbatch(pdl):
    '''
       Each entry in the server list is checked against the FINDSERVERS
       pdl entries.  If any are not found, we have an error, because
       this is only called in batch mode.

       @param pdl - PDL reference
       @param serverlist - A list of all the selected servers
    '''

    debug("Entry check_findservers_forbatch %s" % fixlev)

    serverlist = pdl.get(FULLSERVERS)[0]
    findservers = pdl.get(FIND_SERVERS)[0]
    defaultDCVersion = pdl.get(DEFAULTDCVERSION)[0]
    debug("defaultDCVersion: " + str(defaultDCVersion))
    debug("serverlist from FULLSERVERS: %s" % str(serverlist))
    assert findservers != None, "%s entry not found in the PDL" % FIND_SERVERS
    if len(serverlist) == 0:
        msg = getmsg("CFG1149E","No valid server name has been specified.")
        raise ConfigurationException(msg)

    # Each entry in the FINDSERVERS is a dictionary.

    index = 0
    findservers_indexlist = []

    msServerList = ""
    olderVersionServerList = ""

    # check each entry in the serverlist against the entries in
    # FINDSERVERS, and if there is match, remember the index in
    # FINDSERVERS for later use.

    serverfound = False
    for lentry in serverlist:
        debug("serverlist entry: " + str(lentry))
        for index in range(len(findservers)):
            sentry = findservers[index]
            debug("  find servers sentry: " + str(sentry))

            # For each entry in the server list, check it against
            # this FINDSERVER entry and if the server name matches
            # then make a note of it.
            # A dcversion of None is allowed and considered "not configured"
            if (lentry[WAS_PROFILE].strip() == sentry[WAS_PROFILE].strip() and
                lentry[WAS_CELL].strip() == sentry[WAS_CELL].strip() and
                lentry[WAS_NODE].strip() == sentry[WAS_NODE].strip() and
                lentry[WAS_SERVER].strip() == sentry[WAS_SERVER].strip()):
                debug("FINDSERVERS matched")
                # check if configuration can be applied to the application server
                dcversion = sentry.get("CONFIGURED")
                appServerName = sentry.get("WASCELL") + "." + sentry.get("WASNODE") + "." + sentry.get("WASSERVER")
                debug("The application server: " + appServerName + " has dcversion: " + dcversion + ".")
                if dcversion != None:
                    dcversion = dcversion.strip()
                if dcversion == "VE":
                    if msServerList != "":
                        msServerList = msServerList + ", "
                    msServerList = msServerList + appServerName
                elif dcversion is not None and dcversion != "None" and dcversion != defaultDCVersion and dcversion != "DC":
                    if olderVersionServerList != "":
                        olderVersionServerList = olderVersionServerList + ", "
                    olderVersionServerList = olderVersionServerList + appServerName
                else:
                    findservers_indexlist.append(index)
                    serverfound = True
                    debug("Server entry found in FINDSERVERS- " + str(lentry))
                    break

    if len(msServerList) > 0:
        debug("Application servers configured by ITCAM Managing Server: %s" % msServerList)
        print "%s" % getmsg("CFG1077I", "There are application servers configured by ITCAM Managing Server. Check log file to find these servers")

    if len(olderVersionServerList) > 0:
        debug("Application servers configured by older version of ITCAM Data Collector: %s" % olderVersionServerList)
        print "%s" % getmsg("CFG1078I", "There are application servers configured by older version of ITCAM Data Collector. Check log file to find these servers")

    assert serverfound, "No application servers can be configured with current ITCAM."

    # Transfer the server data to PDL

    transferdata_to_pdl(pdl, findservers_indexlist)

    debug("Exit check_findservers_forbatch")


def transferdata_to_pdl(pdl, findservers_indexlist):
    '''
       Transfers data from FINDSERVERS to PDL

       @param pdl - PDL reference
       @param findservers_indexlist - Index list into FINDSERVERS pdl for selected servers

       Assumption obviously is that USERINSTALLROOT, WPSHOME and WBIHOME are the same
       for every server in the profile. The initial PDL values for these were loaded in populateFindServers.
    '''

    tag = "crtServers"

    debug("Entry transferdata_to_pdl %s" % fixlev)

    if type(pdl["USERINSTALLROOT"][0]) is ListType:
        debug("This is a re-entry of transferdata_to_pdl")

    # Transfer the data from FINDSERVERS to PDL

    fentries = (
                "JVMMODE",
                "SERVERID",
                "SHARECLASSESMODIFIED",
                "STANDALONE",
                "WASCELL",
                "WASNODE",
                "WASSERVER",
                "WASVERSION",
                "CURRENTMAXHEAPSIZE",
                "CURRENTGCLOGLOCATION",
                "WASINSTALLROOT",
                "CONFIGURED",
                "PROFILE",
                "PROFILEHOME",
                "JVMVERSION",
                "MIGRATEDDCHOME",
                "MIGRATEDVERSION",
                "DCVERSION",
                "SUBPRODUCT",
                "CONFIGRETURNCODE",
               )
    for fentry in fentries:
        utils.setData(pdl,fentry,[],tag)
        for serveri in findservers_indexlist:
            utils.getData(pdl,fentry).append(utils.getData(pdl,"FINDSERVERS")[serveri][fentry])

    # If these variables are not lists, create an empty list. They have the same value for every server
    #    USERINSTALLROOT, WPSHOME, WBIHOME, WLPHOME
    # Remember this logic can be redriven during config process so they may be lists which need to be rebuilt.

    runhome = os.path.normpath(os.path.join(utils.getData(pdl,"CONFIGHOME"), "runtime"))

    userinstallroot = utils.getData(pdl,"USERINSTALLROOT")
    wpshome = utils.getData(pdl,"WPSHOME")
    wbihome = utils.getData(pdl,"WBIHOME")
    wlphome = utils.getData(pdl,"WLPHOME")

    # If this is not the first time we are called, these thing will
    # be lists instead of simple strings.  Fortunately they all have the same value.
    if type(userinstallroot) is ListType: userinstallroot = userinstallroot[0]
    if type(wpshome) is ListType: wpshome = wpshome[0]
    if type(wbihome) is ListType: wbihome = wbihome[0]
    if type(wlphome) is ListType: wlphome = wlphome[0]

    utils.setData(pdl,"RUNLOC",[],tag)
    utils.setData(pdl,"RUNTIME",[],tag)
    utils.setData(pdl,"RUNTIMEP",[],tag)
    utils.setData(pdl,"AMPLATFORM",[],tag)
    utils.setData(pdl,"USERINSTALLROOT",[],tag)
    utils.setData(pdl,"WPSHOME",[],tag)
    utils.setData(pdl,"WBIHOME",[],tag)
    utils.setData(pdl,"WLPHOME",[],tag)

    # If the alias list got converted to a dictionary, this is where
    # we must covert it back to a list.
    aliasDict = pdl["ALIASLIST"][0]
    if type(aliasDict) is DictType:
        debug("  Converting ALIASLIST back from dictionary to list")
        pdl["ALIASLIST"][0:2] = [], "createservers.transferdata"

    for serveri in range(len(utils.getData(pdl,"SERVERS"))):
        amplatform = "was" + str(utils.getData(pdl,"WASVERSION")[serveri])
        wascell = utils.getData(pdl, "WASCELL")[serveri]
        wasnode = utils.getData(pdl,"WASNODE")[serveri]
        wasserver = utils.getData(pdl,"SERVERS")[serveri]
        debug("Transfering data for server %s" % wasserver)
        runloc = amplatform + "." + wasnode + "." + wasserver
        runtime = os.path.join(runhome, runloc)
        # Don't use join here; it's not a real path until after substitution
        runtimep = "${ITCAMDCHOME}" + os.sep + "runtime" + os.sep + runloc
        utils.getData(pdl,"RUNLOC").append(runloc)
        utils.getData(pdl,"RUNTIME").append(runtime)
        utils.getData(pdl,"RUNTIMEP").append(runtimep)
        utils.getData(pdl,"AMPLATFORM").append(amplatform)

        utils.getData(pdl,"USERINSTALLROOT").append(userinstallroot)
        utils.getData(pdl,"WPSHOME").append(wpshome)
        utils.getData(pdl,"WBIHOME").append(wbihome)
        utils.getData(pdl,"WLPHOME").append(wlphome)

        # Fill in the ALIASLIST
        if type(aliasDict) is DictType:
            aliasKey = (wascell, wasnode, wasserver)
            if aliasKey in aliasDict.keys():
                debug("  Restoring existing alias - %s : %s" % (aliasDict[aliasKey], aliasKey))
                pdl["ALIASLIST"][0].append(aliasDict[aliasKey])
            else:
                if platformUtil.isZ():
                    default_serveralias = wasserver + "_default"
                else:
                    default_serveralias = wasnode + wasserver
                debug("  Setting default alias - %s : %s" % (default_serveralias, aliasKey))
                pdl["ALIASLIST"][0].append(default_serveralias)

    debug("Exit transferdata_to_pdl")


def process_servers(pdl):
    '''
       Creates SERVERS PDL, based on a cli option or prompting user to select
       a server or servers.

       @param pdl - PDL reference
    '''

    debug("Entry process_servers %s" % fixlev)

    #
    # Check the SERVERS PDL, and check if the server list is already
    # defined on the command line.  Whether the SERVERS is defined or
    # not defined, if there are no previously defined servers, the user
    # is prompted to provide the server list.
    #
    # doprompt has ability to differentiate interactive/batch/silent modes.

    debug("Enter process_servers")
    read_findservers_and_prompt(pdl)

    debug("Exit process_servers")
