#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    This module implements logging functions.  It is suggested that
    it be imported as "log" and the logDebug function be imported
    as "debug".

    This module is used by configDataCollector and unconfigDataCollector,
    so it must be compatible with Jython 2.1 as well as Jython 2.5.2.

    However, the logDumpPCL() and logDumpPDL() routines are never used
    under wsadmin and they may safely employ Jython 2.5.2 features.
'''

fixlev = "38001" # Last APAR, defect or story to update this module

import os
import sys
import traceback
import types
import codecs

from time import strftime

# Note: Because this operates under wsadmin and Jython 2.1 as well as independently
#       under Jython 2.5.2, it should not import any local modules execpt platformUtils.

# Jython 2.1 compatibility
_Jython_vrm = os.sys.version_info[0:3]
assert _Jython_vrm == (2,1,0) or _Jython_vrm >= (2,5,2), "Unsupported Jython version"
if _Jython_vrm == (2,1,0):
    True = 1
    False = 0
else:
    # There is no inspect module in Jython 2.1
    from inspect import stack
    from inspect import getframeinfo

# Define globals

# These are used to determine entries to exclude from traceback
_LOCALMODULENAME = "debugUtil"
_LOCALROUTINES = ("_getCaller", "log", "debug", "logDebug", "logWarning", "logError", "logException", "logDumpStack", "logDumpPCL", "logDumpPDL")

# Log and message files
_logFilename = None
_logfile = None
_linecache = []
_msgfile = None
_msgcache = []


#####################################################################

def set_logging(log_filename=None, msg_filename=None, config_home=None):
    '''
       Opens the trace and message log files.
       There will be buffered log or message records, which will be
       placed into the files once they are opened.

       If called with no arguments, logging will be done to console and
       messaging will be suppressed.
    '''

    global _logFilename, _logfile, _linecache, _msgfile, _msgcache

    debug("Entry set_logging(%s,%s,%s) %s" % (log_filename, msg_filename, config_home, fixlev))

    # Ignore attempts to set the log if it's already been done

    if _logfile is not None:
        debug("Log file previously set, returning after doing nothing")
        return

    # If no log argument is provided, the log entries will be
    # transferred to the console, otherwise, based on the file
    # name provided, the log file is created.  If there is an
    # absolute path then it will be used,, otherwise, the log
    # file will be created in the data dir.

    if log_filename is None:
        debug("Logging to console")
        _logfile = True
        _logFilename = "[console]"
    elif log_filename is False:
        debug("Logging is suppressed")
        _logfile = False
        _logFilename = "[suppressed]"
    else:

        # We make the assumptions that we want to put the log file into
        # [CONFIGHOME]/data/[directory]/[filename]
        # Assumptions about which directories exist are checked until we
        # arrive at some existing location for the new log file.

        debug("Logging requested to file %s" % os.path.basename(log_filename))

        # Break the specified file into directory and base file name parts
        logdirname, logbasename = os.path.split(log_filename)
        debug("Directory for log: '%s', base filename: '%s'" % (logdirname,logbasename))

        candidateList = []

        # If [CONFIGHOME] was specified and exists, put log in there, otherwise use
        # the parent of the current working directory.  Develop a list of possiblities.

        if config_home is not None:
            candidateList.append(os.path.join(config_home,"data",logdirname))
            candidateList.append(os.path.join(config_home,"data"))
            candidateList.append(os.path.join(config_home,logdirname))
            candidateList.append(os.path.join(config_home))

        # No os.path.pardir in Jython 2.1
        if (2,1) != os.sys.version_info[0:2]:
            pardir = os.path.pardir
        else:
            pardir = ".."
        candidateList.append(os.path.join(pardir,"data",logdirname))
        candidateList.append(os.path.join(pardir,"data"))
        candidateList.append(os.path.join(pardir,logdirname))
        candidateList.append(os.path.join(pardir))

        # As a last resort, use a temporary directory
        if "TEMP" in os.environ.keys():
            candidateList.append(os.environ["TEMP"])
        if "TMP" in os.environ.keys():
            candidateList.append(os.environ["TMP"])

        debug("Candidate directories:\n  %s" % "\n  ".join(candidateList))

        # Try really, really hard to find an existing place to put the log
        location = None
        for candidate in candidateList:
            if not os.path.isdir(candidate):
                debug("Path does not exist: %s" % candidate)
                continue
            if (2,1) != os.sys.version_info[0:2]:
                if not os.access(candidate,os.W_OK):
                    debug("Path is not writable: %s" % candidate)
                    continue
                debug("Path exists: %s" % candidate)
            else:
                debug("Path exists and is writable: %s" % candidate)
            location = candidate
            break

        # If we didn't find anything, set the first candidate, take the hit on open, and
        # accept logging to the console.
        if location is None:
            location = candidateList[0]
            debug("Unable to find a suitable location for log file, using %s" % location)

        _logFilename = os.path.normpath(os.path.abspath(os.path.join(location,logbasename)))
        debug("logFilename = %s = %s" % (log_filename,_logFilename))

        # Get rid of any existing file
        _rename(_logFilename)

        #
        # Open the log file.  If there is a problem, revert to the console
        #
        log_dirName = os.path.dirname(_logFilename)
        if os.path.exists(log_dirName):

            try:
                _logfile = codecs.open(_logFilename, mode="a", encoding="utf-8")
            except Exception, errmsg:
                _logfile = False
                templines = _linecache
                _linecache = []
                for line in templines:
                    _logprint(line)
                logException("Error while initializing log object: %s" % errmsg)
                debug("Reraising exception")
                raise
        else:
            logWarning("Directory path %s does not exist, logging to console" % log_dirName)
            _logfile = False

        # Note if there was a failure, we can't issue a globalized message since debugUtil
        # can not depend on resourcebundle.

    # Dump any buffered lines

    templines = _linecache
    _linecache = []
    for line in templines:
        _logprint(line)

    #####################################################################
    # If message file was not provided, we will suppress messaging.
    # Otherwise, this is similar to the trace log.
    #####################################################################

    if msg_filename is None or msg_filename is False:
        debug("Message logging is suppressed")
        _msgfilename = False

    else:
        debug("Message file %s" % os.path.basename(msg_filename))

        # Program around a bug in Jython 2.1, where os.altsep is None in Windows
        altsep = os.altsep
        if altsep is None and os.sep == "\\":
            altsep = "/"

        # Default directory to ./data
        if msg_filename.find(os.sep) == -1 and (altsep == None or msg_filename.find(altsep) == -1):
            debug("Message file does not specify directory, defaulting to data")
            if config_home == None: config_home = os.pardir
            msg_filename = os.path.normpath(os.path.join(config_home, "data", msg_filename))
        _rename(msg_filename)

        #
        # Open the message file
        #
        msg_dirName = os.path.dirname(msg_filename)
        if os.path.exists(msg_dirName):

            try:
                _msgfile = codecs.open(msg_filename, mode="a", encoding="utf-8")
            except Exception, errmsg:
                _msgfile = None
                logException("Error while initializing message object: %s" % errmsg)
                debug("Reraising exception")
                raise
        else:
            logWarning("Directory path %s does not exist, messaging suppressed" % msg_dirName)
            _msgfile = None

    # Dump the line buffer (this might do nothing if the message log is suppressed.)

    templines = _msgcache
    _msgcache = []
    for line in templines:
        _msgprint(line)

    debug("set_logging() exit")


#####################################################################

def getLogFilename():
    '''
        Return the log file name
    '''

    return _logFilename

#####################################################################

def _rename(filename):
    '''
        Rename a (possibly) file for archival purposes
    '''

    # This code is duplicated in util.rename_archival_file to avoid
    # having to make that module compatible with Jython 2.1.
    # Changes made there should be propagated.

    filename = os.path.abspath(filename)
    dirName = os.path.dirname(filename)

    if os.path.exists(filename):
        debug("File %s already exists and must be renamed" % filename)

        #
        # create a string of local time to be inserted in the
        # renamed log file if exists.
        #
        ltime = strftime("%Y%m%d%H%M%S")

        #
        # Get the file name and insert the current time
        # in the file name for rename.
        #

        basefilename = os.path.basename(filename)

        # This is for Jython 2.1 compatibilty, where splitext works differently.
        if basefilename.find(".") == 0 and basefilename[1:].find(".") == -1:
            filebase, fileext = (basefilename,"")
        else:
            filebase, fileext = os.path.splitext(basefilename)
        newfilename = filebase + "." + ltime + fileext
        debug("Existing file will be renamed to %s" % newfilename)

        #
        # Rename the current log file to the new name.
        #
        rename_filename = os.path.normpath(os.path.join(dirName, newfilename))
        try:
            os.rename(filename, rename_filename)
            debug("Rename was successful")
        except OSError, errarg:
            # Note: Can't use getmsg in debugUtil
            logWarning("Could not rename log %s - %s" % (filename, errarg))
    else:
        debug("No previous existing file %s, rename not needed" % filename)


#####################################################################

def _getCaller():
    '''
        Get the caller's name and line number
    '''

    # Get the stack of frames

    #frametuples = _getStack()

    # Find first significant frame

    basename = "<unknown>"
    methodname = "<unknown>"
    lineno = 0

    frame = sys._getframe()
    #for frametuple in frametuples:
    while frame:
        #frame, filename, lineno, methodname, code_context, index = frametuple
        filename = frame.f_code.co_filename or "<unknown>"
        basename = os.path.basename(str(filename))
        rootname, ext = os.path.splitext(basename)
        if ext == ".py":
            basename = rootname
        methodname = frame.f_code.co_name or "<unknown>"
        lineno = frame.f_lineno
        if basename != _LOCALMODULENAME or methodname not in _LOCALROUTINES:
            break
        frame = frame.f_back

    # Special case for wsadmin version of Jython 2.1
    if methodname == "?":
        methodname = "<module>"

    return [basename, methodname, lineno]


#####################################################################

def _logprint(logline=""):
    '''
        Write raw log line into the log file
    '''

    # If logging is suppressed, do nothing
    if _logfile is False: return

    # Make sure what we are logging is a string
    if type(logline) is not types.StringType and type(logline) is not types.UnicodeType:
        logline = str(logline)

    # Either cache the line or write it and all cached lines to the log file
    # If we haven't yet opened the log file, cache the line.
    # Or if the log file is directed to the console, just print it.
    # But most often we write it to the log file.

    if _logfile == None:
        _linecache.append(logline)
    elif _logfile is True or _logfile is False:
        try:
            print unicode(logline, None, "replace")
        except UnicodeEncodeError, errmsg:
            print "<line suppressed due to unicode encode error %s>" % errmsg
    else:
        _logfile.write(logline + "\n")
        _logfile.flush()


#####################################################################

def _msgprint(message=""):
    '''
        Write raw message into the message file
    '''

    # If message logging is suppressed, do nothing
    if _msgfile is False: return

    # Make sure what we are messaging is a string
    if type(message) is not types.StringType and type(message) is not types.UnicodeType:
        message = str(message)

    # Either cache the line or write it and all cached lines to the message file
    # If we haven't yet opened the message file, cache the line.
    # But most often we write it to the message file.

    if _msgfile == None:
        _msgcache.append(message)
    else:
        _msgfile.write("%s\n" % unicode(message, None, "replace"))
        _msgfile.flush()


#####################################################################

def isLogBad():
    '''
        Externalize indication that the log file has not been successfully created.
    '''

    debug("Entry isLogBad() %s called, _logfile is %s %s" % (fixlev, _logfile, type(_logfile)))
    return _logfile is False


#####################################################################

def log(logline, logtype="DEBUG"):

    timestamp = strftime("%Y-%m-%d %H:%M:%S")
    basename, methodname, lineno = _getCaller()
    msgheader = "[%s %s %s (%s:%d)]" % (timestamp, logtype, methodname, basename, lineno)
    _logprint ("\n %s  %s " % (msgheader, logline))


#####################################################################

def messagePrint(message, explanation=None, operatorResponse=None):
    '''
        Print to the message log
    '''

    _msgprint("\n%s" % message)
    if explanation is not None:
        _msgprint("\n  %s" % explanation)
    if operatorResponse is not None:
        _msgprint("\n  %s" % operatorResponse)


#####################################################################

def debug(logline):
    log(logline, "DEBUG")


#####################################################################

def logDebug(logline):
    log(logline, "DEBUG")


#####################################################################

def logWarning(logline):
    log(logline, "WARNING")


#####################################################################

def logError(logline):
    log(logline, "ERROR")


#####################################################################

def logException(logline="Traceback follows"):
    '''
       Traceback into the log
       It requires an active exception
    '''

    log(logline, "EXCEPTION")
    exc_type, exc_value, exc_traceback = sys.exc_info()

    tracelines = traceback.format_exception(exc_type, exc_value, exc_traceback)
    if tracelines == ["None\n"]:
        _logprint("\nNo active exception")
    else:
        s = "\n"+"".join(tracelines)
        if s[-1] == "\n":
            s = s[:-1]
        _logprint(s)

#####################################################################

def format_traceback(exception_only="false"):
    '''
        format_traceback suitable for display
        @return formatted traceback
    '''
    logDebug("format_traceback - entry")
    reptlines = []
    report = ""
    tracelines = ""
    try:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        logDebug("exc_type: %s,  exc_value: %s" % (str(exc_type), str(exc_value)))
        if exception_only == "true":
            tracelines = traceback.format_exception_only(exc_type, exc_value)
        else:
            tracelines = traceback.format_exception(exc_type, exc_value, exc_traceback)
        logDebug(" tracelines: %s" % str(tracelines))

        for traceline in tracelines:
            if traceline.startswith("Traceback"):
                # Keep existing  "\n"
                reptlines.append(traceline)
                continue
            ix = traceline.find(":")
            if ix == -1:
                if traceline[-1] != "\n":
                    traceline = traceline + "\n"
                reptlines.append(traceline)
            else:
                slist = traceline.split(":")
                len_slist = len(slist)
                i = 0
                last = ""
                for s in slist:
                    s = s.strip()
                    i += 1
                    # Do not print duplicate lines
                    if s != last:
                        last = s
                        if i == len_slist:
                            s = s + "\n"
                        else:
                            s = s + ":\n"
                        reptlines.append(s)

        report = "".join(reptlines)
        report = report.rstrip()
    except:
        # Leave report as is in case there is useful information
        logException("Error occurred while formatting traceback")

    logDebug(" formatted traceback: \n%s" % report)
    logDebug("format_traceback - exit")
    return report

#####################################################################

def logDumpStack(logline="Stack Dump:"):
    '''
       Dump stack frames and local variables starting from the oldest exception
    '''

    debug(logline)

    # Force an exception to get a stack frame (same as in _getCaller)

    tb = sys.exc_info()[-1]
    if tb == None:
        try:
            1/0
        except:
            tb = sys.exc_info()[-1]

    # Find the current stack frame

    # Walk back to the top of the trackback chain
    while tb.tb_next:
        tb = tb.tb_next

    # Get all frames into stack
    stack = []
    f = tb.tb_frame
    while f:
        stack.append(f)
        f = f.f_back

    for frame in stack:
        basename = os.path.basename(str(frame.f_code.co_filename)) or "<unknown>"
        if len(basename) > 3 and basename[-3:] == ".py":
            basename = basename[:-3]
        methodname = frame.f_code.co_name or "<unknown>"
        lineno = frame.f_lineno
        if basename == _LOCALMODULENAME and methodname in _LOCALROUTINES:
            continue

        _logprint("\nFrame %s in %s at line %s" % (frame.f_code.co_name, frame.f_code.co_filename, frame.f_lineno))
        #for key, value in sorted(frame.f_locals.items()):
        keypairs = frame.f_locals.keys()
        for keyi in range(len(keypairs)):
            key = keypairs[keyi]
            keypairs[keyi] = [key.lower(), key]
        keypairs.sort()
        for keypair in keypairs:
            key = keypair[1]
            if key == "pdl":
                continue
            value = frame.f_locals[key]
            try:
                _logprint("  %s = '%s' %s" % (key,value,type(value)))
            except Exception, errmsg:
                _logprint("  %s = <ERROR ATTEMPTING TO PRINT VALUE> %s" % (key,errmsg))


#####################################################################

def logDumpPCL(pcl,msg=None):
    '''
        Dump the contents of the PCL
    '''

    # logDumpPCL won't be called under wsadmin because no PCL is used under wsadmin scripts
    assert os.sys.version_info[0] >= 2 and (os.sys.version_info[0] > 2 or os.sys.version_info[1] > 1), "logDumpPCL not valid for Jython 2.1"

    if msg != None and msg != "":
        debug(msg)
    else:
        debug("Contents of PCL follow:")
    _logprint()

    nms = {}
    for i in range(len(pcl)):
        nms[pcl[i][0]] = i

    header_flag = False
    header_line = "       Name            Initial      Format       Env Vars       Flags"

    blanksa = "                                   "
    blanksb = "               "
    blanksc = "                                                "
    for nm in sorted(nms):
        i = nms[nm]
        fmt = pcl[i][1]
        if fmt != "Data":
            init = pcl[i][2]
        else:
            init = "[data]"
        envars = pcl[i][3]
        flags = pcl[i][4]
        printline = "  %4d %-15s %-12s" % (i, nm,init)
        if len(printline) > 35:
            if not header_flag: header_flag = True; _logprint(header_line)
            _logprint(printline)
            printline = blanksa
        printline = printline + " %-12s" % fmt
        for j in range(max(len(envars),len(flags.items()))):
            if j < len(envars):
                printline = printline + " %-14s" % envars[j]
            else:
                printline = printline + blanksb
            if j < len(flags.items()):
                printline = printline + " %s=%s" % (flags.items()[j][0], flags.items()[j][1])
            if not header_flag: header_flag = True; _logprint(header_line)
            _logprint(printline)
            printline = blanksc
        if len(envars)+len(flags.items()) == 0:
            if not header_flag: header_flag = True; _logprint(header_line)
            _logprint(printline)

    if not header_flag:
        _logprint("No PCL entries to print")

#####################################################################

def logDumpPDL(pdl,msg=None,short=True,findservers=False,displaypcl=False):
    '''
        Dump the contents of a PDL
    '''

    # logDumpPDL won't be called under wsadmin because no PDL is used under wsadmin scripts
    assert os.sys.version_info[0] >= 2 and (os.sys.version_info[0] > 2 or os.sys.version_info[1] > 1), "logDumpPDL not valid for Jython 2.1"

    # If short is set True, only data items which are not tagged "Initial" will
    # be dumped.

    if msg != None and msg != "":
        debug(msg)
    else:
        debug("Contents of PDL follow:")
    _logprint()

    pcl = pdl["PCL"][0]

    header_flag = False
    header_line = "                    Name Format      Tag         Value"
    list_flag = False
    for nm in sorted(pdl):
        pd = pdl[nm]
        if short and pdl[nm][1] == "Initial": continue
        pcli = pd[2]
        if nm == "PCL":
            val = "[pcl]"
        elif pcl[pcli][1] == "Data":
            if findservers:
                list_flag = True
            val = "[data]"
        elif type(pd[0]) == type([]) or type(pd[0]) == type(()):
            list_flag = True
            val = "[list]"
        elif type(pd[0]) == type({}):
            list_flag = True
            val = "[dictionary]"
        elif pcl[pcli][1] == "Password":
            val = "********"
        else:
            val = pd[0]
        tag = pd[1]
        if pcli != None:
            fmt = pcl[pcli][1]
        else:
            fmt = ""
        if not header_flag: header_flag = True; _logprint(header_line)
        _logprint("  %22s %-11s %-11s %s" % (nm, fmt, tag, val))
    if not header_flag:
        _logprint("No PDL entries to print")

    if list_flag:
        header_flag = False
        header_line = "\nLists and Dictionaries:"
        for nm in sorted(pdl):
            if nm == "PASSWORDLIST":
                continue
            if nm == "PCL" and not displaypcl:
                continue
            if short and pdl[nm][1] == "Initial": continue
            if type(pdl[nm][0]) == type([]) or type(pdl[nm][0]) == type(()):
                if not header_flag: header_flag = True; _logprint(header_line)
                _logprint("  %s" % nm)
                for vali in range(len(pdl[nm][0])):
                    if nm != "FINDSERVERS":
                        _logprint("%7d. %s" % (vali,pdl[nm][0][vali]))
                    elif findservers:
                        _logprint("%7d." % vali)
                        for key in sorted(pdl[nm][0][vali].keys()):
                            _logprint("       %20s = %s" % (key,pdl[nm][0][vali][key]))
            elif type(pdl[nm][0]) == type({}):
                if not header_flag: header_flag = True; _logprint(header_line)
                _logprint("  %s" % nm)
                for vali in sorted(pdl[nm][0]):
                    if type(pdl[nm][0][vali]) != type({}):
                        _logprint("    %s -> %s" % (vali, pdl[nm][0][vali]))
                    else:
                        _logprint("    %s ->" % vali)
                        for valii in sorted(pdl[nm][0][vali]):
                            _logprint("      %s -> %s" % (valii, pdl[nm][0][vali][valii]))
        if not header_flag:
            _logprint("\nNo Lists or Dictionaries")

    _logprint()


#####################################################################

def _getStack(context=1):

    if os.sys.version_info[0] > 2 or (os.sys.version_info[0] == 2 and os.sys.version_info[1] >= 5):
        return stack(context)

    # Get the current stack frames for Jython 2.1.
    # Note that we don't bother to get the code context, it's too
    # much work for what little payoff it has.

    frame = sys._getframe(context)
    framelist = []
    while frame:
        lineno = frame.f_lineno
        filename = frame.f_code.co_filename
        functionname = frame.f_code.co_name
        if os.sys.version_info[0] < 2 or (os.sys.version_info[0] == 2 and os.sys.version_info[1] <= 1):
            lines = [None]
            index = 0
        else:
            frameinfo = getframeinfo(frame,context)
            lines = frameinfo[3]
            index = frameinfo[4]
        frameinfo = (frame,filename,lineno,functionname,lines,index)
        framelist.append(frameinfo)
        frame = frame.f_back

    return framelist

#####################################################################

if __name__ == "__main__":

    # Test message log

    debug("Debug message %s" % fixlev)
    #_msgprint("Message #1")
    messagePrint("CFG0000I  This is a message")
    messagePrint("CFG0000I  This is a message",explanation="Explanation: Something Happened")
    messagePrint("CFG0000I  This is a message",explanation="Explanation: Something Happened",operatorResponse="Operator Response: Do something else")
    set_logging("./debugUtil.log","./debugUtil-messages.log")
    #set_logging()
    sys.exit(0)

    debug("Debug message #2")
    _msgprint("Message #2")
    sys.exit(0)

    def recurse():
        level = "Recurse #1"
        recurse2()

    def recurse2():
        localvariable = ["a","b","c",r"\test"]

        logDumpStack("Dumping stack")

        # For debugging we get the current traceback entry, follow it back to
        # its origin, and then accumulate frames on the stack.  Since there is
        # no pending exception, we just get the local frames.

        frames = _getStack()[:]
        for frametuple in frames:
            frame, filename, lineno, function, code_context, index = frametuple
            if code_context == None or code_context == [None]:
                code_context = ["<unknown>\n"]

            print "\n  Frame %s in %s" % (frame.f_code.co_name, frame.f_code.co_filename)

            contextline = "".join(code_context)
            #contextline = contextline.strip(" \n")
            while len(contextline) > 1 and (contextline[-1] == "\n" or contextline[-1] == " "):
                contextline = contextline[:-1]
            while len(contextline) > 1 and (contextline[0] == "\n" or contextline[0] == " "):
                contextline = contextline[1:]
            print "  %d. %s" % (lineno,contextline)

            #for key, value in sorted(frame.f_locals.items()):
            keypairs = frame.f_locals.keys()
            for keyi in range(len(keypairs)):
                key = keypairs[keyi]
                keypairs[keyi] = [key.lower(),key]
            keypairs.sort()
            for keypair in keypairs:
                key = keypair[1]
                value = frame.f_locals[key]
                try:
                    print "    %s = '%s' %s" % (key,value,type(value))
                except:
                    print "    %s = <ERROR ATTEMPTING TO PRINT VALUE>" % key
        print

    # Unit testing.

    try:

        print strftime("%Y-%m-%d %H:%M:%S ") + "*Debug*  debugUtil unit testing begun"
        #assert False, "hello assertion"

        if False:

            from createPCL_config import make_pcl
            #from createPCL_createcfg import make_pcl
            from createPDL import make_pdl, proc_envvar, proc_cliopt

            pcl = make_pcl()
            #print "%d entries in real PCL" % len(pcl)
            pdl = make_pdl(pcl)
            print "%d entries in real PDL" % len(pdl)

            set_logging()
            proc_envvar(pdl)
            proc_cliopt(pdl,["program","-silent","/u/ejack/silentfile.txt"])

            logDumpPDL(pdl,"Dumping Full PDL",short=False)
            logDumpPDL(pdl,"Dumping Short PDL")

            raise SystemExit

        if True:

            debug("message before logging")
            #set_logging("../../zzArchive/debugUtil.log")
            set_logging(None)

            logException("No previous")

            try:
                assert False, "False assertion"
            except:
                logException("Testing exception")

            logDumpStack("Dumping stack")
            recurse()

            debug("This is a DEBUG message")
            debug(True)
            debug(None)
            debug(u"Unicode")
            debug(u"\uc2bf")

        print strftime("%Y-%m-%d %H:%M:%S ") + "*Debug*  debugUtil unit testing completed"

    except SystemExit, errmsg:
        print strftime("%Y-%m-%d %H:%M:%S ") + "*Debug*  SystemExit: %s" % errmsg

    except:

        _exctype, _excval, _exctb = sys.exc_info()

        print
        print "***************************************************"
        print "*Error*  Exception encountered, traceback follows *"
        print "***************************************************"
        traceback.print_exception(_exctype, _excval, _exctb)
        print

        print "Dumping stack frames"

        # Walk back to the top of the trackback chain
        # so we find the earliest exception.

        tb = _exctb
        while tb.tb_next:
            tb = tb.tb_next

        # Get all frames above this exception

        frames = []
        f = tb.tb_frame
        while f:
            frames.append(f)
            f = f.f_back

        # List the frames and their local data

        for frame in frames:
            print "\n  Frame %s in %s at line %s" % (frame.f_code.co_name, frame.f_code.co_filename, frame.f_lineno)
            #for key, value in sorted(frame.f_locals.items()):
            for key, value in frame.f_locals.items():
                try:
                    print "    %s = '%s' %s" % (key,value,type(value))
                except Exception, errmsg:
                    print "    %s = <ERROR ATTEMPTING TO PRINT VALUE> %s" % (key,errmsg)
        print "\n"
