#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    prompting routine
    parameters
    - from_key_name PDL key name to be set by the user
    - pdl

    optional parameters
    - target PDL key name if different from source PDL key name
    - prompt to display if no message found in the resource bundle

    example:
    do_prompt("PORT", pdl)
    do_prompt("PORT", pdl, alt_prompt="Enter port number for SOAP Connection")
'''

fixlev = "40544" # Last APAR, defect or story to update this module

import sys
import os
import re

from time import strftime

import utils
import createPCL_config as createPCL
import createPDL
import resourcebundle
import wasdiscover
import platformUtil
import checkversion
import shared_globals as shared

from debugUtil import logDebug as debug
from resourcebundle import getmsg
from ConfigurationException import ConfigurationException

if platformUtil.isWindows():
    import getpass_loc as getpass
elif platformUtil.IS_zOS_OS() or platformUtil.IS_OS400_OS():
    relElements = platformUtil.getJavaRelease().split(".")
    if int(relElements[1]) > 5:
        import getpass_console as getpass
    else:
        if platformUtil.IS_OS400_OS():
            import getpass_console as getpass
        else:
            import getpass
else:
    import getpass_notty as getpass

from constants import constants

# Instantiate the global constants class

common_constants = constants()

_YES = "1"
_NO = "2"
_LEAVE = "3"

yesno_dic = {_YES: True, _NO: False, _LEAVE: None}

PDL_IDENTIFIER = "Prompt"

key_name = None
default_msg = ""
default_local_answer = ""
entervalue_msg = ""
selectall_msg = ""
entervaluelist_msg = ""
multiselect_msg = ""

SELECT_ALL = None

#List of to_keys which user inputs can be an option number or strings the user types in
MIX_INPUT_LIST = ["WASROOT"]

#Multiselect key list, key where multiple items can be selected
MULTI_SELECT_LIST = ["SERVERS"]


def isBatchMode(pdl):
    batchMode = pdl["BATCH"][0]
    assert batchMode == True or batchMode == False, getmsg("CFG5019E", "Unexpected value type {0} for PDL key {1}", [type(batchMode), "BATCH"])
    return batchMode


def isSilentMode(pdl):
    if "SILENT" not in pdl.keys() or pdl["SILENT"][0] is None:
        silentMode = False
    else:
        silentMode = True
    return silentMode


def do_prompt(from_key_name, pdl, to_key_name=None, alt_prompt=None, for_migration=None, force_alt_prompt=False):

    debug("Entry do_prompt %s" % fixlev)

    if isBatchMode(pdl):

        # If prompt is for the SERVERS, in the case of  batch mode, make sure
        # FULLSERVERS is created.

        if to_key_name is None:
            debug("In batch mode, no prompting for %s" % from_key_name)
        else:
            debug("In batch mode, no prompting for %s to %s" % (from_key_name,to_key_name))
        if from_key_name == "SERVERS":
            _process_fullservers_forbatch(pdl)

    elif isSilentMode(pdl):

        # No Prompt needed
        # FULLSERVERS is created in silent processing

        debug("In silent mode, prompting is disabled")
    else:
        _prompt(from_key_name, pdl, to_key_name, alt_prompt, for_migration, force_alt_prompt)

    debug("Exit do_prompt")


def _prompt(from_key_name, pdl, to_key_name=None, alt_prompt=None, for_migration=None, force_alt_prompt=False):
    '''
        prompting code
        parameters
        - pdl key name where to read data from
        - pdl structure
        **OPTIONAL**
        - pdl key where to put data (if not there will use same as from key)
        - alternative prompt message
        will try to prompt user for 5 times, raises exception if valid answer is not received
        shows help message on ? character input
        exits on X character
        throws exception on:
        - termination character (q)
        - number of tries exceeded no valid input received
    '''

    global default_msg, default_local_answer, entervalue_msg, entervaluelist_msg, multiselect_msg, SELECT_ALL, key_name

    debug("Entry _prompt %s - fromkey(%s) tokey(%s)" % (fixlev, from_key_name, to_key_name))

    SHOW_HELP = getmsg("HELP_CHAR","?")          #help
    TERMINATE = getmsg("QUIT_CHAR","q")          #termination character
    SELECT_ALL = getmsg("SELECT_ALL_CHAR","*")   #select all character

    # Special handling for mocking
    if len(SHOW_HELP) > 1 and SHOW_HELP.endswith("]"): SHOW_HELP = SHOW_HELP[-2:-1]
    if len(TERMINATE) > 1 and TERMINATE.endswith("]"): TERMINATE = TERMINATE[-2:-1]
    if len(SELECT_ALL) > 1 and SELECT_ALL.endswith("]"): SELECT_ALL = SELECT_ALL[-2:-1]

    findservers = utils.getData(pdl, "FINDSERVERS")

    default_msg = getmsg("DEFAULT", "default is:")
    entervalue_msg = getmsg("ENTERVALUE", "Enter a number")
    entervaluelist_msg = getmsg("CFG5031I", "Enter a number or numbers separated by a comma, enter {0} to select all items listed", [SELECT_ALL])
    multiselect_msg = getmsg("MULTISELECT", "multiple numbers separated by space or comma")
    ALT_PROMP_DIC = {"WASROOT": getmsg("ENTERWASROOT", "Enter a number or enter the full path to a home directory")}

    local_answer = ""
    errorcount = 0
    is_valid = 0

    if to_key_name == None:
        to_key_name = from_key_name

    if to_key_name not in MULTI_SELECT_LIST:
        entervaluelist_msg = entervalue_msg

    if to_key_name in ALT_PROMP_DIC.keys():
        debug("using alternative prompt for key: %s" % to_key_name)
        entervalue_msg = ALT_PROMP_DIC[to_key_name]
        entervaluelist_msg = entervalue_msg

    format, default, prompt_message_id, help_message_id = _get_pdl_data(from_key_name, to_key_name, pdl)

    if force_alt_prompt == True:
        prompt_message = alt_prompt
    else:
        prompt_message =  getmsg(prompt_message_id, alt_prompt or "Specify value for %s" % (to_key_name))

    help_message = getmsg(help_message_id, "No help available for '%s'" % (to_key_name))
    while not is_valid:
        debug("Prompt for '%s' Pass '%d' out of '%d'" % (to_key_name, errorcount, common_constants.MAX_TRY))
        if errorcount >= common_constants.MAX_TRY:
            print "%s" % getmsg("CFG5003E", "Error: Maximum number of retries have been reached. Unable to prompt for a valid selection")
            raise Exception("Unable to prompt for a valid selection for '%s' as '%s'" % (to_key_name, format))
        print
        if from_key_name != to_key_name:
            local_answer, selection_list, default_list = _get_raw_user_answer_2(pdl, from_key_name, to_key_name, prompt_message)
        else:
            local_answer, selection_list, default_list, servers_pdl_index_mapping   = _get_raw_user_answer(pdl, from_key_name, format, prompt_message, default, for_migration)

        # Do not log password value
        debug("User answer: '%s'" % (format == "Password" and "*"*8 or local_answer))

        if local_answer == TERMINATE:
            debug("User requested to terminate")
            raise KeyboardInterrupt("Received terminate request in doprompt")
        elif local_answer == SHOW_HELP:
            is_valid = 0
            print "\n%s" % (help_message)
            continue
        if from_key_name != to_key_name:
            to_format = utils.getFormat(pdl, to_key_name)
            is_valid, answer = _validate_answer_2(to_key_name, local_answer, to_format, selection_list, default_list, pdl)
        else:
            is_valid, answer = _validate_answer(local_answer, format, selection_list, default_list, pdl)
        # Do not log password value
        debug("Validate: %s for answer '%s'" % (is_valid, format == "Password" and "*"*8 or answer))
        if is_valid:

            # If key is SERVERS, based on the answer, create a FULLSERVERS pdl which
            # has is a dictionary with the key as cell.node.server and its value is index
            # to FINDSERVERS.
            #
            #  Arguments in _process_fullservers(pdl, findservers_index)
            #  indicate that method expects to receive a list of comma separated entries representing indexes into FINDSERVERS,
            #  relative to 1 not 0

            sep = ""
            if to_key_name == "SERVERS":
                debug("Creating FULLSERVERS")
                answer = []
                if local_answer == "": local_answer = default_local_answer
                if len(servers_pdl_index_mapping) > 0 and local_answer != SELECT_ALL:
                    answers_by_findservers_ix = ""
                    re_sep = re.compile("[\s,]+")
                    answer_list = re_sep.split(local_answer.strip())
                    for ans in answer_list:
                        ans = ans.strip()
                        if ans == "": continue
                        answer_findserver_ix = servers_pdl_index_mapping[int(ans) - 1]
                        answer_findserver_ix = int(answer_findserver_ix) + 1
                        answer.append(findservers[answer_findserver_ix-1]["WASSERVER"])
                        if len(answers_by_findservers_ix) > 0:
                            sep = ","
                        answers_by_findservers_ix  = answers_by_findservers_ix + sep + str(answer_findserver_ix)
                    debug("Setting answers_by_findservers_ix: %s" % str(answers_by_findservers_ix))
                    _process_fullservers(pdl, answers_by_findservers_ix)
                else:
                    debug("Raw servers index: %s" % servers_pdl_index_mapping)
                    answers_by_findservers_ix = ",".join([str(ans+1) for ans in servers_pdl_index_mapping])
                    answer = [findservers[ans]["WASSERVER"] for ans in servers_pdl_index_mapping]
                    debug("All findservers ix: %s" % answers_by_findservers_ix)
                    _process_fullservers(pdl, answers_by_findservers_ix)

            pdlentry =  pdl.get(to_key_name)
            pdl[to_key_name] = [answer, PDL_IDENTIFIER, pdlentry[2]]
            # Do not log password value
            display_answer = answer
            if format == "Password": display_answer = "*"*8
            debug("Setting pdl[%s][0]=%s" % (to_key_name, display_answer))
            # Populate message log with answer by using getmsg()
            ans_msg = getmsg("CFG5004I", "Answer for key {0} is: {1}", [to_key_name, display_answer])
        else:
            errorcount += 1
            invalid_reponse_msg = getmsg("CFG5013E", "Invalid response:")
            print "\n%s '%s'" % (invalid_reponse_msg, local_answer)
            local_answer = None

    debug("Exit do_prompt")


def _get_raw_user_answer_2(pdl, from_key, to_key, prompt_message):

    debug("Entry _get_raw_user_answer_2 %s" % fixlev)

    from_format = utils.getFormat(pdl, from_key)
    to_format = utils.getFormat(pdl, to_key)
    debug("From format = '%s' %s, to format = '%s' %s" % (from_format, type(from_format), to_format, type(to_format)))
    local_answer = None
    default_dic = {}
    if from_format.endswith("List") and not to_format.endswith("List"):
        from_pdl_entry = pdl.get(from_key, None)
        to_pdl_entry = pdl.get(to_key, None)
        default_value = to_pdl_entry[0]
        option_list = from_pdl_entry[0]
        debug("default=%s  number of options=%d" % (default_value, len(option_list)))
        debug("Options:\n    %s" % "\n    ".join(option_list))
        #if we have only one option set it as default
        if len(option_list) == 1: default_value = option_list[0]
        msg1 = prompt_message
        index = 1
        items_dic = {}
        options_str = ""
        has_default = False
        for item in option_list:
            options_str += "\n  %d. %s" % (index, item.encode("utf-8"))
            items_dic[str(index)] = item
            if item == default_value: default_dic[str(index)] = item
            index += 1
        if utils.isSet(default_value) and default_value in option_list:
            has_default = True
        # Special handling for WASROOT, where the default may not be in the selection list
        if to_key == "WASROOT" and default_value is not None and default_value not in default_dic.values():
            default_dic[default_value] = str(default_value)
            has_default = True
        msg2 = "\n" + entervalue_msg
        if has_default:
            msg2 +=  " [%s %s]" % (default_msg, dic_to_str(default_dic))
        msg = "%s %s %s: " % (msg1, unicode(options_str, "utf-8"), msg2)
        print msg
        local_answer = raw_input()
        if shared.console_writer is not None: shared.console_writer.log(local_answer + "\n")
    else:
        assert False, getmsg("CFG5020E", "Invalid PDL key format from: {0} to: {1}, unable to prompt", [from_format, to_format])
    debug("Exit _get_raw_user_answer_2: '%s' (default: '%s')" % (local_answer, default_value))
    for keywd, val in sorted(items_dic.items()):
        debug("%8s : %s" % (keywd, val))
    return (local_answer, items_dic, default_value)


def _get_raw_user_answer(pdl, key_name, format, prompt_message, default, for_migration=None):
    '''
        prompt user and receive an answer
        for list format filter out default_list and remove invalid entries
        so if user hits Enter only valid items will be set in PDL
    '''

    debug("Entry _get_raw_user_answer %s" % fixlev)

    selection_list = default
    default_list = default
    from_format = utils.getFormat(pdl,key_name)
    servers_pdl_index_mapping = []
    if from_format.endswith("List"):
        if key_name == "SERVERS":
            unconfigFlag = pdl["UNCONFIG"][0]
            debug("unconfig flag: %s" % unconfigFlag)
            if unconfigFlag:
                prompt_message = getmsg("CFG2045I", "Choose one or more servers to unconfigure for data collection:")
            server_list = _get_server_list(pdl)
            local_answer, selection_list, default_list, servers_pdl_index_mapping = _prompt_server_list(pdl, format, default, prompt_message, server_list, for_migration)
        else:
            local_answer, selection_list, default_list = _prompt_list(format, default, prompt_message, default)
    else:
        options, default_str = _get_prompt_text(format, default)
        local_answer = _prompt_for_answer(prompt_message, options, default_str, format)

    #Do not log PASSWORD input
    display_answer = local_answer
    if format == "Password": display_answer = "*"*8

    debug("Exit get raw input.\n    User answer:  %s\n    servers_pdl_index_mapping: %s" % (display_answer, repr(servers_pdl_index_mapping)))  #  (local_answer))
    return (local_answer, selection_list, default_list, servers_pdl_index_mapping)


def _validate_answer(local_answer, format, selection_list, default_val, pdl=None):
    '''
        validate user answer based on format and default value
    '''

    display_answer = local_answer
    if format == "Password": display_answer = "*"*8
    debug("Entry validate answer %s - %s as %s" % (fixlev, display_answer, format))

    answer = local_answer
    if format.endswith("List"):
        is_valid, pdl_value = _validate_list(local_answer, selection_list, default_val)
    else:
        is_valid, pdl_value = _validate_entry(local_answer, format, selection_list, pdl)
    display_answer = pdl_value
    if format == "Password": display_answer = "*"*8

    debug("Exit _validate_answer: %s  PDL: %r" % (is_valid, display_answer))
    return (is_valid, pdl_value)


def _validate_answer_2(to_key_name, local_answer, to_format, option_list, default_value, pdl = None):

    debug("Entry _validate_answer_2 %s" % fixlev)
    debug("to_format = %s" % to_format)

    is_valid = False
    if not to_format.endswith("List"):
        answer = local_answer
        if len(answer.strip()) == 0:  #user clicked on Enter
            if default_value != None:
                is_valid = True
                answer = default_value
            else:
                is_valid = False
        else:
            is_valid = True
            if utils.isInt(local_answer):
                #we can have a case where user entered custom value, that is not in the list
                answer = option_list.get(local_answer, None)
                if answer == None:
                    is_valid = False
            else:
                #check if the answer can be a string
                #change is_valid = 0 first
                is_valid = False
                if to_key_name in MIX_INPUT_LIST:
                    is_valid = True
            if is_valid:
                #validate against to_key format
                is_valid, answer = _validate_entry(answer, to_format, default_value, pdl)
    else:
        raise ConfigurationException(getmsg("CFG5064E", "Unknown PDL format {0}", [to_format]))

    debug("Exit _validate_answer_2 %s" % is_valid)
    return (is_valid, answer)


def _validate_entry(local_answer, format, default_val, pdl = None):

    debug("Entry _validate_entry %s - %s" % (fixlev, format))

    # Note: We might be validating a password (format == "Password")
    #       Be wary when displaying local_answer or default_val in that case.

    is_valid = False
    answer = local_answer.strip()

    # Did user click on Enter?
    if answer == "":

        # If there is a default value, accept it, with certain exceptions
        # YesNoLeave is a special case, the valid default may be None

        if default_val != None or (format == "YesNoLeave" and default_val is None):
            is_valid = True
            answer = default_val

        # A string or a password may be a zero-length string

        elif format == "String" or format == "Password":
            is_valid = True
            answer = ""

        # Otherwise we don't have a valid value and we must re-prompt.

        else:
            is_valid = False
            answer = None

    else:
        if format == "YesNo":
            local_answer = local_answer.strip()
            if local_answer == _YES or local_answer == _NO:
                is_valid = True
                answer = yesno_dic[local_answer]
        elif format == "YesNoLeave":
            local_answer = local_answer.strip()
            if local_answer.strip() in (_YES, _NO, _LEAVE):
                is_valid = True
                answer = yesno_dic[local_answer.strip()]
        elif format == "FileExists":
            if os.path.isfile(local_answer):
                is_valid = True
        elif format == "PathExists":
            if not os.path.exists(local_answer):
                print "%s" % getmsg("CFG5065E", "Path does not exist: {0}", [local_answer])
            else:
                is_valid = True
        elif  format == "IPAddress":
            answer = local_answer.strip()
            is_valid = utils.isValidIpAddress(local_answer.strip().split("/")[0])
        elif format == "Hostname":
            answer = local_answer.strip()
            is_valid = utils.isValidHostname(answer)
        elif format == "Port":
            answer = local_answer.strip()
            is_valid = utils.isInt(answer) and int(answer) > 0 and int(answer) <= 0xFFFF
        elif format == "PortRange":
            answer = local_answer.strip()
            is_valid = utils.isValidRange(answer)
        elif format == "File":
            is_valid = utils.isValidFilename(local_answer)
        elif format == "FileNoBlank":
            if " " in local_answer:
                print getmsg("NO_BLANK_ALLOWED", "This prompt does not allow any spaces to be added between the words")
                is_valid = False
            else:
                is_valid = utils.isValidLocalPath(local_answer)
        elif format == "Directory":
            answer = local_answer.strip()
            is_valid = True
        elif format == "Alphanumeric":
            is_valid = utils.isAlphaNumericString(answer)
        elif format in ("Discover","Path","String","Password","Data"):
            is_valid = True
        elif format in ("ServerList", "StringList"):
            debug("*Error*  %s is being validated as an entry instead of a list")
            assert False, getmsg("CFG5022E", "Internal error in {0}", ["_validate_entry"])
        else:
            debug("Format '%s' is not recognized" % format)
            assert False, getmsg("CFG5064E", "Unknown PDL format {0}", [format])

    if is_valid and not platformUtil.IS_zOS_OS():
        if key_name == "WASROOT" and platformUtil.getOSFamily() != "OS400":
            is_valid = wasdiscover.verifyValidWasHome(answer, pdl, quiet=False)
            if not is_valid: print "%s" % getmsg("CFG5066E", "Not a valid WebSphere home directory: {0}", [answer])

    debug("Exit _validate_entry: %s" % is_valid)
    return (is_valid, answer)


def _validate_list(local_answer, selection_list, default_list):
    '''
        validate user input for List format
    '''

    global SELECT_ALL      #select all character

    debug("Entry _validate_list %s" % fixlev)
    debug("local_answer: %s, selection_list: %s, default_list: %s" % (local_answer, selection_list, default_list))

    pdl_value = []
    answer = local_answer.strip()
    if answer == "":
        # user clicked on Enter
        if default_list != None and len(default_list) > 0:
            #there is a default value, accepting it
            is_valid = True
            pdl_value = default_list
        else:
            is_valid = False
    elif answer.find(SELECT_ALL) != -1 and answer != SELECT_ALL:
        is_valid = False
    else:
        is_valid = True
        sep = re.compile("[\s,]+")
        answer_list = sep.split(answer)
        if len(set(answer_list)) < len(answer_list):    #check if got any duplicates, set will hold only unique elements
            debug("answer list has duplicate entries")
            is_valid = False
        if (is_valid):
            for key in sep.split(answer):     #split items by space or comma
                key = key.strip()
                dic_value = selection_list.get(key, None)
                if dic_value == None:
                    is_valid = False
                    break
                else:
                    if type(dic_value) == type([]): pdl_value.extend(dic_value)
                    else: pdl_value.append(dic_value)

    debug("Exit _validate_list - is_valid: %s, pdl_value: %s" % (is_valid, pdl_value ))
    return (is_valid, pdl_value)


def _process_fullservers(pdl, findservers_index):
    '''
       This routine creates FULLSERVERS pdl which is a dictionary with cell,
       node, and server name as its items for each selected server.

       @param pdl - PDL reference
       @param findservers_index - A list of Indexes to FINDSERVERS,
                                  relative to 1, not 0.
    '''

    global SELECT_ALL

    debug("Entry _process_fullservers %s" % fixlev)
    debug("  findservers_index: %s" % str(findservers_index))

    if SELECT_ALL is None:
        SELECT_ALL = getmsg("SELECT_ALL_CHAR","*")   #select all character
        if len(SELECT_ALL) > 1 and SELECT_ALL.endswith("]"): SELECT_ALL = SELECT_ALL[-2:-1]

    # Regular expression to split the numbers
    re_sep = re.compile("[\s,]+")

    utils.setData(pdl, "FULLSERVERS", [], "doprompt")
    fullservers = utils.getData(pdl, "FULLSERVERS")
    findservers = utils.getData(pdl, "FINDSERVERS")

    # The find server list is a list of indexes to FINDSERVERS, and are separated by
    # ','.  Also, it could only have '*' which means process all.
    # (Note: Not any more, that asterisk is processed out earlier now.)
    # To do: In next development cycle, remove the SELECT_ALL code since it's handled earlier.

    if findservers_index == SELECT_ALL:

        for serveri in range(len(findservers)):
            fullservers_dict = {}
            findservers_entry = findservers[serveri]
            wasprofile = findservers_entry.get("PROFILE")
            wascell = findservers_entry.get("WASCELL")
            wasnode = findservers_entry.get("WASNODE")
            wasserver = findservers_entry.get("WASSERVER")
            fullservers_dict["PROFILE"] = wasprofile
            fullservers_dict["WASCELL"] = wascell
            fullservers_dict["WASNODE"] = wasnode
            fullservers_dict["WASSERVER"] = wasserver
            fullservers.append(fullservers_dict)

    else:

        indexlist = re_sep.split(findservers_index)
        debug("Raw index list: %s" % indexlist)

        # Index to FINDSERVERS are one less than the actual index because the
        # selected numbers are starting from 1 not zero.
        newindexlist = [int(index)-1 for index in indexlist]

        # Remove any duplicates
        indexlist = []
        for index in newindexlist:
            if index not in indexlist: indexlist.append(index)
        debug("Processed 0-origin index list: %s" % indexlist)

        for findex in indexlist:
            fullservers_dict = {}
            findservers_entry = findservers[findex]
            wasprofile = findservers_entry.get("PROFILE")
            wascell = findservers_entry.get("WASCELL")
            wasnode = findservers_entry.get("WASNODE")
            wasserver = findservers_entry.get("WASSERVER")
            fullservers_dict["PROFILE"] = wasprofile
            fullservers_dict["WASCELL"] = wascell
            fullservers_dict["WASNODE"] = wasnode
            fullservers_dict["WASSERVER"] = wasserver
            fullservers.append(fullservers_dict)

    debug("_process_fullservers complete : %s" % str(fullservers_dict))
    debug("Exit _process_fullservers")


def _process_fullservers_forbatch(pdl):
    '''
       This routine creates FULLSERVERS pdl which is a dictionary with cell,
       node, and server name as its items. This is
       only done for batch mode for ZOS, and there is an assumption that there
       is only default profile and each server name is unique.

       @param pdl - PDL reference
       @param findservers_index - A list of Indexes to FINDSERVERS
    '''
    debug("Entry _process_fullservers_forbatch %s" % fixlev)
    utils.setData(pdl, "FULLSERVERS", [], "doprompt")
    fullservers = utils.getData(pdl, "FULLSERVERS")
    findservers = utils.getData(pdl, "FINDSERVERS")
    servers = utils.getData(pdl, "SERVERS")
    # The servers value may contain ["server1,server2"] when ITCAM_SERVER=server1,server2
    # todo: handle this better
    debug("Servers returned by utils.getData: %s" % str(servers))
    if servers is None or ( type(servers) == type([]) and len(servers) == 0 ):
        msg = getmsg("CFG1148E","No server has been specified.")
        raise ConfigurationException(msg)
    rev_servers_list = []
    for server_name in servers:
        server_names = server_name.split(",")
        for imbedded_server_name in server_names:
            rev_servers_list.append(imbedded_server_name)
    if len(servers) != len(rev_servers_list):
        utils.setData(pdl, "SERVERS", rev_servers_list, "doprompt_pfb")
        servers = rev_servers_list
        debug("Servers returned by utils.getData revised: %s" % str(servers))

    # For each server in the SERVERS list, search the FINDSERVERS and get the info
    # and create the FULLSERVERS

    for server_name in servers:
        debug("Processing server %s" % (server_name))
        for serveri in range(len(findservers)):
            findservers_entry = findservers[serveri]
            wasserver = findservers_entry.get("WASSERVER")
            debug("FINDSERVERS server name: <%s>" % (wasserver))
            if server_name == wasserver:
                fullservers_dict = {}
                wasprofile = findservers_entry.get("PROFILE")
                wascell = findservers_entry.get("WASCELL")
                wasnode = findservers_entry.get("WASNODE")
                fullservers_dict["PROFILE"] = wasprofile
                fullservers_dict["WASCELL"] = wascell
                fullservers_dict["WASNODE"] = wasnode
                fullservers_dict["WASSERVER"] = wasserver
                fullservers.append(fullservers_dict)
    debug("_process_fullservers_forbatch exit")


def _get_server_list(pdl):
    '''
        return list of server names stored in FINDSERVERS PDL key
        TODO: multiple profiles, do we need to display
    '''

    debug("Entry get server list %s" % fixlev)

    findservers = utils.getData(pdl,"FINDSERVERS")
    server_list = []
    for findserver in findservers:
        if type(findserver) == type({}):    #there is a list of dictionaries
            server_list.append(findserver)
        else:
            debug("Other type found where dictionary is expected: %s %s" % (repr(findserver),type(findserver)))

    servers_rept = []
    for serveri in xrange(len(server_list)):
        servers_rept.append("FINDSERVER[%i]:" % serveri)
        for ky in sorted(server_list[serveri].keys()):
            if ky not in ("WASSERVER","WASNODE","WASCELL","PROFILE"): continue
            servers_rept.append("    %16s : %s" % (ky, server_list[serveri][ky]))

    debug("Exit _get_server_list: %i servers found -\n    %s" % (len(server_list), "\n    ".join(servers_rept)))
    return server_list


def _prompt_server_list(pdl, format, default_list, prompt_message, server_list, for_migration):
    '''
        prompt user in case of list format
        server_list - a list from pdl,"FINDSERVERS", not necessarily from findServers.py
        @returns  (local_answer, items_dic, default_list, servers_pdl_index_mapping)
        items_dic - contains a map from the selection choice ( '*' or a number) to the server name (which may not be unique)
        servers_pdl_index_mapping - a list that maps the numbers displayed to the findservers list
    '''

    global SELECT_ALL, entervaluelist_msg   #select all character

    debug("Entry _prompt_server_list %s" % fixlev)

    # Process the asterisk default.
    if server_alldefault(pdl, server_list):
        default_list = pdl["SERVERS"][0]

    server_name_key = "WASSERVER"
    cell_name_key = "WASCELL"
    node_name_key = "WASNODE"
    profile_name_key = "PROFILE"
    configured_key = "CONFIGURED"
    wlp_key = "WLPHOME"
    dcversion_key = "DCVERSION"

    # For an application, it can be
    # 1) pristine (No ITCAM), pristine_server_list; or
    # 2) configured by ITCAM 7.2 or later version, configured_72_server_list; or
    # 3) configured by ITCAM 7.1 or previous version including ecam, olderversion_server_list;
    # 4) has MS installed on it, ms_server_list
    # At prompt, we only provide users pristine_server_list and configured_72_server_list.
    # olderversion_server_list and ms_server_list will be provided to users in log file.

    #process server_list first based on configured_key
    pristine_server_list = []
    configured_72_server_list = []
    olderversion_server_list  = []
    newerversion_server_list = []
    ms_server_list = []
    configured = None
    #configured_server_list is for unconfiguration & -reconfig flag.
    #for both cases, findconfigservers module gets called so CONFIGURED attribute is not populated.
    configured_server_list = []
    all_server_list = []
    #Need to preserve server index after grouping servers in pristine server list and 7.2 server list
    servers_pdl_index_mapping = []   # representing the order and the number displayed on console
    #                                # Entry 0 is the #1. on the console
    pristine_pdl_index_mapping = []
    configured72_pdl_index_mapping = []

    configured_pdl_index_mapping = []
    ms_configured_pdl_index_mapping = []
    newversion_pdl_index_mapping = []
    olderversion_pdl_index_mapping = []
    dcversion_dic = {}
    dcversion_dic_index_mapping = {}

    item_index = 0
    current_version = utils.getData(pdl, "VERSION")
    # servers_dic is for the mapping between serverFullName & serverName
    servers_dic = {}

    # If this is as a result of migrating SOA71, then turn the migration flag on and also
    # set the SOA71 flag on.  In this case we only want to show the pristine servers.

    migrate_soa71_flag = False
    migrate_soa71 = utils.getDataString(pdl, "MIGRATESOA71WASROOT")
    debug("SOA71 Migrate %s" % (migrate_soa71))
    if migrate_soa71 is not None:
        migrate_soa71_flag = True
        for_migration = True
        debug("SOA71 Migration")
    # Construct: configured_server_list, pristine_server_list,
    #            ms_server_list, configured_72_server_list, configured72_pdl_index_mapping,
    #            newerversion_server_list,  olderversion_server_list
    for item in server_list:
        #serverFullName is for display purpose
        wlp_home = item.get(wlp_key, None)
        if wlp_home != None and wlp_home != "None" and len(wlp_home) > 0:
            serverFullName = item.get(server_name_key, "**Not Set**")
        else:
            serverFullName = item.get(cell_name_key, "**Not Set**") + "." + item.get(node_name_key, "**Not Set**") + "." + item.get(server_name_key, "**Not Set**")
            profileName = item.get(profile_name_key, "**Not Set**")
            serverFullName = serverFullName + "(" + item.get(profile_name_key, "**Not Set**") + ")"
        #serverName is the value stored in PDL["SERVERS"][0]
        serverName = item.get(server_name_key, "**Not Set**")
        servers_dic[serverFullName] = serverName
        configured = item.get(configured_key)
        if configured is not None:
            debug("configured value: " + configured)
        dcversion = item.get(dcversion_key)
        if dcversion is not None:
            debug("current dc version: %s" % dcversion)
            if dcversion not in dcversion_dic.keys():
                server_temp_list = []
                temp_index_mapping = []
                dcversion_dic.setdefault(dcversion, []).append(serverFullName)
                #dcversion_dic[dcversion] = server_temp_list.append(serverFullName) 
                dcversion_dic_index_mapping.setdefault(dcversion, []).append(item_index)
            else:
                server_temp_list = dcversion_dic[dcversion]
                temp_index_mapping = dcversion_dic_index_mapping[dcversion]
                debug("server_temp_list %s" % server_temp_list)
                if server_temp_list != None:
                    server_temp_list.append(serverFullName)
                if temp_index_mapping != None:
                    temp_index_mapping.append(item_index)
                dcversion_dic[dcversion] = server_temp_list
                dcversion_dic_index_mapping[dcversion] = temp_index_mapping
                
        if configured is None and not for_migration:
            #unconfiguration and -reconfig flag cases
            configured_server_list.append(serverFullName)
            configured_pdl_index_mapping.append(item_index)  # new
            item_index = item_index + 1                      # new
        elif configured == "None":
            pristine_server_list.append(serverFullName)
            pristine_pdl_index_mapping.append(item_index)
            item_index = item_index + 1
        else:
            if configured == "VE":
                ms_server_list.append(serverFullName)
                ms_configured_pdl_index_mapping.append(item_index) # new
                item_index = item_index + 1                        # new
            elif configured == current_version:
                configured_72_server_list.append(serverFullName)
                configured72_pdl_index_mapping.append(item_index)
                item_index = item_index + 1
            elif checkversion.convert_version(configured) > checkversion.convert_version(current_version):
                newerversion_server_list.append(serverFullName)
                newversion_pdl_index_mapping.append(item_index)   # new
                item_index = item_index + 1                       # new
            else:
                olderversion_server_list.append(serverFullName)
                olderversion_pdl_index_mapping.append(item_index) # new
                item_index = item_index + 1                       # new

    debug("configured_pdl_index_mapping:    %s" % str(configured_pdl_index_mapping))
    debug("ms_configured_pdl_index_mapping: %s" % str(ms_configured_pdl_index_mapping))
    debug("pristine_pdl_index_mapping:      %s" % str(pristine_pdl_index_mapping))
    debug("configured72_pdl_index_mapping:  %s" % str(configured72_pdl_index_mapping))
    debug("newversion_pdl_index_mapping:    %s" % str(newversion_pdl_index_mapping))
    debug("olderversion_pdl_index_mapping:  %s" % str(olderversion_pdl_index_mapping))
    debug("dc version dictionary: %s" % str(dcversion_dic))

    selected_list = []
    is_valid = 0

    # Log server lists

    # We can configure pristine servers
    if len(pristine_server_list) > 0:
        debug("Pristine servers not currently configured with ITCAM:\n\n%s" % "\n".join(pristine_server_list))
    else:
        debug("No servers in pristine server list")

    # We can reconfigure servers configured with the current configuration
    if len(configured_72_server_list) > 0:
        debug("Servers configured with current ITCAM:\n\n%s" % "\n".join(configured_72_server_list))
    else:
        debug("No servers for current ITCAM in server list")

    # We Can migrate servers configured with old versions of ITCAM
    if len(olderversion_server_list) > 0:
        debug("Servers configured with older ITCAM:\n\n%s" % "\n".join(olderversion_server_list))
    else:
        debug("No servers with older ITCAM in server list")

    # We can't do anything with a later version of ITCAM
    if len(newerversion_server_list) > 0:
        debug("Servers configured with newer ITCAM:\n\n%s" % "\n".join(newerversion_server_list))
    else:
        debug("No servers with newer ITCAM in server list")

    # We can't do anything with servers configured with MSVE
    if len(ms_server_list) > 0:
        debug("Servers configured with VE:\n\n%s" % "\n".join(ms_server_list))
    else:
        debug("No servers for VE in server list")

    #Provide a message to tell users that there are old version DCs or
    #there are application servers configured by MSVE
    #Users can check log file to find those servers
    msMsg = ""
    if len(ms_server_list) > 0:
        msMsg = "\n" + getmsg("CFG1077I", "There are application servers configured by ITCAM Managing Server. Check log file to find these servers")

    oldVersionMsg = ""
    if for_migration:
        debug("for_migration: " + str(for_migration))
    if not migrate_soa71_flag and len(olderversion_server_list) > 0:
        if for_migration:
            oldVersionMsg = "\n" + getmsg("CFG1084I", "Choose a Server or Servers to be migrated")
        else:
            oldVersionMsg = "\n\n" + getmsg("CFG1078I", "There are application servers configured by older version of ITCAM Data Collector. Check log file to find these servers")

    msg1 = "\n" + prompt_message
    debug("msg1: " + msg1)
    options_str = ""
    pristine_options_str = ""
    configured_72_options_str = ""
    items_dic = {}
    default_dic = {}
    if default_list == None: default_list = []
    error_list = default_list[:]    #make a copy of the original list
    index = 0

    # For unconfiguration and -reconfig flag cases:

    if not for_migration and len(configured_server_list) > 0:
        for ix in range(len(configured_server_list)):
            item = configured_server_list[ix]
            index += 1
            options_str += "\n %5d. %s" % (index, item)
            debug("options_str: " + options_str)
            items_dic[str(index)] = servers_dic[item]
            all_server_list.append(servers_dic[item])
            debug("Item in configured servers: %s" % item)
            for default_entry in default_list:
                # For Liberty, the item is simply the server name.
                # For WebSphere, it is [cell].[node].[server]([profile])
                if item == default_entry or item.find("."+default_entry+"(") != -1:
                    if default_entry in error_list: error_list.remove(default_entry)
                    default_dic[str(index)] = servers_dic[item]

    # Display pristine server group first
    # Merge pristine_pdl and configured72_pdl index mapping together

    pristineMsg = ""
    pristineServers = len(pristine_server_list)
    debug("Total pristine servers: " + str(pristineServers))
    if (not for_migration and pristineServers > 0) or migrate_soa71_flag and pristineServers > 0:
        pristineMsg = "\n  " + getmsg("CFG1079I", "The following application servers have no ITCAM configured:")
        for ix in range(len(pristine_server_list)):
            item = pristine_server_list[ix]
            index += 1
            pristine_options_str += "\n%5d. %s" % (index, item)
            all_server_list.append(servers_dic[item])
            items_dic[str(index)] = servers_dic[item]
            servers_pdl_index_mapping.append(pristine_pdl_index_mapping[ix])
            debug("Item in pristine servers: %s" % item)
            for default_entry in default_list:
                # For Liberty, the item is simply the server name.
                # For WebSphere, it is [cell].[node].[server]([profile])
                if item == default_entry or item.find("."+default_entry+"(") != -1:
                    if default_entry in error_list: error_list.remove(default_entry)
                    default_dic[str(index)] = servers_dic[item]
        debug("options_str: %s" % pristine_options_str)

    # Display AD 7.2 servers

    configured72Msg = ""
    migrate_ccm_msg = ""
    totalConfigured72Servers = len(configured_72_server_list)
    debug("Total application servers with ITCAM 7.3 or above configuration: %i" % totalConfigured72Servers)
    if totalConfigured72Servers > 0:
        if for_migration and utils.is_ccm(pdl) == True and len(dcversion_dic) > 0:
            #we are migrating servers inside scm ai install
            #all servers from all existing dchomes are listed
            #and displayed grouped by current DC version
            migrate_ccm_msg = "\n" + getmsg("CFG1189I", "Select application servers for migration from the list of configured servers")
            for dc_version in sorted(dcversion_dic.iterkeys()):
                migrate_ccm_msg += "\n" + getmsg("CFG1190I", "Configured by {0}", [dc_version])
                server_list = dcversion_dic[dc_version]
                index_list = dcversion_dic_index_mapping[dc_version]
                for ix in range(len(server_list)):
                    item = server_list[ix]
                    index += 1
                    migrate_ccm_msg += "\n%5d. %s" % (index, item)
                    debug("options_str: " + migrate_ccm_msg)
                    items_dic[str(index)] = servers_dic[item]
                    all_server_list.append(servers_dic[item])
                    servers_pdl_index_mapping.append(index_list[ix])
                    debug("Item in configured 7.3 servers: %s" % item)
        elif not for_migration:    
            configured72Msg = "\n  " + getmsg("CFG1080I", "The following application servers have current version of ITCAM configured:")
            for ix in range(len(configured_72_server_list)):
                item = configured_72_server_list[ix]
                index += 1
                configured_72_options_str += "\n%5d. %s" % (index, item)
                debug("options_str: " + configured_72_options_str)
                items_dic[str(index)] = servers_dic[item]
                all_server_list.append(servers_dic[item])
                servers_pdl_index_mapping.append(configured72_pdl_index_mapping[ix])
                debug("Item in configured 7.2 servers: %s" % item)
                for default_entry in default_list:
                    # For Liberty, the item is simply the server name.
                    # For WebSphere, it is [cell].[node].[server]([profile])
                    if item == default_entry or item.find("."+default_entry+"(") != -1:
                        if default_entry in error_list: error_list.remove(default_entry)
                        default_dic[str(index)] = servers_dic[item]

    

    # Display Older than AD 7.2 servers

    totalOlderServers = len(olderversion_server_list)
    migrate_options_str = ""
    if for_migration and (totalOlderServers > 0):
        for ix in range(len(olderversion_server_list)):
            item = olderversion_server_list[ix]
            index += 1
            migrate_options_str += "\n%5d. %s" % (index, item)
            debug("migrate_option_str: " + migrate_options_str)
            items_dic[str(index)] = servers_dic[item]
            all_server_list.append(servers_dic[item])
            debug("Item in older than 7.2 servers: %s" % item)
            for default_entry in default_list:
                # For Liberty, the item is simply the server name.  (Probably does not apply.)
                # For WebSphere, it is [cell].[node].[server]([profile])
                if item == default_entry or item.find("."+default_entry+"(") != -1:
                    if default_entry in error_list: error_list.remove(default_entry)
                    default_dic[str(index)] = servers_dic[item]
    items_dic[SELECT_ALL] = all_server_list
    for item in error_list: #any items left are invalid default values, they can not be selected
        debug("Default list contains an invalid value '%s', removing..." % (item))
        default_list.remove(item)
    msg2 = "\n" + entervaluelist_msg
    if len(default_list) > 0:
        msg2 += " [%s %s]" % (default_msg, dic_to_str(default_dic))
        debug("msg2: " + msg2)
    #msg2 += "(multiple items separated by space or comma):"
    if for_migration and utils.is_ccm(pdl):
        msg = migrate_ccm_msg + msg2 + ": "
    else:
        msg = msMsg + oldVersionMsg + msg1 + pristineMsg + pristine_options_str + configured72Msg + configured_72_options_str + msg2 + ": "

    if for_migration and not migrate_soa71_flag and not utils.is_ccm(pdl):
        msg = oldVersionMsg + migrate_options_str + msg2 + ": "
    elif configured == None:
        msg = msg1 + options_str + msg2 + ": "
    elif totalConfigured72Servers == 0 and pristineServers == 0:
        noServerMsg = "\n \n" + getmsg("CFG1090I", "No application servers found to be configured, exiting from configuration process.")
        msg = msMsg + oldVersionMsg + noServerMsg
        debug("msg: " + msg)
        print msg
        raise SystemExit

    debug("msg: " + msg)
    print msg
    local_answer = raw_input()
    if shared.console_writer is not None: shared.console_writer.log(local_answer + "\n")
    debug("items_dic: %s" % str(items_dic))
    debug("servers_pdl_index_mapping: %s" % str(servers_pdl_index_mapping))

    debug("Exit _prompt_server_list")
    return (local_answer, items_dic, default_list, servers_pdl_index_mapping)


def _prompt_list(format, default_list, prompt_message, complete_list):
    '''
        prompt user in case of list format
    '''

    global SELECT_ALL, entervaluelist_msg   #select all character

    debug("Entry _prompt_list %s" % fixlev)

    selected_list = []
    is_valid = 0
    msg1 = prompt_message
    options_str = ""
    index = 1
    items_dic = {}
    default_dic = {}
    if default_list == None: default_list = []
    error_list = default_list[:]    #make a copy of the original list
    for item in complete_list:
        options_str += "\n  %d. %s" % (index, item)
        items_dic[str(index)] = item
        if item in default_list:
            error_list.remove(item)
            default_dic[str(index)] = item
        index += 1
    items_dic[SELECT_ALL] = complete_list
    for item in error_list: #any items left are invalid default values, they can not be selected
        debug("Default list contains an invalid value '%s', removing..." % (item))
        default_list.remove(item)
    msg2 = "\n" + entervaluelist_msg
    if len(default_list) > 0:
        msg2 += " [%s %s]" % (default_msg, dic_to_str(default_dic))
    #msg2 += "(multiple items separated by space or comma):"
    msg = msg1 + options_str + msg2 + ": "
    print msg
    local_answer = raw_input()
    if shared.console_writer is not None: shared.console_writer.log(local_answer + "\n")

    debug("_prompt_list exit")
    return (local_answer, items_dic, default_list)


def dic_to_str(dic):
    global default_local_answer
    default_local_answer = ",".join([k for k in sorted(dic.keys())])
    return " and ".join([k for k in sorted(dic.keys())])


def _get_pdl_data(from_key_name, to_key_name, pdl):
    '''
        returns: format, current_value, prompt_message_id, and help_message_id
    '''

    debug("Entry _get_pdl_data(%s, %s) %s" % (from_key_name, to_key_name, fixlev))

    default_value = utils.getData(pdl, from_key_name)
    format = utils.getFormat(pdl,to_key_name)
    prompt_message_id = None
    help_message_id = None
    pcl = pdl["PCL"][0]
    pcli = pdl[from_key_name][2]
    pcl_entry = pcl[pcli]
    if len(pcl_entry) > 5:
        prompt_message_id = pcl_entry[5][0]
        help_message_id = pcl_entry[5][1]

    debug("Exit _get_pdl_data, Key '%s' Format: '%s' Default value '%s' Message_id '%s'" % (to_key_name, format, format == "Password" and "*"*8 or default_value, prompt_message_id))
    return (format, default_value, prompt_message_id, help_message_id)


def _get_prompt_text(format, default):
    '''
        returns default value and options string to be displayed as part of the prompt
    '''

    debug("Entry _get_prompt_text %s" % fixlev)

    options = ""
    default_str = ""
    _YESNO_OPS = getmsg("YESNO", "[1 - YES, 2 - NO]")
    _YESNOLEAVE_OPS = getmsg("YESNOLEAVE", "[1 - YES, 2 - NO, 3 - LEAVE]")
    if format in ("YesNo", "YesNoLeave"):
        options = _YESNOLEAVE_OPS
        if format == "YesNo":
            options = _YESNO_OPS
        if default == True  or str(default) == "1": default_str = _YES
        if default == False or str(default) == "0": default_str = _NO
        if default is None and format == "YesNoLeave": default_str = _LEAVE
    else:
        if default != None: default_str = default

    debug("Exit _get_prompt_text")
    return (options, default_str)


def _prompt_for_answer(prompt_message, options_str, default_value, format):

    debug("Entry _prompt_for_answer: Message is '%s'" % (prompt_message))

    local_answer = ""
    if utils.isSet(default_value):
        display_value = default_value
        if format == "Password": display_value = "*"*8
        msg = "%s %s [%s %s]: " % (prompt_message, options_str, default_msg, display_value.encode(None,"backslashreplace"))
    else:
        if options_str != "":
            msg = "%s %s: " % (prompt_message, options_str)
        else:
            msg = "%s: " % (prompt_message)
    if format == "Password":
        msg = msg + "\n"
        local_answer = getpass.getpass(msg)
    else:
        print msg
        local_answer = raw_input()
        if shared.console_writer is not None: shared.console_writer.log(local_answer + "\n")

    return local_answer


def _prompt_no_pdl(prompt_message, options_str, default_value, format):
    local_answer = ""
    default_msg = getmsg("DEFAULT", "default is:")
    debug("_prompt entrance %s" % (prompt_message))
    if utils.isSet(default_value):
        msg = "%s %s [%s %s]: " % (prompt_message, options_str, default_msg, str(default_value))
    else:
        msg = "%s %s: " % (prompt_message, options_str)
    print msg
    local_answer = raw_input()
    if shared.console_writer is not None: shared.console_writer.log(local_answer  + "\n")

    return local_answer


def prompt_list_no_pdl(pdl, option_list, prompt_msg, default_value=None):

    global entervalue_msg
    global default_msg

    debug("Entry prompt_list_no_pdl %s" % fixlev)

    silent = utils.getData(pdl, "SILENT")
    if silent is None:
        default_msg = getmsg("DEFAULT", "default is:")
        local_answer = None
        if len(option_list) == 0:
            raise Exception("zero length list provided as an input")
        errorcount = 0
        option_str = ""
        index_list = []
        for i in range(len(option_list)):
            index = i + 1
            index_list.append(str(index))
            option_str += "\n  %s. %s" % (str(index), option_list[i])
        while local_answer == None:
            debug("Prompt for %s Pass '%d' out of '%d'" % (option_str, errorcount, common_constants.MAX_TRY))
            if errorcount == common_constants.MAX_TRY:
                print "%s" % getmsg("CFG5003E", "Error: Maximum number of retries have been reached. Unable to prompt for a valid selection")
                raise KeyboardInterrupt("Received terminate request in doprompt")
                #raise Exception("Unable to prompt for a valid selection ")
            msg = "%s %s " % (prompt_msg, option_str)
            if utils.isSet(default_value):
                msg = "%s\n%s [%s %s]: " % (msg, entervalue_msg, default_msg, str(default_value))
            else:
                msg = "%s\n%s:" % (msg, entervalue_msg)

            print msg
            local_answer = raw_input()
            if shared.console_writer is not None: shared.console_writer.log(local_answer + "\n")
            debug("user answer '%s'" % str(local_answer))
            if local_answer == "":
                if utils.isSet(default_value):
                    local_answer = default_value
                else:
                    local_answer = None
            elif local_answer == "q":
                debug("User requested to terminate")
                raise KeyboardInterrupt("Received terminate request in doprompt")
            else:
                if local_answer not in index_list:
                    debug("invalid answer '%s'" % str(local_answer))
                    invalid_reponse_msg = getmsg("CFG5013E", "Invalid response:")
                    print "\n%s '%s'\n" % (invalid_reponse_msg, local_answer)
                    local_answer = None
            errorcount += 1
        debug("prompt_list_no_pdl exit %s" % str(local_answer))
        utils.setData(pdl, "LISTRESP", local_answer, "doprompt")
    else:
        debug("Silent mode")


def _prompt_yesno(prompt_msg, default_value=None):

    debug("Entry _prompt_yesno(%s) %s" % (prompt_msg, fixlev))

    format = "YesNo"
    if default_value == None: default_value = False
    options_str, default_str = _get_prompt_text(format, default_value)
    is_valid = 0
    while not is_valid:
        local_answer = _prompt_no_pdl(prompt_msg, options_str, default_str, format)
        is_valid, answer = _validate_entry(local_answer, format, default_value)

    debug("Exit prompt_yesno, result is %s" % answer)
    return answer


def prompt_file_overwrite(pdl, msg=None):
    if isBatchMode(pdl) or isSilentMode(pdl): return True
    if msg == None:
        msg = getmsg("CFG5005I", "File already exists. Do you want to overwrite?")
    return _prompt_yesno(msg)


def prompt_dir_create(pdl, msg=None):
    if isBatchMode(pdl) or isSilentMode(pdl): return True
    if msg == None:
        msg = getmsg("CFG5006I", "Directory doesn't exist. Is it ok to create?")
    return _prompt_yesno(msg)


def prompt_continue(pdl, default=None):
    if isBatchMode(pdl) or isSilentMode(pdl): return True
    msg = getmsg("CONTINUE", "Do you want to continue?")
    if default == None:
        default = True
    return _prompt_yesno(msg, default)


def prompt_retry(pdl, default=None):

    debug("Entry prompt_retry %s" % fixlev)

    if isBatchMode(pdl):
        return False
    elif isSilentMode(pdl):
        return False
    msg = getmsg("CORRECTRETRY", "Do you want to correct the error and retry?")
    if default == None:
        default = True

    debug("Exit prompt_retry")
    return _prompt_yesno(msg, default)


def server_alldefault(pdl, server_list=None):
    '''
      Process "-server *" by inserting all servers into the default server list
    '''

    global SELECT_ALL

    debug("Entry server_alldefault %s" % fixlev)

    # server_list is the same as pdl["FINDSERVERS"][0]
    if server_list is None: server_list = pdl["FINDSERVERS"][0]

    # If there is an asterisk in the list of servers, we must remove it and append
    # all known servers.

    if SELECT_ALL is None:
        SELECT_ALL = getmsg("SELECT_ALL_CHAR","*")   #select all character
        if len(SELECT_ALL) > 1 and SELECT_ALL.endswith("]"): SELECT_ALL = SELECT_ALL[-2:-1]

    debug("SELECT_ALL = %s, SERVERS = %s" % (SELECT_ALL, pdl["SERVERS"][0]))

    ans = False
    if pdl["SERVERS"][0] is not None and SELECT_ALL in pdl["SERVERS"][0]:
        debug("All servers selected by use of at least one asterisk")
        ans = True
        pdl["FULLSERVERS"][0:2] = [], "doprompt.server_alldefault"
        pdl["SERVERS"][0:2] = [], "doprompt.server_alldefault"
        pdl["ALIASLIST"][0:2] = [], "doprompt.server_alldefault"
        for findserver in server_list:
            default_server = findserver["WASSERVER"]
            debug("  Server %s being added to default servers list for asterisk processing" % default_server)
            pdl["SERVERS"][0].append(default_server)
            pdl["ALIASLIST"][0].append(None)
            fullserver_entry = {}
            for entry in ("PROFILE","WASCELL","WASNODE","WASSERVER"):
                fullserver_entry[entry] = findserver[entry]
            pdl["FULLSERVERS"][0].append(fullserver_entry)
    else:
        debug("All servers not in effect, no asterisk seen")

    debug("Exit server_alldefault: %s" % ans)
    return ans


#####################################################################
# Unit testing
#####################################################################

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL

    from debugUtil import set_logging
    from resourcebundle import set_resourcebundle

    from createPDL import getFormats

    def xdebug(ln):
        print ln

    set_logging("./doprompt.log")
    set_resourcebundle()

    pcl = createPCL.make_pcl()
    pdl = createPDL.make_pdl(pcl)

    valid_formats = getFormats()
    formats = [pcle[1] for pcle in pcl]
    for format in formats:
        assert format in valid_formats, "Format %s is not in list of valid formats" % format
    for format in valid_formats:
        if format not in formats: print "*Unused* %s" % format

    ans = _validate_entry("-1", "Port", "63355")
    print "Port = %s" % repr(ans)

    for pci in xrange(len(pcl)):
        continue
        varname = pcl[pci][0]
        if varname == "FORMATS": continue
        format = pcl[pci][1]

        default = pcl[pci][2]
        if default is None:
            default = "x"
        elif default is True:
            default = "y"
        elif default is False:
            default = "n"
        elif type(default) is type(0):
            default = str(default)
#         print "var = %s  format = %s  default = %s" % (varname,format,default)
        ans = _validate_entry(default,format,default)
        print "  %-10s %-8s %-8s = %s" % (varname, format, default, ans)

    assert _validate_entry("1","YesNo","0")[0], "YesNo = 1"
    assert _validate_entry("127.0.0.1/localhost", "IPAddress", None)

    print "That's all, folks!"

    sys.exit(0)

    do_prompt("SMF",pdl)
    debug("  SMF = %s" % pdl["SMF"][0])
    sys.exit(0)

    server_list = []
    populateFindServers.processFile(pdl, "/data/out.txt", server_list)
    pdl["FINDSERVERS"][0] = server_list
    pdl["FINDSERVERS"][1] = "findServers"
    print server_list
    pdl["SERVERS"][0] = ["server1", "server8", "server15", "my_server"]
    do_prompt("SERVERS", pdl)
    do_prompt("SERVERS", pdl)
    do_prompt("SERVERS", pdl)
    do_prompt("LOG", pdl)
    do_prompt("LOG", pdl)
    do_prompt("PORT", pdl)
    do_prompt("PORT", pdl)
    #do_prompt("PROBERMIPORT", pdl, pcl)
    #do_prompt("RECONFIG", pdl, pcl)
    #do_prompt("LOCALHOSTIP", pdl, pcl)
    #do_prompt("LOCALHOSTNAME", pdl, pcl)
    #do_prompt("MSHOME", pdl, pcl)
    #do_prompt("SMF", pdl, pcl)
