#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    findServers.py
    usage: wsdamin.sh -f findServers.py input.properties
'''

fixlev = "40544" # Last APAR, defect or story to update this module

import sys
import os
import re
import socket
import codecs

from time import strftime
from java.lang import System
from java.net import InetAddress

# On Windows platforms the python.path property has problems.
# Make an explict addition to the the import path.
sys.path.append("lib")

# All imported functions which are invoked must be compatible with Jython 2.1.

import platformUtil
import resourcebundle
import debugUtil

from debugUtil import logDebug as debug
from resourcebundle import getmsg
from constants import constants
from Progress import Progress
from ConfigurationException import ConfigurationException

##Global variables
global sysPropIds

VERSIONSTRING = "ITCAM 7.3 HAAD730 AK00000"
SCRIPT = "findServers"
SCRIPTNAME = SCRIPT+".py"

fileh = None
variableEntriesDic = {}


def isSet(var):
    if var != None and var != "":
        return 1
    return 0


def toList(adminconfig_result):
    return adminconfig_result.splitlines()


def isPropSet(prop_name):
    prop_value = System.getProperty(prop_name)
    if prop_value == None:
        return 0
    return 1


def join (lst, separator=" "):
    strg = ""
    for element in lst:
        strg += str(element) +separator
    strg = strg[0:len(strg)-len(separator)]
    return strg


def wsadminToList(inStr):

    if inStr.startswith("[") and inStr.endswith("]"):
        outList = inStr[1:-1].split() #splits space-separated lists,
    else:
        debug("*Warning*  Invalid input to wsadminToList: %s" % repr(inStr))
        outList = [inStr]

    return outList


def getVariableEntriesMap(nodename):
    '''
        Get the variable entries map for a given node
    '''

    global variableEntriesDic

    # Check if we need to get a new variable map

    # Build a variable map array for the node if doesn't exist
    if nodename not in variableEntriesDic.keys():
        debug("need to build variableEntriesMap for node %s" % nodename)
        variableEntries = toList(AdminConfig.getid("/Node:"+nodename+"/VariableMap:/VariableSubstitutionEntry:/" ))
        variableEntriesMap={}
        for entry in variableEntries:
            # Skip if not VariableSubstitutionEntry
            position = entry.find("#VariableSubstitutionEntry_")
            if position == -1: continue
            try:
                varName = AdminConfig.showAttribute(entry, "symbolicName" )
            except:
                type, value, tbck = sys.exc_info()
                debugUtil.logWarning("No symbolicName for node entry %s . Error: %s" % (entry, str(value)))
                continue
            try:
                varValue = AdminConfig.showAttribute(entry, "value" )
            except:
                type, value, tbck = sys.exc_info()
                debugUtil.logWarning("No value for node entry %s .  Error: %s " % (str(entry), str(value)))
                continue
            variableEntriesMap[varName]=varValue
        variableEntriesDic[nodename] = variableEntriesMap
    else:
        debug("variable entry map for node %s already exists" % nodename)
        variableEntriesMap = variableEntriesDic[nodename]

    return variableEntriesMap


def subsNodeVariable (varstring, nodename):
    '''
        Procedure subsNodeVariable
        Arguments: variable string, node name
        This does recursive replacement of node variables.
    '''

    debug("Entry subsNodeVariable %s - string=%s nodename=%s" % (fixlev, varstring, nodename))

    re_sub = re.compile(r"\${[^\${}]+}")
    variableEntriesMap = getVariableEntriesMap(nodename)

    # Find all replacement strings and substitute
    while 1:
        if varstring is None: break
        m = re_sub.search(varstring)
        if m is None: break
        varname = m.group()[2:-1]

        # Find replacement value and insert into string
        if variableEntriesMap.get(varname, -1) == -1:
            debug("Unable to locate ${%s}" % varname)
            varsub = ""
        else:
            varsub = variableEntriesMap[varname]

        debug("    Replace \"${%s}\" -> \"%s\"" % (varname, varsub))
        varstring = varstring[:m.start()] + varsub + varstring[m.end():]

    debug("Exit subsNodeVariable, result=%s" % varstring)
    return varstring


def getServerProcessEnvironmentValue (name, serverid, processtype, defaultvalue):
    '''
        Procedure getServerProcessEnvironmentValue
        Arguments: property name, server id, process type, default value
    '''

    debug("Entry getServerProcessEnvironmentValue %s" % fixlev)
    debug("name = %s server ID = %s processtype =%s defaultvalue =%s" % (name, serverid, processtype, defaultvalue))

   # Find the process whose server type matches (Control/Servant/Adjunct)
    javaprocessdef = ""
    varValue = ""
    try:
        processids = toList(AdminConfig.showAttribute(serverid, "processDefinitions" ))
        for processid in processids:
            pos = processid.find("#JavaProcessDef_")
            if pos == -1:
                debug("Rejected ProcessDef: %d" % (processid))
                continue
            try:
                adminProcessType = AdminConfig.showAttribute(processid, "processType")
                if processtype == adminProcessType:
                    javaprocessdef = processid
                    debug("Found %s process: %s" % (processtype, processid))
                    break
            except:
                adminProcessType = None
                type, value, tbck = sys.exc_info()
                debug("Unexpected exception: %s" %  value)

        # Find the environment property whose name matches
        if isSet(javaprocessdef):
            try:
                propertyids = toList(AdminConfig.showAttribute(javaprocessdef, "environment"))
                for propertyid in propertyids:
                    debug("Env prop ID = %s"  % (propertyid))
                    try:
                        varName = AdminConfig.showAttribute(propertyid, "name" )
                        debug("name = %s" % (varName))
                        if name == varName:
                            debug("name %s found in id %s " % (name, propertyid))
                            try:
                                varValue = AdminConfig.showAttribute(propertyid, "value" )
                                debug("Value is '%s' " % varValue)
                                break
                            except:
                                type, value, tbck = sys.exc_info()
                                debugUtil.logWarning("No value for JavaProcessDef Environment property %s . Error: %s" % (propertyid, value))
                                continue
                    except:
                        type, value, tbck = sys.exc_info()
                        debugUtil.logWarning("No name for JavaProcessDef Environment property %s . Error: %s" % (propertyid, value))
                        continue
            except:
                type, value, tbck = sys.exc_info()
                debug("Invalid WAS Java Process Def %s . Error: %s" % (javaprocessdef, value))
        else:
            debug("No JavaProcessDef of type %s found " % processtype)
    except:
        type, value, tbck = sys.exc_info()
        debugUtil.logException("Error in getServerProcessEnvironmentValue %s" % value)
    if not isSet(varValue):
        debug("No %s environment variable found, default set" % name)
        varValue = defaultvalue

    debug("Exit getServerProcessEnvironmentValue %s='%s'" % (name, varValue))
    return varValue


def subServerVariables(serverid, nodename, varstring):
    '''
        searches for variables inside a string and replaces them with an actual value
        based on node level variables or server level variables
    '''

    # To do: Cache server substitutions as we do node substitutions.
    #        When we do, reverse lookup order so that server overrides node.

    global variableEntriesDic

    debug("Entry subServerVariables %s server=%s node=%s varstring=%s" % (fixlev, serverid, nodename, varstring))

    variableEntriesMap = variableEntriesDic[nodename] or []

    # Find all replacement strings and substitute
    re_sub = re.compile(r"\${[^\${}]+}")
    while 1:
        if varstring is None: break
        m = re_sub.search(varstring)
        if m is None: break
        varname = m.group()[2:-1]

        # Find replacement value and insert into string
        varsub = getSubstitutionEntryValue(serverid, varname)
        if varsub == None:
            if variableEntriesMap.get(varname, -1) == -1:
                debug("Unable to locate node variable ${%s}, checking server variables" % varname)
                varsub = ""
            else:
                varsub = variableEntriesMap[varname]

        debug("Replace \"${%s}\" -> \"%s\"" % (varname, varsub))
        varstring = varstring[:m.start()] + varsub + varstring[m.end():]

    debug("Exit subServerVariable %s" % varstring, var)
    return var


def hasSubstitutionEntryValue(serverid, var_list):

    # To do: Cache server substitutions as we do node substitutions

    debug("Entry hasSubstitutionEntryValue %s - serverid=%s var_list=%s" % (fixlev, serverid, var_list))

    try:
        variables = toList(AdminConfig.list("VariableSubstitutionEntry", serverid))
        for variable in variables:
            getVarName = AdminConfig.showAttribute(variable, "symbolicName")
            debug("varName %s" % (getVarName))
            if getVarName in var_list:
                varValue = AdminConfig.showAttribute(variable, "value" )
                debug("Exit hasSubstitutionEntryValue, varName %s is set" % (getVarName))
                return varValue
    except:
        debugUtil.logException("Error in hasSubstitutionEntryValue")

    debug("Exit hasSubstitutionEntryValue, no variable from the list %s is set" % var_list)
    return None


def getSubstitutionEntryValue(serverid, var_name):

    debug("Entry getSubstitutionEntryValue %s - %s %s" % (fixlev, serverid, var_name))

    try:
        variables = toList(AdminConfig.list("VariableSubstitutionEntry", serverid))
        for variable in variables:
            getVarName = AdminConfig.showAttribute(variable, "symbolicName")
            debug("varName %s" % getVarName)
            if getVarName == var_name:
                varValue = AdminConfig.showAttribute(variable, "value" )
                debug("Exit getSubstitutionEntryValue, varName %s is set" % (getVarName))
                return varValue
    except:
        debugUtil.logException("Error in getSubstitutionEntryValue")

    debug("Exit getSubstitutionEntryValue, var %s is not set" % var_name)
    return None


def getServerPropertyValue(serverid, property_name):

    debug("Entry getServerPropertyValue %s - %s %s" % (fixlev, serverid, property_name))

    prop_value = None
    try:
        properties = toList(AdminConfig.list("Property", serverid ))
        for property in properties:
            property = property.strip()
            if property == property_name:
                prop_value = AdminConfig.showAttribute(property, "value")
                break
    except:
        debugUtil.logException("Error in getServerPropertyValue")

    debug("Exit getServerPropertyValue, propValue=%s" % prop_value)
    return prop_value


def getPropertyValue(jvmId, property_name):

    debug("Entry getPropertyValue %s - %s %s" % (fixlev, jvmId, property_name))

    propValue = None
    try:
        sysPropIds = toList(AdminConfig.showAttribute(jvmId.strip(), "systemProperties"))
        print "sysPropIds=%s" % sysPropIds
        for propids in sysPropIds:
            propids = wsadminToList(propids)
            debug("%d properties to examine" % len(propids))
            for propid in propids:
                debug("Property ID = %s" % propid)
                index = propid.find("#Property_")
                if index == -1: continue
                propName = AdminConfig.showAttribute(propid, "name")
                debug("propName=%s" % propName)
                if propName == property_name:
                    propValue = AdminConfig.showAttribute(propid, "value" )
                    debug("Exit getPropertyValue, propValue=%s" % propValue)
                    return propValue
    except:
        debugUtil.logException("Error in find servers")

    debug("Exit getPropertyValue, propName=%s not found" % property_name)
    return None


def write_to_findservers_dat(record):
    '''
       Write a record into findServers.dat

       @param record - Record to be written
    '''

    global fileh

    debug("Record to be written to: %s" % (record))
    fileh.write(record)
    fileh.write(os.linesep)
    fileh.flush()


##############################################################
#
#  Main
#
##############################################################

errorCode = ""
errorInfo = ""
sysPropIds = []
progress_obj = None
wasprofile = "None"

errorList = []

# Get some basic properties for the file
# Local cell and node may be overridden by node scan
# Local server may be specified for selection purposes
local_node = System.getProperty("local.node") or ""
local_cell = System.getProperty("local.cell") or ""

itcam_server = System.getProperty("itcam.server") or ""

serverversion = "None"
maxHeapSize = -1
verboseModeGC = "false"
verbosegclog = "None"
isConfigured = "None"
isWPS = "None"
java_version = "None"
itcam_dcversion = "None"

stdoutFilename = "None"
stderrFilename = "None"

global variableEntriesDic
variableEntriesDic = {}

debugUtil.set_logging()

# Initialize the resource bundle

resourcebundle.set_resourcebundle()

# Instantiate the global constants class

common_constants = constants()

# Put out eyecatcher

idline = "#  "+SCRIPT+" for "+VERSIONSTRING + " " + strftime("%Y-%m-%d %H:%M:%S") + "  #"
blanks = "#" + " "*(len(idline)-2) + "#"
hashes = "#"*len(idline)

print
print hashes
print blanks
print idline
print blanks
print hashes
print

# Validate parameter list

print "\narguments count=%d" % (len(sys.argv))
if len(sys.argv) != 1:
    print "Usage: wsadmin -f "+SCRIPTNAME+" <filename>"
    sys.exit(-1)

debug(SCRIPTNAME+" begun")

# Note: Previously we used the itcam.nolist property to abbreviate processing, but
# this has been changed for PK66873.  We now ignore this flag because we may need
# to extract the location of the install roots for all nodes.

# Open the output file and write the version record
# Note: This version will only change when the findServers output data
#       format changes.  It is used by configDataCollector and
#       unconfigDataCollector to verify they are processing the
#       data correctly.

filename = sys.argv[0]
debug("filename=%s" % (filename))


if filename != "":
    print "Input files "+filename

    try:

        # Open the progress file, and read the location of the dat file for saving the
        # servers information

        inputlist_file_loc = filename
        progress_obj = Progress(common_constants.FINDSERVERS, inputlist_file_loc)
        inDesc = progress_obj.open_progressfile()

        # Read the findservers record which has the profile name and dat file location
        # separated by a "^".

        record = inDesc.readline()
        record = record.strip()
        record_entries = record.split("^")
        if len(record_entries) == 2:
            wasprofile = record_entries[0]
            dat_fileloc = record_entries[1]
        else:
            debug("dat record entry does not have required entries - %s" % (record_entries))
            raise

        debug("Dat file location is: %s" % (dat_fileloc))
        inDesc.close()

        # Open the dat file.

        fileh = codecs.open(dat_fileloc, mode="w", encoding="utf-8")
        write_to_findservers_dat("#findServers.version=7.3")

    except Exception, errmsg:
        debug("Error occurred while executing findserver - %s" % (errmsg))
        debugUtil.logException("Unexpected Error: %s . %s ended" % (str(sys.exc_info()[0]), SCRIPTNAME))

        # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.

        if progress_obj != None:
            msgl =[]
            msgl.append(errmsg)
            msg = getmsg("FINDSERVERS_ERROR", "Error occurred while processing findservers - {0}", msgl)
            progress_obj.write_progress(msg)
        sys.exit(1)

else:
    errorCode = None
    raise RuntimeError, "Input file not supplied. findservers abandoned"

loaded_modules = dir()
if "AdminConfig" in loaded_modules and not "AdminTask" in loaded_modules:
    msg = getmsg("CFG5046E", "AdminTask object is required to complete the configuration, but is unavailable. Unable to continue.")
    progress_obj.write_progress(msg)
    sys.exit(1)

if isSet(itcam_server):
    print "  Searching for server: %s" % (itcam_server)

try:

    # Write the start message for the console in the progress file.

    progress_obj.write_progress_start(wasprofile)

    debug("Getting DMGR RMI host and port")
    host = ""
    port = ""
    cell_manager = ""
    dmgr = ""

    try:
        cell_manager = AdminConfig.getid("/CellManager:/" )
        debug("Cell Manager: %s" % cell_manager)
        if len(cell_manager) == 0:
            # Cell manager name is blank for stand alone server
            debug("DMGR RMI port not defined for stand alone server")
        else:
            dmgr = AdminConfig.getid("/Server:dmgr/" )
            debug("dmgr id=%s" % dmgr)
            if dmgr == None or dmgr == "":
                for server in AdminConfig.list("Server").split(os.linesep):
                    if AdminConfig.showAttribute(server,"serverType") == "DEPLOYMENT_MANAGER": dmgr = server
            nameserver = AdminConfig.list("NameServer", dmgr )
            endpoint = AdminConfig.showAttribute(nameserver, "BOOTSTRAP_ADDRESS" )
            host = AdminConfig.showAttribute(endpoint, "host" )
            port = AdminConfig.showAttribute(endpoint, "port" )
            debug("DMGR RMI host="+str(host)+" port="+str(port))
    except:
        type, errmsg , tbck = sys.exc_info()
        debug("error while getting cellmanager, dmgr host and port %s" % errmsg)

    write_to_findservers_dat("#bootstrap.address=%s  %s" % (host, port))

    # We now do three passes looking for local servers.  Once a pass
    # is completed that returns some servers we are done.
    #
    #   0. Use the cell, node, and server (if passed to us) to select
    #      the servers to return.
    #   1. We attempt to match the (unqualified) host name or IP, and
    #      lookup the node's host and match the IP address to determine
    #      the local node.
    #   2. We display servers from all nodes without trying to determine
    #      the local node.
    #
    # This is way more complex than it ought to be.  The problem is some
    # WAS configurations have been identified that have local wsadmin.sh
    # scripts for APPSERVER that won't run.  For this reason we have to
    # allow the DMGR wsadmin to be chosen, which implies a node scan.
    # If we didn't have to provide for this possibility much of the
    # following code could be eliminated.

    # Last node controls extraction of node variables
    last_nodename = ""
    server_count = 0
    pass_cnt = 0  #forStart
    while  pass_cnt <= 2 :  #forTest
        debug("Pass " + str(pass_cnt))
        # We use "if" instead of "switch" so that we can continue to next pass
        if pass_cnt == 0:
            # Get local node and cell name as specified in setupCmdLine.sh.
            # However, they may not be useable, for example if the node name
            # is for the DMGR instead of the APPSERVER.
            debug("Local cell = " + str(local_cell))
            debug("Local node = " + str(local_node))
            debug("Local server = "+str(itcam_server))
            if not isSet(local_node):
                debug("No local node specified, pass "+str(pass_cnt)+" abandoned")
                continue

            if not isSet(local_cell):
                debug("No local cell specified, pass "+str(pass_cnt)+" abandoned")
                continue

            cells = toList(AdminConfig.getid("/Cell:"+local_cell+"/" ))

        if pass_cnt == 1:
            debug("Getting cells, nodes and servers")
            cells = toList(AdminConfig.getid("/Cell:/" ))

            # Get local host name without domain name
            hostname = System.getProperty( "itcam.localhost" )
            hostaddr = System.getProperty( "itcam.localip" )
            socket_hostname=socket.gethostname()
            if not isSet(hostname):
                try:
                    inet = InetAddress.getLocalHost()
                    hostname = inet.getCanonicalHostName()
                    debug("Hostname=" + hostname)
                except:
                    _type_, _value_, _tbck_ = sys.exc_info()
                    ermsg = str(_value_)
                    debugUtil.logWarning("InetAddress.getLocalHost failed: %s" % ermsg)
            if not isSet(hostaddr):
                try:
                    inets = InetAddress.getAllByName(hostname)
                    hostaddr = inets[0].getHostAddress()
                except:
                    type, value, tbck = sys.exc_info()
                    debugUtil.logWarning("InetAddress.getHostAddress failed: %s" % (str(value)))
            shostname = hostname.split("." )[0].lower()
            debug("Short hostname=" + str(shostname) + "  Hostname=" + str(hostname))

            # Locate all IP addresses for the local host
            if not isSet(hostaddr):
                try:
                    inethosts = InetAddress.getAllByName(hostname)
                    hostaddrs = []
                    i = 0
                    while i < len(inethosts):
                        hostaddrs.append(inethosts[i].getHostAddress())
                        i += 1
                except:
                    _type_, _value_, _tbck_ = sys.exc_info()
                    debugUtil.logWarning("InetAddress.getAllByName(" + hostname+ ") failed: " + str(_value_))
                    hostaddrs = [hostaddr]
            else:
                hostaddrs = [hostaddr]
            debug("All Host IP=" + str(hostaddrs))

        debug("Number of cells: %d" % (len(cells)))
        for cell in cells:
            # Skip ID if not for a Cell
            hasCell = cell.find("#Cell_")
            if hasCell == -1:  continue
            cellname = AdminConfig.showAttribute(cell, "name")
            celltype = AdminConfig.showAttribute(cell, "cellType")
            debug("Cell: %s  Type: %s" % (cellname, celltype))

            # If this is a distributed cell but we can't find the deployment manager,
            # something is seriously wrong.  We have seen this situation when -host specifies
            # the host name or port of a server instead of the deployment manager.
            # In that situation, don't put out any servers for this (the only) cell.
            if celltype== "DISTRIBUTED":
                debug("NodeSync check")
                dmgrName = AdminControl.queryNames("type=DeploymentManager,*")
                debug("dmgr = %s" % (str(dmgrName)))
                if len(dmgrName) == 0:
                    debugUtil.logError("Missing deployment manager in distributed cell " + str(cellname) + " Verify the deployment manager host name and port.")
                    continue

            if pass_cnt == 0:
                nodes = toList(AdminConfig.getid("/Cell:"+cellname+"/Node:"+local_node+"/" ))
                nodename = local_node
            else:
                nodes = toList(AdminConfig.getid("/Cell:"+cellname+"/Node:/" ))

            debug("Number of nodes in '" + cellname + "': " + str(len(nodes)))
            for node in nodes:
                # Skip if not Node
                hasNode = node.find("#Node_")
                if hasNode == -1:  continue

                #initialize node parameters
                serverversion = "None"
                java_version = "None"

                nodename = AdminConfig.showAttribute(node, "name")
                # Ignore all non-local nodes that do not match the first qualifier
                # but if it's in IP format, we also check against the local IP addresses
                if pass_cnt == 0:
                    debug("Node: %s" % nodename)
                    local_node_flag = 1
                else:
                    nodehostname = AdminConfig.showAttribute(node, "hostName").lower()
                    debug("Node: %s Host: %s" % (nodename, nodehostname))
                    local_node_flag = 0

                # For pass 2, give up and just list all nodes no matter what
                if pass_cnt == 2:
                    debug("All nodes accepted for Pass " + str(pass_cnt))
                    local_node_flag = 1

                # Check unqualified hostnames

                if  not local_node_flag:
                    snodehostname = nodehostname.split("." )[0]
                    debug("Local Host: %s nodehostname: %s" % (shostname, snodehostname))
                    if isSet(shostname) and snodehostname== shostname:
                        debug("Unqualified hostnames match: %s=%s" % (snodehostname, shostname))
                        local_node_flag = 1

                # Check if node hostname specified is an IP address that
                # matches one of our local IP addresses

                if  not local_node_flag:
                    pattern = "[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+"
                    if re.match(pattern,nodehostname):
                        debug(" IP node host: %s " % (nodehostname))
                        for ip in hostaddrs:
                            if ip!= "" and nodehostname == ip:
                                debug("IP node hostname %s matches local IP" % (nodehostname))
                                local_node_flag = 1
                                break

                # Resolve all IP addresses for the node host and compare to local host's
                if  not local_node_flag:
                    # Attempt to resolve the node's hostname
                    try:
                        inetnodes = InetAddress.getAllByName(nodehostname)
                        nodeaddrs = []
                        nodehosti = 0  #forStart
                        while  nodehosti < len(inetnodes):
                            inetnode = inetnodes[nodehosti]
                            nodeaddrs.append(inetnode.getHostAddress())
                            nodehosti += 1
                    except:
                        type, value, tb = sys.exc_info()
                        debug("*Warning*  getAllByName failed for %s Error: %s " % (nodehostname, value))
                        nodeaddrs = []

                    # If any IP address match, we presume we have a local node

                    for nodeaddr in nodeaddrs:
                        debug("Node IP: %s " % (nodeaddr))
                        for hostaddr in hostaddrs:
                            debug(" Host IP: %s" % (hostaddr))
                            if hostaddr!= "" and nodeaddr == hostaddr:
                                debug("Resolved IP %s for %s matches local IP" % (nodeaddr, nodehostname))
                                local_node_flag = 1
                                break
                        if local_node_flag: break

                if  not local_node_flag: continue

                # Allow specified server to limit the selection until the last pass
                # when we have abandoned all hope of being selective.
                print "itcam_server=%s\n" % (str(itcam_server))
                debug("itcam server %s" % (str(itcam_server)))

                if pass_cnt == 2 and itcam_server != "" :
                    debug("  Pass %s, server selection bypassed" % (pass_cnt))
                    itcam_server = ""

                print "cell-name=%s nodename=%s servername=%s\n" % (cellname, nodename, itcam_server)
                servers = toList(AdminConfig.getid("/Cell:"+cellname+"/Node:"+nodename+"/Server:"+itcam_server+"/" ))
                debug("Number of servers in %s = %d " % (nodename, len(servers)))

                for server in servers:

                    debug("Server ID: %s " % (server))
                    #initialize properties for each server
                    maxHeapSize = -1
                    verboseModeGC = "false"
                    verbosegclog = "None"
                    isConfigured = "None"
                    subProduct = "None"
                    itcam_dcversion = "None"
                    stdoutFilename = "None"
                    stderrFilename = "None"
                    sysPropIds = []
                    is_ve = "false"
                    is_ecam = "false"

                    # Get the server name
                    try:
                        servername = AdminConfig.showAttribute(server, "name" )
                    except:
                        _type_, _value_, _tbck_ = sys.exc_info()
                        debugUtil.logWarning("Server name access failed for server '%s' Error %s" % (server, str(_value_)))
                        continue
                    # Get the server type
                    try:
                        servertype = AdminConfig.showAttribute(server, "serverType" )
                    except:
                        _type_, _value_, _tbck_ = sys.exc_info()
                        debugUtil.logWarning("Server type access failed for server '%s' Error: %s" % (server, str(_value_)))
                        continue
                    debug("Server: %s Type: %s" % (servername, servertype))

                    # Ignore server types other than application server
                    if servertype != "APPLICATION_SERVER":
                        debug("Type is '%s' for server '%s' ignored" % (servertype, servername))
                        continue
                    if serverversion == "None":
                        try:
                            server_props = AdminControl.queryNames("WebSphere:type=Server,*")
                            server_props_list = server_props.split(",")
                            for prop in server_props_list:
                                if prop.find("version=") > -1:
                                    serverversion = prop.split("version=")[1].strip()
                            debug("Server version: %s" % serverversion)
                        except:
                            server_props = ""
                            debugUtil.logException("Unable to get version")

                    #get JVM version for WAS 8 and later
                    if serverversion[0] == "8" and java_version == "None":
                        debug("that is WAS 8, trying to get jvm version")
                        try:
                            objStr = "cell=%s,node=%s,type=Server,*" % (cellname, nodename)
                            debug("objStr=%s" % objStr)
                            serverObj=AdminControl.completeObjectName(objStr)
                            serverVersionList = toList(AdminControl.getAttribute(serverObj,"serverVersion"))
                            #Installed Features    IBM 32-bit SDK for Java, Version 6
                            startFlag = 0
                            for line in serverVersionList:
                                line = line.strip()
                                debug("line='%s'" % line)
                                if line.find("Installed Features") > -1:
                                    startFlag = 1
                                if startFlag == 1:
                                    java_index = line.find("Java")
                                    version_index = line.find("Version")
                                    if java_index > -1 and version_index > java_index:
                                        java_version = line.split("Version")[1].strip()
                                        debug("java_version=%s" % java_version)
                                        break
                        except:
                            type, value, tbck = sys.exc_info()
                            debugUtil.logException("Unable to determine jvm version %s" % value)
                    # Is this a new node?
                    if last_nodename != nodename:
                        last_nodename = nodename
                        print " "
                        print "  Node: "+nodename
                        print " "

                        # If we have identified the local node, verify the cell and node names
                        if local_cell != cellname:
                            debug("Local cell name overridden from "+local_cell+" to "+cellname)
                            local_cell = cellname

                        if local_node != nodename:
                            debug("Local node name overridden from "+local_node+" to "+nodename)
                            local_node = nodename

                        # Identify the cell and node
                        write_to_findservers_dat("#was.cell=%s" % (cellname))
                        write_to_findservers_dat("#was.node=%s" % (nodename))
                        #print >>fileh, "#was.cell=%s" % (cellname)
                        #print >>fileh, "#was.node=%s" % (nodename)

                        # We find the server policy, plugins directory and versionInfo
                        # script from the node variable properties.
                        write_to_findservers_dat("#was.install.root=%s" % (subsNodeVariable("${WAS_INSTALL_ROOT}",nodename)))
                        write_to_findservers_dat("#user.install.root=%s" % (subsNodeVariable("${USER_INSTALL_ROOT}",nodename)))
                        write_to_findservers_dat("#was.plugins.root=%s" % (subsNodeVariable("${WAS_INSTALL_ROOT}",nodename)+"/plugins"))
                        write_to_findservers_dat("#was.config.root=%s" % (subsNodeVariable("${USER_INSTALL_ROOT}", nodename)+"/config"))
                        write_to_findservers_dat("#was.props.dir=%s" % (subsNodeVariable("${WAS_PROPS_DIR}",nodename)))
                        write_to_findservers_dat("#was.log.root=%s" % (subsNodeVariable("${LOG_ROOT}",nodename)))
                        write_to_findservers_dat("#java.home=%s" % (subsNodeVariable("${JAVA_HOME}",nodename)))
                        write_to_findservers_dat("#wps.home=%s" % subsNodeVariable("${WPS_HOME}",nodename))

                        # To do: This too possibly should be checked similarly to wps.home above, if we knew what to check
                        write_to_findservers_dat("#wbi.install.root=%s" % (subsNodeVariable("${WBI_INSTALL_ROOT}",nodename)))

                    # If this is a distributed cell, we must verify that the
                    # Network Agent is alive.
                    # Configuration will fail if he is not running so we will leave
                    # with no servers returned.  (That is, we skip this server choice.)

                    if celltype== "DISTRIBUTED":
                        debug("       NodeSync deployment")
                        if AdminControl.completeObjectName("type=NodeSync,node="+nodename+",*" ) == "":
                            print "\n*Error*  Unable to locate NodeSync for node "+nodename+".\n"
                            print "         Make sure the Network Agent is running before proceeding"
                            print "         with any configuration on this node."
                            print "         Synchronization will fail if this is not corrected.\n"
                            errorCode = None
                            msg = getmsg("CFG1147E", "Unable to locate NodeSync for node {0}. Make sure nodeagent for the node is started.", [nodename])
                            progress_obj.write_progress(msg)
                            errorList.append(msg)
                            raise ConfigurationException(msg)

                    # Get server deployment type standalone is "true" or "false"

                    adminSvc = AdminConfig.list("AdminService", server )
                    debug("     Admin Svc: %s" % (str(adminSvc)))
                    if not isSet(adminSvc):
                        print "*Warning*  No administrative services available for "+server
                        if celltype == "STANDALONE":
                            print "           Standalone deployment will be assumed."
                            standalone = "true"
                        else:
                            print "           Network deployment will be assumed."
                            standalone = "false"
                    else:
                        standalone = AdminConfig.showAttribute(adminSvc, "standalone" )
                        if ( celltype == "STANDALONE" and standalone == "false")  or ( celltype == "DISTRIBUTED" and standalone == "true") :
                            print "*Warning*  cell type "+celltype+" but standalone attribute is "+standalone
                    debug("standalone=%s celltype= %s " % (standalone,celltype))

                    # This alternative method works and is simpler, but it takes
                    # excessive time to go through all the properties.  It is retained
                    # against future changes in WAS.

                    debug("Getting JVM Mode --- does NOT work on distributed")
                    # find the bitMode of the JVM for a given server
                    # if getJVMMode function does not exist invoke the getServerProcessEnvironmentValue function
                    if platformUtil.IS_zOS_OS():
                        try:
                            params = ["-nodeName \"%s\" -serverName \"%s\"" % (nodename, servername)]
                            debug(" getJVMMode params = %s" % params)
                            jvmmode = AdminTask.getJVMMode(params)
                            debug(" getJVMMode function found")
                        except:
                            _type_, _value_, _tbck_ = sys.exc_info()
                            debug(" getJVMMode function not found ")
                            debug(" Trying getServerProcessEnvironmentValue function")
                            jvmmode = getServerProcessEnvironmentValue("was.com.ibm.websphere.zos.jvmmode", server, "Control", "31bit" )
                        debug("JVM Mode: %s" % (jvmmode))
                    elif platformUtil.IS_OS400_OS():
                        try:
                            debug(" getting SDK information on i5")
                            debug(" getServerSDK params: nodename=%s, servername=%s" % (nodename,servername))

                            rawjvmmode = AdminTask.getServerSDK("-nodeName \"%s\" -serverName \"%s\"" % (nodename, servername))
                            jvmmode = serverSDK[-5:-3]

                            debug(" jvmmode=" + jvmmode)
                            debug(" getServerSDK function found")
                        except:
                            debug(" getServerSDK function not found")
                            debug(" will attempt using System.getProperty")
                            jvmmode = System.getProperty("com.ibm.vm.bitmode") or  System.getProperty("sun.arch.data.model") or "None"

                        debug("JVM Mode: %s" % (jvmmode))
                    else:
                        jvmmode = System.getProperty("com.ibm.vm.bitmode") or  System.getProperty("sun.arch.data.model") or "None"

                    # Determine if there are any -Xshareclasses:...modified=... JVM arguments
                    # Walk over Java Process Definitions looking for servant region
                    # Find the JavaVirtualMachine object ID

                    jvmID = ""
                    if not platformUtil.IS_zOS_OS():    #we are on distributed
                        jvmID = AdminConfig.list("JavaVirtualMachine", server)
                        debug("JVM ID: %s " % (jvmID))

                    processDefs = toList(AdminConfig.list("JavaProcessDef", server ))
                    for process in processDefs:
                        debug(" getProcessDefs process = %s" % (process))
                        # z/OS Has multiple ProcessDefs, Control, Servant, Adjunct(V6.0 only)
                        type = AdminConfig.showAttribute(process, "processType" )
                        debug(" type = %s" % (type))

                        if type == "Servant" or not platformUtil.IS_zOS_OS():
                            debug("   getProcessDefs is Servant %s" % process)
                            if platformUtil.IS_zOS_OS():
                                # Note that WAS 5.1 would fail here if "list JavaVirtualMachine $id" were used
                                jvmID = AdminConfig.showAttribute(process, "jvmEntries" )
                                rex = re.compile("\[(.*)\]")
                                jvmID = rex.split(jvmID)[1]
                            break
                        debug("servant JVM ID: %s " % (str(jvmID)))

                    if  jvmID == "" :
                        raise Exception ("Unable to locate JVM or Servant region JVM for $servername, configuration abandoned")
                    else:
                        maxHeapSize = AdminConfig.showAttribute(jvmID, "maximumHeapSize")
                        debug(" maxHeap = %s" % (maxHeapSize))
                        verboseModeGC = AdminConfig.showAttribute(jvmID, "verboseModeGarbageCollection").strip()
                        debug(" verboseModeGC = %s" % (verboseModeGC))
                        #check java.security.policy property to verify if we are on MS
                        itcamdchome_set = hasSubstitutionEntryValue(server, ["ITCAMDCHOME", "ITCAM61HOME"])
                        if itcamdchome_set != None:
                            isConfigured = "DC"

                        dcversion = getSubstitutionEntryValue(server, "ITCAMDCVERSION")
                        if dcversion != None:
                            itcam_dcversion = str(dcversion)

                        itcam_dc_enabled = getServerPropertyValue(server, "ITCAM_DC_ENABLED")
                        if itcam_dc_enabled != None and itcam_dc_enabled == "true":
                            isConfigured = "ECAM"

                        if isConfigured == "None":
                            security_policy = getServerPropertyValue(server, "java.security.policy")
                            if security_policy != None and security_policy.find("ve.policy") > -1:
                                isConfigured = "VE"

                    shareclassesmodified = "false"
                    try:
                        if isSet(jvmID):
                            shareclassesmodified = "false"
                            jvmArgs = AdminConfig.showAttribute(jvmID, "genericJvmArguments").split()
                            for arg in jvmArgs:
                                debug("JVM arg = %s" % (arg))
                                if arg.startswith("-Xshareclasses:"):
                                    debug("-Xshareclasses:")
                                    if arg.find("modified=") > -1:
                                        debug("modified=")
                                        shareclassesmodified = "true"
                                elif arg.startswith("-Xverbosegclog:"):
                                    verbosegclog = arg.split("-Xverbosegclog:")[1].split(",")[0]
                    except:
                        type, value, tbck = sys.exc_info()
                        debug("Unable to get genericJvmArguments Error: %s" % (value))
                    debug("shareclassesmodified = %s " % (shareclassesmodified))

                    # Determine if this is a Portal server
                    # To do: Extend to WBI as well using the same field.
                    if hasSubstitutionEntryValue(server, ["WPS_JVM_ARGUMENTS_EXT"]) is not None: subProduct = "WPS"

                    # put out data.  Note that the server ID must be the last thing on the line.  If the
                    # output line changes, findServers.version above should also be changed.

                    server_count = server_count+1
                    if pass_cnt < 2:
                        print "%5i) %s" % (server_count, servername)
                    else:
                        print "%5i) %7s  Hostname = %s" % (server_count, servername, nodehostname)

                    # Positional data for each server:
                    #
                    #   0. Cell name
                    #   1. Node name
                    #   2. Server name
                    #   3. Standalone or Distributed
                    #   4. JVM bitness
                    #   5. Shared classes
                    #   6. Server version
                    #   7. Maximum heap size
                    #   8. Verbose GC log
                    #   9. ITCAM DC configured
                    #  10. Java version
                    #  11. ITCAM DC version
                    #  12. Portal or other subproduct
                    #  13. Server ID - Always last

                    write_to_findservers_dat("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (cellname,
                                                                                                         nodename,
                                                                                                         servername,
                                                                                                         standalone,
                                                                                                         jvmmode,
                                                                                                         shareclassesmodified,
                                                                                                         serverversion,
                                                                                                         maxHeapSize,
                                                                                                         verbosegclog,
                                                                                                         isConfigured,
                                                                                                         java_version,
                                                                                                         itcam_dcversion,
                                                                                                         subProduct,
                                                                                                         server))

        # If we identified any local servers, no more passes are needed
        print "server_count=%d" % (server_count)
        if server_count > 0: break
        pass_cnt += 1
    if len(errorList) > 0:
        for error in errorList:
            write_to_findservers_dat("#error=%s" % (error))
        debug("findServers finished with warnings: %s" % str(errorList))
        msgl = []
        msgl.append(wasprofile)
        msg = getmsg("FINDSERVERS_ERROR", "Error occurred while processing findservers - {0}", msgl)
        progress_obj.write_progress(msg)
    else:
        msgl =[]
        msgl.append(wasprofile)
        msg = getmsg("FINDSERVERS_SUCCESSFUL", "Finding servers done successfully for profile {0}", msgl)
        progress_obj.write_progress(msg)

    fileh.close()
    fileh = None

    # Write the finished message.

    progress_obj.write_progress_finish(wasprofile, common_constants.SUCCESSFUL)

    # Close the progress file

    if progress_obj != None:
        progress_obj.close_file()
        progress_obj = None

    if server_count == 0:
        debug("No local application servers discovered")

except:
    etype, value, tbck = sys.exc_info()
    debugUtil.logException("Failure in findServers %s" % value)
    debugUtil.logDumpStack()

    # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.

    if progress_obj != None:
        msg = getmsg("FINDSERVERS_ERROR", "Error occurred while processing findservers")
        progress_obj.write_progress(msg)
        progress_obj.write_progress_finish(wasprofile, common_constants.UNSUCCESSFUL)
        progress_obj.close_file()
    if fileh is not None:
        fileh.close()
    sys.exit(1)
