#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module sets the input.properties for a server or servers.  The
   input.properties template is read and the values are substituted, and the
   file is written into the runtime directory for the server.
'''

fixlev = "38001" # Last APAR, defect or story to update this module

import os
import codecs
import debugUtil as log
import platformUtil
import utils

from debugUtil import logDebug as debug
from configUtils import create_substitution_dictionary
from configUtils import create_substitution_dictionary_xd
from resourcebundle import getmsg
import shutil

template_dic = {}
template_filename = None

def read_template(pdl, templatename=None):
    '''
       Read the input.properties template and save each line in a
       list for later use.
       If running in a z/OS environment, the input.properties template
       is in EBCDIC.
    '''
    prop_entries = []

    debug("Entry read_template %s" % fixlev)

    xdtemp = utils.getData(pdl, "CREATETEMPLATE")
    libertyProfile = pdl["WLPINSTALLDIR"][0]
    if templatename == None:
        if xdtemp:
            filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "input.propertiesxd.template"))
        elif libertyProfile != None:  #using WAS Liberty template
            filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "input.propertieswlp.template"))
        else:
            filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "input.properties.template"))
    else:
        filename = templatename

    debug("template filename: %s" % filename)

    if platformUtil.IS_zOS_OS():

        debug("z/OS system to read input.properties in EBCDIC")
        input_properties_file = codecs.open(filename, mode="r", encoding="cp1047")
        prop_datas = input_propterties_file.readlines()
        input_properties_file.close()
        # In case this is a DOS file we remove the CR as well as the LF.
        prop_datas = [prop_data.rstrip("\r\n") for prop_data in prop_datas]

        #
        # Get each record by the new line.
        #   Ignore records that do not apply to z/OS
        #
        for prop_data in prop_datas:
            if prop_data.startswith("#@COMMENT@# server.maximumHeapSize"):
                debug("Dropping this line from read_template: %s" % prop_data)
                continue
            prop_entries.append(prop_data)

    else:

        input_properties_file = open(filename, "r")
        prop_datas = input_properties_file.readlines()
        input_properties_file.close()
        # In case this is a DOS file we remove the CR as well as the LF.
        prop_datas = [prop_data.rstrip("\r\n") for prop_data in prop_datas]

        for prop_data in prop_datas:
            if prop_data.startswith("#@COMMENT@# control.systemproperty=ITCAM_DC_ENABLED"):
                debug("Dropping this line from read_template: %s" % prop_data)
                continue
            prop_entries.append(prop_data)


    debug("Exit read_template -\n\n  %s" % "\n  ".join([ln.rstrip("\n ") for ln in prop_entries if ln.rstrip("\n ") != ""]))
    template_filename = filename
    return prop_entries


def replace_variables(serverlist, prop_entries, pdl):
    '''
        Replaces the variables in the template with the provided
        values for each server.

        @param serverlist - A tuple which each entry represents a server entry
        @param prop_entries - template input.properties as a list
    '''

    debug("Entry replace_variables %s" % fixlev)

    misses = {}
    replaced_prop_entries_dict = {}
    for serveri in range(len(serverlist)):
        serverentry = serverlist[serveri];
        debug("Replacement dictionary for %s -\n\n  %s" % (serveri,"\n  ".join(["%s : %s" % (ky,val) for ky, val in sorted(serverentry.items())])))
        replaced_prop_entries = []
        #
        # Examine the input properties entries.
        # Remove "#@COMMENT@" if present.
        # Replace any keys present.
        #
        lineno = -1
        tpv = utils.getData(pdl, "TPVLIST", serveri)
        for prop_entry in prop_entries:
            lineno = lineno+1
            #
            # Check the prop entry, and if it is ITCAM_DC_ENABLED, then based on the TPV
            # setting for the server, the property is set or just ignored.
            #
            if platformUtil.IS_zOS_OS():
                if tpv == "1":
                    if prop_entry.startswith("#@COMMENT@# server.systemproperty=ITCAM_DC_ENABLED"):
                        continue
                else:
                    if prop_entry.startswith("#@COMMENT@# server.systemproperty=ITCAM_DC_ENABLED"):
                        continue
                    elif prop_entry.startswith("#@COMMENT@# control.systemproperty=ITCAM_DC_ENABLED"):
                        continue
            else:
                if tpv == "0":
                    if prop_entry.startswith("#@COMMENT@# server.systemproperty=ITCAM_DC_ENABLED"):
                        continue

            #
            # Check for servername, cell, node, and profilename, and check it against the
            # modified one (they are modified in the case of non ascii to its ascii
            # presentations), and if they are equal then the original property is not
            # set.

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wascell=@{WAS_CELL}"):
                if serverentry["WAS_CELL"] == serverentry["WAS_CELL_MODIFIED"]:
                    continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wasprofile=@{PROFILE}"):
                if serverentry["PROFILE"] == serverentry["PROFILE_MODIFIED"]:
                    continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wasnode=@{WAS_NODE}"):
                if serverentry["WAS_NODE"] == serverentry["WAS_NODE_MODIFIED"]:
                    continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wasserver=@{SERVER}"):
                if serverentry["SERVER"] == serverentry["SERVER_MODIFIED"]:
                    continue

            if prop_entry.startswith("#@COMMENT@# "):
                prop_entry = prop_entry[len("#@COMMENT@# "):len(prop_entry)]

            prop_entry.rstrip()
            for key in serverentry.keys():
                #assert type(serverentry[key]) == type(""), "Non-string value in replacement dictionary entry for %s = %s" % (key,type(serverentry[key]))
                if serverentry[key] != None:
                    prop_entry = prop_entry.replace("@{"+key+"}",serverentry[key])

            replaced_prop_entries.append(prop_entry)

            #
            # Check each entry for variable that is not substituted, and put
            # a message in the log.
            #
            i = 0
            while True:
                i = prop_entry.find("@{",i)
                if i == -1:
                    break
                i = i + 2
                j = prop_entry.find("}",i)
                if j == -1:
                    break
                miss = prop_entry[i-2:j+1]
                misses[miss] = (lineno,prop_entry)

        #
        # Add the replaced prop list to the prop dictionary based on the
        # WAS version, Node Name, and Server name as VER.NODE.SERVER
        #
        inputkey = utils.getServerBasedFilePrefix(pdl, serveri)
        debug("inputkey: " + inputkey)
        replaced_prop_entries_dict[inputkey] = replaced_prop_entries

    if len(misses.keys()) > 0:
        print "\n%s" % getmsg("CFG1004E","Some substitution values were not replaced in the input.properties file:")
        for miss in misses.keys():
            print "    %s" % miss

    debug("Exit replace_variables")

    return replaced_prop_entries_dict


def replace_variables_xd(serverlist, prop_entries, pdl):
    '''
        Replaces the variables in the template with the provided
        values for xd template.

        @param serverlist - A tuple which each entry represents a server entry
        @param prop_entries - template input.properties as a list
    '''

    debug("Entry replace_variables_xd %s" % fixlev)

    misses = {}
    replaced_prop_entries_dict = {}
    platform = utils.getData(pdl, "TEMPLATEOS")
    for serveri in range(len(serverlist)):
        serverentry = serverlist[serveri];
        replaced_prop_entries = []
        #
        # Examine the input properties entries.
        # Remove "#@COMMENT@" if present.
        # Replace any keys present.
        #
        lineno = -1
        tpv = utils.getData(pdl, "TPVLIST", serveri)
        for prop_entry in prop_entries:
            lineno = lineno+1
            #
            # Check the prop entry, and if it is ITCAM_DC_ENABLED, then based on the TPV
            # setting for the server, the property is set or just ignored.
            #
            if platform is "zos":
                if tpv == "1":
                    if prop_entry.startswith("#@COMMENT@# server.systemproperty=ITCAM_DC_ENABLED"):
                        continue
                else:
                    if prop_entry.startswith("#@COMMENT@# server.systemproperty=ITCAM_DC_ENABLED"):
                        continue
                    elif prop_entry.startswith("#@COMMENT@# control.systemproperty=ITCAM_DC_ENABLED"):
                        continue
            else:
                if tpv == "0":
                    if prop_entry.startswith("#@COMMENT@# server.systemproperty=ITCAM_DC_ENABLED"):
                        continue

            #
            # Skip adding the following properties as they are not needed
            #
            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wascell=@{WAS_CELL}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wasprofile=@{PROFILE}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wasnode=@{WAS_NODE}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.orig.wasserver=@{SERVER}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.wascell=@{WAS_CELL_MODIFIED}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.wasprofile=@{PROFILE_MODIFIED}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.wasnode=@{WAS_NODE_MODIFIED}"):
                continue

            if prop_entry.startswith("#@COMMENT@# server.genericJvmArguments=-Dam.wasserver=@{SERVER_MODIFIED}"):
                continue

            if prop_entry.startswith("#@COMMENT@# "):
                prop_entry = prop_entry[len("#@COMMENT@# "):len(prop_entry)]
            prop_entry.rstrip()
            for key in serverentry.keys():
                #assert type(serverentry[key]) == type(""), "Non-string value in replacement dictionary entry for %s = %s" % (key,type(serverentry[key]))
                if serverentry[key] != None:
                    prop_entry = prop_entry.replace("@{"+key+"}",serverentry[key])

            replaced_prop_entries.append(prop_entry)


        #
        # There is only one set, so the key to the dictionary is xd.
        #        #
        replaced_prop_entries_dict["XD"] = replaced_prop_entries

    debug("Exit replace_variables_xd")

    return replaced_prop_entries_dict

def get_template_name(pdl, serveri):
    '''
        return name of the template file used for the server type
    '''
    debug("get_template_name Entry %s" % fixlev)
    is_wlp_server = False
    wlphome = utils.getData(pdl, "WLPHOME")[serveri]
    if wlphome != None and len(wlphome) > 0:
        is_wlp_server = True
    xdtemp = utils.getData(pdl, "CREATETEMPLATE")
    if xdtemp == True:
        filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "input.propertiesxd.template"))
    elif is_wlp_server:  #using WAS Liberty template
        filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "input.propertieswlp.template"))
    else:
        filename = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "itcamdc", "etc", "input.properties.template"))
    debug("get_template_name Exit %s" % filename)
    return filename


def create_prop_entries_dic_for_template(pdl, templatename):
    '''
        update template_dictionary with new template type key
    '''
    global template_dic

    debug("create_prop_entries_dic_for_template Entry %s - %s" % (fixlev,templatename))
    xdtemp = utils.getData(pdl, "CREATETEMPLATE")
    if xdtemp == True:
        servers = create_substitution_dictionary_xd(pdl)
    else:
        servers = create_substitution_dictionary(pdl)

    prop_entries = read_template(pdl, templatename)
    if xdtemp == True:
        replaced_serverlist_dict = replace_variables_xd(servers, prop_entries, pdl)
    else:
        replaced_serverlist_dict = replace_variables(servers, prop_entries, pdl)
    debug(replaced_serverlist_dict)
    template_dic[templatename] = replaced_serverlist_dict
    debug("create_prop_entries_dic_for_template Exit")
    return replaced_serverlist_dict


# The content of <server specific>.input.properties file will be written to config.log for debugging purpose.
def write_input_properties(pdl,replaced_prop_entries_dict):
    '''
       Runs thru the serverlist, and creates input.properties for each server.
       The input.properties file will reside in CONFIGHOME/runtime directory,
       its name uses the format: <was_cell_name>.<was_node_name>.<was_server_name>.input.properties
    '''

    global template_dic

    debug("Entry write_input_properties %s" % fixlev)

    runtimedir = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "runtime"))

    #
    # Write the input.properties in ASCII
    #
    serverlist = pdl["SERVERS"][0]

    for serveri in range(len(serverlist)):

        templatename = get_template_name(pdl, serveri)
        debug("templatename=%s" % templatename)
        if templatename in template_dic:
            debug("getting values from dictionary")
            replaced_prop_entries_dict = template_dic[templatename]
        else:
            debug("need to create new dic entry for template")
            replaced_prop_entries_dict = create_prop_entries_dic_for_template(pdl, templatename)

        xdtemp = utils.getData(pdl, "CREATETEMPLATE")
        if xdtemp:
            input_properties_file_prefix = "XD"
        else:
            input_properties_file_prefix = utils.getServerBasedFilePrefix(pdl, serveri)

        debug("input_properties_file_prefix: " + input_properties_file_prefix)

        input_properties_filename = runtimedir + "/" + input_properties_file_prefix + "input.properties"
        input_properties_backup = input_properties_filename + "_org"
        debug("input.properties file name: " + input_properties_filename)

        if os.path.exists(input_properties_filename):
            debug("creating backup file  of %s as %s" % (input_properties_filename, input_properties_backup))
            shutil.copyfile(input_properties_filename, input_properties_backup)

        value = replaced_prop_entries_dict[input_properties_file_prefix]
        input_properties_file = codecs.open(input_properties_filename, mode="w", encoding="utf-8")
        prop_data_tolog = []
        for prop_data in value:
            # Strip any line separation which may have been inserted.
            prop_data = prop_data.rstrip("\n")
            prop_data_tolog.append(prop_data)
            input_properties_file.write(prop_data+"\n")
            # Apparently OS/400 needs an extra line feed.  (Suspect this is bogus)
            if xdtemp:
                platform = utils.getDataString(pdl, "TEMPLATEOS")
                if platform.startswith("OS400"):
                    input_properties_file.write("\n")
            else:
                if platformUtil.IS_OS400_OS():
                    input_properties_file.write("\n")

        debug("===============================================")
        debug("Begin file %s\n\n  %s" % (input_properties_filename,"\n  ".join(prop_data_tolog)))
        debug("End of " + input_properties_filename)
        debug("===============================================")

        input_properties_file.close()
        debug("input.properties saved successfully in - " + input_properties_filename)

    debug("Exit write_input_properties")


def process_inputproperties(pdl):
    '''
       Create input.properties
       @param pdl - PDL reference
    '''

    debug("Entry process_inputproperties %s" % fixlev)

    global template_dic
    global template_filename

    try:

        #
        # Create the servers dictionary
        #
        xdtemp = utils.getData(pdl, "CREATETEMPLATE")
        if xdtemp:
            servers = create_substitution_dictionary_xd(pdl)
        else:
            servers = create_substitution_dictionary(pdl)

        #
        # Read the input.properties template and save the entries in a
        # list for later use.
        #
        prop_entries = read_template(pdl)
        #
        # Replace the server specific variables into the template, and
        # save the as input.properties under the runtime directory for
        # the processed server.
        #
        if xdtemp:
            replaced_serverlist_dict = replace_variables_xd(servers, prop_entries, pdl)
        else:
            replaced_serverlist_dict = replace_variables(servers, prop_entries, pdl)


        debug("Replaced server list -\n\n    %s" % "\n\n    ".join(["%s :\n\n        %s" % (k,"\n        ".join([ln.rstrip("\n ") for ln in e if ln.rstrip("\n ") != ""])) for k, e in replaced_serverlist_dict.items()]))
        template_dic[template_filename] = replaced_serverlist_dict
        #
        # Write the modified input.properties into the runtime dir.
        #
        write_input_properties(pdl,replaced_serverlist_dict)

    except:
        log.logException("Error while processing input properties")
        raise

    debug("Exit process_inputproperties")


if __name__ == "__main__":

    import sys
    import createPDL as createPDL
    import createPCL_config as createPCL
    import resourcebundle

    pdl = createPDL.make_pdl(createPCL.make_pcl())
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)

    # Init log
    log.set_logging(utils.getData(pdl,"LOG"))
    print " set logging to: " + str(utils.getData(pdl,"LOG"))
    log.logDumpPDL(pdl,short=False,findservers=False)

    # Initialize the resource bundle
    resourcebundle.set_resourcebundle()

    utils.setData(pdl,"SERVERS",   ["b8sr00a","b8sr00b","b8sr00c"],   "*Debug*")
    utils.setData(pdl,"JVMMODE",   ["32","64","32"],                  "*Debug*")
    utils.setData(pdl,"PROFILES",  ["default","default","default"],   "*Debug*")
    utils.setData(pdl,"WSADMIN",   "/u/WAS800B1/AppServer/bin/wsadmin.sh", "*Debug*")
    utils.setData(pdl,"PROFILE",   ["default","default","default"],   "*Debug*")
    utils.setData(pdl,"PROFILEHOME",["/u/WAS800B1/beta/tvt7004/AppServer/profiles/default",
                                     "",
                                     "/u/WAS800B3/beta/tvt7004/AppServer/profiles/default"],
                                                                       "*Debug*")
    utils.setData(pdl,"STANDALONE",   ["","",""],                      "*Debug*")
    utils.setData(pdl,"SMFLIST",      [False,False,False],             "*Debug*")
    utils.setData(pdl,"USERINSTALLROOT", ["/u/uir","/u/uir","/u/uir"], "*Debug*")
    utils.setData(pdl,"ITCAMDCINPUT", ["","",""],                      "*Debug*")
    utils.setData(pdl,"WASCELL",      ["beta","beta","mycell"],        "*Debug*")
    utils.setData(pdl,"WASNODE",      ["tvt7006","tvt7006","mynode"],  "*Debug*")
    utils.setData(pdl,"WASSERVER",    ["b8sr00a","b8sr00b","b8sr00c"], "*Debug*")
    utils.setData(pdl,"WASVERSION",   ["80","80","85"],                "*Debug*")
    utils.setData(pdl,"WASVER",       ["8.0.0.0 ND","8.0.0.0 ND","8.0.0.0 ND"], "*Debug*")
    utils.setData(pdl,"CURRENTMAXHEAPSIZE",[500000,500000,30000],      "*Debug*")
    utils.setData(pdl,"RUNLOC",       ["/u/config1","/u/config2","/u/config3"], "*Debug*")
    utils.setData(pdl,"GCOUTPUTDIST", "/u/config1/gc",                 "*Debug*")
    utils.setData(pdl,"HEAPSIZE",     128,                             "*Debug*")
    utils.setData(pdl,"TPV",          True,                            "*Debug*")

    utils.setData(pdl,"JVMVERSION", ["1.6", "1.6", "1.6"],            "*Debug*")
    utils.setData(pdl,"TPVLIST",    ["0", "0", "0"],                  "*Debug*")
    #utils.setData(pdl,"SERVERALIAS", ["Alias1", "Alias2", "Alias3"],  "*Debug*")
    utils.setData(pdl,"ALIASLIST",  ["Alias1", "Alias2", "Alias3"],  "*Debug*")

    process_inputproperties(pdl)

    log.logDumpPDL(pdl, "Final PDL")
