#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module unconfigures the DC from the Managing Server.
#
#
#####################################################################

#####################################################################
# Notes about ProbeUnconfigurator:
#  1. If ProbeUnconfigurator.main(controllerId) is called using java,
#     we need an updated CLASSPATH to include the itcam jars,
#  2. The system.exit(0) it uses stops the jython script even if 
#     the call is within a try/except.
#  3.CLASSPATH updates needed if you call ProbeUnconfigurator using java: 
#      CP=$CONFIG_HOME/itcamdc/lib/ext/ppe.probe.jar
#      CP=$CP:$CONFIG_HOME/itcamdc/lib/ext/kernel.common.jar
#      CP=$CP:$CONFIG_HOME/itcamdc/lib/ext/itcam.mscommon.jar
#      CP=$CP:$CONFIG_HOME/toolkit/lib/ext/tk_jflt.jar
#
# ProbeUnconfigurator return codes 
#  RC = 0    Successful 
#  RC = 1   "ProbeUnconfigurator: kernel.rfs.address not set"
#           "ProbeUnconfigurator: Probe will not be unconfigured"
#  RC = 2   "ProbeUnconfigurator: invalid rfs port found in "+rfsAddress
#  RC = 3   "ProbeUnconfigurator: Probe with ID "+ probeid + " not found in MS"
#  RC = 4   "ProbeUnconfigurator: Exception"
#
#
#
#
######################################################################

import sys
import os
import re
import glob
import codecs
import urllib
from types import *

import java.io.File
from java.lang import System
from java import lang
from java import util
from java import io
from java.io import File
from subprocess import call
import subprocess

import utils
import configUtils
import platformUtil
import debugUtil as log
import resourcebundle

from debugUtil import logDebug as debug
from resourcebundle import getmsg

# Needed if we use java class invocation from Jython.
#  from com.ibm.tivoli.itcam.dc.install import ProbeUnconfigurator


SCRIPT_NAME = "msunconfig.py"

PDL_DUMPED = False

############################################
#  msunconfig for a single runtime
############################################
def _msunconfig_runtime(pdl, runtime, java_home, wasnode, wascell, wasprofile, server, progress_obj):
    '''
        _msunconfig_runtime(runtime, java_home, wasnode, wascell, wasprofile, server, progress_obj)
        @param runtime      path to app server runtime inside configuration home
        @param java_home    path to JAVA_HOME
        ..
        returns True   if call made to msunconfig
                False  if call to msunconfig not possible
    '''
    debug("_msunconfig_runtime entry")
    if runtime != None:
        debug("runtime: " + runtime)
    if java_home != None:
        debug("javahome: " + java_home)

    classpathdelimiter = ":"
    if platformUtil.isWindows():
        classpathdelimiter = ";"
        runtime = utils.replaceBackSlash(runtime)
        java_home = utils.replaceBackSlash(java_home)

    debug("**msunconfig executing for: " + runtime )
    debug("   java_home: %s" % java_home)
    debug("   node: %s, cell: %s, profile: %s, server: %s" %(wasnode, wascell, wasprofile, server)) 

    runtime_base = os.path.basename(runtime)
    # Determine CONFIG_HOME
    rt_split = runtime.split('/runtime')
    config_home =  rt_split[0]
    debug("   CONFIG_HOME: %s " % (config_home) )

    operation_mode      = 'dc.operation.mode'
    kernel_codebase     = 'kernel.codebase'
    rmi_server_codebase = 'java.rmi.server.codebase'
    kernel_rfs_address  = 'kernel.rfs.address'

    dcjprops_file   = runtime+'/dc.java.properties'
    dcprops_file    = runtime+'/datacollector.properties'
    id_file         = runtime+'/id'
    security_policy = config_home+'/itcamdc/etc/datacollector.policy'
    ipprops_file    = config_home+'/runtime/%s.%s.%s.%s.input.properties' %(wasprofile,wascell,wasnode,server)
    debug("input.properties file: %s" % ipprops_file ) 
    # Saved properties we may need in input.properties
    ipprops_dict = {"KERNELIP":"", "KERNELPORT01":"", "MSHOME":"", "RFSPORT01":""}   
  
    # Test for existence of id file. 
    if not os.path.exists(id_file):
        args = []
        args.append(id_file)
        debug( "  No id file exists; there has been no connection to the MS to be unconfigured")
        debug( "  msunconfig: MS unconfig bypassed")
        print "\n"
        progress_obj.write_progress(getmsg("CFG5032E", "Cannot access file: {0}.", args))
        return False
        
    # Test for existence of: dc.java.properties, datacollector.properties, security policy,
    # and input.properties 
    if not os.path.exists(ipprops_file): 
        ipprops_file = ipprops_file + ".bak"  
        debug(" using backup input.properties file")  
    files = [dcjprops_file, dcprops_file, security_policy, ipprops_file ]
    for file in files:
        if not os.path.exists(file):
            args = []
            args.append(file)
            debug("  Supporting file not found at: " + file )
            debug("  msunconfig: MS unconfig bypassed" )
            print "\n"
            progress_obj.write_progress(getmsg("CFG5032E", "Cannot access file: {0}.", args))
            return False

    # Locate specific properties
    
    # 1. java.rmi.server.codebase 
    #    Note: A dc.operation.mode set to "MS" is no longer required, 
    #          since reconfig may remove the "MS" entry. 
    
    dcjprops_dict = {"dc.operation.mode":"", "java.rmi.server.codebase":""}
    process_flag = _process_prop_files(dcjprops_file, dcjprops_dict, progress_obj)
    if process_flag != None and not process_flag:
        return process_flag
    dc_operation_mode = dcjprops_dict["dc.operation.mode"]
    rmi_codebase = dcjprops_dict["java.rmi.server.codebase"]
    debug("rmi_codebase: %s" % rmi_codebase)
    debug("dc_operation_mode: %s" % dc_operation_mode)
    opmode = dc_operation_mode.lower()
    ms_active = True
    ms =  opmode.find('ms')
    if ms == -1:
        ms_active = False
        debug( "  dc.operation.mode does not specify \"MS\" ") 
     
    # 2. kernel.codebase and kernel.rfs.address
    dcprops_dict = {"kernel.codebase":"", "kernel.rfs.address":""}
    process_flag = _process_prop_files(dcprops_file, dcprops_dict, progress_obj)
    if process_flag != None and not process_flag:
        return process_flag
    k_codebase = dcprops_dict["kernel.codebase"]
    debug("k_codebase: " + k_codebase)
    krfs_address = dcprops_dict["kernel.rfs.address"]
    debug("krfs_address: " + krfs_address)

    # 3. octigate id
    idprops_dict = {"probe.probecontroller.octigate_id":""}
    process_flag = _process_prop_files(id_file, idprops_dict, progress_obj)
    if process_flag != None and not process_flag:
        return process_flag
    controller = idprops_dict["probe.probecontroller.octigate_id"]
    debug("controller: " + controller)


    # 4. Read the saved input.properties for specific kernel connection properties 
    #    This also works:    ipprops_dict = configUtils.read_saved_properties(ipprops_file, ipprops_dict)
    #    Unconfig process is renaming the input.properties file 
    #    Remember, the input.properties version of ms.home has not been url encoded.  
    _process_prop_files(ipprops_file, ipprops_dict, progress_obj, zos_uses_utf8=True)
    
   
    # If reconfig has been run and 'MS' not configured, the MS connection properties
    # may have substitution-characters in these properties   
      
    # 5. Perform substitution on these properties:    
    #     k_codebase    @{ms.hostname}          from "KERNELIP" , saved in input.properties
    #                   @{kernel.codebase.port} from "KERNELPORT01"
    #     krfs_address  @{ms.hostname}          from "KERNELIP"
    #                   @{kernel.rfs.port}      from "RFSPORT01"     
    #     rmi_codebase  @{ms.home}              from "MSHOME"
    ms_home_encoded = urllib.quote(ipprops_dict["MSHOME"],"\\/:").replace("\\","/")
    substlist =  [ "@{ms.hostname}:" + ipprops_dict["KERNELIP"],
                   "@{kernel.codebase.port}:" + ipprops_dict["KERNELPORT01"],
                   "@{kernel.rfs.port}:" + ipprops_dict["RFSPORT01"], 
                   "@{ms.home}:" + ms_home_encoded 
                 ]  
    debug("Substitutions list: %s" % str(substlist))               
    k_codebase   = configUtils.stringReplace(k_codebase,   substlist) 
    krfs_address = configUtils.stringReplace(krfs_address, substlist)     
    rmi_codebase = configUtils.stringReplace(rmi_codebase, substlist)   
    
    debug("Kernel connection properties:\n\
      k_codebase:   %s \n\
      krfs_address: %s \n\
      rmi_codebase: %s   " %  (k_codebase, krfs_address, rmi_codebase) )
 
    # Class path
    cp1 = config_home + "/itcamdc/lib/ext/ppe.probe.jar" + classpathdelimiter
    cp2 = config_home + "/itcamdc/lib/ext/kernel.common.jar" + classpathdelimiter
    cp3 = config_home + "/itcamdc/lib/ext/itcam.mscommon.jar" + classpathdelimiter
    cp4 = config_home + "/toolkit/lib/ext/tk_jflt.jar" + classpathdelimiter
    cpath = []
    cp_list = [cp1, cp2, cp3, cp4]
    for cp in cp_list:
        cpath.append(cp)
    CP = ''.join(cpath)

    # Java properties
    jp1 = " -Djava.security.policy=" + security_policy
    jp2 = " -Djava.rmi.server.codebase=" + "\"" + rmi_codebase + "\""
    jp3 = " -Dkernel.codebase="          + "\"" + k_codebase   + "\""
    jp4 = " -Dkernel.rfs.address="       + "\"" + krfs_address + "\""
    jp5 = " -Dam.kernel.dynamicconfig=false "  

    cmd_java_home = java_home + "/bin/java"
    if platformUtil.isWindows(): 
        cmd_java_home = utils.add_quotes(cmd_java_home)
    
    javacmd = cmd_java_home   \
              + " -cp " + CP   \
              + jp1           \
              + jp2           \
              + jp3           \
              + jp4           \
              + jp5           \
              +  " com.ibm.tivoli.itcam.dc.install.ProbeUnconfigurator " +   str(controller)
              
    datadir = pdl["CONFIGHOME"][0] + os.sep + "data"
    logfile_loc  = datadir + "/" + wasprofile + "." + wascell + "." + wasnode + "." + server + "." + "msunconfig.log"  
    logfile_loc = utils.add_quotes(logfile_loc)  
    debug("MS unconfig log loc - '%s'" % logfile_loc)
    javacmd = javacmd +  "  " + logfile_loc      

    debug("  java exec to call ProbeUnconfigurator ")
    debug("  " + javacmd )
    
    rc = 0
    rtn = True 
    
    try:
        
        if platformUtil.isWindows():       
            p = subprocess.Popen(javacmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        else:
            p = subprocess.Popen(javacmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        p.wait()
        rc = p.returncode
        response = p.stdout.read().strip()
        
        #print " msunconfig popen rc: " + str(rc) + " -  " + response 
       
        if rc == 1:
            progress_obj.write_progress(getmsg("CFG5049E", "The ProbeUnconfigurator failed: kernel.rfs.address is not set."))   
        elif rc == 2:
            progress_obj.write_progress(getmsg("CFG5050E", "The ProbeUnconfigurator failed: invalid rfs port "))    
        elif rc == 3:
            progress_obj.write_progress(getmsg("CFG5051I", "The DC has already been unconfigured from the MS")) 
        elif rc == 4:
            progress_obj.write_progress(getmsg("CFG5052W", "ProbeUnconfigurator unable to execute")) 
                         
        debug( "  call ProbeUnconfigurator..complete, rc=%s - %s " % (str(rc),response) )

    except:
        log.logException("Exception in ProbeUnconfigurator; msunconfig failed")
        return False

    # If rc = 0 or 3, rename  the id file
    if  rc == 0 or rc == 3:
        try:
            os.rename(id_file,  id_file + ".bak")
            debug("The id file successfully renamed to: %s" % id_file + ".bak")     
        except OSError: 
            log.logException("Unable to rename id file: %s" % id_file)
            
    if rc != 0:
        rtn = False
 
    debug("** msunconfig completed for runtime %s **" % ( runtime))
    debug("_msunconfig_runtime exit rc:%s rtn: %s" %(str(rc), str(rtn)))
    return rtn 

#From dc.java.properties file,  get:  java.rmi.server.codebase
#From datacollector.properties, get   kernel.codebase   and kernel.rfs.address
#From input.properties, get "KERNELIP", "KERNELPORT01", "MSHOME", "RFSPORT01" 
def _process_prop_files(file_name, prop_dict, progress_obj, zos_uses_utf8=False):
    '''
        _process_prop_files(file_name, prop_dict, progress_obj)
    ''' 
    debug("_process_prop_files entry: " + file_name + ", " + str(prop_dict))
    props = None
    encode = 'utf-8'    # default encoding
    if platformUtil.IS_zOS_OS():
        if zos_uses_utf8 == False: 
           encode = 'cp1047' 
    try:
        props = codecs.open(file_name, mode='r', encoding=encode)
        
        for line in props.readlines():
            if line != None:
                line = line.strip()
            if line.startswith("# "):
                line = line[len('# '):len(line)]
            ws = line.split('=')
            if ws != None and len(ws) > 0:
                # debug("ws[0]: " + ws[0])
                for key in prop_dict.keys():
                    if ws[0] == key:
                        # debug("key: " + key)
                        # debug("ws[1]: " + ws[1])
                        prop_dict[key] = ws[1]
                    else:
                        continue

        for key in prop_dict.keys():
    
            if prop_dict[key] is None or prop_dict[key] == "":
                args = []
                args.append(key)
                args.append(file_name)
                debug( "  Unable to locate " + key + " in " + file_name)
                print "\n"
                progress_obj.write_progress(getmsg("CFG2044E", "Unable to locate {0} in {1}.", args))
                debug( "  msunconfig: MS unconfig bypassed" )
                debug("_process_prop_files exit")
                return False
            else:
                debug("  " + key + ' = ' + prop_dict[key])
    except:
        log.logException("_process_prop_files")
        if props == None:
            args = []
            args.append(file_name)
            debug("  Supporting file not found at: " + file_name )
            debug("  msunconfig: MS unconfig bypassed" )
            print "\n"
            print getmsg("CFG5032E", "Cannot access file: {0}.", args)
    finally:
        if props != None:
            props.close()

    debug("_process_prop_files exit")

#####################################################################
#  Call ProbeUnconfig for serveri                                   #
#####################################################################
def msunconfig( pdl, serveri, progress_obj):
    '''
        unconfig_all( pdl)
        returns True   if normal processing completed
                False  if processing failed
    '''
    method_id = "msunconfig.msunconfig"
    all_success = True
    server = ""
    try:
        debug( method_id + " executing")
        if pdl["SERVERS"][0] is None:
            assert 0, getmsg("CFG5017E", "Data element {0} of PDL has no entries", ["SERVERS"])

        confighome = utils.getData(pdl, "CONFIGHOME")
        runtime_root =  confighome + "/runtime/"
        javahome   = utils.getData(pdl, "JAVAHOME",   serveri)
        wasversion = utils.getData(pdl, "WASVERSION", serveri)
        server, wasnode, wascell, wasprofile = _get_server_cell_node_profile(pdl, serveri)
        
        #  Locate the app server's runtime using  cell.node.profile.server 
        #  Call _msunconfig_runtime
        appsrv_runtime = runtime_root + "was%s.%s.%s.%s.%s"  % (str(wasversion), wascell, wasnode, wasprofile, server)   
        pattern_dir = runtime_root + "was%s.*%s.%s.%s.%s"  % (str(wasversion), wascell, wasnode, wasprofile, server) 
        debug("  appsrv_runtime:%s " % (appsrv_runtime) )  
        debug("  pattern_dir:%s " % (pattern_dir) )  
        if os.path.isdir(appsrv_runtime):
            debug("  matching runtime found: %s" % str(appsrv_runtime))
            success = _msunconfig_runtime(pdl, appsrv_runtime, javahome, wasnode, wascell, wasprofile, server, progress_obj )
            if not success: all_success = False  
        else:
            runtime_list =  glob.glob(pattern_dir)
            debug("  matching runtime dirs/files: %s " % (str(runtime_list)) )
            for runtime  in runtime_list:
                if os.path.isdir(runtime):
                    success = _msunconfig_runtime(pdl, runtime, javahome, wasnode, wascell, wasprofile, server, progress_obj )
                    if not success: all_success = False

        debug(method_id  + " completed; successful = " + str(all_success) )
        return all_success

    except:
        log.logException("**ERROR** Error in %s for server %s" % ( method_id, server) )
        return False

#####################################################################
#  server_MS_configured ?                                            #
#  Recall that dc.operation.mode may not contain "MS" because       #
#  reconfig may have removed it.                                    #
#  In Windows, path statements need to have backslashes('\')        #
#  changed to forwardslashes('/') for the os.path statements.       #
#####################################################################
def server_MS_configured(pdl, serveri):
    '''
        Determines if the server has been MS-configured and therefore 
        is eligible for msunconfig processing
        @param  - pdl 
        @param  - serveri - server index # into SERVERS pdl entry 
        @return - True | False   
        
        Currently we test for the existence of the "id" file
    '''
    debug("server_MS_configured - entry - serveri: %s" % str(serveri))
    global PDL_DUMPED 
    if PDL_DUMPED is False:
        log.logDumpPDL(pdl, "PDL at server_MS_configured ")  
        PDL_DUMPED = True
      
    rtn = False
    # Determine if id file is present. If so, consider this server a candidate for msunconfig. 
    confighome = utils.getData(pdl, "CONFIGHOME")
    if platformUtil.isWindows():
        confighome = utils.replaceBackSlash(confighome)
    runtime_root =  confighome + "/runtime/"
    wasversion = utils.getData(pdl, "WASVERSION", serveri)
    
    server, wasnode, wascell, wasprofile = _get_server_cell_node_profile(pdl, serveri)
    
    #  Locate the app server's runtime using  cell.node.profile.server.
    #  In theory there could be more than one but it's unlikely
    appsrv_runtime = runtime_root + "was%s.%s.%s.%s.%s" % (str(wasversion), wascell, wasnode, wasprofile, server)   
    pattern_dir = runtime_root + "was%s.*%s.%s.%s.%s"   % (str(wasversion), wascell, wasnode, wasprofile, server) 
    
    debug("  appsrv_runtime:%s " % appsrv_runtime )  
    debug("  pattern_dir:%s " % pattern_dir )
    
    #if directory_exists(appsrv_runtime): 
    if os.path.isdir(appsrv_runtime):
        debug("  matching appsrv runtime found: %s" % str(appsrv_runtime))
        if os.path.isfile(appsrv_runtime+'/id'):
            debug("  id file exists in: %s " % appsrv_runtime )
            rtn = True 
        else:
            debug("  id file does not exists in: %s " % appsrv_runtime )    
    else:
        runtime_list =  glob.glob(pattern_dir)
        debug("  matching runtimes using glob: %s " % (str(runtime_list)) )
        for runtime  in runtime_list:
            if os.path.isdir(runtime):
                if os.path.exists(runtime+'/id'):   
                    debug("  id file exists in: %s " % runtime )
                    rtn = True
                else:
                    debug("  id file does not exists in: %s " % runtime )     
  
    debug("server_MS_configured - exit: rtn=%s" % rtn)
    return rtn

def _get_server_cell_node_profile(pdl, serveri):
    '''
        Obtains server, cell, node, and profile in a non-ascii world
        @return server, cell, node, profile
        
        If the variables are stored in the non-ascii representation 'MODIFIED' variable, use them.
        
        To use: server, wasnode, wascell, wasprofile = get_server_cell_node_profile(pdl, serveri)
        TODO: does this need to be expanded to cover the conversion if not done ? 
    '''
    debug("get_server_cell_node_profile - entry")
    server  = ""
    wasnode = ""  
    wascell = ""   
    wasprofile = ""  
    serversm = utils.getData(pdl,"SERVERSMODIFIED")
    nodesm   = utils.getData(pdl,"WASNODEMODIFIED")
    cellsm   = utils.getData(pdl,"WASCELLMODIFIED")
    profsm   = utils.getData(pdl,"PROFILEMODIFIED")
    if type(serversm) is ListType and len(serversm) > 0:
        server      = utils.getData(pdl,"SERVERSMODIFIED", serveri)
    if type(nodesm)   is ListType and len(nodesm) > 0:
        wasnode     = utils.getData(pdl,"WASNODEMODIFIED", serveri)
    if type(cellsm)   is ListType and  len(cellsm) > 0:
        wascell     = utils.getData(pdl,"WASCELLMODIFIED", serveri)
    if type(profsm)   is ListType and  len(profsm) > 0:
        wasprofile  = utils.getData(pdl,"PROFILEMODIFIED", serveri)
    if server   == "":   server    = utils.getData(pdl, "SERVERS", serveri) 
    if wasnode  == "":   wasnode   = utils.getData(pdl, "WASNODE", serveri)
    if wascell  == "":   wascell   = utils.getData(pdl, "WASCELL", serveri)
    if wasprofile == "": wasprofile= utils.getData(pdl, "PROFILE", serveri) 
   
    debug_line = unicode_line("get_server_cell_node_profile: \n server: %s\n cell: %s\n node: %s\n profile: %s" % \
                                                          (server, wascell, wasnode, wasprofile))  
    debug(debug_line) 
    debug("get_server_cell_node_profile - exit  ") 
    
    return server, wasnode, wascell, wasprofile
    
def unicode_line(line):
    try:
        unicode(line, None, "replace")
    except UnicodeEncodeError, errmsg:
        line = "<line suppressed due to unicode encode error %s>" % errmsg
    return line        


        
#####################################################################
#  test_msunconfig                                                  #
#####################################################################
def test_msunconfig():
    import createPCL_config as createPCL
    import createPDL
    from Progress import Progress
    from constants import constants
    
    # Code here for testing only
    pcl=createPCL.make_pcl()
    pdl=createPDL.make_pdl(pcl)
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    #log.logDumpPDL(pdl,short=False,findservers=True)
    print "Testing msunconfig"
    common_constants = constants()
    # Init log
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])

    # Initialize the resource bundle
    print " Set resource bundle"
    resourcebundle.set_resourcebundle()
    progress_file_loc = pdl["CONFIGHOME"][0] + os.sep + "data" + os.sep + "msunconfig_test_progress.txt"
    progress_obj = Progress(common_constants.MSUNCONFIG, None, progress_file_loc)
    JAVAHOME= "/u/WAS800B/beta/tvt7006/AppServer/java64"
    runtime = "/u/mcoley/jython/CONFIG/runtime/was80.beta.tvt7006.default.b8sr00b"
    wasnode = "tvt7006"
    wascell = "beta"
    wasprofile = "default"
    server = b8sr00b
    _msunconfig_runtime(pdl, runtime, javahome, wasnode, wascell, wasprofile, server, progress_obj )
  
    return pdl


def test_msunconfig2():
    import createPCL_config as createPCL
    import createPDL
    from Progress import Progress
    from constants import constants
    
    # Code here for testing only
    pcl=createPCL.make_pcl()
    pdl=createPDL.make_pdl(pcl)
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    print "Testing msunconfig2"
    common_constants = constants()
    pdl["SERVERS"][0]    = ['b8sr00a', 'b8sr00b']
    pdl["CONFIGHOME"][0] = '/u/mcoley/jython/CONFIG'
    pdl["WASVERSION"][0] = ['80', '80']
    pdl["WASNODE"][0]    = ['tvt7006', 'tvt7006']
    pdl["WASCELL"][0]    = ['beta',  'beta']
    pdl["PROFILE"][0]    = ['default', 'default']
    pdl["JAVAHOME"][0]   = ["/u/WAS800B/beta/tvt7006/AppServer/java64", "/u/WAS800B/beta/tvt7006/AppServer/java64"]
    # Init log
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
    # Initialize the resource bundle
    print " Set resource bundle"
    resourcebundle.set_resourcebundle()

    progress_file_loc = pdl["CONFIGHOME"][0] + "/data/msunconfig_test_progress.txt"
    progress_obj = Progress(common_constants.MSUNCONFIG, None, progress_file_loc)
    
    JAVAHOME= "/u/WAS800B/beta/tvt7006/AppServer/java64"
    runtime = "/u/mcoley/jython/CONFIG/runtime/was80.beta.tvt7006.default.b8sr00a"
    msunconfig(pdl, 0,  progress_obj)
    msunconfig(pdl, 1,  progress_obj)
    return pdl

    
    
def test_msunconfig3():
    import createPCL_config as createPCL
    import createPDL
    pcl=createPCL.make_pcl()
    pdl=createPDL.make_pdl(pcl)
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(
    pdl,sys.argv)
    print "Testing msunconfig3 - call to server_MS_configured  "
  
    pdl["SERVERS"][0]    = ['b8sr00a', 'b8sr00b']
    pdl["CONFIGHOME"][0] = '/u/mcoley/jython/CONFIG'
    pdl["WASVERSION"][0] = ['80', '80']
    pdl["WASNODE"][0]    = ['tvt7006', 'tvt7006']
    pdl["WASCELL"][0]    = ['beta', 'beta']
    pdl["PROFILE"][0]    = ['default', 'default']
    pdl["JAVAHOME"][0]   = ["/u/WAS800B/beta/tvt7006/AppServer/java64", "/u/WAS800B/beta/tvt7006/AppServer/java64"]
    # Init log
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
    # Initialize the resource bundle
    print " Set resource bundle"
    resourcebundle.set_resourcebundle()
    print "log at: " +  pdl["LOG"][0]
    server_indexlist = [0,1]
    for ix in server_indexlist:
        print "   testing ix: %s:" %str(ix) 
        rtn = server_MS_configured(pdl, ix )      
        print "     server MS configured: %s" % str(rtn)
        if rtn == False:
            server_indexlist.remove(ix)  

    print "...results: %s" % str(server_indexlist)
    return pdl
    
