#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    platformUtil.py
    platform specific utilities
'''

fixlev = "39298" # Last APAR, defect or story to update this module

import os
import re
from java.lang import System

from debugUtil import debug
from ConfigurationException import ConfigurationException

# global platform constants
OS_NAME_WINNT = "Windows NT"
OS_NAME_WIN2K = "Windows 2000"
OS_NAME_WIN2K3 = "Windows 2003"
OS_NAME_WINSERVER2K3 = "Windows Server 2003"
OS_NAME_WINSERVER2K8 = "Windows Server 2008"
OS_NAME_WINSERVER2K12 = "Windows Server 2012"
OS_NAME_WINXP = "Windows XP"
OS_NAME_WINVISTA = "Windows Vista"
OS_NAME_WIN7 = "Windows 7"
OS_NAME_WIN8 = "Windows 8"
OS_NAME_SUNOS = "sunos"
OS_NAME_HPUX = "hp-ux"
OS_NAME_AIX = "AIX"
OS_NAME_ZOS = "z/OS"
OS_NAME_LINUX = "linux"
OS_NAME_OS390 = "OS/390"
OS_NAME_OS400 = "OS/400"

'''
   * Interpret name for 32-bit pLinux.
   *
   * <li>os.name: Linux, os.arch: ppc, os.version: 2.4.19-ul1-ppc64-SMP</li>
'''
LINUX_PPC = "linux-ppc"

'''
   * Interpret name for 64-bit pLinux.
   *
   * <li>os.name: Linux, os.arch: ppc64, os.version: 2.6.5-7.244-pseries64
   * </li>
'''
LINUX_PPC64 = "linux-ppc64"

'''
   * Interpret name for 32-bit zLinux.
   *
   * <li>os.name: Linux, os.arch: s390, os.version: 2.4.19-3suse-SMP</li>
'''
LINUX_S390 = "linux-s390"

'''
   * Interpret name for 64-bit zLinux.
   *
   * <li>os.name: Linux, os.arch: s390x, os.version: 2.6.5-7.252-s390x</li>
'''
LINUX_S390_64 = "linux-s390-64"

'''
   * Interpret name for HP-UX vB10.x.
'''
HPUX10 = "hpux10"

'''
   * Interpret name for 32-bit HP-UX vB11.x.
   *
   * <li>os.name: HP-UX, os.arch: PA_RISC2.0, os.version: B.11.11</li>
'''
HPUX11 = "hpux11"

'''
   * Interpret name for 64-bit HP-UX vB11.x and up.
   *
   * <li>os.name: HP-UX, os.arch: PA_RISC2.0W, os.version: B.11.11</li>
'''
HPUX_64 = "hpux-64"

'''
   * Interpret name for 32-bit JVM on HP-UX Itanium 64bit vB11.x and up.
   *
   * <li>os.name: HP-UX, os.arch: Itanium, os.version: B.11.11</li>
'''
HPUX_IA64N = "hpux-ia64n"

'''
   * Interpret name for 64-bit JVM on HP-UX Itanium 64bit vB11.x and up.
   *
   * <li>os.name: HP-UX, os.arch: Itanium, os.version: B.11.11</li>
'''
HPUX_IA64 = "hpux-ia64"

'''
   * Interpret nme for 32-bit AIX v4.x and up.
   *
   * <li>os.name: AIX, os.arch: ppc, os.version: 5.1</li>
'''
AIX4_R1 = "aix4-r1"

'''
   * Interpret nme for 64-bit AIX v5.x and up.
   *
   * <li>os.name: AIX, os.arch: ppc64, os.version: 5.1</li>
'''
AIX_64 = "aix-64"

'''
   * Interpret name for 32-bit Solaris v2.x and up.
   *
   * <li>os.name: SunOS, os.arch: sparc, os.version: 5.8</li>
'''
SOLARIS2 = "solaris2"

'''
   * Interpret name for 64-bit Solaris v2.x and up.
   *
   * <li>os.name: SunOS, os.arch: sparcv9, os.version: 5.8</li>
'''
SOLARIS_64 = "solaris-64"

'''
   * Interpret name for 32-bit Solaris v2.x and up on x86 platform.
   *
   * <li>os.name: SunOS, os.arch: sparc, os.version: 5.8</li>
'''
SOLARIS2_IX86 = "solaris2-ix86"

'''
   * Interpret name for 64-bit Solaris v2.x and up on x86 platform.
   *
   * <li>os.name: SunOS, os.arch: sparc, os.version: 5.8</li>
'''
SOLARIS2_IX86_64 = "solaris2-ix86-64"

'''
   * Interpret name for 32-bit xLinux.
   *
   * os.name: Linux, os.arch: x86, os.version: 2.4.9-e.3smp
'''
LINUX_IX86 = "linux-ix86"

'''
   * Interpret name for 64-bit xLinux.
   *
   * <li>os.name: Linux, os.arch: amd64, os.version: 2.6.9-5.ELsmp</li>
'''
LINUX_IX64 = "linux-ix64"

'''
   * Interpret name for 64-bit Itanium Linux.
   *
   * <li>os.name: Linux, os.arch: ia64</li>
'''
LINUX_IA64 = "linux-ia64"

'''
   * Interpret name for OS/400.
'''
OS400 = "OS400"

'''
   * Interpret name for 32-bit Windows, any version or chipset.
   *
   * <li>os.name: Windows XP, os.arch: x86, os.version: 5.1</li>
   * <li>os.name: Windows 2003, os.arch: amd64, os.version: 5.2</li>
'''
W32_IX86 = "w32-ix86"

'''
   * Interpret name for 64-bit Windows, any version or chipset.
   *
   * <li>os.name: Windows 2003, os.arch: amd64, os.version: 5.2</li>
'''
W64_X64 = "w64-x64"

'''
   * Old Interpret name for z/OS.
   *
   * @deprecated Use the constant <code>ZOS</code> instead.
'''
OS390 = "os390"

'''
   * Interpret name for z/OS.
   *
   * <li>os.name: z/OS, os.arch: 390, os.version: 01.04.00</li>
'''
ZOS = "zos"

#The constants below are for toolkit arch name in toolkit native library path: toolkit/lib/<arch>
ARCH_AIX4_R1 = "aix533"
ARCH_AIX_64 = "aix536"
ARCH_LINUX_IX86 = "li6263"
ARCH_LINUX_IX64 = "lx8266"
ARCH_LINUX_PPC = "lpp263"
ARCH_LINUX_PPC64 = "lpp266"
ARCH_LINUX_S390 = "ls3263"
ARCH_LINUX_S390_64 = "ls3266"
ARCH_SOLARIS2 = "sol293"
ARCH_SOLARIS_64 = "sol296"
ARCH_SOLARIS2_IX86 = "sol603"
ARCH_SOLARIS2_IX86_64 = "sol606"
ARCH_HPUX10 = "hp11"
ARCH_HPUX_64 = "hp116"
ARCH_HPUX_IA64 = "hpi116"
ARCH_HPUX_IA64N = "hpi113"
ARCH_W32_IX86 = "win32"
ARCH_W64_X64 = "win64"
ARCH_OS400 = "os400-pase"
ARCH_OS400_64 = "os400-pase64"

def getJavaRelease():
    '''
       Java release (java.version) in ths format: 1.6.0
    '''
    return  System.getProperty("java.version")

def getPlatform():
    '''
       platform name
    '''
    return System.getProperty("os.name")

def getOSArch():
    '''
       os.arch information
    '''
    return System.getProperty("os.arch")

def getOSVersion():
    '''
       os.version information
    '''
    return System.getProperty("os.version")

def isWindows(platform=None):
    '''
       if platform name matches Windows, MS or NT
    '''
    if platform == None: platform = getPlatform()
    if re.match("win|ms|nt", platform, flags=re.IGNORECASE) : return 1
    return 0

def isZ(platform=None):
    '''
       Returns True if this is z/OS, False otherwise.
    '''
    if platform == None: platform = getPlatform()
    return platform == OS_NAME_ZOS

def getEncoding(platform=None):
    '''
       For z/OS return CP-1047, all other platforms UTF-8
    '''
    if isZ(platform):
        enc = "CP-1047"
    else:
        enc = "UTF-8"
    return enc

def getOSFamily(platform=None):
    '''
       returns os family name
    '''

    if platform == None: platform = getPlatform()
    osFamily = None
    if platform:
        platform = platform.lower()
        if platform.find("aix") > -1: osFamily = "AIX"
        elif platform.find("hp") > -1:  osFamily = "HP-UX"
        elif platform.startswith("z"):  osFamily = "ZOS"
        elif platform.find("linux") > -1: osFamily = "LINUX"
        elif platform.find("400") > -1: osFamily = "OS400"
        elif re.compile("sun|sol").match(platform): osFamily = "SOLARIS"
        elif re.compile("win|ms|nt").match(platform): osFamily = "WINDOWS"
    return osFamily

def getCommandSeparator(platform=None):
    '''
        command separator for multiple commands
    '''
    if platform == None: platform = getPlatform()
    if isWindows(platform): return "&"
    return ";"

def getEnLocale(platform=None):
    '''
        name of the default English locale
    '''

    debug("Entry getEnLocale(%s) %s" % (platform, fixlev))
    if platform == None:
        platform = getPlatform()
    osFamily = getOSFamily(platform)
    assert osFamily, "Unknown OS Family, can not determine default English locale"
    enlocale = "en_US"
    if osFamily == "WINDOWS" :
        enlocale = "437"
    elif osFamily == "ZOS":
        enlocale = "En_US.IBM-1047"
    elif osFamily == "OS400":
        enlocale = "/QSYS.LIB/EN_US.LOCALE"
    else:
        enlocale = "en_US"

    debug("Exit getEnLocale(), result = %s" % enlocale)
    return enlocale

def setLocale(platform=None, locale=None):
    '''
       sets LANG env variable
       todo: this need to  be fixed
    '''
    debug("Entry setLocale(%s,%s) %s" % (platform, locale, fixlev))
    if platform == None: platform = getPlatform()
    var = "LANG"
    if locale == None: locale = "C"
    if getOSFamily(platform) == "ZOS":
        var = "LC_ALL"
        locale = "En_US.IBM-1047"
    os.putenv(var, locale)
    debug("Exit selocale(), %s=%s" % (var,locale))
#end

def getWSAdminScript(platform=None):
    '''
       return name of the wsadmin script based on platform type
    '''
    if getOSFamily(platform) == "OS400": return "wsadmin"
    if isWindows(platform): return "wsadmin.bat"
    return "wsadmin.sh"

def getBitMode(confighome, jvmmode):
    '''
       Get 32-bit or 64-bit information for a specific Operating System
    '''
    bitMode = "unknown"
    platform = getPlatform()
    if getOSFamily(platform) == "ZOS":
        #For z/OS, native library is directly under toolkit/lib, no need to find out bitmode information.
        bitMode = ""
    else:
        bitMode = "/" + getToolkitArch(confighome, jvmmode)
    return bitMode

def IS_WINDOWS_OS():
    return (IS_NT_OS() or IS_WIN2K_OS() or IS_XP_OS() or IS_WIN2003_OS() or IS_WINSERVER2003_OS() or IS_WINSERVER2008_OS() or IS_WINSERVER2012_OS() or IS_VISTA_OS() or IS_WINDOWS7_OS() or IS_WINDOWS8_OS() )

def IS_WINDOWS_64_OS():
    return (getPlatformInTivoliFormat() == W64_X64)

def IS_XP_OS():
    '''
        true=on a w32 XP machine, false=not a w32 XP machine
    '''
    return (getPlatform() == OS_NAME_WINXP)

def IS_VISTA_OS():
    '''
        true=on a w32 Vista machine, false=not a w32 Vista machine
    '''
    return (getPlatform().startswith(OS_NAME_WINVISTA) or (getPlatform().startswith(OS_NAME_WINNT) and getOSVersion() == "6.0"))

def IS_WINDOWS7_OS():
    '''
         true=on a w32 or w64 windows7 machine, false=not a windows7 machine
    '''
    return (getPlatform().startswith(OS_NAME_WIN7) or (getPlatform().startswith(OS_NAME_WINNT) and getOSVersion().startswith("6.1")))

def IS_WINDOWS8_OS():
    '''
         true=on a w32 or w64 windows8 machine, false=not a windows8 machine
    '''
    return (getPlatform().startswith(OS_NAME_WIN8))

def IS_WIN2K_OS():
    '''
         true=on a Win2000 machine, false=not a Win2000 machine
    '''
    return (getPlatform() == OS_NAME_WIN2K or getPlatform().startswith(OS_NAME_WIN2K))

def IS_WIN2003_OS():
    '''
         true=on a Win2003 machine, false=not a Win2003 machine
    '''
    return ((getPlatform().startswith(OS_NAME_WIN2K) and getOSVersion() == "5.2") or getPlatform().startswith(OS_NAME_WIN2K3))

def IS_WINSERVER2003_OS():
    '''
        Determine if running on a Windows 2003 Server.
        true=on a Win Server 2003 machine, false=not a Win Server 2003 machine
    '''
    return (getPlatform().startswith(OS_NAME_WINSERVER2K3))

def IS_WINSERVER2008_OS():
    '''
        Determine if running on a Windows 2008 Server.
        true=on a Win Server 2008 machine, false=not a Win Server 2008 machine
    '''
    return (getPlatform().startswith(OS_NAME_WINSERVER2K8))

def IS_WINSERVER2012_OS():
    '''
        Determine if running on a Windows 2012 Server.
        true=on a Win Server 2012 machine, false=not a Win Server 2012 machine
    '''
    return (getPlatform().startswith(OS_NAME_WINSERVER2K12))

def IS_NT_OS():
    '''
        true=on a w32 NT machine, false=not a w32 NT machine
    '''
    return (getPlatform() == OS_NAME_WINNT)

def IS_UNIX_OS():
    '''
        true=on a unix machine, false=not a unix machine
    '''
    return (not IS_WINDOWS_OS())

def IS_SOL_OS():
    '''
        true=on a Solaris machine, false=not a Solaris machine
    '''
    return (getPlatformInTivoliFormat() == SOLARIS2)

def IS_SOL_64_OS():
    return (getPlatformInTivoliFormat() == SOLARIS_64)

def IS_SOL_IX86_OS():
    return (getPlatformInTivoliFormat() == SOLARIS2_IX86)

def IS_SOL_IX86_64_OS():
    return (getPlatformInTivoliFormat() == SOLARIS2_IX86 and isJDK64Bit())

def IS_HPUX_OS():
    '''
        true=on an HP/UX machine, false=not an HP/UX machine
    '''
    return (getPlatform().lower() == OS_NAME_HPUX.lower())

def IS_HPUX11_OS():
    '''
        true=on an HP/UX machine, false=not an HP/UX machine
    '''
    return (getPlatformInTivoliFormat() == HPUX11)

def IS_HPUX_64_OS():
    return (getPlatformInTivoliFormat() == HPUX_64)


def IS_HPUX_IA64_OS():
    return (getPlatformInTivoliFormat() == HPUX_IA64)


def IS_HPUX_IA64N_OS():
    return (getPlatformInTivoliFormat() == HPUX_IA64N)


def IS_AIX_OS():
    '''
        true=on an AIX machine, false=not an AIX machine
    '''
    return (getPlatform() == OS_NAME_AIX)

def IS_AIX_64_OS():
    return (getPlatformInTivoliFormat() == AIX_64)

def IS_AIX5_OS():
    '''
        true=on an AIX5 machine, false=AIX4 machine
    '''
    if getPlatform() == OS_NAME_AIX:
        versionParts = getOSVersion().split(".")
        version = versionParts[0]

        if version >= 5:
            return 1

    return 0

def IS_LINUX_IX86_OS():
    '''
        true=on a LINUX_IX86 machine, false=not a LINUX_IX86 machine
    '''
    return (getPlatformInTivoliFormat() == LINUX_IX86)

def IS_LINUX_IX64_OS():
    return (getPlatformInTivoliFormat() == LINUX_IX64)

def IS_LINUX_IA64_OS():
    return (getPlatformInTivoliFormat() == LINUX_IA64)

def IS_LINUX_PPC_OS():
    '''
        true=on a LINUX_PPC machine, false=not a LINUX_PPC machine
    '''
    return (getPlatformInTivoliFormat() == LINUX_PPC)


def IS_LINUX_PPC64_OS():
    return (getPlatformInTivoliFormat() == LINUX_PPC64)


def IS_LINUX_S390_OS():
    '''
        true=on a LINUX_S390 machine, false=not a LINUX_S390 machine
    '''
    return (getPlatformInTivoliFormat() == LINUX_S390)

def IS_LINUX_S390_64_OS(confighome):
    '''
        true=on a LINUX_S390 machine, false=not a LINUX_S390 machine
    '''
    return (getPlatformInTivoliFormat(confighome) == LINUX_S390_64)

def IS_LINUX_OS():
    '''
        true=on any LINUX machine, false=not a LINUX machine
    '''
    return (getPlatform().lower() == OS_NAME_LINUX.lower())

def IS_SUSE_LINUX_OS():
    '''
        true=on any SuSE LINUX machine, false=not a SuSE LINUX machine
    '''
    suseVersionFile = open("/etc/SuSE-release", "r")
    return (IS_LINUX_OS() and os.path.isfile(suseVersionFile))


def IS_REDHAT_LINUX_OS():
    '''
        true=on any RedHat LINUX machine, false=not a RedHat LINUX machine
    '''

    redhatVersionFile = open("/etc/redhat-release", "r")
    return (IS_LINUX_OS() and os.path.isfile(redhatVersionFile))

def IS_zOS_OS():
    '''
        true=on a z/OS USS machine, false=not a z/OS machine
        Prefer to use isZ() instead.
    '''
    return isZ()

def IS_OS390_OS():
    '''
        true=on an OS/390 USS machine, false=not an OS/390 machine
    '''
    return (getPlatform() == OS_NAME_OS390)

def IS_OS400_OS():
    '''
        true=on an OS400 machine, false=not a OS400 machine
    '''
    return (getPlatform() == OS_NAME_OS400)

def getPlatformInTivoliFormat(confighome=None):
    '''
        Returns the string representation of the platform in Tivoli format, such
        as solaris2, w32-ix86, aix4-r1, linux-ix86, etc. If the platform is not a
        known type, "unknown" is returned.

        the platform of this machine in Tivoli format (e.g. solaris2,
        w32-ix86, aix4-r1, linux-ix86, etc. or "unknown" if the platform cannot
        be determined).
    '''

    platform = "unknown"

    if getPlatform().lower() == OS_NAME_SUNOS.lower():
        if getOSArch() == "sparcv9":
            platform = SOLARIS_64
        elif getOSArch() == "sparc":
            platform = SOLARIS2
        elif getOSArch() == "x86":
            platform = SOLARIS2_IX86

    elif getPlatform().lower() == OS_NAME_AIX.lower():
        if getOSArch() == "ppc64":
            platform = AIX_64
        elif getOSArch() == "ppc":
            platform = AIX4_R1

    elif getPlatform().lower() == OS_NAME_HPUX.lower():
        if getOSVersion().startswith("B.11") and getOSArch() == "PA_RISC2.0":
            platform = HPUX11
        elif getOSVersion().startswith("B.11") and getOSArch() == "PA_RISC2.0W":
            platform = HPUX_64
        elif getOSVersion().startswith("B.11") and (getOSArch().startswith("ia64n") or getOSArch().startswith("IA64N")):
            platform = HPUX_IA64N
        elif getOSVersion().startswith("B.11") and (getOSArch().startswith("ia64w") or getOSArch().startswith("IA64W")):
            platform = HPUX_IA64
        elif getOSVersion().startswith("B.10"):
            platform = HPUX10

    elif getPlatform().lower() == OS_NAME_LINUX.lower():
        if (getOSArch().lower() == "s390".lower()) or (getOSArch().lower() == "s390x".lower()):
            platform = LINUX_S390
        elif getOSArch().startswith("ppc"):
            platform = LINUX_PPC
        elif getOSArch().startswith("ppc64"):
            platform = LINUX_PPC64
        elif getOSArch().startswith("x86_64") or getOSArch().startswith("amd64"):
            platform = LINUX_IX64
        elif getOSArch().startswith("x86") or getOSArch().startswith("i386"):
            platform = LINUX_IX86
        elif getOSArch().startswith("ia64") or getOSArch().startswith("IA64"):
            platform = LINUX_IA64

    elif getPlatform().lower() == OS_NAME_ZOS.lower():
        platform = ZOS

    elif getPlatform().lower() == OS_NAME_OS390.lower():
        platform = OS390

    elif getPlatform().lower() == OS_NAME_OS400.lower():
        platform = OS400

    return platform

def is64BitWindows(confighome):
    '''
       Get Windows 32-bit or 64-bit information
    '''
    systemInfoFile = confighome + "/data/systemInfo.data"
    cmd = "systeminfo.exe > " + systemInfoFile
    rc = os.system(cmd)
    if rc > 0:
        raise ConfigurationException("Cannot get Windows system information.")

    #Parse systemInfo.data file to get Windows bit information
    infile = open(systemInfoFile, "r")
    count = 0
    lines = infile.readlines()
    for line in lines:
        #skip empty line
        if not line.strip():
            count = count + 1
            continue
        else:
            if line.find("Processor") > -1:
                #processor detail information is in next line, read now
                line = lines[count+1]
                debug("processor detail information: " + line)
                if line.find("64") > -1:
                    return 1
                else:
                    return 0
                #No need to continue to process the file, exit from the for loop
                break
            count = count + 1
    return 0

def isJDK64Bit(jvmmode):
    '''
       Check if the target JVM is 32-bit or 64-bit.
       This check is based on the following system properties:
       "com.ibm.vm.bitmode" (IBM JDK),
       "sun.arch.data.model".
    '''

    debug("Entry isJDK64Bit(%s) %s" % (jvmmode, fixlev))

    if jvmmode != None:
        jvmmode = jvmmode.strip()
    assert jvmmode == "64" or jvmmode == "32", "Unknown com.ibm.vm.bitmode: %s" % jvmmode

    if jvmmode == "64":
        return 1
    return 0

def getToolkitArch(confighome, jvmmode):
    '''
       Construct toolkit arch used in toolkit/lib/<arch>
    '''

    debug("Entry getToolkitArch(%s, %s) %s" % (confighome, jvmmode, fixlev))

    toolkit_arch = None
    if IS_AIX5_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_AIX4_R1
        else:
            toolkit_arch = ARCH_AIX_64
    elif IS_LINUX_IX86_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_LINUX_IX86
        else:
            toolkit_arch = ARCH_LINUX_IX64
    elif IS_LINUX_IX64_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_LINUX_IX86
        else:
            toolkit_arch = ARCH_LINUX_IX64
    elif IS_LINUX_PPC_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_LINUX_PPC
        else:
            toolkit_arch = ARCH_LINUX_PPC64
    elif IS_LINUX_S390_64_OS(confighome):
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_LINUX_S390
        else:
            toolkit_arch = ARCH_LINUX_S390_64
    elif IS_LINUX_S390_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_LINUX_S390
        else:
            toolkit_arch = ARCH_LINUX_S390_64
    elif IS_SOL_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_SOLARIS2
        else:
            toolkit_arch = ARCH_SOLARIS_64
    elif IS_SOL_IX86_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_SOLARIS2_IX86
        else:
            toolkit_arch = ARCH_SOLARIS2_IX86_64
    elif IS_HPUX_IA64N_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_HPUX_IA64N
        else:
            toolkit_arch = ARCH_HPUX_IA64
    elif IS_HPUX_IA64_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_HPUX_IA64N
        else:
            toolkit_arch = ARCH_HPUX_IA64
    elif IS_HPUX_64_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_HPUX10
        else:
            toolkit_arch = ARCH_HPUX_64
    elif IS_HPUX_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_HPUX10
        else:
            toolkit_arch = ARCH_HPUX_64
    elif IS_WINDOWS_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_W32_IX86
        else:
            toolkit_arch = ARCH_W64_X64
    elif IS_OS400_OS():
        if not isJDK64Bit(jvmmode):
            toolkit_arch = ARCH_OS400
        else:
            toolkit_arch = ARCH_OS400_64

    # This will hit if, by mistake, we call this routine under z/OS.
    assert toolkit_arch is not None, "The '%s' platform is not supported for toolkit architecture" % getPlatform()

    debug("Exit getToolkitArch(), result = %s" % toolkit_arch)
    return toolkit_arch


#######################################################################
# Unit Test
#######################################################################

if __name__ == "__main__":

    import debugUtil as log

    print "Is Windows = %s" % isWindows()
    print "Is Z = %s" % isZ()
    print "getEnLocale() = %s" % getEnLocale()

    log.set_logging()
