#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''

   This module handles getting the configured servers configuration data for AD72
   and previous releases.  The required config files to be read and PDL entries created.

'''

fixlev = "41683" # Last APAR, defect or story to update this module

import os
import codecs

import java.io.File

import debugUtil as log
import platformUtil
import configUtils
import utils
import readconfigfiles_zos

from debugUtil import logDebug as debug
from resourcebundle import getmsg
from constants import constants


RUNTIME_DIR = "RUNTIME_DIR"
RUNTIME_NAME = "RUNTIME_NAME"
ITCAMDCHOME = "ITCAMDCHOME"
DC_OPER_MODE ="DC_OPER_MODE"

#
# Instantiate the global constants class
#
common_constants = constants()


def read_pre_ad72_config_files(pdl,
                               pdl_entry_names,
                               config_dirlist,
                               dcinput_props,
                               ptype,
                               version,
                               vtype,
                               dcinput_fname = "dcInput.txt",
                               appserver_prop_name = "APP_SERVER_NAME"):
    '''
       For each server directory, read its configuration properties from the configuration files based on the
       provided properties to be acquired.

       @param pdl - pdl reference
       @param pdl_entry_names - A list of all the pdl names that are needed in FINDSERVERS
       @param config_dirlist - Configured Directory
       @param dcinput_props - A dictionary that contains the configured property names to be acquired and its
                              value is the pdl name that is inserted into FINDSERVERS.
       @param ptype - Determines the type of processing such as migrate, reconfig,..
       @param version - Version of the process type
       @param vtype -  process type like AD71, AD72, ecam,....
       @param dcinput_fname - DC input file name, the default is "dcInput.txt".  This file name is different
                              in DC 61 and ecam 7.1.1.
       @param appserver_prop_name - the property name for app server name which contains the complete
                                    name as cell, node, server.  The default property name is "APP_SERVER_NAME"
                                    but for DC61 and ecam, the property name is different.
       returns process, find_serverlist

    '''

    global common_constants

    debug("Entry read_pre_ad72_config_files %s" % fixlev)

    find_serverlist = []
    process = True
    seriousError = None
    migrate_path = pdl["MIGRATEHOME"][0]

    for config_dir in config_dirlist:
        debug("config_dir: %s" % config_dir)
        try:

            findservers_prop_entries = {}

            # Initialize the findserver dictionary entry.

            for pdl_entry in pdl_entry_names:
                findservers_prop_entries[pdl_entry] = None

            # Get the  properties from the configuration file based on the platform.  For
            # ZOS not all the required data is available in one file, so
            # the input.properties is also read to get the data, and also for
            # TT subsystem name, the toolkit_custom.properties in custom directory
            # is search.

            if platformUtil.IS_zOS_OS():
                readconfigfiles_zos.process_zos(pdl,
                                                config_dir,
                                                version,
                                                vtype,
                                                findservers_prop_entries)

            # Check dc input file.  Note that dc input file is available in all the distributed
            # and also for AD 71 ZOS.

            dcinput_file = os.path.normpath(os.path.join(config_dir, dcinput_fname))
            process, runtime_name, runtime_dir_loc = _read_dcinput_file(pdl,
                                                                        dcinput_file,
                                                                        dcinput_props,
                                                                        findservers_prop_entries,
                                                                        appserver_prop_name,
                                                                        version)
            if process == True:

                # Check the kwjdc properties file to get the tema info.
                # We need to make sure runtime directory is present, and in the
                # case of distributed, it is always present because it is always
                # available in dc input file, but in the case of Z for ecam and dc61,
                # there is no dc input file, and the runtime directory is the actual config
                # directory.

                if vtype == common_constants.ECAM or vtype == common_constants.DC61 or vtype == common_constants.AD71:
                    if runtime_dir_loc is None:
                        runtime_dir_loc = config_dir
                        debug("Runtime directory location is %s" % (runtime_dir_loc))
                        runtime_name = os.path.basename(runtime_dir_loc)
                        debug("Runtime directory name is %s " % (runtime_name))

                    _read_kwjdc_file(pdl,
                                     version,
                                     vtype,
                                     runtime_name,
                                     runtime_dir_loc,
                                     findservers_prop_entries)

                # If this is migrate, then set the pdl entries, otherwise
                # the default is already set to None.
                # Also, Check the server input.properties and if it does exist
                # issue a message and do not process the server.

                if ptype == common_constants.MIGRATE:
                    findservers_prop_entries["MIGRATEDDCHOME"] = migrate_path
                    findservers_prop_entries["MIGRATEDVERSION"] = version
                    findservers_prop_entries["MIGRATEDCONFIGHOME"] = config_dir
                    server_ok = _check_server(pdl, findservers_prop_entries)
                    if server_ok:
                        find_serverlist.append(findservers_prop_entries)
                else:
                    find_serverlist.append(findservers_prop_entries)

        except Exception, errmsg:
            process = False
            seriousError = "read_pre_ad72_config_files: %s" % errmsg
            print "\n%s" % getmsg("CFG5022E", "internal error {0}", [seriousError])
            log.logException("An error occurred processing AD 71 - %s" % (errmsg))

    # Now save the find server list in FINDSERVERS pdl, and prompt the user
    # to choose the servers.

    if len(find_serverlist) == 0:
        debug("Server list is empty, or an error has occurred.  Message: %s" % seriousError)
        if seriousError is None:
            print "\n%s" % getmsg("CFG1186I", "Server list is empty")
        else:
            print "%s" % getmsg("CFG1058E", "No servers discovered due to serious error previously issued")
        process = False
    else:

        # Once the files are processed, and the findserver list is created,
        # based on the findserver data, and some variable existence, create and
        # set other variables.

        process = True
        _check_findserver_list(find_serverlist)

    debug("Exiting read_pre_ad72_config_files, return %s - %s" %(process, find_serverlist))
    return process, find_serverlist


def _read_dcinput_file(pdl,
                       dcinput_filename,
                       dcinput_props,
                       findservers_prop_entries,
                       appserver_prop_name,
                       version):
    '''
       Reads the dc input file and sets the required properties in the find servers dictionary.

       @param pdl - pdl reference
       @param dcinput_filename - dc input file name
       @param dcinput_props - A dictionary that contains the configured property names to be acquired and its
                              value is the pdl name that is inserted into FINDSERVERS.
       @param findservers_prop_entries - A dictionary that has its key as the pdl names and value from the dc input property file
       @param appserver_prop_name - the property name for app server name which contains the complete
                                    name as cell, node, server.  The default property name is "APP_SERVER_NAME"
                                    but for DC61 and ecam, the property name is different.
       @param version - Version of the pre AD72
       @return - true/false if processed ok, runtime dir path, runtime dir name

    '''

    global common_constants

    debug("Entry _read_dcinput_file %s - %s - %s - %s - %s" % (fixlev, dcinput_filename, dcinput_props, appserver_prop_name, version))

    runtime_dir_loc = None
    runtime_name = None
    dcinput_property_value = None
    itcam_dc_enabled = None
    process = True

    dcinput_file = java.io.File(dcinput_filename)
    if dcinput_file.exists():
        try:
            debug("Processing file %s" % (dcinput_filename))
            if platformUtil.IS_zOS_OS():
                properties = configUtils.loadPropertiesBasic(dcinput_filename)
            else:
                properties = configUtils.loadProperties(dcinput_filename)

            # Read each property and match it against the dcinput
            # properties in "dcinput_props" and save it in find server
            # list for later use.

            for dcinput_property in properties.keys():
                debug("dcInput property:  <%s> " % (dcinput_property))
                dcinput_property_value = properties.get(dcinput_property)
                debug("dcInput property value:  <%s> " % (dcinput_property_value))

                # Check for RUNTIME_DIR property and if found, save it, as it is needed to
                # search for kwjdc.properties file for ecam and dc61.
                # Also, the ITCAMDCHOME is checked against the migrated home to make sure that
                # we are migrating the right server.  IN AD 71 with ifixes, the directory that
                # saves the configurd servers could be a mixture of previous releases, so we
                # check here to make sure to pickup the right one.

                if dcinput_property == RUNTIME_DIR:
                    debug("RUNTIME_DIR found - %s" % (dcinput_property))
                    runtime_dir_loc = dcinput_property_value
                    debug("RUNTIME_DIR location - %s" % (runtime_dir_loc))
                elif dcinput_property == RUNTIME_NAME:
                    debug("RUNTIME_NAME found - %s" % (dcinput_property))
                    runtime_name = dcinput_property_value
                    debug("RUNTIME_NAME is - %s" % (runtime_name))
                elif dcinput_property == ITCAMDCHOME:
                    debug("ITCAMDCHOME found - %s" % (dcinput_property))
                    itcamdchome = dcinput_property_value
                    debug("ITCAMDCHOME is - %s" % (itcamdchome))
                    migratehome = utils.getData(pdl, "MIGRATEHOME")

                    if platformUtil.getOSFamily() != "OS400":
                        tmpitcamdchome = itcamdchome
                        tmpmigratehome = migratehome
                    else:
                        tmpitcamdchome = itcamdchome.lower()
                        tmpmigratehome = migratehome.lower()

                    if tmpitcamdchome != tmpmigratehome:
                        process = False
                        debug("Migrate home does not match - %s - %s" % (tmpitcamdchome, tmpmigratehome))
                        break;
                else:
                    prop_pdl_entry = dcinput_props.get(dcinput_property)
                    if prop_pdl_entry is not None:
                        if dcinput_property == DC_OPER_MODE:

                            # Check for ecam or tpv in DC_OPER_MODE, and set the
                            # flag accordingly.

                            debug("DC_OPER_MODE found - %s" % (dcinput_property))
                            if (dcinput_property_value.find("ecam") >= 0 or
                                dcinput_property_value.find("tpv") >= 0):
                                itcam_dc_enabled = "1"
                            else:
                                itcam_dc_enabled = "0"

                            debug("ITCAM_DC_ENABLED value is - %s" % (itcam_dc_enabled))
                            findservers_prop_entries[prop_pdl_entry] = itcam_dc_enabled

                        # To get the cell, node, and server name, there is no specific property for them, so
                        # we do get it from provided property  which contains the
                        # cells/cell name/nodes/node name/servers/servername, for example:
                        # APP_SERVER_NAME=cells/IBM-6DA7F9C6EE6Node01Cell/nodes/IBM-6DA7F9C6EE6Node01/servers/server1
                        # This property is only available in distributed and the property name is provided based on
                        # the ITCAM version, as it is different.
                        # Also the profile home is set in "USERINSTALLROOT".

                        elif not platformUtil.IS_zOS_OS():
                            if dcinput_property == appserver_prop_name:
                                value = dcinput_property_value
                                appitems = value.split("/")
                                if len(appitems) > 2:
                                    findservers_prop_entries[prop_pdl_entry] = appitems[1]
                                    findservers_prop_entries["WASNODE"] = appitems[3]
                                    findservers_prop_entries["WASSERVER"] = appitems[5]
                            elif dcinput_property == "WAS_HOME_SHORT_PATH":
                                findservers_prop_entries["USERINSTALLROOT"] = dcinput_property_value
                                debug("Property %s matched and value replaced to %s" % ("USERINSTALLROOT", dcinput_property_value))
                                findservers_prop_entries[prop_pdl_entry] = dcinput_property_value
                            else:
                                findservers_prop_entries[prop_pdl_entry] = dcinput_property_value
                        else:
                            findservers_prop_entries[prop_pdl_entry] = dcinput_property_value

                        debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, dcinput_property_value))
            if process == True:
                debug(  "dcInput.txt findservers_prop_entries: %s" % str(findservers_prop_entries))

                # Save the file in the find server list and also add the
                # config location as an entry in the FINDSERVERS.

                findservers_prop_entries["CONFIGURED"] = "DC"

        except Exception, errmsg:
            log.logError("An error occurred processing PRE AD 71 - %s" % (errmsg))
            raise
    else:
        process = False
        debug("File not found - %s" % (dcinput_filename))

    debug ("Exiting _read_dcinput_file, Returning %s - %s - %s" % (process, runtime_name, runtime_dir_loc))

    return process, runtime_name, runtime_dir_loc


def _read_kwjdc_file(pdl,
                     version,
                     vtype,
                     runtime_name,
                     runtime_dir_loc,
                     findservers_prop_entries):
    '''
       Reads the kwjdc.properties file.  This file contains the TEMA information.  This routine
       only reads the file if processing for migration from ecam or DC 61 or AD 71.

       @param pdl - pdl reference
       @param version - Version
       @param vtype -  process type like AD71, AD72, ecam,....
       @param runtime_name - Runtime name
       @param runtime_dir_loc = Runtime directory location
       @param findservers_prop_entries - A dictionary that has its key as the pdl names and value from the dc input property file

    '''

    # This dictionary has the property names that are in input.properties, and
    # a search is done for all the properties in this list.
    # The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
    # Each entry in the FINDSERVERS is a dictionary which contains the same name
    # as the value of this dictionary.  The value of the dictionary are the names
    # that are matched in PDL

    kwjdc_props = {
                   "com.ibm.tivoli.kwj.agentport": "TEMAPORTLIST",
                   "com.ibm.tivoli.kwj.agenthostname": "TEMAHOSTLIST",
                   "com.ibm.tivoli.kwj.serveralias": "ALIASLIST",
                  }

    global common_constants

    debug("Entry _read_kwjdc_file %s" % fixlev)

    process = False

    try:

        # Check the version, and only process for DC61 and ecam version.  The kwjdc property file
        # name is different for ecam and dc61.  DC61 has the runtime name as a suffix, and ecam
        # does not.

        if (vtype == common_constants.ECAM or
            vtype == common_constants.AD71):
            kwjdc_properties_filename = os.path.normpath(os.path.join(runtime_dir_loc, "kwjdc.properties"))
            process = True
        elif vtype == common_constants.DC61:
            kwjdc_properties_filename = os.path.normpath(os.path.join(runtime_dir_loc, runtime_name + ".kwjdc.properties"))
            process = True
        if process == True:
            kwjdc_file = java.io.File(kwjdc_properties_filename)
            if kwjdc_file.exists():
                debug("kwjdc file is - %s" % (kwjdc_properties_filename))
                properties = configUtils.loadProperties(kwjdc_properties_filename)

                # Read each property and match it against the kwjdc
                # properties in "kwjdc_props" and save it in find server
                # list for later use.

                for kwjdc_property in properties.keys():
                    debug("kwjdc property:  <%s> " % (kwjdc_property))
                    kwjdc_property_value = properties.get(kwjdc_property)
                    debug("kwjdc property value:  <%s> " % (kwjdc_property_value))
                    prop_pdl_entry = kwjdc_props.get(kwjdc_property)
                    if prop_pdl_entry is not None:
                        debug("kwjdc property: %s - %s " % (prop_pdl_entry, kwjdc_property_value) )
                        findservers_prop_entries[prop_pdl_entry] = kwjdc_property_value
                        debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, kwjdc_property_value))
            else:
                debug("File not found - %s" % (kwjdc_properties_filename))

    except Exception, emsg:
        log.logException("Error occurred while processing kwjdc.properties in z - %s " % (emsg))
        raise

    debug ("Exiting _read_kwjdc_file")


def _read_AD72_kwjdc_file(pdl,
                          runtime_dir_loc,
                          findservers_prop_entries):
    '''
       Reads the kwjdc.properties file.  This file contains the TEMA information.

       @param pdl - pdl reference
       @param runtime_dir_loc = Runtime directory location
       @param findservers_prop_entries - A dictionary that has its key as the pdl names and value from the dc input property file

    '''

    debug("Entry _read_AD72_kwjdc_file %s runtime_dir_loc=%s" % (fixlev, runtime_dir_loc))

    # This dictionary has the property names that are in input.properties, and
    # a search is done for all the properties in this list.
    # The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
    # Each entry in the FINDSERVERS is a dictionary which contains the same name
    # as the value of this dictionary.  The value of the dictionary are the names
    # that are matched in PDL

    kwjdc_props = {
                   "com.ibm.tivoli.kwj.agentport": "TEMAPORTLIST",
                   "com.ibm.tivoli.kwj.agenthostname": "TEMAHOSTLIST",
                   "com.ibm.tivoli.kwj.serveralias": "ALIASLIST",
                  }

    prop_entries = {}

    try:
        kwjdc_properties_filename = os.path.normpath(os.path.join(runtime_dir_loc, "kwjdc.properties"))
        kwjdc_file = java.io.File(kwjdc_properties_filename)
        if kwjdc_file.exists():
            debug("kwjdc file is - %s" % (kwjdc_properties_filename))

            if platformUtil.IS_zOS_OS():
                input_properties_file = codecs.open(kwjdc_properties_filename, mode="r", encoding="cp1047")
            else:
                input_properties_file = codecs.open(kwjdc_properties_filename, mode="r", encoding="utf-8")

            # In case this is a DOS file we remove the CR as well as the LF.
            records = [record.rstrip("\r\n") for record in input_properties_file.readlines()]
            input_properties_file.close()
            debug("Records are:\n  %s" % "\n  ".join(records))

            for prop_data in records:
                if prop_data.startswith("#"): continue
                ws = prop_data.split("=",1)
                if len(ws) != 2: continue
                prop_entries[ws[0]] = ws[1]

            # Read each property and match it against the kwjdc
            # properties in "kwjdc_props" and save it in find server
            # list for later use.

            for kwjdc_property in prop_entries.keys():
                debug("kwjdc property:  <%s> " % (kwjdc_property))
                kwjdc_property_value = prop_entries.get(kwjdc_property)
                debug("kwjdc property value:  <%s> " % (kwjdc_property_value))
                prop_pdl_entry = kwjdc_props.get(kwjdc_property)
                if prop_pdl_entry is not None:
                    debug("kwjdc property: %s - %s " % (prop_pdl_entry, kwjdc_property_value) )
                    findservers_prop_entries[prop_pdl_entry] = kwjdc_property_value
                    debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, kwjdc_property_value))
        else:
            debug("File not found - %s" % (kwjdc_properties_filename))

    except Exception, emsg:
        log.logException("Error occurred while processing kwjdc.properties in z - %s " % (emsg))
        raise

    debug ("Exiting _read_AD72_kwjdc_file")


def _check_findserver_list(findserver_list):
    '''
       Checks the find server list which is a dictionary and checks some of the
       variables in the dictionary for their existence, and based on their existence,
       some variables will be set accordingly.

       @param findserver_list - A list that contains the FINDSERVER entries
    '''

    debug("Entry _check_findserver_list %s" % fixlev)

    os_platform = platformUtil.getOSFamily()
    for findserver_entry in findserver_list:

        # If this is not ZOS, the SMF list will have a value of NO.

        if not platformUtil.IS_zOS_OS():
            findserver_entry["SMFLIST"] = "No"

        # Set the SOA value to 0 which means there is no SOA

        findserver_entry["SOADCLIST"] = "0"

        # Set the TPV value to "0" if not already set

        if findserver_entry["TPVLIST"] is None:
            findserver_entry["TPVLIST"] = "0"

        # Set the DE value to 0 which means there is no DE

        findserver_entry["DELIST"] = "0"

        # Check the modified pdls for profile, cell, node, and server, and if
        # it has not been set, thenset the default.

        if findserver_entry["PROFILEMODIFIED"] is None:
            findserver_entry["PROFILEMODIFIED"] = findserver_entry["PROFILE"]
        if findserver_entry["WASCELLMODIFIED"] is None:
            findserver_entry["WASCELLMODIFIED"] = findserver_entry["WASCELL"]
        if findserver_entry["WASNODEMODIFIED"] is None:
            findserver_entry["WASNODEMODIFIED"] = findserver_entry["WASNODE"]
        if findserver_entry["SERVERSMODIFIED"] is None:
            findserver_entry["SERVERSMODIFIED"] = findserver_entry["WASSERVER"]

        # Check for TEMA existence and if it does exists, set a variable to
        # indicate its existence

        tema_port = findserver_entry.get("TEMAPORTLIST")
        debug("Tema Port: %s" % (tema_port))
        if (tema_port is "none" or
            tema_port is None):
            findserver_entry["TEMACONNECTLIST"] = "0"
        else:
            findserver_entry["TEMACONNECTLIST"] = "1"

        # Check for MS existence and if it does exists, set a variable to
        # indicate its existence, and also check probes for ports.

        ms_host = findserver_entry.get("KERNELIPLIST")
        ms_port = findserver_entry.get("KERNELPORT01LIST")
        probe_controller_rmi = findserver_entry.get("PROBECONTROLLERRMIPORTLIST")
        probe_rmi = findserver_entry.get("PROBERMIPORTLIST")
        debug("MS host: %s  MS port: %s" % (ms_host, ms_port))
        debug("Probe Controller: %s  Probe Rmi: %s" % (probe_controller_rmi, probe_rmi))
        if (ms_host is None or
            ms_port is None):
            findserver_entry["MSCONNECTLIST"] = "0"
            findserver_entry["MSHOMECONNECTLIST"] = "0"
        else:
            findserver_entry["MSCONNECTLIST"] = "1"
            findserver_entry["MSHOMECONNECTLIST"] = "1"
        if probe_controller_rmi is None:
            findserver_entry["PROBECONTROLLERRMIPORTLIST"] = "8300-8399"
        if probe_rmi is None:
            findserver_entry["PROBERMIPORTLIST"] = "8200-8299"

        # Check the TT (TTAPI) variables.
        # In z/OS, for AD71 we cannot conclusively locate TTAPIDIR, TTAPILIBPATH or TTAPICLASSPATH
        # since WAS was updated manually, not thru setupwas.sh. TTAPI libpath and classpath entries may not be obvious.
        # Since these WAS libpath/classpath entries likely do not contain ${ITCAMDCHOME}, config will not likely remove them.
        # Therefore, locating them is not required for TT migration to succeed.

        if platformUtil.IS_zOS_OS():
            TT = findserver_entry.get("TTAPILIST")
            TTssn = findserver_entry.get("TTAPISSNLIST")
            TTdir = findserver_entry.get("TTAPIDIRLIST")
            debug("TT is: %s  TTssn is: %s   TTdir is: %s" % (TT, TTssn, TTdir))
            if TT is None or TT == "0":
                findserver_entry["TTAPILIST"] = "0"
            else:
                findserver_entry["TTAPILIST"] = "1"
            # A value of 'None' doesn't work well in input.properties substitution
            findserver_entry["TTAPILIBPATH"]   = ""
            findserver_entry["TTAPICLASSPATH"] = ""
        else:
            TThost = findserver_entry.get("TTTCHOSTLIST")
            TTport = findserver_entry.get("TTTCPORTLIST")
            debug("TT Host: %s  TT Port: %s" % (TThost, TTport))
            if (TThost is None or TThost == "none" or
                TTport is None or TTport == "none"):
                findserver_entry["TTAPILIST"] = "0"
            else:
                findserver_entry["TTAPILIST"] = "1"

        # Check the current gc log location, and if it has been provided
        # then it will be used, and gc flag is set accordingly.

        if not platformUtil.IS_zOS_OS():
            gclogpath = findserver_entry.get("CURRENTGCLOGLOCATION")
            debug("Current Gc log path is: %s" %(gclogpath))
            if gclogpath is None or gclogpath == "":
                findserver_entry["GC_PARMS"] = "-verbosegc "
                findserver_entry["GCOUTPUTDISTLIST"] = "0"
                findserver_entry["GCLOGPATHLIST"] = "None"
            else:
                findserver_entry["GCOUTPUTDISTLIST"] = "1"

                # Set the gc log path to None as the current gc location is
                # already present.

                findserver_entry["GCLOGPATHLIST"] = "None"

                # If gc log path starts with "${SERVER_LOG_ROOT}", the modification
                # to the path is not taking place as the path will be unique, otherwise,
                # check the path for "/" or "\".  We are not using os.sep
                # as the user could specify "/" for windows and that is an
                # ok specification, so we check for both characters.

                if not gclogpath.startswith("${SERVER_LOG_ROOT}"):
                    pathe = gclogpath.split("/")
                    if len(pathe) == 1:
                        pathe = gclogpath.split("\\")
                    index = len(pathe)
                    if len(pathe) == 1:
                        gclogfile = pathe
                    else:
                        gclogfile = pathe[index-1]
                    debug("GC log file - %s" % (gclogfile))

                    wasprofile = findserver_entry["PROFILE"]
                    wascell = findserver_entry["WASCELL"]
                    wasnode = findserver_entry["WASNODE"]
                    wasserver = findserver_entry["WASSERVER"]
                    new_gclogfile = wasprofile + "." + wascell + "." + wasnode + "." + wasserver + "." + gclogfile
                    gclogpath = ""
                    if len(pathe) == 1:
                        gclogpath = new_gclogfile
                    else:
                        for gclogi in range(len(pathe) - 1):
                            if gclogi == 0:
                                gclogpath = pathe[gclogi]
                            else:
                                gclogpath = os.path.normpath(os.path.join(gclogpath, pathe[gclogi]))
                        gclogpath = os.path.normpath(os.path.join(gclogpath, new_gclogfile))

                    # based on a platform set the GC entries.  If the gc log path
                    # starts with , there is no need to set the gc log location, as it
                    # is already set.

                    if os_platform == "HP-UX" or os_platform == "SOLARIS":
                        gcparms = "-XX:+PrintGCTimeStamps -verbosegc -Xloggc:%s" %(gclogpath)
                    else:
                        gcparms = "-verbosegc -Xverbosegclog:%s" %(gclogpath)

                    # Set the GC_PARMS pdl.

                    findserver_entry["GC_PARMS"] = gcparms
                else:
                    findserver_entry["GC_PARMS"] = "-verbosegc "
                    findserver_entry["CURRENTGCLOGLOCATION"] = "None"
                    findserver_entry["GCOUTPUTDISTLIST"] = "0"

        # The heap size for migrate will not be touched so the PDL entry will
        # have a vlaue of zero which indicates not to touch the heap size.

        if not platformUtil.IS_zOS_OS():
            findserver_entry["MAXHEAPSIZE"] = "0"

        # In ZOS,to find out if this is ND, the STANDALONE property is checked which
        # contains the data from  "DMGR_RMI" in dcInput.txt.  If it has a value, then a
        # true value is set for "STANDALONE" key otherwise, false is set.

        if platformUtil.IS_zOS_OS():
            standalone = findserver_entry["STANDALONE"]
            debug("Stand Alone is: %s" % (standalone))
            if standalone == "":
                findserver_entry["STANDALONE"] = "true"
            else:
                findserver_entry["STANDALONE"] = "false"

        debug("Find Server Entry - %s" % (findserver_entry))

        debug("Exit _check_findserver_list")


def read_ad72_inputproperty_file(pdl,
                                 pdl_entry_names,
                                 inputproperties_props,
                                 filelist,
                                 ptype,
                                 version,
                                 printError=True):
    '''
       For each file in the files list, read it and get the properties that are
       listed in "inputproperties_props" and save properties value in a dictionary for each server.
       Each dictionary is saved in a list which saved as FINDSERVERS in pdl

       @param pdl - pdl reference
       ? @param dcinput_props - A dictionary that contains the inputproperty property names to be acquired and its
                              value is the pdl name that is inserted into FINDSERVERS.
       @param inputproperties_props - A dictionary that the key is the property name in the
                                      input.properties and its value is the pdl name
       @parma filelist - A list of files that ends with input.properties in "runtime" directory.
                         List contains the full path.
       @param ptype - Process type
                       1 - reconfig
                       2 - upgrade
       @param version - Version of config
       @param printError - print message to the console. By default the message will be printed
    '''

    debug("Entry read_ad72_inputproperty_file %s" % fixlev)
    debug("..pdl_entry_names:\n    %s" % "\n    ".join(pdl_entry_names))
    debug("..inputproperties_props:\n    %s" % "\n    ".join(["%s->%s" % (ent, inputproperties_props[ent]) for ent in sorted(inputproperties_props.keys())]))
    debug("..filelist:\n    %s" % "\n    ".join(filelist))
    debug("..ptype: %s" % ptype)
    debug("..version: %s" % version)

    find_serverlist = []
    infile = None
    runtime_dir_loc = None
    process = False
    seriousError = None
    defaultDCVersion = pdl["VERSION"][0]

    for config_file in filelist:
        # To do: This try should be just covering the reading of the file, not everything that could go wrong
        try:
            debug("Processing file %s " % (config_file))
            infile = codecs.open(config_file, mode="r", encoding="utf-8")
            matched = 0
            findservers_prop_entries = {}

            # Initialize the findserver dictionary entry.

            for pdl_entry in pdl_entry_names:
                findservers_prop_entries[pdl_entry] = None

            # Read input.properties for the server, and read the properties that are
            # defined in "inputproperties_props" and set the dictionary value based on
            # the property value.

            wlp_home_found = False
            if "WLP_HOME" not in inputproperties_props.keys():
                inputproperties_props["WLP_HOME"] = "WLPHOME"
            # In case this is a DOS file we remove the CR as well as the LF.
            records = [record.rstrip("\r\n") for record in infile.readlines() if record.startswith("# ")]
            infile.close();

            debug("Input properties file contents:\n\n    %s" % "\n    ".join(records))

            for line in records:

                # Only examine the commented properties
                if not line.startswith("# "): continue
                line = line[2:]
                prop = line.split("=",1)
                if len(prop) != 2: continue
                debug("Processing Record: <%s=%s> " % (prop[0],prop[1]))
                prop[0] = prop[0].strip()
                prop[1] = prop[1].strip()
                prop_pdl_entry = inputproperties_props.get(prop[0])
                if prop_pdl_entry is None: continue

                matched = matched + 1

                # If processing "WAS_HOMEPROFILE", get the data and also set it
                # in the pdl as "USERINSTALLROOT".

                if prop[0] == "WAS_HOMEPROFILE":
                    findservers_prop_entries["USERINSTALLROOT"] = prop[1]
                    findservers_prop_entries[prop_pdl_entry] = prop[1]
                    debug("Property %s matched and value replaced to %s" % ("USERINSTALLROOT", prop[1]))
                elif prop[0] == "WLP_HOME":
                    debug("WLP_HOME found")
                    debug("prop_pdl_entry=%s prop[1]=%s" % (prop_pdl_entry, prop[1]))
                    findservers_prop_entries[prop_pdl_entry] = prop[1]
                    debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, prop[1]))
                    wlp_home_found = True
                # Special processing for TEMACONNECT is due to a latent buglet.
                elif prop[0] == "TEMACONNECT":
                    debug("TEMACONNECT found with value '%s'" % prop[1])
                    if prop[1] not in ("0","1"):
                        debug("Corrected to '0'")
                        prop[1] = "0"
                    findservers_prop_entries[prop_pdl_entry] = prop[1]
                    debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, prop[1]))
                else:
                    findservers_prop_entries[prop_pdl_entry] = prop[1]
                    debug("Property %s matched and value replaced with %s" % (prop_pdl_entry, prop[1]))

            if not wlp_home_found:
                debug("no WLP_HOME for %s" % config_file)
                inputproperties_props.pop("WLP_HOME", None)

            # Must default SUBPRODUCT because it was not originally present in 7.2 base
            if findservers_prop_entries["SUBPRODUCT"] is None:
                debug("Defaulting SUBPRODUCT to None")
                findservers_prop_entries["SUBPRODUCT"] = "None"
                matched += 1

            # Must default CONFIGRETURNCODE because it was not originally present in 7.2 base
            if findservers_prop_entries["CONFIGRETURNCODE"] is None:
                debug("Defaulting CONFIGRETURNCODE to 0")
                findservers_prop_entries["CONFIGRETURNCODE"] = 0
                matched += 1
            # save the selected properties in the find server list and close the file.
            # Check the matched numbers and if this is migrate, then the entries
            # in the input.properties for this release might have new entries that
            # are not available in the previous release, so no need to check to see
            # if we have all the data, but for any other requests, it is checked to
            # make sure that all entries are available.

            infile = None
            if matched > 0:
                process = True
                findservers_prop_entries["CONFIGURED"] = defaultDCVersion
                if ptype != common_constants.MIGRATE and matched != len(inputproperties_props):
                    log.logError("No matching property or missing property found in %s" % (config_file))
                    print "%s" % getmsg("CFG1059E", "No matching property or missing property found in - {0}", [config_file])
                    for prop_entry in inputproperties_props.keys():
                        if findservers_prop_entries[inputproperties_props[prop_entry]] is None:
                            print getmsg("CFG1057E", "Property {0} has no value or doesn't exist - {1}", [prop_entry, config_file])
                    process = False
                else:
                    if ptype == common_constants.MIGRATE:

                        # Check the kwjdc properties file to get the tema info.
                        # The runtime directory is not saved anywhere, because it is
                        # built by the runtime builder when DC is coming up, so we
                        # need to determine the runtime directory.  It is in the
                        # format of APPSERVER.STACK.CELL.NODE.PROFILE.SERVER, so
                        # the combination of CELL.NODE.PROFILE.SERVER makes it a
                        # unique name.  A search is done in the runtime directory
                        # for a match.

                        migrate_path = pdl["MIGRATEHOME"][0]
                        wascell = findservers_prop_entries["WASCELLMODIFIED"]
                        wasnode = findservers_prop_entries["WASNODEMODIFIED"]
                        wasserver = findservers_prop_entries["SERVERSMODIFIED"]
                        wasprofile = findservers_prop_entries["PROFILEMODIFIED"]
                        if (wasprofile is None or len(wasprofile) == 0):
                            debug("WAS Liberty found")
                            runtime_prefix = wascell + "." + wasnode + "." + wasserver
                        else:
                            runtime_prefix = wascell + "." + wasnode + "." + wasprofile + "." + wasserver
                        debug("Runtime Prefix - %s" % (runtime_prefix))
                        runtimedir_name = os.path.normpath(os.path.join(migrate_path,  "runtime"))
                        runtimedir_path = java.io.File(runtimedir_name)
                        if runtimedir_path.exists():
                            listing = runtimedir_path.list()
                            for infile in listing:
                                infile = os.path.normpath(os.path.join(runtimedir_path.getCanonicalPath(), infile))
                                runtimedir_file = java.io.File(infile)
                                if runtimedir_file.isDirectory():
                                    runtimedir_filename = runtimedir_file.getName()
                                    debug("Runtime File name: %s" % (runtimedir_filename))
                                    if runtimedir_filename.endswith(runtime_prefix):
                                        debug("Found runtime file for migrate - %s" % (infile))
                                        runtime_dir_loc = infile
                                        process = True
                                        break
                                    else:
                                        debug("Runtime directory does not exist - %s" % (runtimedir_name))
                                        process = False

                        if process == True:
                            _read_AD72_kwjdc_file(pdl,
                                                  runtime_dir_loc,
                                                  findservers_prop_entries)

                            # If this is migrate, then set the pdl entries, otherwise
                            # the default is already set to None.
                            # Also, Check the server input.properties and if it does exist
                            # issue a message and do not process the server.

                            findservers_prop_entries["MIGRATEDDCHOME"] = migrate_path
                            findservers_prop_entries["MIGRATEDVERSION"] = version
                            findservers_prop_entries["MIGRATEDCONFIGHOME"] = config_file
                            server_ok = _check_server(pdl, findservers_prop_entries)
                            if server_ok:
                                find_serverlist.append(findservers_prop_entries)
                    else:
                        find_serverlist.append(findservers_prop_entries)
            else:
                debug("No property matched ")

        except Exception, errmsg:
            seriousError = "read_ad72_inputproperty_file: %s" % errmsg
            print "\n%s" % getmsg("CFG5022E", "internal error {0}", [seriousError])
            log.logError("An error occurred processing AD 7.2 input properties - %s" % (errmsg))
            if infile is not None:
                infile.close();
            process = False

    # Now save the find server list in FINDSERVERS pdl, and prompt the user
    # to choose the servers.

    if len(find_serverlist) == 0:
        debug("Server list is empty")
        if seriousError is None:
            if printError:
                print "\n%s" % getmsg("CFG1186I", "Server list is empty")
        else:
            print "%s" % getmsg("CFG1058E", "No servers discovered due to serious error previously issued")
        process = False
    else:
        process = True

    debug("Exit read_ad72_inputproperty_file")
    return process, find_serverlist


def _check_server(pdl, findservers_prop_entries):
    '''
       Checks the existence of the input properties for a specific server in "runtime" directory, and
       if it does exist, it returns the flag accordingly.  Also, the WAS version is checked.  Only,
       WAS version 7 and up is supported for migration.

       @param pdl - PDL reference
       @param findservers_prop_entries - A dictionary for a servetr which contains the server's info
       @return - True if file exists, otherwise False
    '''

    debug("Entry _check_server %s" % fixlev)

    server_stat = True

    path = utils.getData(pdl, "CONFIGHOME") + "/runtime"
    wascell = findservers_prop_entries["WASCELL"]
    wasnode = findservers_prop_entries["WASNODE"]
    wasserver = findservers_prop_entries["WASSERVER"]
    wasversion = findservers_prop_entries["WASVERSION"]
    wasprofile = findservers_prop_entries["PROFILE"]
    inputproperties_file_path = os.path.normpath(os.path.join(path, wasprofile + " ." + wascell + "." + wasnode + "." + wasserver +  ".input.properties"))

    # Check the existence of the input.properties for the server, and if it does exist,
    # then return a flag accordingly.

    if os.path.exists(inputproperties_file_path):
        debug("Server has already been configured in this release - %s" % (inputproperties_file_path))
        msgl =[]
        msgl.append(wasserver)
        msgl.append(wascell + "." + wasnode)
        errmsg = getmsg("CFG1108I", "Server {0} has already been configured in this release - {1}", msgl)
        print errmsg
        server_stat = False
    else:

        # Check the WAS version, and if it is not version 7 or higher, then set the
        # flag accordingly.

        wver = wasversion[0]
        if utils.isInt(wver):
            wver_int = int(wver)
            if wver_int < 7:
                debug("Server has a WAS version that is not supported - %s" % (wver))
                msgl =[]
                msgl.append(wasserver + "." + wascell + "." + wasnode)
                msgl.append(wver)
                errmsg = getmsg("CFG1136I", "Server {0} has WAS version {1}, and is not supported.", msgl)
                print errmsg
                server_stat = False
        else:
            debug("Server has a WAS version that is not supported - %s" % (wver))
            msgl =[]
            msgl.append(wasserver + "." + wascell + "." + wasnode)
            msgl.append(wver)
            errmsg = getmsg("CFG1136I", "Server {0} has WAS version {1}, and is not supported.", msgl)
            print errmsg
            server_stat = False

    debug("Exit _check_server - %s" % (server_stat))

    return server_stat



def set_probes_for_reconfig(pdl, pdl_entries):
    '''
       Sets the probe info for the reconfig.  For each selected servers, the probe info
       is read from pdl, and it is set in the CONFIGUREPROBES pdl.

       @param pdl - pdl reference
       @param pdl_entries - A list of pdl entries which are the probe entries for a server
       @return - True if processing ok, otherwise False

    '''

    debug("Entry set_probes_for_reconfig %s" % fixlev)

    process = False
    probe_entries = {}

    fullservers = utils.getData(pdl, "FULLSERVERS")

    # Create probe pdls.

    utils.setData(pdl, "CONFIGUREDPROBES", {}, "readconfigfiles")

    for serveri in range(len(fullservers)):
        probe_entry = {}
        for pdl_probe_entry in pdl_entries:
            probe_entry[pdl_probe_entry] = utils.getData(pdl, pdl_probe_entry, serveri)
        probe_entries[serveri] = probe_entry
        process = True

    if process == True:
        debug("Probe entries - %s" % (probe_entries))
        utils.setData(pdl, "CONFIGUREDPROBES", probe_entries, "readconfigfiles")

    debug("Exit set_probes_for_reconfig")

    return process



def set_probes(pdl,
               inputproperties_probe_props,
               filelist,
               keylist):
    '''
       For each file in the files list, read it and get the properties that are
       listed in "inputproperties_props" and save properties value in a dictionary for each server.
       Each dictionary is saved in a list which saved as CONFIGUREDPROBES in pdl

       @param pdl - pdl reference
       @param inputproperties_probe_props - A dictionary that the key is the property name in the
                                      input.properties and its value is the pdl name
       @parma filelist - A list of input.properties files.
                         List contains the full path.
       @param keylist - A list that contains the server index.
       @return - True if processing ok, otherwise False
    '''

    debug("Entry set_probes %s" % fixlev)

    index = 0
    process = True
    infile = None
    probe_entries = {}

    # Create probe pdls.

    utils.setData(pdl, "CONFIGUREDPROBES", {}, "readconfigfiles")

    for config_file in filelist:
        probe_key = keylist[index]
        try:
            debug("Processing file %s " % (config_file))
            infile = codecs.open(config_file, mode="r", encoding="utf-8")
            matched = 0
            probe_entry = {}

            # Read input.properties for the server, and read the properties that are
            # defined in "inputproperties_props" and set the dictionary value based on
            # the property value.

            # In case this is a DOS file we remove the CR as well as the LF.
            lines = [line.strip("\r\n") for line in infile.readlines()]
            infile.close();
            debug("Lines in %s file:\n\n      %s" % (config_file, "\n      ".join(lines)))

            for line in lines:
                if line.startswith("# "):
                    line = line[len("# "):len(line)]
                    prop = line.split("=")
                    if len(prop) == 2:
                        debug("Processing Record: <%s> " % (prop))
                        prop_pdl_entry = inputproperties_probe_props.get(prop[0])
                        if prop_pdl_entry is not None:
                            matched = matched + 1
                            debug("Property <%s> found, value: <%s>" % (prop[0], prop[1]))
                            probe_entry[prop_pdl_entry] = prop[1]

            # Check that the all the probe values are acquired

            infile = None
            index = index + 1
            if matched != len(inputproperties_probe_props):
                log.logError("No matching property or missing property found in - %s" % (config_file))
                msgl =[]
                msgl.append(config_file)
                errmsg = getmsg("CFG1059E", "No matching property or missing property found in - {0}", msgl)
                print errmsg
                for prop_entry in inputproperties_probe_props.keys():
                    if not probe_entry.has_key(prop_entry):
                        print getmsg("CFG1057E", "Property {0} has no value or doesn't exist - {1}", [prop_entry, config_file])
                process = False
                break
            else:
                probe_entries[probe_key] = probe_entry


        except Exception, errmsg:
            log.logException("An error occurred processing set probes - %s" % (errmsg))
            if infile is not None:
                infile.close();
            process = False

        if process == False:
            break


    if process == True:
        debug("Probe entries - %s" % (probe_entries))
        utils.setData(pdl, "CONFIGUREDPROBES", probe_entries, "readconfigfiles")

    debug("Exit set_probes")

    return process
