#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''

 This module handles getting the configured servers configuration data for pre
 AD72 for ZOS.  The required config files are read and PDL entries
 are created.

'''

fixlev = "38001" # Last APAR, defect or story to update this module

import os
import debugUtil as log
import configUtils

from debugUtil import logDebug as debug
from resourcebundle import getmsg
from constants import constants


#
# This dictionary has the property names that are in input.properties, and
# a search is done for all the properties in this list.
# The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
# Each entry in the FINDSERVERS is a dictionary which contains the same name
# as the value of this dictionary.  The value of the dictionary are the names
# that are matched in PDL
#
inputproperties_props = {
                         "WAS_CELL": "WASCELL",
                         "WAS_NODE": "WASNODE",
                         "WAS_SERVER": "WASSERVER",
                         "WAS_HOME": "WASINSTALLROOT",
                         "JAVA_HOME": "JAVAHOME",
                         "LOG_ROOT": "PROFILEHOME",
                         "server.smf": "SMFLIST",
                         "server.systemproperty": "CURRENTGCLOGLOCATION",
                         "server.id": "SERVERID",
                         "server.version": "WASVERSION",
                        }


#
# This dictionary has the property names that are in input.properties, and
# a search is done for all the properties in this list.
# The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
# Each entry in the FINDSERVERS is a dictionary which contains the same name
# as the value of this dictionary.  The value of the dictionary are the names
# that are matched in PDL.
# This is for dc61.
#
inputproperties_props_dc61 = {
                               "WAS_CELL": "WASCELL",
                               "WAS_NODE": "WASNODE",
                               "WAS_SERVER": "WASSERVER",
                               "WAS_HOME": "WASINSTALLROOT",
                               "JAVA_HOME": "JAVAHOME",
                               "LOG_ROOT": "PROFILEHOME",
                               "server.smf": "SMFLIST",
                               "server.systemproperty": "CURRENTGCLOGLOCATION",
                               "server.id": "SERVERID",
                               "server.version": "WASVERSION",
                               "server.variable": "MSHOMELIST",
                             }



#
# Instantiate the global constants class
#
common_constants = constants()


def _read_pre_ad72_inputproperty_file(pdl,
                                      filelocation,
                                      findservers_prop_entries,
                                      inputproperties_props):
    '''
       Read the input.properties from the runtime directory for a specific server.
       based on the provided propertries, the properties are read and set in the
       find servers dictionary.

       @param pdl - pdl reference
       @param filelocation - Location of the input.properties
       @param findservers_prop_entries - A dictionary which contains the server's information
       @param inputproperties_props - A dictionary providing the inutproperties properties to be acquired

    '''


    infile = None

    debug("Entry _read_pre_ad72_inputproperty_file %s" % fixlev)

    try:
        debug("_read_pre_ad72_inputproperty_file entry  - for %s" % str(filelocation))
        infile = open(filelocation, "r")
        matched = 0
        doset = False
        #
        # Read input.properties for the server, and read the properties that are
        # defined in "inputproperties_props" and set the dictionary value based on
        # the property value.
        #
        # In case this is a DOS file we remove the CR as well as the LF.
        lines = [line.rstrip("\r\n") for line in infile.readlines()]
        debug("input.properties(z/OS) Records:\n  %s" % "\n  ".join(lines))
        infile.close();
        infile = None

        for line in lines:
            doset = False
            value = None
            if line.startswith("# "):
                line = line[len("# "):]
                prop = line.split("=")
                if len(prop) == 2:
                    value = prop[1]
                    doset = True
            else:
                prop = line.split("=")
                if len(prop) >= 2:
                    value = prop[1]
                    doset = True
            if doset == True:
                prop_pdl_entry = inputproperties_props.get(prop[0])
                debug("prop_pdl_entry: %s for %s " % (str(prop_pdl_entry), prop[0]))
                if prop_pdl_entry is not None:
                    matched = matched + 1
                    #
                    # If this is LOG_ROOT data, we get the profile name from it, as this
                    # is the only place that has the profile in its path.  Note that this
                    # is only done for Z, so we check for '/'
                    #
                    if prop[0] == "LOG_ROOT":
                        file_entries = value.split("/")
                        profileloc = ""
                        for index in range(len(file_entries) -1):
                            if index == 0:
                                profileloc = "/" + file_entries[index]
                            else:
                                profileloc+= "/" + file_entries[index]
                        debug("  profile location entry: %s" % profileloc)
                        #proplr = profileloc.split("=")
                        #value = proplr[1]
                        findservers_prop_entries["PROFILEHOME"] = profileloc
                        findservers_prop_entries["USERINSTALLROOT"] = profileloc
                        debug("Property %s matched and value replaced to %s" % ("USERINSTALLROOT", profileloc))
                    elif prop[0] == "server.systemproperty":
                        prop = line.split("=")
                        if len(prop) == 3:
                            if prop[1] == "TEMAGCCollector.gclog.path":
                                findservers_prop_entries[prop_pdl_entry] = prop[2]
                                debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, prop[2]))
                    elif prop[0] == "server.variable":
                        prop = line.split("=")
                        if len(prop) == 3:
                            if prop[1] == "MS_AM_HOME":
                                findservers_prop_entries[prop_pdl_entry] = prop[2]
                                debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, prop[2]))
                    else:
                        findservers_prop_entries[prop_pdl_entry] = value
                        debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, value))
        #
        # The GC_PARMS value for ZOS is always "-verbose:gc ", so we set the
        # value in  find server list, also there is no JVMMODE so we create
        # the entry as None.
        #
        findservers_prop_entries["GC_PARMS"] = "-verbose:gc "
        findservers_prop_entries["JVMMODE"] = "None"
        #
        # save the file in the find server list and close the file.
        #
        if matched < 1 or matched != len(inputproperties_props):
            log.logError("No matching property or missing property found in - %s" % (filelocation))
            msgl =[]
            msgl.append(filelocation)
            errmsg = getmsg("CFG1059E", "No matching property or missing property found in - {0}", msgl)
            print errmsg
            for prop_entry in inputproperties_props.keys():
                if findservers_prop_entries[inputproperties_props[prop_entry]] is None:
                    debug("Property is missing value  %s " % (prop_entry))
                    debug("Property %s is missing value or does not exist - %s" % (prop_entry, filelocation))
            raise

        debug("Exit _read_pre_ad72_inputproperty_file")

    except Exception, emsg:
        if infile is not None: infile.close();
        log.logException("Error occurred while processing input.properties in z - %s " % (emsg))
        raise


def _read_pre_ad72_reconfig_data_file(pdl, filelocation, findservers_prop_entries):
    '''
       Read the reconfig.data file from the runtime directory for a specific server.
       Read property "CONNTYPE" ex "CONNTYPE=-conntype SOAP" for connection type to wsadmin
       Read property "HOST"     ex: HOST="-host tvt7006.tivlab.raleigh.ibm.com"
       Read property "PORT"     ex: PORT="-port 8879" or PORT=""

       @param pdl - pdl reference
       @param filelocation - Location of reconfig.data
       @param findservers_prop_entries - A dictionary which contains the server's information
    '''

    #
    # This dictionary has the property names that are in reconfig.data, and
    # a search is done for all the properties in this list.
    # The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
    # Each entry in the FINDSERVERS is a dictionary which contains the same name
    # as the value of this dictionary.  The value of the dictionary are the names
    # that are matched in PDL
    #
    reconfig_props = {
                       "CONNTYPE": "CONNTYPELIST",
                       "HOST": "HOSTLIST",
                       "PORT": "PORTLIST",
                     }

    debug("Entry _read_pre_ad72_reconfig_data_file %s" % fixlev)

    try:
        properties = configUtils.loadProperties(filelocation)
        #
        # Read each property and match it against the reconfig
        # properties in "reconfig_props" and save it in find server
        # list for later use.
        #
        for reconfig_property in properties.keys():
            debug("reconfig property:  <%s> " % (reconfig_property))
            reconfig_property_value = properties.get(reconfig_property)
            debug("reconfig property value:  <%s> " % (reconfig_property_value))
            prop_pdl_entry = reconfig_props.get(reconfig_property)
            if prop_pdl_entry is not None:
                if reconfig_property_value is not None:
                    reconfig_property_value=reconfig_property_value.strip('\"')
                    if prop_pdl_entry == "CONNTYPELIST"  :
                        reconfig_property_value = reconfig_property_value.replace("-conntype ","")
                    elif prop_pdl_entry == "HOSTLIST":
                        reconfig_property_value = reconfig_property_value.replace("-host ","")
                    elif prop_pdl_entry == "PORTLIST" :
                        reconfig_property_value = reconfig_property_value.replace("-port ","")
                findservers_prop_entries[prop_pdl_entry] = reconfig_property_value
                debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, reconfig_property_value))

        debug("Exit _read_pre_ad72_reconfig_data_file")

    except Exception, msgVar:
        log.logException("Unable load reconfig.data -  exception %s" % msgVar)
        raise


def _read_pre_ad72_toolkit_custom_file(pdl, filelocation, findservers_prop_entries):
    '''
       Read the toolkit_custom.properties from the runtime.custom directory for a specific server.
       Read property "com.ibm.tivoli.itcam.dc.ttapi.enable" [=true|false] to determine if TTAPI enabled
       Read property "com.ibm.tivoli.itcam.dc.ttapi.ttas.transport" to get the ssn name.  The
       ssn is in the form of  ssn:CYTZ

       @param pdl - pdl reference
       @param filelocation - Location of the toolkit_custom.properties
       @param findservers_prop_entries - A dictionary which contains the server's information

    '''

    #
    # This dictionary has the property names that are in toolkit_custom.properties, and
    # a search is done for all the properties in this list.
    # The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
    # Each entry in the FINDSERVERS is a dictionary which contains the same name
    # as the value of this dictionary.  The value of the dictionary are the names
    # that are matched in PDL
    #
    toolkit_custom_props = {
                             "com.ibm.tivoli.itcam.dc.ttapi.enable": "TTAPILIST",
                             "com.ibm.tivoli.itcam.dc.ttapi.ttas.transport": "TTAPISSNLIST",
                           }


    tt_enabled = False
    ssnvalue = None
    ssn_acquired = False

    debug("Entry _read_pre_ad72_toolkit_custom_file %s" % fixlev)

    try:
        properties = configUtils.loadProperties(filelocation)
        #
        # Read each property and match it against the toolkit_custom.prooerties
        # properties in "toolkit_custom_props" and save it in find server
        # list for later use.
        #
        for custom_property in properties.keys():
            debug("custom property:  <%s> " % (custom_property))
            custom_property_value = properties.get(custom_property)
            debug("custom property value:  <%s> " % (custom_property_value))
            prop_pdl_entry = toolkit_custom_props.get(custom_property)
            if prop_pdl_entry is not None:
                if prop_pdl_entry == "com.ibm.tivoli.itcam.dc.ttapi.enable":
                    debug("Found property com.ibm.tivoli.itcam.dc.ttapi.enable")
                    enable = custom_property_value.strip().lower()
                    if enable == "true":
                        tt_enabled = True
                        findservers_prop_entries[prop_pdl_entry] = "1"
                    else:
                        findservers_prop_entries[prop_pdl_entry] = "0"
                    debug("TTAPI value is %s - " % str(tt_enabled))
                    continue
                elif prop_pdl_entry == "com.ibm.tivoli.itcam.dc.ttapi.ttas.transport":
                    debug("Found property com.ibm.tivoli.itcam.dc.ttapi.ttas.transport")
                    ssn = custom_property_value
                    ssnprop = ssn.split(":")
                    if len(ssnprop) == 2:
                        ssnvalue = ssnprop[1]
                        findservers_prop_entries[prop_pdl_entry] = ssnvalue
                        debug("SSN value is %s - " % (ssnvalue))
                        ssn_acquired = True
                    continue
        if not ssn_acquired:
            debug("SSN value not provided")

        debug("Exit _read_pre_ad72_toolkit_custom_file - return - %s" % (ssn_acquired))

    except Exception, emsg:
        log.logException("Error occurred while processing toolkit_custom.properties in z - %s " % (emsg))
        raise


    return ssn_acquired


def _read_pre_ad72_datacollector_file(pdl, filelocation, findservers_prop_entries):
    '''
       Reads the data collector properties to get config data such as MS info, and sets the
       find servers dictionary for a specific server.

       @param pdl - pdl reference
       @param filelocation - Location of datacollector.properties
       @param findservers_prop_entries - A dictionary which contains the server's information
    '''

    #
    # This dictionary has the property names that are in reconfig.data, and
    # a search is done for all the properties in this list.
    # The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
    # Each entry in the FINDSERVERS is a dictionary which contains the same name
    # as the value of this dictionary.  The value of the dictionary are the names
    # that are matched in PDL
    #
    datacollector_props = {
                            "kernel.codebase": "KERNELPORT01LIST",
                            "kernel.rfs.address": "KERNELIPLIST",
                            "probe.rmi.port": "PROBERMIPORTLIST",
                            "probe.controller.rmi.port": "PROBECONTROLLERRMIPORTLIST",
                            "deploymentmgr.rmi.host" : "STANDALONE",
                          }

    debug("Entry _read_pre_ad72_datacollector_file %s" % fixlev)

    try:
        properties = configUtils.loadProperties(filelocation)
        #
        # Read each property and match it against the datacollector
        # properties in "datacollector_props" and save it in find server
        # list for later use.
        #
        for dc_property in properties.keys():
            debug("datacollector property:  <%s> " % (dc_property))
            dc_property_value = properties.get(dc_property)
            debug("datacollector property value:  <%s> " % (dc_property_value))
            prop_pdl_entry = datacollector_props.get(dc_property)
            if prop_pdl_entry is not None:
                #
                # If the value starts with '@' it inidcates that the property is not set,
                # so it will be skipped
                #
                if not dc_property_value.startswith("@"):
                    #
                    # If this is the kernel codebase, then get the port number from it.  Code base is in the
                    # form of http://HOST:PORT/JAR...
                    #
                    if dc_property == "kernel.codebase":
                        kcodebase = dc_property_value.split(":")
                        debug("Kernel code base is - %s" % (kcodebase))
                        if len(kcodebase) >= 3:
                            kportjar = kcodebase[2]
                            kport = kportjar.split("/")
                            port = kport[0]
                            debug("Kenrel port is - %s" % (port))
                            findservers_prop_entries[prop_pdl_entry] = port
                        else:
                            debug("Invalid kernel code base")
                    elif dc_property == "kernel.rfs.address":
                        krfsadd = dc_property_value.split(":")
                        debug("Kernel rfs address is - %s" % (krfsadd))
                        if len(krfsadd == 2):
                            host = krfsadd[0]
                            findservers_prop_entries[prop_pdl_entry] = host
                    else:
                        findservers_prop_entries[prop_pdl_entry] = dc_property_value

                    debug("Property %s matched and value replaced to %s" % (prop_pdl_entry, dc_property_value))
                else:
                    debug("datacollector property is not set, starts with '@' - %s" % (dc_property_value))

        debug("Exit _read_pre_ad72_datacollector_file")

    except Exception, msgVar:
        log.logException("Unable load datacollector.properties in z -  exception %s" % msgVar)
        raise



def _process_ecam_dc61(pdl,
                       config_dir,
                       version,
                       findservers_prop_entries):
    '''
       Reads all the required zos configuration files for ecam or DC61 and creates pdl entries.

       @param pdl - PDL reference
       @param config_dir - Config directory location
       @param version - Version
       #param findservers_prop_entries - A dictionary which contains the config data
    '''

    debug("Entry _process_ecam_dc61 %s" % fixlev)

    #
    # Check for input.propereties
    #
    inputproperties_file = os.path.normpath(os.path.join(config_dir, "input.properties"))
    if os.path.exists(inputproperties_file):
        if version == common_constants.ECAMVERSION:
            input_props = inputproperties_props
        else:
            input_props = inputproperties_props_dc61
        _read_pre_ad72_inputproperty_file(pdl,
                                          inputproperties_file,
                                          findservers_prop_entries,
                                          input_props)
    else:
        debug("File not found - %s" % (inputproperties_file))

    #
    # Check for custom directory and look for toolkit_custom.properites
    #
    custom_dir = os.path.normpath(os.path.join(config_dir, "custom"))
    if os.path.isdir(custom_dir):
        toolkit_custom_properties_file = os.path.normpath(os.path.join(custom_dir, "toolkit_custom.properties"))
        if os.path.exists(toolkit_custom_properties_file):
            debug("Processing file %s - " % (toolkit_custom_properties_file))
            _read_pre_ad72_toolkit_custom_file(pdl,
                                               toolkit_custom_properties_file,
                                               findservers_prop_entries)
        else:
            debug("File not found - %s" % (custom_dir))

    #
    # Check for datacollector.propereties.  The file name is different for dc61 and ecam.
    # DC61 has the runtime name as a suffix, and ecam does not.
    #
    if version == common_constants.ECAMVERSION:
        datacollector_file = os.path.normpath(os.path.join(config_dir, "datacollector.properties"))
    else:
        runtime_name = os.path.basename(config_dir)
        datacollector_file = os.path.normpath(os.path.join(config_dir, runtime_name + ".datacollector.properties"))
    debug("datacollector properties file location is - %s" % (datacollector_file))
    if os.path.exists(datacollector_file):
        _read_pre_ad72_datacollector_file(pdl,
                                          datacollector_file,
                                          findservers_prop_entries)
    else:
        debug("File not found - %s" % (datacollector_file))


    debug("Exit _process_ecam_dc61")


def _process_ad71(pdl,
                  config_dir,
                  version,
                  findservers_prop_entries):
    '''
       Reads all the required zos configuration files for AD71 and creates pdl entries.

       @param pdl - PDL reference
       @param config_dir - Config directory location
       @param version - Version
       #param findservers_prop_entries - A dictionary which contains the config data
    '''

    debug("Entry _process_ad71 %s" % fixlev)

    #
    # Check for input.propereties.asc
    #
    inputproperties_file = os.path.normpath(os.path.join(config_dir, "input.properties.asc"))
    if os.path.exists(inputproperties_file):
        _read_pre_ad72_inputproperty_file(pdl,
                                          inputproperties_file,
                                          findservers_prop_entries,
                                          inputproperties_props)
        #input_properties_processed = True
    else:
        debug("File not found - %s" % (inputproperties_file))
    #
    # Check for custom directory and look for toolkit_custom.properites
    #
    custom_dir = os.path.normpath(os.path.join(config_dir, "custom"))
    if os.path.isdir(custom_dir):
        toolkit_custom_properties_file = os.path.normpath(os.path.join(custom_dir, "toolkit_custom.properties"))
        if os.path.exists(toolkit_custom_properties_file):
            debug("Processing file %s - " % (toolkit_custom_properties_file))
            _read_pre_ad72_toolkit_custom_file(pdl,
                                               toolkit_custom_properties_file,
                                               findservers_prop_entries)
            #toolkit_custom_processed = True
        else:
            debug("File not found - %s" % (custom_dir))
    #
    # Check for reconfig.data
    #
    reconfig_data_file = os.path.normpath(os.path.join(config_dir, "reconfig.data"))
    if os.path.isfile(reconfig_data_file):
        debug("Processing file %s - " % (reconfig_data_file))
        _read_pre_ad72_reconfig_data_file(pdl,
                                          reconfig_data_file,
                                          findservers_prop_entries)
    else:
        debug("File not found - %s" % (reconfig_data_file))

    debug("Exit _process_ad71")



def process_zos(pdl,
                config_dir,
                version,
                vtype,
                findservers_prop_entries):
    '''
       Reads all the required zos configuration files and creates pdl entries based on
       the provided information.
       Based on the version, required configuration files are read.  Each version has its
       own specific config file which saves the config data.

       @param pdl - PDL reference
       @param config_dir - Config directory location
       @param version - Version
       @param vtype -  process type like AD71, AD72, ecam,....
       #param findservers_prop_entries - A dictionary which contains the config data
    '''

    debug("Entry process_zos %s" % fixlev)


    #
    # Check the version, and call appropriate routine to read the config files
    #
    if vtype == common_constants.AD71:
        _process_ad71(pdl,
                      config_dir,
                      version,
                      findservers_prop_entries)
    else:
        _process_ecam_dc61(pdl,
                           config_dir,
                           version,
                           findservers_prop_entries)



    debug("Exit process_zos")
