#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Resource Bundle processing
    This module implements the message resource bundle.
    The java resource bundle class is used to translate a key passed
    to us into a globalized message.
'''

import re

import java.util.Locale
import java.util.ResourceBundle
import java.text.MessageFormat
import java.util.MissingResourceException

import debugUtil as log
from debugUtil import logDebug as debug

resourceBundle = None
_explanationPrefix = None
_operatorResponsePrefix = None


#####################################################################

def set_resourcebundle(locale=None):
    '''
       Create resource bundle
       Load the resource bundle as a class from the resource
       bundle jar found in the CLASSPATH.
    '''

    global resourceBundle

    bundleName = "com.ibm.tivoli.itcam.install.resources.InstallResourceBundle"
    debug("About to load the resource bundle %s" % bundleName)

    if locale is None:
        locale = java.util.Locale.getDefault()
    
    debug("Resource bundle locale = %s" % locale)

    try:
        resourceBundle = java.util.ResourceBundle.getBundle(bundleName, locale)
        if resourceBundle == None:
            raise java.util.MissingResourceException("No resource bundle handle returned.", None, None)
        debug("Resource Bundle created successfully")

    except java.util.MissingResourceException, errmsg:
        log.logException("Error occurred while getting resource bundle: %s" % (errmsg))
        # Note this will never successfully resolve the key because the bundle didn't open.
        # The backup text will always be used.
        print getmsg("CFG5010W","Unable to load resource bundle, continuing with globalization disabled")
        resourceBundle = None

#####################################################################

def getmsg(key, default_message=None, params=None):
    '''
        Get message text from the resource bundle.

        @param key - Message key in resource bundle
        @param default_message - Optional default message to return if not found in bundle
        @param params - optional parameters to be inserted into the text
    '''

    global resourceBundle
    global _explanationPrefix, _operatorResponsePrefix

    if resourceBundle != None:
        # Unfortunately containsKey() fails in some older versions of Java
        # so we are forced to use exceptions to catch missing keys.
        try:
            message = resourceBundle.getString(key)
        except java.util.MissingResourceException:
            log.logException("Resource bundle not available to resolve message key %s" % key)
            message = None
    else:
        log.logError("Resource bundle not available to resolve message key %s" % key)
        message = None

    if message == None:
        if not default_message is None:
            message = key + " " + default_message
        else:
            message = "*** Message key %s can not be resolved from resource bundle ***" % key

    # Special handling for mocked messages
    if not params is None:
        sub_flag = "'" in message
        if sub_flag:
            message = message.replace("'","<apost/>")
        message = java.text.MessageFormat.format(message, params)
        if sub_flag:
            message = message.replace("<apost/>","'")

    # Check for messages which need to be sent to the message file
    if re.compile("^CFG\d{4}[IWE]$").match(key) is not None and message.startswith(key):

        # Get the explanation and operator response prefixes for messages

        explanation = None
        operatorResponse = None

        # Attempt to retrieve the explanation and operator response
        # This needs to be activated in the build with -param explanation E -param operatorresponse O

        if resourceBundle is not None:

            _explanationPrefix = "Explanation:"
            if resourceBundle is not None:
                try:
                    _explanationPrefix = resourceBundle.getString("MESSAGE_EXPLANATION")
                except java.util.MissingResourceException:
                    pass

            _operatorResponsePrefix = "Operator Response:"
            if resourceBundle is not None:
                try:
                    _operatorResponsePrefix = resourceBundle.getString("MESSAGE_OPERATORRESPONSE")
                except java.util.MissingResourceException:
                    pass

            try:
                explanation = "%s %s" % (_explanationPrefix, resourceBundle.getString(key+"E"))
                debug("%s explanation: %s" % (key, explanation))
            except java.util.MissingResourceException:
                debug("Missing explanation for %s message" % key)
            try:
                operatorResponse = "%s %s" % (_operatorResponsePrefix, resourceBundle.getString(key+"O"))
                debug("%s operator response: %s" % (key, operatorResponse))
            except java.util.MissingResourceException:
                debug("Missing operator response for %s message" % key)

        log.messagePrint(message, explanation, operatorResponse)

    return message


#####################################################################

if __name__ == "__main__":

    # Initialize the log utility
    log.set_logging("./resourcebundle-trace.log","./resourcebundle-message.log")

    #locale = java.util.Locale.CHINESE

    print
    #set_resourcebundle()
    set_resourcebundle(java.util.Locale.GERMANY)
    
    print "getmsg(CFG0002E) = '%s'" % getmsg("CFG0002E", "Default message for ID CFG0002E p1={0} p2={1}", params=("param(a)","param(b)"))

    '''
    print "1116E: %s" % getmsg("CFG1116E", params=["subComponent","symbolicDirectoryName"])
    print "1123E: %s" % getmsg("CFG1116E", params=["subComponent","symbolicDirectoryName"])

    print "5004I: %s" % getmsg("CFG5004I", params=["*test*"])

    version = ".".join([str(n) for n in [7,2,0]])
    product = getmsg("ITCAMAD","ITCAM for Application Diagnostics (preliminary)")

    print "version = '%s', product = '%s'" % (version, product)
    print getmsg("CFG1089I", "No application servers previously configured with version {0} of {1} found, nothing to do.", [version, product])

    print getmsg("CFG0002E")
    print "getmsg(CFG3100I) = '%s'" % getmsg("CFG3100I","default CFG3100I message")

    print "getmsg(XCFG0002E) =",
    print "'%s'" % getmsg("XCFG0002E", "Default message for ID CFG0002E p1={0} p2={1}", params=("param(a)","param(b)"))

    print "getmsg(CFG0002E) = '%s'" % getmsg("CFG0002E", "Default message for ID CFG0002E p1={0} p2={1}", params=("param(a)","param(b)"))
    print "getmsg(CFG0002E) = '%s'" % getmsg("CFG0002E", params=("param(a)","param(b)"))
    print "getmsg(UNKNOWN,Parm(0) = {0} Parm(1) = {1}) ="
    print getmsg("UNKNOWN","Parm(0) = {0} Parm(1) = {1}",["First","Second"])

    print
    print "getmsg(UNKNOWN) = '%s'" % getmsg("UNKNOWN")

    print
    print "getmsg(UNKNOWN,Key UNKNOWN not found) = '%s'" % getmsg("UNKNOWN","Key Unknown not found")

    print
    print "getmsg(UNKNOWN,Parm(0) = {0} Parm(1) = {1}) = '%s'" % getmsg("UNKNOWN","Parm(0) = {0} Parm(1) = {1}",["First","Second"])

    print
    print "getmsg(CFG0001I) = '%s'" % getmsg("CFG0001I")

    print
    print "getmsg(CFG0001I,Key CFG0001I not found) = '%s'" % getmsg("CFG0001I","Key Unknown not found")
    '''

    print


