#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62 
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
# This module contains the TCP/IP interface inquiry function.  It will
# determine the local host name and IP address.  Note that it may
# be necessary to prompt for selection if multiple IP addresses are
# found. 
#####################################################################

'''
    Get the host name and IP address using the InetAddr object.
'''

import java.net.InetAddress as InetAddress
import java.net.NetworkInterface as NetworkInterface

import doprompt
import debugUtil as log
from ConfigurationException import ConfigurationException
from debugUtil import logDebug as debug
from resourcebundle import getmsg
import platformUtil
import jarray

def prompt_iphost(pdl):
    '''
       Get the TCP/IP details and prompt to choose interface
    '''

    debug("prompt_iphost() entry")

    # Get the InetAddress object for the local host, get it's host name and IP address,
    # and get the interfaces defined for that host name.

    inet = InetAddress.getLocalHost()
    hostname = inet.getHostName()
    ipaddr = inet.getHostAddress()
    interfaces = InetAddress.getAllByName(hostname)

    debug("Local host = %s, host name = %s, IP address = %s" % (inet, hostname, ipaddr))

    # Get all interfaces and put the coresponding IP addresses and host name in the PDL.
    pdl["TCPIPINTERFACES"][0] = []
    for i, interface in [(interfacei,interfaces[interfacei]) for interfacei in range(len(interfaces))]:
        hostip = interface.getHostAddress()
        canonicalhostname = interface.getCanonicalHostName()
        debug("%3d canonical host name = %s, IP address = %s" % (i, canonicalhostname, hostip))
        if canonicalhostname != hostip:
            pdl["TCPIPINTERFACES"][0].append(hostip+"/"+canonicalhostname)
        else:
            pdl["TCPIPINTERFACES"][0].append(hostip)
        pdl["TCPIPINTERFACES"][1] = "get_iphost"

    
    # In case no interfaces are defined, use the host address and hostname.
    if pdl["TCPIPINTERFACES"][1] != "get_iphost":
        if ipaddr != hostname:
            pdl["TCPIPINTFERACES"][0] = [ipaddr+"/"+hostname]
        else:
            pdl["TCPIPINTFERACES"][0] = [ipaddr]
        pdl["TCPIPINTERFACES"][1] = "get_iphost"

    #check if we got no addresses or only a loopback address back
    if len(pdl["TCPIPINTERFACES"][0]) == 0 or (len(pdl["TCPIPINTERFACES"][0]) == 1 and pdl["TCPIPINTERFACES"][0][0].startswith("127.")):
        debug("TCPIPINTERFACES has a single local address in")
        nis = NetworkInterface.getNetworkInterfaces()
        
        b = jarray.array ([127,0, 0, 1],"b")
        inetAddress = InetAddress.getByAddress(b)
        loopbackNI = NetworkInterface.getByInetAddress(inetAddress)
        
        relElements = platformUtil.getJavaRelease().split(".")

        while nis.hasMoreElements():
            ni = nis.nextElement()
            debug("network interface: %s" %  ni.getName())
            if int(relElements[1]) > 5 and ni.isLoopback() or int(relElements[1]) < 6 and ni.equals(loopbackNI): 
                debug("network interface: %s is a loopback interface, skipping" %  ni.getName())
                continue
            addrs = ni.getInetAddresses()
            while addrs.hasMoreElements():
                addr = addrs.nextElement()
                hostip = addr.getHostAddress()
                debug("network interface: %s address: %s" %  (ni.getName(), hostip))
                if not addr.isLoopbackAddress() and hostip not in pdl["TCPIPINTERFACES"][0]:
                    pdl["TCPIPINTERFACES"][0].append(hostip)
            
    if len(pdl["TCPIPINTERFACES"][0]) > 0:
        pdl["TCPIPINTERFACE"][0] = pdl["TCPIPINTERFACES"][0][0]
        pdl["TCPIPINTERFACE"][1] = pdl["TCPIPINTERFACES"][1]
    
    doprompt.do_prompt("TCPIPINTERFACES",pdl,"TCPIPINTERFACE")
    log.logDumpPDL(pdl)
    
    
    
    
    if pdl["TCPIPINTERFACE"][0] == None:
        raise ConfigurationException(getmsg("CFG1111E", "Prompt failed for TCP/IP interface"))

    pipaddr = pdl["TCPIPINTERFACE"][0].split("/",1)
    if len(pipaddr) == 2:
        pipaddr, phostname = pipaddr
    elif len(pipaddr) == 1:
        pipaddr = pipaddr[0]
        phostname = pipaddr
    else:
        pipaddr, phostname = ("", "")

    debug("prompt_iphost exit, Selected interface = host(%s) IP(%s)" % (phostname,pipaddr))
    return (pipaddr,phostname)


def get_iphost(pdl):
    '''
        Get the host name and IP address using the InetAddr object.
    '''

    # If there is no default IP address, prompt for it.
    # If there is no default host name, set it to what we responded

    debug("get_iphost() entry")

    debug("DEFAULTHOSTIP = %s" % pdl["DEFAULTHOSTIP"][0])
    hostname = None
    ipaddr = None
    if not pdl["DEFAULTHOSTIP"][0]:
        ipaddr, hostname = prompt_iphost(pdl)
        debug("ipaddr = %s, hostname = %s" % (ipaddr,hostname))
    if pdl["DEFAULTHOSTNAME"][0]:
        hostname = pdl["DEFAULTHOSTNAME"][0]
    if not hostname:
        hostname = ipaddr
        
    if not pdl["DEFAULTHOSTIP"][0]:
        pdl["DEFAULTHOSTIP"][0:2] = ipaddr, "get_iphost"

    if not pdl["DEFAULTHOSTNAME"][0]:
        pdl["DEFAULTHOSTNAME"][0:2] = hostname, "get_iphost"

    if not pdl["LOCALHOSTIP"][0]:
        pdl["LOCALHOSTIP"][0:2] = pdl["DEFAULTHOSTIP"][0], "get_iphost"

    if not pdl["LOCALHOSTNAME"][0]:
        pdl["LOCALHOSTNAME"][0:2] = pdl["DEFAULTHOSTNAME"][0], "get_iphost"

    if not pdl["WSHOST"][0]:
        pdl["WSHOST"][0:2] = pdl["DEFAULTHOSTNAME"][0], "get_iphost"

    if not pdl["BINDIP"][0]:
        pdl["BINDIP"][0:2] = pdl["DEFAULTHOSTNAME"][0], "get_iphost"

    if not pdl["EXPORTIP"][0]:
        pdl["EXPORTIP"][0:2] = pdl["DEFAULTHOSTNAME"][0], "get_iphost"
        
    # Note that DMGRHOST is set from the results of findServers.py, not
    # from the local TCP/IP interface.

    debug("get_iphost() exit")

            
########################################################
# Unit Testing
########################################################

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL

    from resourcebundle import set_resourcebundle

    log.set_logging("./tcpipstack.log")
    set_resourcebundle()
    
    try:

        debug("Unit testing of tcpipstck")

        pdl = createPDL.make_pdl(createPCL.make_pcl())
        #createPDL.proc_cliopt(pdl, ("", "-defaulthostip","192.168.100.105"))
        get_iphost(pdl)
        log.logDumpPDL(pdl)

    except:

        log.logException("\Traceback Follows")
        #log.logDumpStack()
