#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Utilities
'''

fixlev = "40234" # Last APAR, defect or story to update this module

import sys
import os
import re
import codecs
import socket
import tempfile
import subprocess

import java.nio.ByteBuffer as Bytebuffer
import java.nio.charset.Charset as Charset
import java.nio.charset.CharacterCodingException
import java.lang.String
import java.lang.Integer

from java.lang import System
from java.text.MessageFormat import format

from types import ListType, IntType
from time import strftime

import platformUtil
import debugUtil as log

from debugUtil import logDebug as debug
from resourcebundle import getmsg


def replaceBackSlash(windowsPath):
    '''
       utility function to replace "\" in Windows file path to "/" so the file path can work with wsadmin
    '''

    debug("Entry replaceBackSlash(%s) %s" % (windowsPath,fixlev))

    # Note that for Jython 2.1 on Windows, os.altsep is None.  So we don't use os.sep and os.altsep.

    # For some test cases, windowsPath may be None.
    if `windowsPath`.count("\\") > 0:
        windowsPath = re.sub(r"\\", "/", windowsPath)

    debug("Exit replaceBackSlash(), result is %s" % windowsPath)
    return windowsPath


def createDataDir(configHome):
    '''
       create a data directory under CONFIGHOME to store data files and log files
    '''

    debug("Entry createDataDir(%s) %s" % (configHome,fixlev))

    # Create the data directory if it doesn't already exist

    datadir = os.path.normpath(os.path.join(configHome, "data"))
    ans = None
    if not os.path.exists(datadir):
        debug("%s does not exist, will attempt to create." % datadir)
        try:
            os.mkdir(datadir)
            debug("Successfully created directory %s" % datadir)
            ans = True
        except:
            debug("Unable to create %s" % datadir)
            debug("Data and logs will be written to CONFIGHOME %s " % configHome)
            ans = False
    else:
        debug("Directory already exists: %s" % datadir)
        ans = True

    debug("Exit createDataDir(), result is %s" % ans)
    return ans


def getServerBasedFilePrefix(pdl, serveri):
    '''
       utility function to get the file prefix using the format:
       <wascell>.<wasnode>.<wasserver>.
    '''

    debug("Entry getServerBasedFilePrefix(pdl, serveri=%s) %s" % (serveri,fixlev))

    # If the modified pdl entries are not set yet, then read the pdl entries for
    # cell, node, profile, and server, and do the conversion here in case there
    # is a non ascii character.

    wascellName = getDataFlex(pdl,"WASCELLMODIFIED")
    wasnodeName = getData(pdl,"WASNODEMODIFIED")
    wasserverName = getData(pdl,"SERVERSMODIFIED")
    wasprofileName = getData(pdl, "PROFILEMODIFIED")
    debug("cell=%s node=%s server=%s profile=%s" % (wascellName, wasnodeName, wasserverName, wasprofileName))
    if wascellName is None or wasnodeName is None or wasserverName is None or wasprofileName is None:
        wascellName = convert_nonascii(getData(pdl,"WASCELL", serveri))
        wasnodeName = convert_nonascii(getData(pdl,"WASNODE", serveri))
        wasserverName = convert_nonascii(getData(pdl,"SERVERS", serveri))
        wasprofileName = convert_nonascii(getData(pdl, "PROFILE", serveri))
    else:
        wascellName = getData(pdl,"WASCELLMODIFIED", serveri)
        wasnodeName = getData(pdl,"WASNODEMODIFIED", serveri)
        wasserverName = getData(pdl,"SERVERSMODIFIED", serveri)
        wasprofileName = getData(pdl, "PROFILEMODIFIED", serveri)

    assert wascellName is not None, "Missing WAS cell name"
    assert wasnodeName is not None, "Missing WAS node name"
    assert wasserverName is not None, "Missing WAS server name"
    assert wasprofileName is not None, "Missing WAS profile name"

    wascellName = wascellName.strip()
    wasnodeName = wasnodeName.strip()
    wasserverName = wasserverName.strip()
    wasprofileName = wasprofileName.strip()

    if len(wasprofileName) > 0:
        serverBasedFilePrefix = wasprofileName + "." + wascellName + "." + wasnodeName + "." + wasserverName + "."
    else:
        serverBasedFilePrefix = wascellName + "." + wasnodeName + "." + wasserverName + "."

    debug("Exit getServerBasedFilePrefix(), result = %s" % serverBasedFilePrefix)
    return serverBasedFilePrefix


def getServerName(pdl, serveri):

    debug("Entry getServerName(pdl, %s) %s" % (serveri,fixlev))

    wascellName = getData(pdl,"WASCELL")[serveri]
    wasnodeName = getData(pdl,"WASNODE")[serveri]
    wasserverName = getData(pdl,"SERVERS")[serveri]
    wasprofileName = getData(pdl, "PROFILE")[serveri]

    assert wascellName is not None, "Missing WAS cell name"
    assert wasnodeName is not None, "Missing WAS node name"
    assert wasserverName is not None, "Missing WAS server name"
    assert wasprofileName is not None, "Missing WAS profile name"

    wascellName = wascellName.strip()
    wasnodeName = wasnodeName.strip()
    wasserverName = wasserverName.strip()
    wasprofileName = wasprofileName.strip()

    serverName = wascellName + "." + wasnodeName + "." + wasserverName + "(" + wasprofileName + ")"

    debug("Exit getServerName(), result = %s" % serverName)
    return serverName


def create_dir_tree(dir_path):
    '''
        create directory tree
    '''

    debug("Entry create_dir_tree(%s) %s" % (dir_path,fixlev))

    if not os.path.exists(dir_path):
        try:
            os.makedirs(dir_path)
            debug("Successfully created directory")
        except OSError, errmsg:
            debug("Unable to create directory %s - %s" % (dir_path,errmsg))
    exist_flag = os.path.exists(dir_path)

    debug("Exit create_dir_tree(), result is %s" % exist_flag)
    return exist_flag


def isSet(var):
    '''
       checks is variable is set
    '''

    # Don't log low-level routines
    if var is not None and var != "": return True
    return False


def isInt(num):
    '''
        check if variable is an integer
    '''

    # Don't log low-level routines
    try:
        int(num)
        return True
    except:
        return False


def toList(stritem):
    '''
        converts a string to a list (can be used with wsadmin Admin comamnd outputs)
        (Note: There is a splitlines() method of string that does this much better.)
    '''

    # Don't log low-level routines
    lineSeparator = System.getProperty("line.separator")
    _list = stritem.split(lineSeparator)

    return _list


def isPropSet(prop_name):

    prop_value = System.getProperty(prop_name)
    if prop_value != None:
        return 1
    else:
        return 0


def find_file(dirName, fileNameFilter, max_depth=None):
    '''
        return a list of files in directory dirName with names matching pattern
        input parameters
        absolute path directory name to search in
        file name reg ex to match, i.e. script.sh|script.bat
        OPTIONAL: maximum depth of nested directory, if not specified will use 6
        example: find_file("E:/IBM/WebSphere/AppServer", "wsadmin.bat|wsadmin.sh")
        ouput: a list of absolute paths to files found
    '''

    #maximum level to search nested directories
    MAX_DIR_DEPTH = 6

    if max_depth is None:
        max_depth = MAX_DIR_DEPTH
    found_files = []
    fileNameRegEx = re.compile(fileNameFilter)
    _recursiveFind(dirName, 1, fileNameRegEx, found_files, max_depth)

    return found_files


def _isSelfOrRootDir(dirName):

    if dirName == "." or dirName == "..": return 1
    return 0


def _recursiveFind(dirName, depth, fileNameFilter, foundList, max_depth):

    # todo: consider remembering the fact that we had a permission error
    #       and issuing a message later, in case we got nothing.

    # We can't look in directories where we don't have permission
    try:
        fileList = os.listdir(dirName)
    except OSError, instance:
        debug("Unable to list directory: %s" % instance)
        fileList = []
    for f in fileList:
        full_name = os.path.normpath(os.path.join(dirName, f))
        if not _isSelfOrRootDir(f):
            if depth < max_depth and os.path.isdir(full_name):
                _recursiveFind(full_name, (depth + 1), fileNameFilter, foundList, max_depth)
            if os.path.isfile(full_name) and fileNameFilter.match(f):
                foundList.append(full_name)
                #print full_name
                debug("found file=%s" % (full_name))


def runCommand(command, commandForDebug=None):

    debug("Entry runCommand %s" % fixlev)

    if platformUtil.isWindows():
        return runCommandWindows(command, commandForDebug)
    else:
        return runCommandOsSystem(command, commandForDebug)


def runCommandOsSystem(command, commandForDebug=None):
    '''
        executes system command using os.system which is compatible with 2.1 jython.
        later version of jython have much nicer ways of running commands
        returns rc and command output
        throws exception if command can not be run
        rc == 0 success, otherwise failure
        output will contain result or error message
        example:
        rc, output = runCommand("ls -l /opt/ibm")
        if rc > 0:
        print "command failed"
    '''

    if commandForDebug == None: commandForDebug = command
    debug("Entry runCommandOsSystem(%s) %s" % (commandForDebug,fixlev))

    out_list = []
    try:
        outfile = tempfile.mktemp(".txt")
        debug("temp file=%s" % outfile)
        command_str = "%s > %s 2>&1" % (command, outfile)
        rc = os.system(command_str)
        filh = open(outfile)
        while True:
            line = filh.readline()
            if not line: break
            out_list.append(line)
        filh.close()
    except Exception, errmsg:
        value = sys.exc_info()[1]
        out_list.append("Unable to execute command %s Error: %s" % (command, value))
        out_list.append("Error message: %s" % errmsg)
        rc = 1

    if len(out_list) == 0:
        debug("runCommand output: None")
    else:
        debug("runCommand output:\n\n    %s" % "\n    ".join([ln.strip("\r\n") for ln in out_list if ln.strip() != ""]))

    debug("Exit runCommand(), result is %s" % rc)
    return (rc, out_list)


def runCommandWindows(command, commandForDebug=None):
    '''
        execute command on Windows, without using redirect symbol in the command string.
        Latest java SR's (SR12 for java 6 and SR4 for java 7) put double quotes around redirect character on Windows
        and make command syntax invalid
    '''

    if commandForDebug == None: commandForDebug = command
    debug("Entry runCommandWindows(%s) %s" % (commandForDebug,fixlev))

    rc = -1
    out_list = []
    #create temprorary file
    outfile = tempfile.mktemp(".txt")
    debug("outfile file %s" % outfile)
    try:
        fhandler = open(outfile, "w")
        rc = subprocess.call(command, stdout = fhandler, stderr = fhandler)
        fhandler.close()
    except Exception, errmsg:
        debug("error while executing command %s" % errmsg)

    if os.path.isfile(outfile):
        filh = open(outfile, "r")
        while True:
            line = filh.readline()
            if not line: break
            out_list.append(line)
        filh.close()

    if len(out_list) == 0:
        debug("runCommand output: None")
    else:
        debug("runCommand output:\n\n    %s" % "\n    ".join([ln.strip("\r\n") for ln in out_list if ln.strip() != ""]))

    debug("Exit runCommandWindows(), result is %s" % rc)
    return (rc, out_list)


def isValidIpAddress(address_str):
    '''
        validates if input string is a valid ip address
    '''

    if address_str != None:
        address_str = address_str.strip()
    else:
        debug("address_str is None.")
        return False

    if isValidIpv4Address(address_str) or isValidIpv6Address(address_str):
        return True
    else:
        return False


def isValidIpv4Address(address_str):
    '''
        Use Java Inet to validate an IP V4 address
    '''

    debug("Entry isValidIpv4Address(%s) %s" % (address_str,fixlev))

    is_valid = False
    try:
        socket.inet_pton(socket.AF_INET, address_str)
        is_valid = True
    except AttributeError:
        debug("Got AttributeError for inet_pton, attempting inet_aton")
        try:
            socket.inet_aton(address_str)
            is_valid = True
        except socket.error:
            debug("Got socket.error inside handling AttributeError exception.")
    except socket.error:
        debug("Got socket.error")
    except:
        debug("Got a general exception.")

    debug("Exit isValidIpv4Address(), result = %s" % is_valid)
    return is_valid


def isValidIpv6Address(address_str):
    '''
        Use Java Inet to validate an IP V6 address
    '''

    debug("Entry isValidIpv6Address(%s) %s" % (address_str,fixlev))

    is_valid = False
    try:
        socket.inet_pton(socket.AF_INET6, address_str)
        is_valid = True
    except socket.error:
        debug("Got socket.error")
    except:
        debug("Got a general exception.")

    debug("isValidIpv6Address() exit, result is %s" % is_valid)
    return is_valid


def isValidLocalPath(filename_str):
    '''
        validates if string is a valid path to a file on a local machine
        this is restrictive checking, invalidates paths that contain characters
        not allowed by WebSphere
    '''

    debug("Entry isValidLocalPath(%s) %s" % (filename_str,fixlev))

    #check that filename is not exceeding 255 char length
    if len(filename_str) > 255: return 0

    file_name = os.path.basename(filename_str)
    debug("file_name: " + file_name)
    dir_path = os.path.dirname(filename_str)
    debug("dir_path: " + dir_path)

    #Check if file path exists
    if not os.path.exists(dir_path):
        debug("The file path: " + dir_path + " does not exist.")
        print "%s" % getmsg("CFG5030E", "The file directory in the file name: {0} does not exist. The file must be under an existing directory.", [filename_str])
        return 0

    folders = []
    folders.append(file_name)
    while 1:
        dir_path, dir = os.path.split(dir_path)
        if dir != "":
            folders.append(dir)
        else:
            folders.append(dir_path)
            break

    debug("path tokens %s" % folders)
    pattern = '[`\*:<\\\>\?/\"\|]'
    for index in range(len(folders) - 1):
        dir_name = folders[index]
        is_valid = not re.search(pattern, dir_name)
        if is_valid == 0:
            print "%s" % getmsg("NO_SPECIAL_CHARS_ALLOWED", "The following characters are not allowed as part of a directory or file name: *, \", <, >, :, ?, |, `")
            break

    if is_valid == 0: debug("Invalid characters in path: %s" % dir_name)

    debug("isValidFLocalPath exit %s" % str(is_valid))
    return is_valid


def isValidHostname(hostname_str):
    '''
        validates if string is a valid hostname
    '''

    if isValidIpAddress(hostname_str):
        return True
    else:
        debug("Not a valid IP address: " + hostname_str)

    #check if hostname is not exceeding 255 char length
    if len(hostname_str) > 255: return False

    #strip . from the end of the hostname (if it is there)
    if hostname_str.endswith("."):
        hostname_str = hostname_str[:-1]
    pattern = "[A-Z\d-]{1,63}$"
    is_valid = True
    for token in hostname_str.split("."):
        if not re.match(pattern, token, flags=re.IGNORECASE):
            is_valid = False
            break

    return is_valid


def isValidRange(range_str):
    '''
        validates if string is a range in format xxx - yyy, where xxx < yyy
    '''

    pattern = "^[0-9]+[\s]*-[\s]*[0-9]+$"
    is_valid = 0
    if re.match(pattern,range_str):
        tokens = range_str.split("-",1)
        try:
            if int(tokens[0].strip()) < int(tokens[1].strip()): is_valid = 1
        except:
            is_valid = 0

    return is_valid


def isValidFilename(filename_str):
    '''
        validates if string is a valid filename
    '''

    debug("Entry isValidFilename(%s) %s" % (filename_str,fixlev))

    #check that filename is not exceeding 255 char length
    if len(filename_str) > 255: return 0

    file_name = os.path.basename(filename_str)
    debug("file_name: " + file_name)
    dir_path = os.path.dirname(filename_str)
    debug("dir_path: " + dir_path)

    #Check if file path exists
    if not os.path.exists(dir_path):
        debug("The file path: " + dir_path + " does not exist.")
        print "%s" % getmsg("CFG5030E", "The file directory in the file name: {0} does not exist. The file must be under an existing directory.", [filename_str])
        return 0

    pattern = '[`\*:<\\\>\?/\"\|]'
    is_valid = not re.search(pattern, file_name)

    debug("isValidFilename exit %s" % str(is_valid))
    return is_valid


#####################################################################
# These routines manipulate the PDL:
#
# getData(pdl,name,[index=index])
# getDataString(pdl,name,[index=index])
# setData(pdl,name,data,[tag],[index=index])
# getPCL(pdl)
# getFormat(pdl,name)
#####################################################################

def getData(pdl,key,index=None):
    '''
        get a data element from the PDL by name.
    '''

    assert key in pdl.keys(), "Data element %s is not present in the PDL" % key
    if index == None:
        ans = pdl[key][0]
    else:
        assert isinstance(pdl[key][0],ListType), "Data element %s of PDL is not a list, superfluous index %s specified" % (key, index)
        ans = pdl[key][0][index]

    return ans


def getDataString(pdl,key,index=None):
    '''
        get a data element from the PDL by name.

        For integers and booleans, a string value is returned.
    '''

    assert key in pdl.keys(), "Data element %s is not present in the PDL" % key
    if index == None:
        ans = pdl[key][0]
    else:
        assert isinstance(pdl[key][0],ListType), "Data element %s of PDL is not a list, superfluous index %s specified" % (key, index)
        ans = pdl[key][0][index]

    # This is for compatibilty with Jython 2.1.
    if ans == True: ans = "1"
    if ans == False: ans = "0"

    return ans


def getDataFlex(pdl,key,index=None):
    '''
        getDataFlex supports an index, used only if the variable is a List
    '''

    msgl = []
    msgl.append(key)
    assert key in pdl.keys(), getmsg("CFG5015E", "Data element {0} is not present in the PDL", msgl)
    if index is None:
        ans = pdl[key][0]
    elif isinstance(pdl[key][0],ListType):
        ans = pdl[key][0][index]
    else:
        ans = pdl[key][0]

    return ans


def setData(pdl,key,value,tag,index=None):
    '''
         Set a data element in the PDL
    '''

    assert key in pdl.keys(), "Data element %s is not present in the PDL" % key
    if index == None:
        pdl[key][0] = value
        pdl[key][1] = tag
    else:
        assert isinstance(pdl[key][0],ListType), "Data element %s of PDL is not a list, superfluous index %s specified" % (key,index)
        pdl[key][0][index] = value
        pdl[key][1] = tag

    return


def getPCL(pdl):
    '''
        Retrieve the PCL from its anchor in the PDL
    '''

    assert "PCL" in pdl.keys(), "Data element for PCL is not present in the PDL"
    pcl = pdl["PCL"][0]

    return pcl


def getFormat(pdl,key):
    '''
        Retrieve the format name from the PCL for a PDL data element
    '''

    assert key in pdl.keys(), "Data element {0} is not present in the PDL" % key
    pcl = pdl["PCL"][0]
    pcli = pdl[key][2]
    ans = pcl[pcli][1]

    return ans


def getValidation(pdl,key):
    '''
        Retrieve the verification routine from the PCL for a PDL data element
    '''

    assert "FORMATS" in pdl, "Data element FORMATS is not present in the PDL"
    valid_formats = pdl["FORMATS"][0]

    assert key in pdl.keys(), "Data element {0} is not present in the PDL" % key
    pcl = pdl["PCL"][0]
    pcli = pdl[key][2]
    fmt = pcl[pcli][1]
    ans = valid_formats[fmt][0]

    return ans


def getMsgIDs(pdl,key):
    '''
        Retrieve the prompt and help message IDs associated with a PDL data element as a tuple of strings
    '''

    assert key in pdl.keys(), "Data element %s is not present in the PDL" % key
    pcl = pdl["PCL"][0]
    pcli = pdl[key][2]
    if len(pcl[pcli]) >= 6 and pcl[pcli] != None:
        ans = pcl[pcli][5]
    else:
        ans = ("CFG0000I", "CFG0000I")

    return ans


def get_win_registry_key(key_name, value=None):
    '''
        run reg query to get key or value
    '''

    debug("Entry get_win_registry_key(%s, %s) %s" % (key_name, value, fixlev))

    if value is not None:
        reg_query = 'reg query \"%s\" /v %s' % (key_name, value)
    else:
        reg_query = 'reg query \"%s\"' % (key_name)
    subkey_list = []
    debug("Querying registry key %s" % reg_query)
    rc, subkey_list = runCommand(reg_query)

    debug("return code from registry: %s, lines found %d" % (str(rc), len(subkey_list)))
    return subkey_list


def get_win_registry_key_all(key_name):
    '''
        run reg query with /s flag, outputs all subkeys, values for key_name
    '''

    debug("Entry get_win_registry_key_all(%s) %s" % (key_name, fixlev))

    reg_query = 'reg query \"%s\" /s' % (key_name)
    debug("Querying registry key %s" % reg_query)
    rc, subkey_list = runCommand(reg_query)

    debug("return code from registry: %s, lines found %d" % (str(rc), len(subkey_list)))
    return subkey_list


def strip_last_path_separator(path):
    '''
        removes path separator character from the end of path
    '''

    # Workaround for Jython 2.1 where os.altsep is None for Windows
    altsep = os.altsep
    if altsep is None and os.sep == "\\": altsep = "/"

    if path is not None and len(path) > 0 and (path[-1:] == os.sep or path[-1:] == altsep):
        path = path[:-1]

    return path


def add_quotes(mystr):
    '''
        Encloses string within double-quotes if it contains spaces.
        Otherwise returns an original string unchanged.
        May be used when running a script or script parameters that
        might contain spaces. (As in "C:/Program Files" for example).

        Note that more work would be required if quotes could be present in the string.
    '''

    if mystr.find(" ") != -1:
        mystr =  "\"" + mystr + "\""

    return mystr


def get_was_error_msg(log_file):
    '''
        return the first occurance of WebSphere error message from a log file
        identifies an error message as a line with WASX.*E: prefix
    '''

    debug("Entry get_was_error_msg(%s) %s" % (log_file, fixlev))

    if platformUtil.IS_zOS_OS():
        enc = "cp1047"
    else:
        enc = "utf-8"

    msg_prefix = "WASX"
    user_passwd_msg_id = "WASX7246E"
    was_connection_msg_id = "WASX7023E"
    was_cannot_find = "WASX7011E"
    was_import_failed = "WASX7487E"
    error_msg = ""
    filh = None
    lines = None
    is_user_passwd_problem = False
    is_was_connection_problem = False
    try:
        filh = codecs.open(log_file, "r", enc)
        # In case this is a DOS file we remove the CR as well as the LF.
        lines = [line.rstrip("\r\n") for line in filh.readlines() if line.strip() != ""]
        #finally block does not work well so close file here and in except.
        filh.close()
    except:
        debug("Unable to read %s" % log_file)
        if filh is not None: filh.close()

    if lines is not None:
        debug("Selected WebSphere wsadmin output from %s:\n\n    %s" % (log_file,"\n    ".join([line for line in lines if line.startswith(msg_prefix)])))
        for line in lines:
            if line.strip() == "": continue
            words = line.split()
            if len(words) < 2: words.append("")
            if words[0].startswith(user_passwd_msg_id) or words[1].startswith(user_passwd_msg_id):
                debug("Password problem detected: %s" % line)
                is_user_passwd_problem = True
                error_msg = line
                break
            elif words[0].startswith(was_connection_msg_id):
                debug("Connection problem detected: %s" % line)
                is_was_connection_problem = True
                error_msg = line
                break
            # Ignore this: WASX7011E: Cannot find file "/tmp/dummy.py"
            elif words[0].startswith(was_cannot_find) and line.endswith("dummy.py\""):
                debug("Missing dummy Python file error message ignored: %s" % line)
            # Ignore this: WASX7487E: Failed to import script libraries modules: [WASHOME]\scriptLibraries\perfTuning\V70\applyPerfTuningTemplate.py
            elif words[0].startswith(was_import_failed) and line.find("applyPerfTuningTemplate.py") != -1:
                debug("APAR PM60715 problem ignored: %s" % line)
            elif words[0].startswith(msg_prefix) and words[0][-2] == "E":
                debug("Error message detected: %s" % line)
                error_msg = line
                break

    debug("get_was_error_msg() Exit, Password=%s  Connection=%s  ErrorMsg='%s'" % (is_user_passwd_problem,is_was_connection_problem,error_msg))
    return error_msg, is_user_passwd_problem, is_was_connection_problem


def is_initial_value(pdl, key_name):
    '''
        returns true if pdl key was not set before
    '''

    if pdl[key_name][1] == "Initial":
        return 1
    return 0


def get_groupname(pdl, wascell, wasnode, profile, wlphome=None):
    '''
       Get the gorup name which is the combination of cell.node.profile.

       @param pdl - PDL reference
       @param wascell - WAS cell name
       @param wasnode - WAS node name
       @param profile - WAS profile name
    '''

    debug("Entry get_groupname %s - Cell %s - Node %s - Profile %s" % (fixlev, wascell, wasnode, profile))

    if wlphome == None or len(wlphome) == 0 or wlphome == "None":    #not a Liberty server
        groupname = wascell + "." + wasnode + "." + profile
    else:
        groupname = "liberty"
        #groupname = "#liberty." + wascell + "." + wasnode

    debug ("Exit get_groupname - %s" % (groupname))
    return groupname


def group_findservers(pdl):
    '''
       Get total number of servers  for each profile in the FINDSERVERS based on a cell, node,
       and profile name, and save it in a dictionary.

       @param pdl - PDL reference
    '''

    debug("Entry group_findservers %s" % fixlev)

    numberof_servers = 0
    findservers_group = {}

    findservers = getData(pdl, "FINDSERVERS")
    for server_entry in findservers:
        wascell = server_entry["WASCELL"]
        wasnode = server_entry["WASNODE"]
        profile = server_entry["PROFILE"]
        wlp     = server_entry["WLPHOME"]
        groupname = get_groupname(pdl, wascell, wasnode, profile, wlp)
        if findservers_group.has_key(groupname):
            numberof_servers = findservers_group[groupname]
            numberof_servers = numberof_servers + 1
        else:
            numberof_servers = 1
        findservers_group[groupname] = numberof_servers

    for groupservers_entry in findservers_group.keys():
        debug("Group Servers key: %s   Total number of servers: %d" % (groupservers_entry, findservers_group[groupservers_entry]))

    debug("Exit group_findservers - %s" % (findservers_group))
    return findservers_group


def group_servers(pdl, server_index=None):
    '''
       Groups all the servers with the same cell, node.  The index into the
       SERVERS pdl is picked up for the same cell/node, and it is saved in
       a dictionary.

       @param pdl - PDL reference
       @param server_index - A list of server indexes
       @return - Dictionary which key is the cellname.nodename and its value is
                 a list which contains the index into SERVERS list
    '''

    group_servers = {}

    if server_index is None:
        for serveri in range(len(pdl["SERVERS"][0])):
            set_group_servers(pdl, group_servers, serveri)
    else:
        for serveri in server_index:
            set_group_servers(pdl, group_servers, serveri)

    return group_servers


def set_group_servers(pdl, group_servers, serveri):
    '''
       Groups all the servers with the same cell, node.  The index into the
       SERVERS pdl is picked up for the same cell/node, and it is saved in
       a dictionary.

       @param pdl - PDL reference
       @param group_servers - A dictionary that holds the indexes for a set of servers
                              that belong to the same cell and node.
       @param serveri - Server index

    '''

    debug("Entry set_group_servers() %s" % fixlev)

    wasservername = getData(pdl,"SERVERS")[serveri]
    wasservername = wasservername.strip()
    wascellname = getData(pdl,"WASCELL")[serveri]
    wascellname = wascellname.strip()
    wasnodename = getData(pdl,"WASNODE")[serveri]
    wasnodename = wasnodename.strip()
    wasprofile = getData(pdl,"PROFILE")[serveri]
    wasprofile = wasprofile.strip()
    wlphome = getData(pdl, "WLPHOME")[serveri]
    groupname = get_groupname(pdl, wascellname, wasnodename, wasprofile, wlphome)
    if group_servers.has_key(groupname):
        value = group_servers[groupname]
        value.append(serveri)
    else:
        indexes = []
        indexes.append(serveri)
        group_servers[groupname] = indexes

    debug("set_group_servers() exit")


def run_versionInfo(was_root):
    '''
        subroutine to get WAS version info from the output of versionInfo.sh|bat
        input: was_root directory
        to run: run_versionInfo(was_root)
        output: version string
    '''

    debug("Entry run_versionInfo(%s) %s" % (was_root,fixlev))

    platform = platformUtil.getPlatform()
    version = ""
    name= ""
    package = ""
    param = ""
    value = ""
    versionInfoOutput = []

    is_readable = _check_hist_dir(was_root)
    if is_readable:
        binDir = os.path.normpath(os.path.join(was_root, "bin"))
        versionInfoScript = os.path.normpath(os.path.join(binDir, _get_versionInfo_script(platform)))
        if not os.path.isfile(versionInfoScript):
            error_msg = "Script '%s' not found" % (versionInfoScript)
            raise Exception(error_msg)
        getVersionInfoCmd = add_quotes(versionInfoScript)

        #set locale to English to get expected output
        platformUtil.setLocale(platform)
        if platform.isZ():
            import configUtils
            rc, versionInfoOutput = configUtils.CommandResponseRC(getVersionInfoCmd)
        else:
            rc, versionInfoOutput = runCommand(getVersionInfoCmd)
        if rc != 0:
            # todo: Issue message instead of exception
            raise Exception ("%s script failed. %s" % (versionInfoScript, versionInfoOutput))
    else:
        debug("**Error** - history directory is not readable, unable to get version info")

    for line in versionInfoOutput:
        if line.find(" ") == -1: continue
        param, value = line.split(" ", 1)
        param = param.strip()
        value = value.strip()
        if param == "Version":
            version = value
        elif param == "ID":
            package = value
        elif param == "Name":
            name = value
    debug("Package = %s, Name = %s" % (package,name))

    debug("run_versionInfo() exit, result = %s" % version)
    return version


def _get_versionInfo_script(platform=None):
    '''
        return name of the versionInfo script based on the platform
    '''

    debug("Entry _get_versionInfo_script(%s) %s" % (platform,fixlev))

    _VERSIONINFO = "versionInfo"
    scriptName = _VERSIONINFO + ".sh"
    if platformUtil.isWindows():
        scriptName = _VERSIONINFO + ".bat"
    elif platformUtil.IS_OS400_OS():
        scriptName = _VERSIONINFO

    debug("_get_versionInfo_script() exit, result = %s" % scriptName)
    return scriptName


def _check_hist_dir(was_install_root):
    '''
        check if Websphere history directory is readable
    '''

    debug("Entry _check_hist_dir(%s) %s" % (was_install_root,fixlev))

    r_ok = True
    history = os.path.normpath(os.path.join(was_install_root, "properties", "version", "history"))
    debug("History path: %s" % history)
    if os.path.isdir(history):
        if hasattr(os, "R_OK"):
            if not os.access(history, os.R_OK): r_ok = False

    debug("_check_hist_dir() exit, result = %s" % r_ok)
    return r_ok


def rename_archival_file(log_fname):
    '''
        rename a file for archival
    '''

    debug("Entry rename_archival_file(%s) %s" % (log_fname,fixlev))

    # This code is duplicated in debugUtil.py to avoid having to make util.py
    # compatible with Jython 2.1.  Changes made there should also be done here.

    log_fname = os.path.abspath(log_fname)
    dirName = os.path.dirname(log_fname)

    if os.path.exists(log_fname):
        debug("File %s already exists and must be renamed" % log_fname)

        # create a string of local time to be inserted in the
        # renamed log file if exists.

        ltime = strftime("%Y%m%d%H%M%S")

        # Get the file name and insert the current time
        # in the file name for rename.

        filename = os.path.basename(log_fname)

        # This check is for Jython 2.1, it is not needed for 2.5.2 where
        # os.path.splitext() works as desired.
        if filename.find(".") == 0 and filename[1:].find(".") == -1:
            filebase, fileext = (filename,"")
        else:
            filebase, fileext = os.path.splitext(filename)
        logf = filebase + "." + ltime + fileext
        debug("Existing file will be renamed to %s" % logf)

        # Rename the current log file to the new name.

        rename_filename = os.path.normpath(os.path.join(dirName, logf))
        try:
            os.rename(log_fname, rename_filename)
            debug("Rename was successful")
        except OSError, errarg:
            # Note: Can't use getmsg in resourcebundle
            log.logWarning("Could not rename log %s - %s" % (log_fname, errarg))

    else:
        debug("No previous existing file %s, rename not needed" % log_fname)

    debug("Exit rename_archival_file()")


def format_message(message, params):
    '''
       Formats the message by replacing the parameters into  the message using
       from java.text.MessageFormat.
       @param message - Message text
       @param params - Parms to be replaced into the message
       @return - Modified message
    '''

    tmessage = format(message, params)

    return tmessage


def ispure_ascii(strchar):
    '''
       Checks the string for non ascii characters, and if it does exist,
       sets a return code of true, otherwise, false

       @param strchar - String to be converted.
       @return - true if pure ascii, otherwise, false
    '''

    assert strchar is not None, "utils.ispure_ascii(None) is not allowed"

    stat = True;

    jstring = java.lang.String(strchar)
    bytearray = jstring.getBytes()
    charset = Charset.forName("US-ASCII")
    charset_decoder = charset.newDecoder()

    try:
        bytebuffer = Bytebuffer.wrap(bytearray)
        charset_decoder.decode(bytebuffer)

    except java.nio.charset.CharacterCodingException:
        stat = False

    return stat

def isAlphaNumericString(str_input):
    '''
       validates if string contains only alpha numeric, ".", "_" and "-" characters
    '''

    debug("Entry isAlphaNumericString(%s) %s" % (str_input,fixlev))

    is_valid = re.match("^[\w\.-]+$", str_input)

    debug("isValidFilename exit %s" % is_valid)
    return is_valid

def convert_nonascii(strchar):
    '''
       Converts a nonascii character within a string to its ascii presentation.

       @param strchar - String which contains the non ascii characgter
    '''

    # Check see if the string has non ascii or not

    charoutput = strchar
    if strchar is None:
        charoutput = "None"
    elif not ispure_ascii(strchar):
        charoutput = ""
        debug("Character string has non ascii characters - %s" % (strchar))

        for ch in strchar:
            int_ch = ord(ch)

            # If character is non ascii, then add an @ as an escape character
            # to make it a unique string, so we won't have conflicts.
            # Also, if there is @ as a character then it is replaced with
            # @ and its hex representation.

            if int_ch > 127:
                ichar = java.lang.Integer.toHexString(int_ch)
                charoutput+= "@" + ichar
            elif int_ch == 64:
                charoutput+= "@40"
            else:
                charoutput+= ch

        debug("Converted string from (%s) to (%s)" % (strchar, charoutput))

    return charoutput


def convert_nonascii_ad71(strchar):
    '''
       Converts a nonascii character within a string to its ascii presentation.
       It adds a "u" to the ascii text.  This is based on AD 71 conversion.

       @param strchar - String which contains the non ascii characgter
    '''

    charoutput = strchar

    # Check see if the string has non ascii or not

    if not ispure_ascii(strchar):
        charoutput = ""
        debug("Character string has non ascii characters - %s" % (strchar))

        for ch in strchar:
            int_ch = ord(ch)
            if int_ch > 127:
                ichar = java.lang.Integer.toHexString(int_ch)
                if len(ichar) == 2:
                    cnvt = ichar.upper()
                    charoutput+= "u00" + cnvt
                elif len(ichar) == 3:
                    cnvt = ichar.upper()
                    charoutput+= "u0" + cnvt
                else:
                    cnvt = ichar.upper()
                    charoutput+= "u" + cnvt
            else:
                charoutput+= ch

        debug("Converted string from (%s) to (%s)" % (strchar, charoutput))

    return charoutput


def get_wlp_selected(pdl):
    '''
        Checks if Liberty servers selected for reconfiguration
        Returns 2 parameters
        -boolean flag, True if ONLY liberty servers are selected
        -list of liberty servers selected
    '''

    debug("Entry get_wlp_selected %s" % fixlev)

    wlp_servers = []
    wlp_indexes = []
    serverlist = getData(pdl, "SERVERS")
    only_wlp_selected = False
    for serveri in range(len(serverlist)):
        wlphome = getData(pdl, "WLPHOME")
        if wlphome is not None:
            wlphome = getData(pdl, "WLPHOME")[serveri]
            if wlphome != None and wlphome != "None" and len(wlphome) > 0:
                wlp_indexes.append(serveri)
                wlp_servers.append(serveri)
        only_wlp_selected = len(wlp_indexes) == len(serverlist)

    debug("Exit get_wlp_selected: only_wlp_selected=%s wlp_servers=%s" % (only_wlp_selected, wlp_servers))
    return (only_wlp_selected, wlp_servers)

def is_ccm(pdl):
    '''
    check if this is an SCM AI version
    returns 
    - true if this is SCM AI install
    - false if not SCM AI install
    '''
    debug("is_ccm entry")
    smai_pdl_data = getData(pdl, "SMAI")
    debug("is_ccm from pdl: %s" % smai_pdl_data)
    if smai_pdl_data is None:
        is_smai = True
        if platformUtil.isWindows():
            if pdl["CONFIGHOME"][0] is None:
                candleHome = "C:/IBM/ITM"
            else:
                configHome = pdl["CONFIGHOME"][0]
                dcHome = os.path.join(configHome, os.pardir)
                candleHome = os.path.join(dcHome, os.pardir) 
            yn_cnfglist_file = candleHome + os.sep + "BIN" + os.sep + "smai-agent.bat"
            debug("Checking is file exists %s" % yn_cnfglist_file)
            if os.path.exists(yn_cnfglist_file):
                is_smai = True
        else:
            confighome = getData(pdl, "CONFIGHOME")
            debug("checking if smai install on Linux")
            if confighome.find("ibm" + os.sep + "ccm" + os.sep) > -1:
               is_smai = True
        debug("is_smai_install %s" % is_smai)
        setData(pdl, "SMAI", is_smai, "Discover")
        smai_pdl_data = is_smai
    debug("is_ccm Exit %s" % smai_pdl_data)
    return smai_pdl_data
    
def getProperties(filepath, encoding='utf-8'):
    properties={}
    try:
        debug("Opening file %s." % (filepath))
        file = codecs.open(filepath, 'r', encoding)
        for line in file.readlines():
            line = line.rstrip("\r\n")
            if line.startswith('#') :
                continue
            if len(line.strip()) == 0:
                continue
            key, value = line.split('=')
            properties[key] = value
        return properties    
    except Exception, errmsg:        
        debug("Problem opening file %s. Error details - %s" % (filepath, errmsg))
    
def getProperty(filepath, propertyname, encoding='utf-8'):
    properties = getProperties(filepath, encoding)
    for k,v in properties.items():
        if k == propertyname:
            return v

########################################################
# Unit Test
########################################################

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL

    log.set_logging("./utils.log")
    debug("Unit testing of utils")
    #isValidIpv4Address("127.0.0.1")
    #raise SystemExit

    print "done"
    pdl = createPDL.make_pdl(createPCL.make_pcl())
    createPDL.proc_cliopt(pdl, ("", "-server","server01","-server","server02"))

    # Test the getValidation() routine
    ver = getValidation(pdl,"SMF")
    print "SMF verification = %s %s" % (ver,type(ver))
    print "ver(SMF,Environment,xray) = %s" % ver("SMF","Environment","xray")
    print "ver(SMF,Environment,Yes) = %s" % ver("SMF","Environment","Yes")
    raise SystemExit

    log.logDumpPDL(pdl,short=True)
    setData(pdl,"JVMMODE",["FALSE","FALSE"],"*Debug*")
    jvmMode = getData(pdl,"JVMMODE")
    jvmMode[0] = "TRUE"
    setData(pdl,"SMF",True,"*Debug*")
    print "actual SMF = '%s' %s" % (getData(pdl,"SMF"),type(getData(pdl,"SMF")))
    print "data string SMF = '%s' %s" % (getDataString(pdl,"SMF"),type(getDataString(pdl,"SMF")))
    print "actual JVMMODE = %s %s" % (getData(pdl,"JVMMODE"),type(getData(pdl,"JVMMODE"))        )
    print "actual JVMMODE[0] = %s %s" % (getData(pdl,"JVMMODE",0),type(getData(pdl,"JVMMODE",0)))
    #raise SystemExit(0)

    log.logDumpPDL(pdl,short=True)
    pcl = getPCL(pdl)

    print "*Debug*  SERVERS = %s" % getData(pdl,"SERVERS")
    print "*Debug*  SERVERS[0] = %s" % getData(pdl,"SERVERS",0)
    print "*Debug*  SERVERS[1] = %s" % getData(pdl,"SERVERS",1)
    print "*Debug*  SERVERS format = %s" % getFormat(pdl,"SERVERS")
