#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    utilities for discovering WAS home dir and getting profiles

    discover_was() - Get a list of WAS home directories
    get_was_profiles(was_dir) - Get a list of profiles for a WAS home
    get_was_profile_home(was_dir, profile_name) - Get profile home directory
'''

fixlev = "51424" # Last APAR, defect or story to update this module

import os
import re
import codecs

import java.io.File

from xml.dom import minidom
from subprocess import call
from time import strftime

from java.lang import System

import utils
import platformUtil
import doprompt
import debugUtil as log
import wlpConfigUtils

from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg
from ConfigurationException import ConfigurationException


REG_FILE_LIST  = [".was.iminstalllocations.registry", ".was.installlocations.registry"]
DOCS_DIR = "Documents and Settings"
WIN_USER_DIR = ["Administrator", "All Users"]
UNIX_DIR = ["/root", "/opt/.ibm", "/usr/lib/objrepos"]

ADMIN_USER = "Administrator"
IBM_SFW_REG_KEY = "HKEY_CURRENT_USER\\Software\\IBM"
KEY_VALUE = "InstallLocation"
WAS_PRODUCT = "WebSphere Application Server"

CMD_LIST_PROFILES = "-listProfiles"
CMD_GET_PROFILE_PATH = "-getPath -profileName"
CMD_GET_DFLT_PROFILE = "-getDefaultName"


WAS_UNIX_KEY = "WSB"        #package name WAS is under on UNIX

WAS_PROFILES={}


def getManageProfileCmd(was_home):
    '''
        returns location of the manageprofiles script
        depending on the WAS version we are looking for
        manageprofiles[.bat|.sh] or wasprofile[.bat|.sh]
    '''

    debug("Entry getManageProfileCmd %s" % fixlev)

    binDir = os.path.normpath(os.path.join(was_home, "bin"))
    script_list = ["manageprofiles", "wasprofile"]
    ext = ".sh"
    if platformUtil.isWindows(): ext = ".bat"
    for script in script_list:
        cmd = os.path.normpath(os.path.join(binDir, script + ext))
        if os.path.exists(cmd):
            debug("manage profile script: %s" % cmd)
            return '"' + cmd + '"'

    debug("failed to find manageprofile script in %s" % binDir)
    return None


def _getWasFromRegistry():

    debug("Entry _getWasFromRegistry() %s" % fixlev)

    if platformUtil.isWindows():
        return _getWasFromWinRegistry()
    else:
        return _getUnixPackageInfo()


def _getUnixPackageInfo():
    '''
        the idea is to read installed package information using method appropriate for a given platform
    '''

    debug("Entry getUnixPackageInfo %s" % fixlev)

    dir_list = []
    osFamily = platformUtil.getOSFamily()
    debug("os family: %s" % osFamily)
    user_home = System.getProperty("user.home") or "/"
    user_name = System.getProperty("user.name")
    debug("User name: %s  User home: %s" % (user_name, user_home))
    if user_home not in UNIX_DIR: UNIX_DIR.insert(0, user_home)
    vpd_properties_file = get_vpd_properties_file(UNIX_DIR)
    if vpd_properties_file != None:
        dir_list = getWasFromVpdProperties(vpd_properties_file)
    if osFamily == "SOLARIS":
        rc, pkglist = utils.runCommand("pkginfo 2>/dev/null | grep WSB | cut -f 2 -d' '")
        if rc != 0:
            debug("Unable to get package info, %s" % pkglist)
        else:
            for package in pkglist:
                rc, output = utils.runCommand("LANG=C; export LANG; pkginfo -l %s | grep BASEDIR" % (package.strip()))
                if rc != 0:
                    debug("Unable to get info for package '%s', error: %s" % (package, output))
                else:
                    for line in output:
                        tokens = line.split("BASEDIR:")
                        if len(tokens) == 2:
                            dir_list.append(tokens[1].strip())
                            debug("Found WAS home at %s" % tokens[1].strip())
    elif osFamily == "HP-UX":
        rc, pkglist = utils.runCommand("/usr/sbin/swlist 2>/dev/null| grep %s " % WAS_UNIX_KEY)
        if rc != 0:
            debug("Unable to get package info, %s" % pkglist)
        else:
            for pkg_line in pkglist:
                #WSBAA70      /opt/IBM/WebSphere/AppServer
                #info format
                package = pkg_line.split()[0].strip()
                rc, output = utils.runCommand("/usr/sbin/swlist -a location  -l product %s" % package)
                if rc != 0:
                    debug("Unable to get info for package '%s', error: %s" % (package, output))
                else:
                    for line in output:
                        tokens = line.split("\t")
                        if len(tokens) == 2:
                            dir_list.append(tokens[1].strip())
                            debug("Found WAS home at %s" % tokens[1].strip())
    im_was_list = get_im_was()
    if len(im_was_list) > 0:
        dir_list.extend(im_was_list)

    debug("Exit getUnixPackageInfo")
    return dir_list


def get_vpd_properties_file(dir_list):

    debug("Entry get_vpd_properties_file %s" % fixlev)

    file_name = "vpd.properties"
    ans = None
    for dir in dir_list:
        debug("looking for vpd.properties under %s" % dir)
        full_file_name = os.path.normpath(os.path.join(dir, file_name))
        if os.path.exists(full_file_name):
            ans = full_file_name
            break

    debug("Exit get_vpd_properties_file: %s" % ans)
    return ans


def getWasFromVpdProperties(file):
    '''
        read vpd.properties file (Linux/AIX 'registry') to get a list of installed WAS
    '''

    debug("Entry getWasFromVpdProperties %s - %s" % (fixlev,file))

    dir_list = []
    debug("reading registry file %s" % file)
    rc, output = utils.runCommand("cat %s 2>/dev/null | grep %s " % (file, WAS_UNIX_KEY))
    if rc != 0:
        debug("Unable to get data from vpd.properties")
    else:
        debug("found %d lines for websphere components" % len(output))
        for line in output:
            if len(line.split("|")) > 12:
                dir_list.append(line.split("|")[12])

    debug("Exit getWasFromVpdProperties %s" % str(dir_list))
    return dir_list


def _getWasFromWinRegistry():
    '''
        reads windows registry to get InstallLocation value for WebSphere
        example of the output we are looking for:
        HKEY_CURRENT_USER\Software\IBM\WebSphere Application Server\8.0.0.0
        BinPath     REG_SZ  E:\IBM\WebSphere\AppServer8\bin
        LibPath     REG_SZ  E:\IBM\WebSphere\AppServer8\lib
        InstallLocation     REG_SZ  E:\IBM\WebSphere\AppServer8
        MajorVersion        REG_SZ  8

        returns a list of possible WAS home dirs
    '''

    debug("Entry getWasFromRegistry %s" % fixlev)

    reg_key = IBM_SFW_REG_KEY
    subkey_list = []
    dir_list = []
    subkey_list = utils.get_win_registry_key_all(reg_key)
    key_found = 0
    for subkey in subkey_list:
        was_key = reg_key + "\\" + WAS_PRODUCT
        subkey = subkey.strip()
        if subkey.find(IBM_SFW_REG_KEY) > -1: #that's a key line
            key_found = 0
            if subkey.find(was_key) > -1:
                debug("Found WAS key %s" % subkey)
                key_found = 1
        else:
            if key_found:
                if subkey.find(KEY_VALUE) > -1:
                    was_home = subkey.split("REG_SZ")[1].strip()
                    dir_list.append(was_home)
                    debug("Found WAS home at %s" % was_home)
    debug("Found %d WAS homes in registry" % len(dir_list))

    debug("Exit getWasFromRegistry %s" % str(dir_list))
    return dir_list


def _getWasRegistryFileList():
    '''
        locates all .WASRegistry files on the machine
    '''

    debug("Entry _getWasRegistryFileList %s" % fixlev)

    user_home = System.getProperty("user.home") or "/"
    user_name = System.getProperty("user.name")
    debug("user=%s home dir=%s" % (user_name, user_home))
    discovered_reg_file_list = []
    if user_home not in UNIX_DIR: UNIX_DIR.insert(0, user_home)
    #check user home
    for file in REG_FILE_LIST:
        regFilePath = os.path.normpath(os.path.join(user_home, file))
        if os.path.exists(regFilePath):
            discovered_reg_file_list.append(regFilePath)

    dirs_to_search = UNIX_DIR
    if platformUtil.isWindows():
        dirs_to_search = []
        index = user_home.find(":")
        systemDrive = "C:\\"
        if index == 1: systemDrive = user_home[0:3]
        docsAndSettings = os.path.join(systemDrive, DOCS_DIR)
        for user_dir in WIN_USER_DIR:
            dirs_to_search.append(os.path.join(docsAndSettings, user_dir))

    #check all default locations
    for dir in dirs_to_search:
        for file in REG_FILE_LIST:
            debug("looking for file '%s' under '%s" % (file, dir))
            regFilePath = os.path.normpath(os.path.join(dir, file))
            if os.path.exists(regFilePath) and regFilePath not in discovered_reg_file_list:
                discovered_reg_file_list.append(regFilePath)
    debug("found files: %s" % (discovered_reg_file_list))

    debug("Exit _getWasRegistryFileList")
    return discovered_reg_file_list


def _readWasRegistryFile(fileName):
    '''
        reads .WASRegsitry file returns a list of entries as possible WAS install home dirs
    '''

    debug("Entry _readWasRegistryFile %s - %s" % (fixlev,fileName))

    if fileName == None or not os.path.isfile(fileName): return
    filh = open(fileName)
    dir_list = []
    while 1:
        line = filh.readline()
        if not line: break
        line = line.strip()
        if len(line) > 0:
            dir_list.append(line)
    filh.close()

    debug("Exit _readWasRegistryFile")
    return dir_list


def verifyValidWasHome(dirName, pdl=None, quiet=True):
    '''
        validates that directory is WAS install directory
        verifies that there is WAS>product under <dir>/properties/version
    '''

    debug("Entry verifyValidWasHome %s - %s" % (fixlev,dirName))

    isWasHome = False

    # Note: in z/OS: the properties/version product file in WASHOME specifies a symlink to WAS.product, WBI.product, etc.
    #                but it may not be not be necessary to test it.
    #                The presence of profiles/default should distinguish WASHOME from the WAS install directory.

    if platformUtil.IS_zOS_OS():
        was_home_profile_dir = os.path.normpath(os.path.join(dirName, "profiles", "default"))
        was_home_versionDirPath = os.path.normpath(os.path.join(dirName, "properties", "version"))
        if os.path.isdir(was_home_profile_dir):
            if os.path.isdir(was_home_versionDirPath):
                isWasHome = True
                debug("  Is z/OS WAS home, version properties directory found: %s" % was_home_profile_dir)
            else:
                debug("  Not z/OS WAS home, missing version properties directory")
                if not quiet: print "%s" % getmsg("CFG5067E", "No properties/version path under {0}", [dirName])
        else:
            debug("  Not z/OS WAS home, missing default profile")
            if not quiet: print "%s" % getmsg("CFG5068E", "No profiles/default path under {0}", [dirName])
    elif platformUtil.IS_OS400_OS():
        isWasHome = True
    else:
        versionDirPath = os.path.normpath(os.path.join(dirName, "properties", "version"))
        if os.path.isdir(versionDirPath):
            dirList = os.listdir(versionDirPath)
            for f in dirList:
                if f.endswith(".product") and f.startswith("WAS"):
                    isWasHome = True
                    debug("  Is WAS home, found WAS product file: %s" % f)
                    break
    if not isWasHome:
        debug("  Not tWAS home, product version file not found")
        debug ("Checking if WebSphere Liberty")
        is_wlp = isWLPHome(dirName, quiet=quiet)
        if is_wlp:
            debug("WLP found, setting Liberty Profile flags")
            isWasHome = True
            if pdl != None:
                pdl["WLPINSTALLDIR"][0:2] = dirName, "wasdiscover"
                pdl["WLPUSRDIR"][0:2] = wlpConfigUtils.getWlpUserDir(dirName), "wasdiscover"
        else:
            debug("  Not WAS home, missing version properties file or directory")
            if not quiet: 
                if os.path.isdir(versionDirPath): 
                    print "%s" % getmsg("CFG5069E", "The WebSphere product file WAS.product was not found in {0}", [versionDirPath])
                else:
                    print "%s" % getmsg("CFG5070E", "No WebSphere properties/version path found under {0}", [dirName])
    debug("Exit verifyValidWasHome result: %s" % isWasHome)
    return isWasHome


def isWLPHome(dir_path, quiet=True):
    '''
    verify if directory is WebSphere Liberty home
    '''

    debug("Entry isWLPHome %s  - is this directory WAS Liberty: %s" % (fixlev,dir_path))

    ############################################################
    # Disable Liberty detection (for back-ported version only) #
    ############################################################

    if False:
        debug("isWLPHome() always returns False for back-ported version.")
        return False

    is_wlp = False
    versionDirPath = os.path.normpath(os.path.join(dir_path, "lib", "versions"))
    if os.path.isdir(versionDirPath):
        dirList = os.listdir(versionDirPath)
        for f in dirList:
            if f.lower() == "websphereapplicationserver.properties":
                debug("Found WebSphere Liberty home properties file: %s" % os.path.join(versionDirPath,f))
                is_wlp = True
                break
        if not is_wlp:
            debug("Did not find WebSphere Liberty home properties file")
            if not quiet: print "%s" % getmsg("CFG5071E", "No Liberty home properties file found in lib/versions under {0}", [dir_path])
    else:
        debug("Did not find WebSphere lib/versions path")
        if not quiet: print "%s" % getmsg("CFG5072E", "Not WebSphere Liberty: lib/properties path found under {0}", [dir_path])

    debug("Exit isWLPHome: %s" % is_wlp)
    return is_wlp


def getwsadmin(dirName,depth=5):
    '''
        validates that directory is WAS install directory
        gets a list of WSADMIN scripts
        @param dirName directory to search
        @param depth   max depth to search
        @return        list of wsadmin paths
    '''

    debug("Entry getwsadmin %s - %s" % (fixlev,dirName))

    wsadmin = platformUtil.getWSAdminScript() + "$"
    debug("wsadmin script pattern for this platform: '%s'" % (wsadmin))
    wsadmin_list = utils.find_file(dirName, wsadmin, max_depth=depth)

    debug("Exit getwsadmin")
    return wsadmin_list


def is_security_enabled(profile_home):

    debug("Entry is_security_enabled %s - %s" % (fixlev,profile_home))

    cells_dir = os.path.normpath(os.path.join(profile_home, "config", "cells"))
    enabled = {}
    #from xml.dom import minidom
    cells_path = java.io.File(cells_dir)
    if cells_path.exists():
        fileList = os.listdir(cells_dir)
        for f in fileList:
            if os.path.isfile(f): continue   #we are looking for directory only
            cellName = f
            enabled_str = None
            debug("cell name: %s" % cellName)
            security_xml = os.path.normpath(os.path.join(profile_home, "config", "cells", cellName, "security.xml"))
            debug("security xml='%s'" % security_xml)
            security_path = java.io.File(security_xml)
            try:
                if security_path.exists():
                    doc = minidom.parse(security_xml)
                    root = doc.documentElement
                    enabled_str = root.getAttribute("enabled").strip()
                    #enabled[cellName] = enabled_str
                    debug("Exit is_security_enabled")
                    return enabled_str
                else:
                    continue
            except:
                log.logException("Unable to get security enabled value")
                continue
    debug("Unable to get security.xml for '%s'" % profile_home)
    return enabled


def get_was_profiles(pdl, was_home):
    '''
        This routine gets the profiles for the WAS home.  It reads the profiles registry
        file and gets the profile names and paths.

        The list of profiles returned may be empty if WAS_HOME is determined to be invalid
        or no acceptable profile is discovered.

        @param pdl - PDL reference
        @param was_home - WAS home
        @return - A list of profiles, and a list of profile paths
    '''

    debug("Entry get_was_profiles(%s) %s" % (was_home, fixlev))

    profile_name_list = []
    profile_path_list = []
    default_profile_name = None
    is_os400 = platformUtil.IS_OS400_OS()

    try:

        debug("Getting profiles for WAS home: %s" % (was_home))

        # Search for the registry profile file name.  The registry usually resides in
        # [WAS_HOME]/properties/profileRegistry.xml, but that location is found in
        # the property WS_PROFILE_REGISTRY in [WAS_HOME]/properties/wasprofile.properties.

        if not is_os400:
            profile_registry_found = False
            profile_registry_file = _get_was_profile_registry(was_home)
            debug("Looking for profile registry file as: %s" % profile_registry_file)
            profile_registry_found = os.path.isfile(profile_registry_file)
        else:
            # For OS/400 we just assume the registry file exists (Where is Carlos when we need him?)
            profile_registry_file = os.path.normpath(os.path.join(was_home, "profileRegistry", "profileregistry.xml"))
            profile_registry_found = True
            debug("Profile registry file for OS/400 assumed to be: %s" % profile_registry_file)

        if not profile_registry_found:
            print "%s" % getmsg("CFG2041E", "Profile registry file was not found: {0}", [profile_registry_file])

        else:

            debug("Profile registry file found: %s" % profile_registry_file)

            profiles = minidom.parse(profile_registry_file)
            profile_list = profiles.getElementsByTagName("profile")
            debug("Profile list is: %s" % (profile_list))

            #
            # For each profile tag, get its name and its path attribute
            #
            for profile_entry in profile_list:
                debug("Profile entry is: %s" % (profile_entry))
                profile_name = profile_entry.getAttribute("name").strip()
                profile_path = profile_entry.getAttribute("path").strip()
                profile_template = profile_entry.getAttribute("template").strip()
                isDefault = profile_entry.getAttribute("isDefault").strip()
                if isDefault == "true":
                    default_profile_name = profile_name
                    debug("default profile: %s" % default_profile_name)

                #
                # Sample tag for DMGR profile, it is created using 'management' or 'dmgr' template
                #   <profile isAReservationTicket="false" isDefault="true" name="Dmgr01" path="/opt/IBM/WAS8/profiles/Dmgr01" template="/opt/IBM/WAS8/profileTemplates/management">
                # Sample tag for non-DMGR profile
                #   <profile isAReservationTicket="false" isDefault="false" name="odr" path="/opt/IBM/WAS8/profiles/odr" template="/opt/IBM/WAS8/profileTemplates/default">
                #

                isDmgr = False
                if profile_template != None and (profile_template.endswith("management") or profile_template.endswith("dmgr")):
                    isDmgr = True
                    debug("Found DMGR profile '%s'" % profile_name)
                createtemplateflag = pdl["CREATETEMPLATE"][0]
                deletetemplateflag = pdl["DELETETEMPLATE"][0]
                if createtemplateflag == True or deletetemplateflag == True:
                    if isDmgr:
                        profile_name_list.append(profile_name)
                        profile_path_list.append(profile_path)
                        debug("Profile name is: %s   Profile Path is: %s" % (profile_name, profile_path))
                elif not isDmgr:
                    profile_name_list.append(profile_name)
                    profile_path_list.append(profile_path)
                    debug("Profile name is: %s   Profile Path is: %s" % (profile_name, profile_path))
                else:
                    debug("Profile is DMGR, will not be added - %s" % (profile_name))

    except Exception, errmsg:
        log.logException("Error occurred while getting WAS profiles - %s" % (errmsg))
        raise

    debug("Exit get_was_profiles")
    if len(profile_name_list) == 0:
        debug("  No profile names discovered")
    else:
        debug("  Profile names:\n    %s" % "\n    ".join(profile_name_list))
    if len(profile_path_list) == 0:
        debug("  No profile paths discovered")
    else:
        debug("  Profile paths:\n    %s" % "\n    ".join(profile_path_list))
    debug("  Default profile: %s" % default_profile_name)
    return profile_name_list, profile_path_list, default_profile_name


def _get_was_profile_registry(was_home):
    '''
        Passed the [WAS_HOME], we locate the profile registry file.
    '''

    debug("Entry _get_profile_registry(%s) %s" % (was_home, fixlev))

    enc = platformUtil.getEncoding()
    filename = os.path.join(was_home, "properties", "wasprofile.properties")
    debug("Attempting to read wasprofile.properties as %s from file %s" % (enc,filename))
    lines = []
    try:
        infile = codecs.open(filename, "r", enc)
        lines = [line.rstrip("\r\n") for line in infile.xreadlines() if line.strip() != "" and not line.startswith("#")]
        infile.close()
    except IOError, errmsg:
        debug("I/O error reading wasprofile.properties: %s" % errmsg)

    debug("Contents of wasprofile.properties:\n\n    %s" % "\n    ".join(lines))
    profile_registry_file = None
    for line in lines:
        props = line.split("=",1)
        if len(props) != 2: continue
        if props[0].strip() == "WS_PROFILE_REGISTRY":
            profile_registry_file = props[1].strip()
            profile_registry_file = profile_registry_file.replace("${was.install.root}",was_home)
            profile_registry_file = os.path.normpath(profile_registry_file)
            debug("Normalized profile registry name: %s" % profile_registry_file)
            if not os.path.isfile(profile_registry_file):
                debug("File not found: %s" % profile_registry_file)
                profile_registry_file = None
            break

    if profile_registry_file is None:
        debug("Unable to resolve profile registry name, will make the default assumption")
        profile_registry_file = os.path.normpath(os.path.join(was_home, "properties", "profileRegistry.xml"))

    debug("Exit _get_profile_registry: %s" % profile_registry_file)
    return profile_registry_file


def get_im_was():
    '''
        Returns a list of WebSphere homes installed using IBM Installation Manager
        input: none
        output: list of directory names
    '''

    debug("Entry get_im_was %s" % fixlev)

    dir_list = []
    user_home = System.getProperty("user.home") or "/"
    im_home_list = []
    user_im_home = os.path.normpath(os.path.join(user_home, "var", "ibm"))

    installed_xml_file = "installed.xml"
    im_home_list.append(user_im_home)
    if user_home != "/": im_home_list.append("/var/ibm")

    for im_home in im_home_list:
        if not os.path.exists(im_home):
            debug("checking IM installations, directory %s not found on the system" % im_home )
            continue
        fileList = os.listdir(im_home)
        for file in fileList:
            full_name = os.path.normpath(os.path.join(im_home, file))
            if os.path.exists(os.path.join(full_name, installed_xml_file)):
                installed_xml = os.path.normpath(os.path.join(full_name, installed_xml_file))
                debug("found IM dir in %s" % full_name)
                dir_list.extend(get_installed_product_list(installed_xml))

    debug("Exit get_im_was %s" % str(dir_list))
    return dir_list


def get_installed_product_list(installed_xml):
    '''
        Parses IM xml file to get a list of installed products
        input: xml file full name
        output: list of directory names
    '''

    debug("Entry get_installed_product_list %s - %s" % (fixlev,installed_xml))

    dir_list = []
    try:
        from xml.dom import minidom
        doc = minidom.parse(installed_xml)
        installInfo = doc.getElementsByTagName("installInfo")
        if installInfo != None and len(installInfo) == 1:
            products = installInfo[0].getElementsByTagName("location")
            for product in products:
                id = product.getAttribute("id")
                debug("installed product: %s" % id)
                if id != None and id.find("WebSphere") > -1:
                    path = product.getAttribute("path")
                    dir_list.append(path)
                    debug("found WebSphere at: %s" % path)
    except:
        log.logException("error parsing installed.xml")

    debug("Exit get_installed_product_list")
    return dir_list


def discover_was():
    '''
        returns a list of WAS install directories based on:
        - entries in .WASRegsitry
        - windows registry under HKEY_CURRENT_USER\\Software\\IBM
        - unix entries in .was.iminstalllocations.registry
        - IM repository
    '''

    debug("Entry discover_was %s" % fixlev)

    was_home_dir_list = []
    reg_file_list = _getWasRegistryFileList()
    result_list = []
    for file in reg_file_list:
        was_home_dir_list.extend(_readWasRegistryFile(file))
    was_home_dir_list.extend(_getWasFromRegistry())
    was_home_dir_list = list(set(was_home_dir_list))
    for dir in was_home_dir_list:
        if verifyValidWasHome(dir): result_list.append(dir)

    debug("Exit discover_was")
    return result_list


def process_backup(pdl, backup_file, serveri, is_wlp=False):
    '''
        process WAS backup based on WAS type
    '''

    debug("Entry process_backup %s" % fixlev)

    if is_wlp:
        return backup_wlp(pdl, backup_file, serveri)
    else:
        return backup_was(pdl, backup_file, serveri)


def backup_wlp(pdl, backup_file, serveri):
    '''
        sub for "backing up" liberty profile. Creates copies of server.xml, server.env and jvm.options files
    '''

    debug("Entry backup_wlp %s" % fixlev)

    wlpInstallRoot = utils.getData(pdl, "WLPHOME", serveri)
    wlpUsrDirPath = utils.getData(pdl, "USERINSTALLROOT", serveri)
    wasserver = utils.getData(pdl,"SERVERS", serveri)
    debug("backing up server %s" % wasserver)
    files_to_backup = ["server.xml", "server.env", "jvm.options"]
    #Liberty configuration files for the server
    wlpServerRoot = os.path.normpath(os.path.join(wlpUsrDirPath, "servers", wasserver))
    rc = 0
    suffix = "itcam" + "_" + strftime("%Y%m%d%H%M%S")
    backup_files = []
    output = ""
    try:
        for file in files_to_backup:
            filepath = os.path.normpath(os.path.join(wlpServerRoot, file))
            if os.path.isfile(filepath):
                file_bak = wlpConfigUtils.create_file_backup_copy(filepath, suffix)
                backup_files.append(file_bak)
            rc = 0
        output = ", ".join(backup_files)
    except Exception, errmsg:
        rc = 1
        output = errmsg

    debug("Exit backup_wlp (rc=%s, %s)" % (rc, output))
    return (rc, output)


def backup_was(pdl, backup_file, serveri):
    '''
        command for backing up websphere profile using backupConfig.sh|bat script
        will use the following options
        -nostop - in order not to stop the WAS server
        -quiet - to suppress console messages
        -logfile - log file is created under <confighome>/data
        -user, -password for specifying user credentials
        -profileName for specifying a profile

        returns return_code, output pair from backupConfig execution
    '''

    debug("Entry backup_was %s" % fixlev)

    backup_script = "backupConfig.sh"
    if platformUtil.isWindows():
        backup_script = "backupConfig.bat"
    elif platformUtil.IS_OS400_OS():
        backup_script = "backupConfig"
    confighome = pdl["CONFIGHOME"][0]
    wasroot = os.path.normpath(os.path.join(pdl["WASINSTALLROOT"][0][serveri], "bin"))
    profile = pdl["PROFILE"][0][serveri]
    if profile != None:
        debug("profile: " + profile)
    wasProfileHome = utils.getData(pdl, "PROFILEHOME")[serveri]
    if wasProfileHome != None:
        wasProfileHome = wasProfileHome.strip()
        debug("wasProfileHome: " + wasProfileHome)
    #
    # Check for security
    #
    wroot = pdl["WASINSTALLROOT"][0][serveri]
    checkWASSecurity(pdl, wasProfileHome, wroot, profile)
    user = None
    password = None
    userlist = pdl["USERLIST"][0]
    if userlist != None:
        if userlist.has_key(wasProfileHome):
            user = userlist[wasProfileHome]
            debug("user: " + user)
    passwordlist = pdl["PASSWORDLIST"][0]
    if passwordlist != None:
        if passwordlist.has_key(wasProfileHome):
            password = passwordlist[wasProfileHome]
    if password == None:
        if user != None:
            raise Exception("WAS admin user: " + user + " has no password")
    log_path = confighome
    if utils.createDataDir(confighome):
        log_path = os.path.normpath(os.path.join(log_path, "data"))
    log_file = os.path.normpath(os.path.join(log_path, "backupConfig.log"))
    command = os.path.normpath(os.path.join(wasroot, backup_script))

    if command.find(" ") > -1:
        command = utils.add_quotes(command)
    cmd_params = []

    cmd_params.append("-nostop")
    cmd_params.append("-quiet")
    cmd_params.append("-logfile %s" % utils.add_quotes(log_file))
    if profile != None: cmd_params.append("-profileName %s" % profile)

    if user != None and password != None:
        cmd_params.append("-username %s -password %s" % (user,password))
    command += " %s" % (utils.add_quotes(backup_file))

    for param in cmd_params:
        command += " %s" % param
    msgl =[]
    msgl.append(profile)
    msgl.append(wasroot)
    infomsg = getmsg("CFG2015I", "Running Backup processing for profile: {0} home: {1} ...", msgl)
    print infomsg
    command_debug = re.sub("-password\s+%s" % password, "-password ********", command)
    output = []
    if platformUtil.isWindows():
        rc = call(command)
    else:
        rc = call(command, shell=True)
    #rc, output = utils.runCommand(command, command_debug)

    debug("Exit backup_was rc=%s" % str(rc))
    return (rc, output)


def checkWASSecurity(pdl, wasprofilehome, wasroot, profilename, retry=None):
    '''
       This call checks the WAS for global security.  If it is on,
       it prompts the user to specify user and the password.  The
       user and password is saved in its own pdl, as a dictionary
       which the key of the dictionary is the was profile home.
       Before prompting, the dictionary for uid and pwd is checked and
       if it already has been specified for a profile, the prompting
       is skipped.

       @param pdl - PDL reference
       @param wasProfilehome - WAS profile home
       @param wasroot - WAS root path
       @param profilename - Profile name
    '''

    debug("Entry checkWASSecurity %s" % fixlev)
    debug("wasprofilehome: " + wasprofilehome)

    userentry = None
    passwordentry = None
    useclientpropsentry = None

    osfamily = platformUtil.getOSFamily()
    #
    # Check the dictionary for a specific profile, if the uid/pws has already
    # been specified.
    #
    userdict = utils.getData(pdl, "USERLIST")
    passworddict = utils.getData(pdl, "PASSWORDLIST")
    useclientpropsdict = utils.getData(pdl, "USECLIENTPROPSLIST")
    debug("  userdict: %s " % str(userdict) )
    if passworddict is None:
        debug("  passworddict : None")
    else:
        debug("  passworddict : {%s}" % ", ".join(["%s : ********" % k for k in passworddict.keys()]))
    debug("  useclientpropsdict : %s " % str(useclientpropsdict))
    if useclientpropsdict is not None:
        useclientpropsentry = useclientpropsdict.get(wasprofilehome)
    if userdict is not None and passworddict is not None:
        userentry = userdict.get(wasprofilehome)
        passwordentry = passworddict.get(wasprofilehome)
    if useclientpropsentry is not None and useclientpropsentry == True:
        debug("User client props is set")
    elif userentry is not None and passwordentry is not None:
        debug("Userid and password found for %s" % (wasprofilehome))
    else:
        securityOn = is_security_enabled(wasprofilehome)
        debug("WAS securityOn: " + str(securityOn))
        if securityOn == "true":
            washome_label = getmsg("WASHOME", "WAS Home: ")
            wasprofile_label = getmsg("WASPROFILE", "WAS Profile: ")
            wasmsg = washome_label + wasroot + "    " + wasprofile_label + profilename
            print wasmsg
            print "\n"
            global_security_msg = getmsg("GLOBALSECURITY", "WAS security is on.")
            print global_security_msg
            #Check if users want to use soap.client.props or other client.props
            #If *.client.props file has no user/password setting,
            #then retry will happen.
            #In retry, we should not ask users if they want to use *.client.props again.
            debug("retry: " + str(retry))
            if retry == None and not pdl["USECLIENTPROPS"][0]:
                doprompt.do_prompt("USECLIENTPROPS", pdl)
            useclientprops = utils.getData(pdl, "USECLIENTPROPS")
            debug("USECLIENTPROPS: %s" % (useclientprops))
            useclientpropsdict = utils.getData(pdl, "USECLIENTPROPSLIST")
            if useclientpropsdict is None:
                useclientpropsdict = {}
                utils.setData(pdl, "USECLIENTPROPSLIST", useclientpropsdict, "utils")
            useclientpropsdict[wasprofilehome] = useclientprops
            # Recall that batch mode supports entry of user and password
            if not pdl["USECLIENTPROPS"][0] or pdl["USECLIENTPROPS"][0] == False:
                #Populate USER/PASSWORD in PDL
                doprompt.do_prompt("USER", pdl)
                user = utils.getData(pdl, "USER")
                if user is not None:
                    user = user.strip()
                if user is not None and len(user) > 0:
                    doprompt.do_prompt("PASSWORD", pdl)
                    passwordentry = utils.getData(pdl, "PASSWORD")
                    if passwordentry is not None:
                        passwordentry = passwordentry.strip()
                    if passwordentry != None and len(passwordentry) > 0:
                        #
                        # Save the user and password in pdl (USERLIST, PASSWORDLIST)
                        # Each entry in the list is a dictionary which its key is the
                        # WASHOME.WASPROFILE and its value is userid and password.
                        #
                        userdict = utils.getData(pdl, "USERLIST")
                        passworddict = utils.getData(pdl, "PASSWORDLIST")
                        if userdict is None:
                            userdict = {}
                            utils.setData(pdl, "USERLIST", userdict, "utils")
                        if passworddict is None:
                            passworddict = {}
                            utils.setData(pdl, "PASSWORDLIST", passworddict, "utils")
                        userdict[wasprofilehome] = user
                        passworddict[wasprofilehome] = passwordentry
                        #
                        # Clear USER and PASSWORD pdl entries
                        #
                        utils.setData(pdl, "USER", None, "utils")
                        utils.setData(pdl, "PASSWORD", None, "utils")
                    else:
                        debug("Blank value entered for password")
                        #
                        # If userid was entered as blank, then set it as blank within double quotes
                        # as this will cause the wsadmin to get an error which is detected to do
                        # a retry.
                        #
                        utils.setData(pdl, "USER", '"  "', "utils")
                        utils.setData(pdl, "PASSWORD", '"  "', "utils")
                else:
                    debug("Blank value entered for userid")
                    #
                    # If password was entered as blank, then set it as blank within double quotes
                    # as this will cause the wsadmin to get an error which is detected to do
                    # a retry.
                    #
                    utils.setData(pdl, "USER", '"  "', "utils")
                    utils.setData(pdl, "PASSWORD", '"  "', "utils")
            else:
                #USECLIENTPROPS
                utils.setData(pdl, "USER", None, "utils")
                utils.setData(pdl, "PASSWORD", None, "utils")

    debug("Exit checkWASSecurity")


def clear_WASsecurity(pdl, wasprofilehome):
    '''
       This call clears the user/password for a specific profile in the
       pdl.

       @param pdl - PDL reference
       @param wasProfilehome - WAS profile home
    '''

    debug("Entry clear_WASsecurity %s" % fixlev)

    osfamily = platformUtil.getOSFamily()
    if osfamily != "OS400":
        debug("wasprofilehome: " + wasprofilehome)
        userdict = utils.getData(pdl, "USERLIST")
        passworddict = utils.getData(pdl, "PASSWORDLIST")
        useclientpropsdict = utils.getData(pdl, "USECLIENTPROPSLIST")
        if userdict is not None and len(userdict) > 0 and passworddict is not None and len(passworddict) > 0:
            del userdict[wasprofilehome]
            del passworddict[wasprofilehome]
        if useclientpropsdict is not None and len(useclientpropsdict) > 0:
            del useclientpropsdict[wasprofilehome]

    debug("Exit clear_WASsecurity")


def isWASPortal(profile_home_dir):
    '''
        checks is profile home is a WebSphere Portal Installation
    '''

    debug("Entry isWASPortal %s" % profile_home_dir)

    isPortal = False
    if os.path.exists(os.path.join(profile_home_dir, "ConfigEngine")):
        isPortal = True

    debug("Exit isWASPortal %s" % isPortal)
    return isPortal

def getPortalVersion(profile_home_dir):
    '''
        returns version string from wps.properties file
    '''
    debug("Entry getPortalVersion entry %s - %s " % (fixlev,profile_home_dir))
    properties_file = os.path.normpath(os.path.join(profile_home_dir, "PortalServer",  "wps.properties"))
    debug("checking file %s" % properties_file)
    version = None
    if os.path.isfile(properties_file):
        filh = open(properties_file, "r")
        for line in filh.readlines():
            if not line.strip(): continue
            line = line.strip()
            if line.startswith("version"):
                debug("line=%s" % line)
                version = line.split("=")[1]
                version = version.strip()
                break
        filh.close()
    else:
        debug("Warning: file not found")
    
    debug("Exit getPortalVersion %s" % version)
    return version

def getPortalVersion(profile_home_dir):
    '''
        returns version string from wps.properties file
    '''
    debug("Entry getPortalVersion entry %s - %s " % (fixlev,profile_home_dir))
    properties_file = os.path.normpath(os.path.join(profile_home_dir, "PortalServer",  "wps.properties"))
    debug("checking file %s" % properties_file)
    version = None
    if os.path.isfile(properties_file):
        filh = open(properties_file, "r")
        for line in filh.readlines():
            if not line.strip(): continue
            line = line.strip()
            if line.startswith("version"):
                debug("line=%s" % line)
                version = line.split("=")[1]
                version = version.strip()
                break
        filh.close()
    else:
        debug("Warning: file not found")
    
    debug("Exit getPortalVersion %s" % version)
    return version


def getDefaultPortalSoapPort(profile_home_dir):
    '''
        returns Portal server soap port.
        By default wsadmin connects to server1 in wp_portal profile
        need to force it to connect to the Portal Server
    '''

    debug("Entry getDefaultPortalSoapPort entry %s - %s " % (fixlev,profile_home_dir))

    properties_file = os.path.normpath(os.path.join(profile_home_dir, "ConfigEngine", "properties", "wkplc.properties"))
    debug("checking file %s" % properties_file)
    soap_port = None
    if os.path.isfile(properties_file):
        filh = open(properties_file, "r")
        for line in filh.readlines():
            if not line.strip(): continue
            line = line.strip()
            if line.startswith("WPS_SOAP_PORT"):
                debug("line=%s" % line)
                soap_port = line.split("=")[1]
                soap_port = soap_port.strip()
                break
        filh.close()
    else:
        debug("Warning: file not found")

    debug("Exit getDefaultPortalSoapPort %s" % soap_port)
    return soap_port


###############
# Unit Testing
###############

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL

    log.set_logging("./wasdiscover.log")
    pdl = createPDL.make_pdl(createPCL.make_pcl())

    was_home_list = []
    log.set_logging()
    was_home_list = discover_was()
    for was_dir in was_home_list:
        print was_dir
        profile_name_list, profile_path_list, default_profile_name = get_was_profiles(pdl, was_dir)
        print "Profile name list: %s" % profile_name_list
        print "profile path list: %s" % profile_path_list
        print "default_profile_name: %s" % default_profile_name
