#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Utilities to assist in getting WLP configuration parameters
'''

fixlev = "41080" # Last APAR, defect or story to update this module

import os
import sys
import codecs
import re
from time import strftime
import shutil
import socket

import utils
import platformUtil
from resourcebundle import getmsg as getmsg
from debugUtil import debug as debug
import debugUtil


def getWlpUserDir(wlpInstallDir):
    '''
        get WLP_USER_DIR for the WLP_INSTALL_DIR
        unless <wlp_install_dir>/etc/server.env has WLP_USER_DIR variable set
        WLP_USER_DIR is defaulted to <wlp_install_dir>/usr
    '''

    debug("Entry getWlpUserDir %s -- %s" % (fixlev, wlpInstallDir))

    wlpUserDirKey = "WLP_USER_DIR"
    wlpUserDir = wlpInstallDir + os.sep + "usr"
    globalServerEnv = wlpInstallDir + os.sep + "etc" + os.sep + "server.env"
    if os.path.isfile(globalServerEnv):
        var_names_list, var_values_list = readPropertiesFile(globalServerEnv)
        if wlpUserDirKey in var_names_list:
            wlpUserDir = var_values_list[var_names_list.index(wlpUserDirKey)]

    debug("Exit getWlpUserDir %s" % wlpUserDir)
    return wlpUserDir


def getWlpServersList(wlpInstallDir):

    server_list = []
    usrDirPath = getWlpUserDir(wlpInstallDir)
    serversDirPath = usrDirPath + os.sep + "servers"
    if os.path.isdir(serversDirPath):
        dirList = os.listdir(serversDirPath)
        for entry in dirList:
            debug("Getting liberty servers from %s" % serversDirPath)
            server_xml_file = serversDirPath + os.sep + entry + os.sep + "server.xml"
            if os.path.exists(server_xml_file):
                isConfigured = "None"
                itcam_dcversion = "None"
                server_list.append(entry)

    return server_list


def getEnvVar(usrDir, serverName, varName):

    debug("Entry getEnvVar %s - %s %s %s" % (fixlev, usrDir, serverName, varName))

    serverConfigDir = usrDir + os.sep + "servers" + os.sep + serverName
    envFileName = serverConfigDir + os.sep + "server.env"
    debug ("reading environment entries from file %s" % envFileName)
    var_names_list = []
    var_values_list = []
    varValue = None
    if os.path.exists(envFileName):
        var_names_list, var_values_list = readPropertiesFile(envFileName)
    else:
        debug("File %s doesn't exist" % envFileName)
    if varName in var_names_list:
        varValue = var_values_list[var_names_list.index(varName)]
    debug("getEnvVar exit %s" % varValue)

    return varValue


def getEnvVarList(usrDir, serverName, wlpInstallRoot=None):

    debug("Entry getEnvVarList %s - %s %s" % (fixlev, usrDir, serverName))

    serverConfigDir = usrDir + os.sep + "servers" + os.sep + serverName
    envFileName = serverConfigDir + os.sep + "server.env"

    debug ("reading environment entries from file %s" % envFileName)
    var_names_list = []
    var_values_list = []
    varValue = None
    if os.path.exists(envFileName):
        var_names_list, var_values_list = readPropertiesFile(envFileName)
    else:
        debug("File %s doesn't exist, checking server.env under etc" % envFileName)
        if wlpInstallRoot != None:
            etc_envFileName = wlpInstallRoot + os.sep + "etc" + os.sep + "server.env"
            if  os.path.exists(etc_envFileName):
                debug("Found server.env %s" % etc_envFileName)
                var_names_list, var_values_list = readPropertiesFile(etc_envFileName)

    debug("Exit getEnvVarList %s" % var_names_list)
    return var_names_list, var_values_list


def readPropertiesFile(fileName):

    debug("Entry readPropertiesFile %s - %s" % (fixlev, fileName))

    var_names_list = []
    var_values_list = []
    if os.path.exists(fileName):
        infile = None
        try:
            infile = codecs.open(fileName, mode="r", encoding="utf-8")
            lines = ""
            for line in infile.readlines():
                debug ("line: %s" % line)
                line = line.rstrip("\r\n")
                if len(line.strip()) == 0: continue
                if line.startswith("#"):
                    var_name = line
                    var_value = ""
                else:
                    ws = line.split("=", 1)
                    if len(ws) < 2:
                        debug("Invalid entry in server.env %s" % line)
                        continue
                    var_name = ws[0]
                    var_value = ws[1]
                var_names_list.append(var_name)
                var_values_list.append(var_value)

        finally:
            if infile != None:
                infile.close();
                debug("finished reading WLP properties")
    else:
        debug("File %s doesn't exist" % fileName)

    debug("Exit readPropertiesFile %s" % str(var_names_list))
    return var_names_list, var_values_list


def writeDataToFile(fileName, lineList, removeIfEmpty=False):

    debug("Entry writeDataToFile() %s - %s " % (fixlev, fileName))

    fileh = None
    result = False
    try:
        if len(lineList) == 0 and removeIfEmpty:
            debug("no data to write into file %s, removing" % fileName)
            os.remove(fileName)
        else:
            fileh = codecs.open(fileName, mode="w", encoding="utf-8")
            debug("opened file %s for writing" % fileName)
            for line in lineList:
                debug("writing line: '%s'" % line)
                fileh.write(line)
                fileh.write(os.linesep)
                #fileh.flush()
        result = True
    except Exception, err:
        result = False
        print "Unable to write into file %s" % err
        raise RuntimeError, "Unable to write data into a file: '%s'" % err
    finally:
        if fileh != None:
            #fileh.flush()
            fileh.close()

    return result


def create_file_backup_copy(fileName, suffix = None):
    '''
        create a copy on an original file
    '''

    debug("Entry create_backup_file %s (%s)" % (fixlev, fileName))

    # This code is duplicated in debugUtil.py to avoid having to make util.py
    # compatible with Jython 2.1.  Changes made there should also be done here.

    fileName = os.path.abspath(fileName)
    dirName = os.path.dirname(fileName)
    rename_filename = None
    if os.path.exists(fileName):
        debug("File %s exists backup will be created" % fileName)

        # create a string of local time to be inserted in the
        # renamed log file if exists.

        if suffix == None:
            ltime = strftime("%Y%m%d%H%M%S")
            suffix = ltime

        # Get the file name and insert the current time
        # in the file name for rename.

        basename = os.path.basename(fileName)

        # This check is for Jython 2.1, it is not needed for 2.5.2 where
        # os.path.splitext() works as desired.
        if basename.find(".") == 0 and basename[1:].find(".") == -1:
            filebase, fileext = (basename,"")
        else:
            filebase, fileext = os.path.splitext(basename)
        logf = filebase + "." + suffix + fileext
        debug("Existing file will be renamed to %s" % logf)

        #
        # Rename the current log file to the new name.
        #
        rename_filename = dirName + os.sep + logf
        try:
            shutil.copy2(fileName, rename_filename)
            debug("Rename was successful")
        except OSError, errarg:
            debug("Could not rename log %s - %s" % (fileName, errarg))
            raise Exception

    else:
        debug("No previous existing file %s, rename not needed" % fileName)


    debug("Exit rename_archival_file(), new file '%s'" % rename_filename)
    return rename_filename


def create_backup_file(fileName):
    '''
        create a copy on an original file
    '''

    debug("Entry create_backup_file %s (%s)" % (fixlev, fileName))

    # This code is duplicated in debugUtil.py to avoid having to make util.py
    # compatible with Jython 2.1.  Changes made there should also be done here.

    fileName = os.path.abspath(fileName)
    dirName = os.path.dirname(fileName)
    rename_filename = None
    if os.path.exists(fileName):
        debug("File %s exists backup will be created" % fileName)

        # create a string of local time to be inserted in the
        # renamed log file if exists.

        ltime = strftime("%Y%m%d%H%M%S")

        # Get the file name and insert the current time
        # in the file name for rename.

        basename = os.path.basename(fileName)

        # This check is for Jython 2.1, it is not needed for 2.5.2 where
        # os.path.splitext() works as desired.
        if basename.find(".") == 0 and basename[1:].find(".") == -1:
            filebase, fileext = (basename,"")
        else:
            filebase, fileext = os.path.splitext(basename)
        logf = filebase + "." + ltime + fileext
        debug("Existing file will be renamed to %s" % logf)

        # Rename the current log file to the new name.

        rename_filename = dirName + os.sep + logf
        try:
            shutil.copyfile(fileName, rename_filename)
            debug("---------- Copied file %s to %s" % (fileName, rename_filename))
        except OSError, errarg:
            debug("Could not rename log %s - %s" % (fileName, errarg))
            raise Exception

    else:
        debug("No previous existing file %s, rename not needed" % fileName)

    debug("Exit create_backup_file, new file '%s'" % rename_filename)
    return rename_filename


def restoreFileFromBackup(bak_fileName, fileName):

    debug("Entry restoreFileFromBackup %s - %s %s" % (fixlev, bak_fileName, fileName))

    if bak_fileName != None and os.path.isfile(bak_fileName):
        shutil.copy(bak_fileName, fileName)
        debug("restored original file %s from backup %s" % (fileName, bak_fileName))
    else:
        debug("backup file %s not found" % bak_fileName)


def displayJvmArgs (title, jvmargs):

    if title != "":
        if jvmargs == "":
            debug("%s: (none)" % title)
            return
        debug("%s: " % title)
    hit_flag = 0
    for arg in jvmargs:
        if arg == "":  continue
        hit_flag = 1
        if arg.find(":") == arg.rfind(":") :
            debug("\t%s" % arg)
        else:
            args = arg.split(":")
            argi = 0  #forStart
            while  argi < len(args)  :  #forTest
                debug("%d %s" % (argi, args[argi]))
                argi += 1  #forNext
    if  not hit_flag: debug("  (none)")


def mergePaths ( basePaths, newPaths ):
    '''
        Procedure  mergePaths
        Arguments: base classpath
                   new classpath
        Returns  Updated classpath string

        Given a new classpath and an old classpath, merges the
        new paths into the old paths, with the new paths before the
        existing paths.

        Note: Empty braces may have been introduced in previous configuration
    '''

    debug("Entry mergePaths %s" % fixlev)

    if basePaths != None:
        basePaths = basePaths.strip()
    if newPaths != None:
        newPaths = newPaths.strip()
    debug("Base:   %s" % basePaths)
    debug("New:    %s" % newPaths)

    results = []
    used = []
    # Put in the new path entries first, which includes the option keyword
    for path in split_path(newPaths):
        if path.strip( ) == "": continue
        if path.strip( ) == "{}": continue
        debug("New path entry: %s" % path)
        used.append(path)
        results.append(path)

    # Now put in all base path entries which are not duplicate, ITCAM product, or obsolete
    # Note that removing the duplicates will handle the keyword option, if present
    for path in split_path(basePaths):
        if path.strip( ) == "":  continue
        if path.strip( ) == "{}":  continue
        debug("Base path entry: %s" % path)

        # Do not add entries we have already added
        if path in used:
            debug("Duplicate entry %s dropped." % path)
            continue
        if path.find("${ITCAMDCHOME}") != -1:
            debug("ITCAM product entry dropped.")
            continue

        results.append(path)

    results = join(results, ":")
    result = results

    debug("Exit mergePaths = %s " % results)
    return result


def getArg ( arg, argList ):
    '''
        Procedure  getArg
        Arguments: arg
                   argList
        Returns argument from list if present, "" if not
    '''

    debug("Entry getArg %s arg='%s'" % (fixlev, arg))

    retArg = ""
    for argument in argList:
        debug("\targument=%s" % argument)
        if argument.find(arg) == 0:
            retArg = argument
            break

    debug("Exit getArg input arg='%s' return arg='%s'" % (arg, retArg))
    return retArg


def diffJvmArgs ( exist_opts, minus_opts ):
    '''
        Procedure  diffJvmArgs
        Arguments:

        Find the difference between two strings.  That is, remove every word
        found in the second string from the first string.

        Note that debug tracing is commented out in this routine
        because the voluminous output obscures the flow of code
        more likely to fail.  This low-level routine was debugged
        and tested outside the wsadmin environment.
    '''

    debug("Entry diffJvmArgs() %s" % fixlev)
    debug(" current: %s" % exist_opts)
    debug(" minus: %s" % minus_opts)

    extras = []
    for item in exist_opts:
        for olditem in minus_opts:
            if item == olditem:
                item = ""
                break
        if item != "": extras.append(item)

    debug(" diffJvmArgs() result = %s" % diff)
    return diff


def split_path(path_str):
    '''
        splits path by unix or windows style separator
        replacement for split(";:") in jacl
    '''

    rex = re.compile("[:]")
    if platformUtil.isWindows():
        rex = re.compile("[;]")

    return rex.split(path_str)


def regexp(pattern, string, flags=0):

    if re.compile(pattern, flags).search(string)==None: return 0
    else: return 1


def regexpn(pattern, string, flags=0):

    r = re.compile(pattern, flags).subn("", string)

    return r[1]


def join(lst, separator=" "):

    strg = ""
    for element in lst:
        strg += str(element) +separator
    strg = strg[0:len(strg)-len(separator)]

    return strg


def unmergePaths ( currentPaths, addedPaths ):
    '''
        Procedure  unmergePaths
        Arguments: existing paths
                   added paths
        Returns  Updated classpath string

        Given an existing classpath, removes paths provided in
        the addedPath parameter.

        Note: Empty braces may have been introduced in previous configuration
    '''

    debug("Entry unMergePaths %s" % fixlev)
    debug("Current %s" % currentPaths)
    debug("Added: %s" % addedPaths)

    # Build a lookup array of the added paths
    used = []
    for path in split_path(addedPaths):
        if path.strip( ) == "": continue
        if path.strip( ) == "{}": continue
        debug(" Path entry: %s " % path)
        used.append(path)

    results = []

    # Remove added paths, ITCAM product paths, and obsolete paths
    debug("Current paths: %s " % currentPaths)

    for path in split_path(currentPaths):
        if path.strip( ) == "": continue
        if path.strip( ) == "{}": continue
        debug("Path entry: %s" % path)
        if path in used:
            debug("Added entry dropped.")
            continue

        if path.find("${ITCAMDCHOME}") != -1:
            debug("ITCAM product entry dropped.")
            continue

        results.append(path)
    result = results

    debug("Exit unmergePaths %s" % result)
    return results


def getNodeText(nodelist):

    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)

    return "".join(rc)


def getCellName():
    '''
        returns cell name (usually set to hostname
    '''

    debug("Entry getCellName %s" % fixlev)

    cellname = "liberty"
    try:
        hostname= socket.gethostname()
        debug("hostname=%s" % hostname)
        if hostname != None and len(hostname) > 0:
            cellname=hostname
        else:
            cellname=socket.gethostbyaddr(hostname)[0]
        debug("cell name=%s" % cellname)
    except:
        debug("Unable to determine hostname, using 'liberty' as cellname")
    cellname = cellname.split(".")[0]

    debug("Exit getCellName %s" % cellname)
    return cellname


def getNodeName(wlpInstallRoot):
    '''
        returns node name from wlp install path
        convert path into a nodename, replace space with _
    '''

    debug("Entry getNodeName(%s) %s" % (wlpInstallRoot, fixlev))
    
    # In case some smart alec uses forward slashes on Windows

    # Protect this code from Jython 2.1, where os.altsep is None
    assert sys.version_info >= (2, 5), getmsg("CFG5022E", "Internal error in {0}", ["wlpConfigUtils.getNodeName"])
    wlpInstallPath = os.path.splitdrive(wlpInstallRoot)[1]
    if os.altsep is not None and os.altsep != os.sep:
        wlpInstallPath = wlpInstallPath.replace(os.altsep, os.sep)
    nodename = ".".join([node for node in wlpInstallPath.split(os.sep) if node != ""]).replace(" ","_").strip(".")

    debug("Exit getNodeName, result = %s" % nodename)
    return nodename

def get_feature_name_version(feature):
    debug("get_feature_name_version Entry %s" % feature)
    props = feature.split("-", 1)
    if len(props) == 2:
        feature_name = props[0]
        feature_version = props[1]
    else:
        debug("Invalid feature definiotion, no version: %s" % current_feature)
        feature_name = props[0]
        feature_version = -1
    if feature_name.find(":") > -1:
        feature_name = feature_name.split(":", 1)[1]
        debug("striping out prefix: %s" % feature_name)
    debug("get feature_name_version Exit %s, %s" % (feature_name, feature_version))
    return feature_name, feature_version
