#####################################################################
# Licensed Materials - Property of IBM
#
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module upgrades previous AD releases (7.1) to new AD release.
# There is a version.properties which includes the version number for
# install.  The version is checked against the current release, and it must
# be a lower release than the current one, otherwise, upgrade does not apply.
# Once the release is valid, all the required PDL is created, When the PDL is
# created, a regular configuration is implemented.
#
#####################################################################

import os, sys, traceback

# Eliminate need to prefix import modules with "lib."
sys.path.append("lib")

import createPCL_config as createPCL
import createPDL
import tcpipstack
import resourcebundle
import debugUtil
import checkversion
import findconfigservers
import inputproperties
import configplugin
import createDCInput
import configDC
import migratecustomfiles
import utils
import platformUtil
import configzos
import configtema
import installSRVPGMs
import configUtils
import codecs
import doprompt as prompt
import silentInputFileProcessor
import get_wasroot
import populateFindServers
import configms
import configtt
import configsoa
import configsmf
import configgc
import wasbackup
import display_config
import configmaxheap
import createservers
import configtpv
import configde
import migrate_unconfigservers
import debugUtil as log
import configproxy

from ConfigurationException import ConfigurationException
from resourcebundle import getmsg as getmsg
from constants import constants
from debugUtil import logDebug as debug
from ConsoleWriter import ConsoleWriter 
from PrintWrapper import PrintWrapper


product = "ITCAM/AD 7.3"
fixlev = "HAAD730 AK00000"



def migrate_soa71(pdl):
    '''
       Migrate SOA71.  This migration processing for SOA71 is different than other
       migration, as it will be configured as a regular configuration by asking the
       questions, because SOA71 is only a plugin.

       @param pdl - PDL reference
    '''

    prompt.do_prompt("MIGRATESOA71WASROOT", pdl)
    mdata = utils.getDataString(pdl, "MIGRATESOA71WASROOT")
    utils.setData(pdl, "WASROOT", mdata, "migrate")
    get_wasroot.get_wasroot(pdl, True)
    server_found = populateFindServers.populateFindServers(pdl)
    if (server_found):
        #
        # At this point we need a loop to allow reprompting after the summary is displayed.
        #
        while True:
            # Create servers
            createservers.process_servers(pdl)
            # Configure SOA DC (not for OS400)
            if not platformUtil.IS_OS400_OS():
                configsoa.config_soa(pdl, True)

            # Configure Tema
            configtema.config_tema(pdl, True)

            # Configure Managing Server
            configms.config_ms(pdl, True)
            

            # Config TTAPI
            continue_config = configtt.config_tt(pdl, True)
            if not continue_config:
                raise ConfigurationException(getmsg("CFG1998I", "Configuration of Data Collector has been bypassed"))

            # Configure SMF
            configsmf.config_smf(pdl, True)
            
            # Configure TPV
            configtpv.config_tpv(pdl, True)
            
            # Configure DE
            configde.config_de(pdl, True)

            # Configure GC
            configgc.config_gc(pdl, True)

            # Configure max heap
            # configmaxheap.config_maxheap(pdl, True)

            # Display a configuration summary page and prompt for next action.
            doConfig = display_config.display(pdl)
            if doConfig:
                break

        # Dump the contents of the now-completed PDL.
        log.logDumpPDL(pdl,"findServers has been applied to the PDL", findservers=False)

        #Prompt for Websphere backup and backup current configuration if requested
        continue_config = wasbackup.do_backup(pdl)

        if not continue_config:
            raise ConfigurationException(getmsg("CFG1998I", "Configuration of Data Collector has been bypassed"))

        ###############################################################################
        # At this point the data in the PDL should be complete.  We are ready to start
        # creating the runtimes.
        ###############################################################################

        # If z/OS, execute platform specific configuration tasks
        if platformUtil.IS_zOS_OS():
            configzos.config_zos(pdl)

        # Config plugin
        configplugin.process_plugin(pdl)


        # The runtime directory will be created at DC startup, based on the manual inputs file.
        createDCInput.createManualInputFile(pdl)

        # Create input.properties files
        inputproperties.process_inputproperties(pdl)

        # Call configDataCollector if not bypassed and no return code has been set
        assert isinstance(pdl["RETURNCODE"][0],int), "Invalid return code has been set"
        if not pdl["BYPASSCONFIG"][0] and pdl["RETURNCODE"][0] == 0:
            configDC.configDataCollector(pdl)
        else:
            print "%s" % getmsg("CFG1132I", "Migration of Data Collector has been bypassed at user's request")


    sys.exit(0)
    
    
def get_install_history_file():
    '''
        Return the path/name of the file in which GDC install history is saved
    '''
    debug("get_install_history_file entry")

    if platformUtil.isWindows():
        if pdl["CONFIGHOME"][0] is None:
            candleHome = "C:/IBM/ITM"
        else:
            configHome = pdl["CONFIGHOME"][0]
            dcHome = os.path.join(configHome, os.pardir)
            candleHome = os.path.join(dcHome, os.pardir)
    else:
        candleHome = os.getenv("CANDLE_HOME")
        if (candleHome == None or len(candleHome) == 0):
            candleHome = "/opt/ibm/ccm/agent/"

    file = "gdc_install_history.txt"
    if platformUtil.isWindows():
        path = "TMAITM6"
    else:
        path = "config"

    debug("get_install_history_file exit %s" % os.path.join(candleHome, path, file))
    return os.path.join(candleHome, path, file)

def read_install_history(history_file, pdl):
    '''
        Returns a list of DC installation directories from the gdc_install_history file
        Current DC home is not included
        Doesn't check if DC installations are valid
    '''
    dc_home_list = []
    debug("read_install_history entry %s" % history_file)
    #Current DC intallation directory
    confighome = utils.getData(pdl, "CONFIGHOME")
    dchome_path, current_version = os.path.split(confighome)
    if os.path.isfile(history_file):
        try:
            fdescr = open(history_file, "r")
            lines = []
            while 1:
                buffer = fdescr.readline()
                if not buffer: break
                if len(buffer.strip()) > 0:
                    lines.append(buffer)
                
            fdescr.close()
            for line in lines:
                dir_name = line.split('\t')[0].strip()
                version = line.split('\t')[1].strip()
                dc_home_path = os.path.join(dir_name, version)
                dc_home_path = os.path.normpath(dc_home_path)
                debug("Found dc at %s" % dc_home_path)
                if dc_home_path not in dc_home_list and version != current_version:
                    dc_home_list.append(dc_home_path)
                else:
                    debug("skipping directory %s" % dc_home_path)
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            debug("Failed to read from the history file: %s. Error %s" % (history_file, errmsg))

   
    debug("dc home list %s" % str(dc_home_list))
    return dc_home_list    
    

    
try:

    #############
    # Initialize
    #############
    regular_process = True
    
    #
    # Instantiate the global constants class
    #
    common_constants = constants()
    
    #
    # Set the print wrapper for stdout and stderr.
    #
    PrintWrapper(common_constants.STDOUT)
    PrintWrapper(common_constants.STDERR)

    debug("Initialization of "+product+" "+fixlev)

    #
    # Initialize the resource bundle
    #
    resourcebundle.set_resourcebundle()

    #
    # Constants for the migrate display.  The order should not be changed, but
    # if it there is a need to change the order, the variable SOA71_RESP should
    # also be changed to reflect the index.
    #
    MIGRATE_PRODUCTS = [getmsg("MIGRATE_DC61", "ITCAM for WebSphere 6.1  (fix pack 4 or later)"),
                        getmsg("MIGRATE_WR62", "ITCAM WebSphere Agent 6.2  (fix pack 4 or later) [ITCAM for Web Resources 6.2]"),
                        getmsg("MIGRATE_AD71", "ITCAM Agent for WebSphere Applications 7.1 [ITCAM for Application Diagnostics 7.1]"),
                        getmsg("MIGRATE_AD72", "ITCAM for WebSphere Application Server 7.2"),
                        getmsg("MIGRATE_SOA711", "ITCAM for SOA 7.1.1"),
                        getmsg("MIGRATE_ITCAM72", "Maintenance level update for ITCAM Data Collector for WebSphere 7.2 and later [ITCAM Agent for WebSphere Applications 7.2 and later, ITCAM for SOA 7.2 and later, IBM Application Performance Diagnostics Lite]")
                       ]

    MIGRATE_PRODUCTS_I5 = [getmsg("MIGRATE_DC61", "ITCAM for WebSphere 6.1  (fix pack 4 or later)"),
                           getmsg("MIGRATE_AD71", "ITCAM Agent for WebSphere Applications 7.1 [ITCAM for Application Diagnostics 7.1]"),
                           getmsg("MIGRATE_AD72", "ITCAM for WebSphere Application Server 7.2")
                          ]
    SOA71_RESP = "5"
    MIGRATE_ITCAM72 = "6"

    ############################
    # Display a startup message
    ############################

    print
    print
    print "%s" % getmsg("FLOWERBOX_MIGRATE","Migrate Data Collector")
    print


    #
    # Create the data structures
    #
    pdl = createPDL.make_pdl(createPCL.make_pcl())
    debugUtil.logDumpPCL(pdl["PCL"][0], "Initial creation of PCL with %d entries" % len(pdl["PCL"][0]))
    debugUtil.logDumpPDL(pdl, "Initial creation of PDL with %d entries" % len(pdl), short=False)

    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    debugUtil.logDumpPDL(pdl, "PDL after environment variables and command line arguments")
   
    # Perhaps all we are going to do is display the help text.
    if pdl["HELP"][0]:
        #debugUtil.set_logging(False)
        print "%s" % getmsg("HELP_MIGRATE")
        print
        sys.exit(0)
     
    # Default log and message file names
    if pdl["LOG"][0] is None:
        pdl["LOG"][0:2] = "migrate-trace.log", "migrate"
    if pdl["MESSAGE"][0] is None:
        pdl["MESSAGE"][0:2] = "migrate-message.log", "migrate"

    # Default and validate the config home.
    if pdl["CONFIGHOME"][0] is None:
        cwd = os.getcwd()
        if cwd.endswith("/bin"):
            config_home = cwd.replace("/bin","")
            pdl["CONFIGHOME"][0:2] = config_home, "migrate.py"
        else:
            print "%s" % getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home")
            sys.exit(1)

    # Initialize the log utility and interrogate version.properties.
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
    if log.isLogBad():
        print "%s" % getmsg("CFG5027W", "Unable to open log file {0}, diagnostic output will be directed to the console.", [pdl["LOG"][0]])
    else:
        # Redirect sys.stdout
        if pdl["LOGCONSOLE"][0] is True:
            log_file = debugUtil.getLogFilename()
            log_dir = os.path.dirname(log_file)
            console_wtr = ConsoleWriter(log_dir + os.sep + "migrate-console.log" )
            # Start with flower box printed above 
            console_wtr.log("%s" % getmsg("FLOWERBOX_MIGRATE","Migrate Data Collector"))
    configUtils.logVersionProperties(pdl)
    
    print "%s" % getmsg("CFG5043I", "Log file name: {0}", [log.getLogFilename()])

    #
    # Silent file not used in migrate
    #
    if pdl["SILENT"][0]:
        debug("Silent migration file is supplied: %s" % pdl["SILENT"][0])
        silentInputFileProcessor.processFile(pdl,pdl["SILENT"][0],4)
    #
    #Determine type of install we are dealing with
    #
    ccm_install = utils.is_ccm(pdl)

    #
    # Determine the local host name
    #
    tcpipstack.get_iphost(pdl)
    #
    # Indicate in DL that this is migrate processing
    #
    utils.setData(pdl, "MIGRATE", True, "migrate")
    #
    # Get the current version of this release, and save it.
    #
    version = checkversion.get_current_version(pdl)
    assert version != -1, getmsg("CFG1070E", "Invalid version number  - {0}", [version])
    utils.setData(pdl, "VERSION", version, "migrate")
    #
    # Get the current version of liberty feature
    #
    feature_version = checkversion.get_feature_version(pdl)
    if feature_version == "-1":
        raise ConfigurationException(getmsg("CFG1070E", "Invalid liberty feature version number - {0}", [feature_version]))
    pdl["ITCAMFEATUREVERSION"][0:2] = feature_version, "migrate"
    #
    # Prompt to choose the product to migrate.  For OS400, SOA71 is not
    # supported .  Do not change the positions of the messages as we are
    # specifically looking for an soa711 index.
    #
    if platformUtil.IS_OS400_OS():
        migrate_products = MIGRATE_PRODUCTS_I5
    else:
        migrate_products = MIGRATE_PRODUCTS

    if ccm_install == False:
        migrate_resp = prompt.prompt_list_no_pdl(pdl,
                                             migrate_products,
                                             getmsg("CFG1130I", "List of ITCAM agents whose data collector can be upgraded to the ITCAM Data Collector  for WebSphere 7.2:"))
    else:
        #Defaulting to ITCAM73
        migrate_resp = MIGRATE_ITCAM72
    migrate_resp = utils.getData(pdl, "LISTRESP")
    if (migrate_resp == SOA71_RESP):
        debug("Migrating SOA71...")
        regular_process = False
        migrate_soa71(pdl)
    else:
        debug("Migrating other products but SOA71...")

        #
        # Prompt the user to provide the upgrade location, and check its
        # version and making sure it is a valid upgrade version
        #
        dc_home_list = None
        print_error = True
        if ccm_install == True:
            #read gdc_install history file and get a list of installed data collectors
            gdc_install_history_file_path = get_install_history_file()
            dc_home_list = read_install_history(gdc_install_history_file_path, pdl)
            pdl["CCMDCHOMES"][0:2] = dc_home_list, "migrate"
            print_error = False
            
        #version_checked, migrate_version = checkversion.check_migrate_version(pdl, dc_home_list)
        migratehome_dic = checkversion.check_migrate_version(pdl, dc_home_list, print_error)
            
        #
        # Once the version checking is done, the configured servers are
        # checked from the upgrade home, and will be displayed for the
        # user to choose for upgrade.
        # This is done based on AD 7.1 or 7.2.  In AD 7.1, the list
        # of configured servers are in a different location.
        #
        if len(migratehome_dic) > 0:
            debug("migrate homes found=%s" % migratehome_dic.keys())
            if ccm_install:
                pdl["CCMMIGRATEHOMES"][0:2] = migratehome_dic, "migrate"
                serversfound = findconfigservers.find_configservers_for_migrate_73(pdl, common_constants.MIGRATE, None, migratehome_dic)
                migrate_version = migratehome_dic.values()[0]
            else:
                for migratehome in migratehome_dic.keys():
                    debug("migratehome=%s" % migratehome )
                    pdl["MIGRATEHOME"][0:2] = migratehome, "migrate"
                    debug("migratehome in pdl=%s" % utils.getData(pdl, "MIGRATEHOME"))
                    migrate_version = migratehome_dic[migratehome]
                    serversfound = findconfigservers.find_configservers(pdl, common_constants.MIGRATE, None, migrate_version)
            debug("servers found=%s" % pdl["SERVERS"][0])
            if (serversfound == True):
                #
                # Configure Tema
                #
                configtema.config_tema(pdl)
                #
                # Config plugin
                #
                configplugin.process_plugin(pdl)

                # If zOS, execute platform specific configuration tasks.
                if platformUtil.IS_zOS_OS():
                    configtt.config_tt_zOS(pdl)
                    configzos.set_jvmmode(pdl)
                    configzos.config_zos(pdl)

                #
                # The runtime directory will be created at the time DC starts.
                #

                ####################################
                # Install SRVPGMs if this is OS/400
                ####################################

                try:
                    if platformUtil.IS_OS400_OS():
                        srvpgmRC = installSRVPGMs.installSRVPGMs()

                        if srvpgmRC > 0:
                            raise SystemExit

                except SystemExit:
                    raise SystemExit

                except:
                    raise SystemExit("An error occurred while trying to install the required Server programs. Please check the logs for more details.")

                createDCInput.createManualInputFile(pdl)
                #
                # Create input.properties files
                #
                inputproperties.process_inputproperties(pdl)
                #
                # Call configDataCollector to config the app server
                #
                
                configDC.configDataCollector(pdl)
                #
                # Check the RETRUNCODE pdl and if it is a not zero, then do the
                # clean up, as the config did not execute successfully, otherwise, copy
                # the custom directory property files from the old release to the
                # new one.
                #
                
                
                rc = pdl["RETURNCODE"][0]
                if (rc == 0):
                    #configure port consolidator scripts if there is an MS connection
                    configproxy.config_proxyserverctrl_script(pdl, False)
                    
                    #
                    # Unconfig the migrated config files.
                    #
                    unconfigured = migrate_unconfigservers.unconfig_servers(pdl, migrate_version)
                    if ccm_install:
                        copied = migratecustomfiles.copy_files_smai(pdl)
                    else:
                        copied = migratecustomfiles.copy_files(pdl, migrate_version)
                    if (copied == True and unconfigured == True):
                        print "\n%s\n" % getmsg("CFG2005I","Migration of the Data Collector has successfully completed with return code {0}.",(pdl["RETURNCODE"][0],))
                    else:
                        print "\n%s\n" % getmsg("CFG2007E","Copying the custom files or migrate unconfiguring failed, Migration of the Data Collector did not completed successfully.")
                        print "\n%s\n" % getmsg("CFG2006E","Migration of the Data Collector did not completed successfully with return code {0}.",(pdl["RETURNCODE"][0],)) 
                else:
                    print "\n%s\n" % getmsg("CFG2006E","Migration of the Data Collector did not completed successfully with return code {0}.",(pdl["RETURNCODE"][0],))
            else:
                print "\n%s\n" % getmsg("CFG1083I","There are no servers to migrate")
        else:
            debug("Invalid migrate home provided")

    # If z/OS, update access permisssions and ownership
    if platformUtil.IS_zOS_OS():
        configUtils.update_runtime_access_zOS(pdl)

#former assert, unrecoverable error caused by environment settings or input parameters
#fixable by user, not a program error, error text goes to the console
except ConfigurationException, (instance):
    print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["migrate.py"])
    print "%s" % instance.parameter #print exception message
    sys.exit(1)

except AssertionError:

    print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["migrate.py"])
    try:
        debugUtil.logException("Assertion error in migrate.py")
        debugUtil.logDumpStack("Stack data for diagnostic purposes")
    except:
        _type, _val, _tb = sys.exc_info()
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
        traceback.print_exception(_type,_val,_tb)
    sys.exit(1)

except KeyboardInterrupt:   #User entered terminate character in prompt
    print "%s" % getmsg("CFG1133I", "Exiting the ITCAM Data Collector for WebSphere Migration Utility.")
    sys.exit(0)

except SystemExit:   # Generic exit (after an exception was already caught and handled)
    print "%s" % getmsg("CFG1133I", "Exiting the ITCAM Data Collector for WebSphere Migration Utility.")
    sys.exit(1)

except:
    _exctype, _excval, _exctb = sys.exc_info()

    try:
        cfg1002E_msg = getmsg("CFG1002E", "Unexpected exception detected in migrate.py")
    except:
        cfg1002E_msg = "Unexpected exception detected in migrate.py"
    print "\n%s\n" % cfg1002E_msg

    try:
        debugUtil.logException('Error in main routine')
    except:
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
    traceback.print_exception(_exctype,_excval,_exctb)
    print
    sys.exit(1)

