#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This is a top level script whose purpose is to unconfigure a Data
# Collector.  The steps are as follows:
#
#   1) Create the PCL and PDL to hold the persistent data.
#      The PCL is the data dictionary.  It holds the name,
#      description, initial value and prompt message key for
#      all common data items.
#
#   2) Apply any relevant environment variables to the PDL.
#      For each data item there may be zero or more environment
#      variables which may set the value of the data item.
#
#   3) Interpret the command line flags and put the results in the PDL.
#      For each data item there may be zero or more flags (command
#      line options) which set the value of the data item.
#
#   4) [todo] Handle the silent install file.
#
#   5) Make various inquiries to the operating system and prompts to
#      the user to set the final state of the PDL, such as running
#      findServers.  Routines which update the value of data items
#      in the PDL are also responsible for setting a tag to indicate
#      the source of the data item value.
#
#   6) Run unconfigDataCollector, passing the input.properties files.
#
#####################################################################

'''
    Unconfigure the data collector for ITCAM/AD 7.2
'''

fixlev = "41818" # Last APAR, defect or story to update this module

import os
import sys
import platform
import zipfile
import traceback
import codecs

# This try is the last resort.  If an exception bubbles up to this level, we have
# failed and all that remains is to print debugging information.

try:

    # Eliminate need to prefix import modules with "lib."
    sys.path.append("lib")

    # Jython 2.5.2 compatibility
    _Jython_vrm = os.sys.version_info[0:3]
    assert _Jython_vrm >= (2,5,2), "Unsupported Jython version %s" % ".".join([str(_vrm) for _vrm in _Jython_vrm])

    # Java 1.5 compatibility
    _Java_vrm = [int(_vrm) for _vrm in platform.java_ver()[0].split(".")[0:2]]
    assert _Java_vrm >= [1,5], "Unsupported Java JVM version %s" % ".".join([str(_vrm) for _vrm in _Java_vrm])

    from ConfigurationException import ConfigurationException

    import createPCL_config as createPCL
    import createPDL
    import tcpipstack
    import findconfigservers
    import unconfigDC
    import platformUtil
    import wasbackup
    import display_config
    import silentInputFileProcessor
    import resourcebundle
    from resourcebundle import getmsg as getmsg
    import debugUtil
    import debugUtil as log
    from debugUtil import logDebug as debug
    import doprompt
    import checkversion
    import utils
    import configUtils
    import codecs
    from constants import constants

    from ConsoleWriter import ConsoleWriter
    from PrintWrapper import PrintWrapper
    import wlpUnconfigDC

    #############
    # Initialize
    #############

    # Instantiate the global constants class

    common_constants = constants()

    # Set the print wrapper for stdout and stderr.

    PrintWrapper(common_constants.STDOUT)
    PrintWrapper(common_constants.STDERR)

    # Initialize the resource bundle
    resourcebundle.set_resourcebundle()

    # There should be no debug() statement before this point
    debug("Entry config %s" % fixlev)

    debug("  Jython version: %s" % ".".join([str(_vrm) for _vrm in os.sys.version_info[0:3]]))
    debug("  Java version: %s" % platform.java_ver()[0])
    debug("  Java home: %s" % os.environ.get("JAVA_HOME"))

    # Get the approximate build timestamp from the install bundle zip directory
    try:
        zipFilename = None

        # Try environment
        if zipFilename is None:
            if "ITCAM_CONFIGHOME" in os.environ:
                zipFilename = os.path.normpath(os.path.join(os.environ["ITCAM_CONFIGHOME"], "bin", "lib", "installbundle.jar"))
                if not os.path.isfile(zipFilename):
                    debug("Install bundle file %s not found from ITCAM_CONFIGHOME" % zipFilename)
                    zipFilename = None

        # Try command line
        if zipFilename is None:
            if len(sys.argv) > 0 and sys.argv[0]:
                zipFilename = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), os.pardir, "bin", "lib", "installbundle.jar"))
                if not os.path.isfile(zipFilename):
                    debug("Install bundle file %s not found from sys.argv[0]" % zipFilename)
                    zipFilename = None

        # Try current working directory
        if zipFilename is None:
            zipFilename = os.path.normpath(os.path.join(os.getcwd(), "lib", "installbundle.jar"))
            if not os.path.isfile(zipFilename):
                debug("Install bundle file %s not found from os.getcwd()" % zipFilename)
                zipFilename = None

        if zipFilename is not None:
            zipObject = zipfile.ZipFile(zipFilename,"r")
            zipInfos = zipObject.infolist()
            zipObject.close()
            if len(zipInfos) > 0: debug("  Build timestamp: %04i-%02i-%02i %02i:%02i:%02i" % zipInfos[0].date_time)
        else: debug("Unable to locate installbundle.jar, build timestamp not available.")

    except Exception, errmsg:
        debug("Error encountered getting build timestamp:\n\n%s" % traceback.format_exc())
        debug("Execution continuing")

    ############################
    # Display a startup message
    ############################

    print
    print
    print "%s" % getmsg("FLOWERBOX_UNCONFIG","Unconfigure Data Collector")
    print

    #########################################
    # Create the PDL and insert data from
    # environment and command line arguments
    #########################################

    pdl = createPDL.make_pdl(createPCL.make_pcl())

    log.logDumpPCL(pdl["PCL"][0], "Initial creation of PCL with %d entries" % len(pdl["PCL"][0]))
    log.logDumpPDL(pdl, "Initial creation of PDL with %d entries" % len(pdl), short=False)

    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    pdl["UNCONFIG"][0:2] = True, "unconfig"

    # Perhaps all we are going to do is display the help text.
    if pdl["HELP"][0]:
        log.set_logging(False)
        print "%s" % getmsg("HELP_UNCONFIG")
        print
        sys.exit(0)

    log.logDumpPDL(pdl, "PDL after environment variables and command line arguments")

    # Default log and message file names
    if pdl["LOG"][0] is None:
        pdl["LOG"][0:2] = "unconfig-trace.log", "unconfig"
    if pdl["MESSAGE"][0] is None:
        pdl["MESSAGE"][0:2] = "unconfig-message.log", "unconfig"

    # Default the config home.
    if pdl["CONFIGHOME"][0] is None:
        if len(sys.argv) > 0 and sys.argv[0]:
            cwd = os.path.dirname(os.path.abspath(sys.argv[0]))
            debug("Current working directory based on sys.arg[0]: %s" % cwd)
        else:
            cwd = os.path.abspath(os.getcwd())
            debug("Current working directory based on os.getcwd(): %s" % cwd)
        if os.path.basename(cwd) == "bin":
            config_home = os.path.join(cwd, os.pardir)
            debug("Configuration home default: %s" % config_home)
            pdl["CONFIGHOME"][0:2] = config_home, "config"
        else:
            raise ConfigurationException(getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home"))

    # Normalize the configuration home
    config_home = os.path.normpath(pdl["CONFIGHOME"][0])
    if config_home != pdl["CONFIGHOME"][0]:
        debug("Configuration home normalized to: %s" % config_home)
        pdl["CONFIGHOME"][0:2] = config_home, "config"


    # Initialize the log utility and interrogate version.properties.
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
    if log.isLogBad():
        print "%s" % getmsg("CFG5027W", "Unable to open log file {0}, diagnostic output will be directed to the console.", [pdl["LOG"][0]])
    else:
        # Redirect sys.stdout
        if pdl["LOGCONSOLE"][0] is True:
            log_file = debugUtil.getLogFilename()
            log_dir = os.path.dirname(log_file)
            console_wtr = ConsoleWriter(log_dir + os.sep + "unconfig-console.log" )
            # Start with flower box printed above
            console_wtr.log("%s" % getmsg("FLOWERBOX_UNCONFIG","Unconfigure Data Collector"))
    configUtils.logVersionProperties(pdl)

    print "%s" % getmsg("CFG5043I", "Log file name: {0}", [log.getLogFilename()])

    ###################
    # Identify servers
    ###################

    # Get the current version of this release, and save it.

    version = checkversion.get_current_version(pdl)
    assert version != "-1", getmsg("CFG1070E", "Invalid version number  - {1}", [version])
    utils.setData(pdl, "VERSION", version, "unconfig")


    ########################
    # Get Silent Properties
    ########################

    try:
        if pdl["SILENT"][0]:
            debug("Silent unconfiguration file is supplied: %s" % pdl["SILENT"][0])
            inputproperties_list = silentInputFileProcessor.processFile(pdl,pdl["SILENT"][0],3)
            server_found = findconfigservers.find_configservers(pdl, common_constants.UNCONFIG, inputproperties_list)
        else:
            server_found = findconfigservers.find_configservers(pdl, common_constants.UNCONFIG)

    except UserWarning:
        _exctype, _excval, _exctb = sys.exc_info()
        print _excval
        retry = doprompt.prompt_retry(pdl)
        if not retry:
            raise KeyboardInterrupt("No retry in unconfig")
    except:
        raise

    # Dump the contents of the PDL.
    log.logDumpPDL(pdl, "PDL for unconfiguration")

    # Call unconfigDataCollector
    utils.setData(pdl, "RETURNCODE", 0, "unconfig")
    if server_found == True:
        #Prompt for Websphere backup and backup current configuration if requested
        wasbackup.do_backup(pdl)
        unconfigDC.unconfigDataCollector(pdl)


######################################################
# This is the last-ditch attempt to gather diagnostic
# information for unexpected failures.
######################################################

except ConfigurationException, (instance):

    print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["unconfig.py"])
    print "%s" % instance.parameter

    # In case logging was never started, we need to dump the log buffer.
    log.logException("Configuration error in config.py")
    log.logDumpStack("Stack data for diagnostic purposes")
    sys.exit(1)

except AssertionError:

    print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["unconfig.py"])
    try:
        log.logException("Assertion error in unconfig.py")
        log.logDumpStack("Stack data for diagnostic purposes")
    except:
        _type, _val, _tb = sys.exc_info()
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
        traceback.print_exception(_type,_val,_tb)
    sys.exit(1)

except KeyboardInterrupt:   #User entered terminate character in prompt

    print "%s" % getmsg("CFG1088I", "Exiting...")
    sys.exit(0)

except SystemExit:   # Generic exit (after an exception was already caught and handled)

    print "%s" % getmsg("CFG1088I", "Exiting...")
    sys.exit(1)

except:

    # In unlikely case an exception happened before imports were
    # completed, we need to do some minimal imports to insure
    # we can put out diagnostic information.

    import debugUtil as log
    from resourcebundle import getmsg as getmsg

    _exctype, _excval, _exctb = sys.exc_info()

    print
    try:
        print "%s" % getmsg("CFG1002E", "Unexpected exception detected in config.py")
    except Exception:
        print "Unexpected exception detected in config.py"
    print

    try:
        log.logException("Error in main routine")
        log.logDumpStack("Stack data for diagnostic purposes")
    except Exception:
        _type, _val, _tb = sys.exc_info()
        print
        try:
            print "%s" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "Logging routine failed, traceback follows"
        print
        traceback.print_exception(_type,_val,_tb)
        print
        print "Original traceback"
    traceback.print_exception(_exctype,_excval,_exctb)
    print

    # In case logging was never started, we need to dump the log buffer.
    log.set_logging()
    sys.exit(1)
